/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#if !defined(AFX_HTTPINJECTORSTATUS_I_H__E806AF04_2293_4D4A_AF92_2976BB529ABE__INCLUDED_)
#define AFX_HTTPINJECTORSTATUS_I_H__E806AF04_2293_4D4A_AF92_2976BB529ABE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "InjectorStatus_i.h"
#include "InjectorStatus.h"
#include "TestExecuter.h"
#include "TestManager.h"
#include "RemoteRepository.h"
#include <OSTAORB/OSTAORB.h>
#include <string>
#include "InjectorControl_i.h"

///////////////////////////////////////////////////////////////////////////////
// @class   HTTPInjectorStatusImpl | This is the implementation class for the
//          HTTPInjectorStatus CORBA interface. The Injector Status object resides
//          in the OpenSTA Daemon process on each Injector (an Injector is a
//          host on which OpenSTA has been installed and on which one or more
//          OpenSTA Executers may run). There is exactly one InjectorStatus
//          object per Injector.
//
//          The InjectorStatus object is responsible for retrieving status data
//          from all the Executers on an Injector, while a Test is running.
//          This data is requested by the Test Manager, on behalf of Commander
//          and is only required when a Test is being monitored using
//          Commander. 
//
//			HTTPInjectorStatusImpl extends InjectorStatusImpl

class HTTPInjectorStatusImpl :public virtual POA_HTTP_InjectorStatus, public InjectorStatusImpl
{

public:
	
	HTTPInjectorStatusImpl(CORBAServer* corba, InjectorControlImpl& ic);
	virtual ~HTTPInjectorStatusImpl();

    // HTTP_InjectorStatus operations.
	virtual CORBA::Boolean StartMonitoring(const char* strFileName, CORBA::Long lPace);
	virtual CORBA::Boolean StopMonitoring();
	virtual HTTP_TestExecuterSnapshot* HTTP_GetStatsSnapshot();
	virtual CORBA::Boolean HTTP_GetBothStatsSnapshots(HTTP_TestExecuterSnapshot_out HTTP_InjectorSnapshot, TESnapList_out InjectorSnapList);

protected:
	long m_lInterval;
	static void* WriteToFile(void* args);
	bool m_bStopMonitoring;
// @access Private Members.

private:
	

	HTTP_TestExecuterSnapshot* HTTP_GetStatsSnapshot2();
	CORBA::Boolean HTTP_GetBothStatsSnapshots2(HTTP_TestExecuterSnapshot_out HTTP_InjectorSnapshot, TESnapList_out InjectorSnapList);

	CTMutex   m_CacheInUseMutex;

    string m_statsFileName;
	TESnapList m_cachedInjectorSnapList;
	HTTP_TestExecuterSnapshot m_cachedHTTP_InjectorSnapshot;
	bool m_bCachedInjectorSnapList;
	bool m_bCachedHTTP_InjectorSnapshot;
	// @cmember Pointer to thread that logs stats to file
	omni_thread*  m_pStatWriter;
	omni_thread*  m_pCheckActive;
};


#endif // !defined(AFX_HTTPINJECTORSTATUS_I_H__E806AF04_2293_4D4A_AF92_2976BB529ABE__INCLUDED_)

