/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     StepByStep_ControlImpl
// @module  StepByStep_ControlImpl|implement StepByStep_Control
//
///////////////////////////////////////////////////////////////////////////////
#ifndef _StepByStep_ControlImpl_H_
#define _StepByStep_ControlImpl_H_

#include "StepByStep_Control.h"
#include "StepByStep_TestExecuter.h"
#include <string>
#include <fstream>
#include <mutex.hpp>
#include <OSTAORB/OSTAORB.h>
#include "InjectorControl_i.h"


class StepByStepControlImpl : public virtual POA_StepByStepControl,public PortableServer::RefCountServantBase
{
public:
    StepByStepControlImpl(CORBAServer* corba, InjectorControlImpl& ic);
    virtual ~StepByStepControlImpl();

    // StepByStep_Control operations:
    virtual void SetCommander(StepByStepCommander_ptr refCommander);
    virtual void SetTestExecuter(StepByStepScriptExecuter_ptr refTestExecuter);
    virtual CORBA::Boolean Next(CORBA::Long lTaskGroupID, CORBA::Long lVUId);
    virtual CORBA::Boolean IsStepByStep();
    virtual void NextRequest(const char* ScriptName, CORBA::Long lScriptLineNumber, CORBA::Long lTaskGroupID, CORBA::Long lVUId, CORBA::Long lConId, const char* strRequest);
    virtual void RequestComplete(CORBA::Long lTaskGroupID, CORBA::Long lVUId, CORBA::Long lConId, CORBA::Boolean bIsBinaryData, const char* strHeaders, const char* strBody, CORBA::Long lBodyLength);

private:

    //// Data:
    CORBAServer*                    m_corba;
    std::string                     m_registered_name;
    std::string                     m_LogFile, m_LogFileBaseName;
    InjectorControlImpl&            m_ic;
    StepByStepScriptExecuter_var    m_refTestExecuter;
    StepByStepCommander_var         m_refCommander;
    std::ofstream*                  m_pLogFile;
    CTMutex                         m_logFileMutex;

    //// Helpers:
    void WriteLogMsg(const std::string& msg);
};

#define GetLogStream()      { CTLockGuard<CTMutex> lock(m_logFileMutex); \
                              *m_pLogFile
#define ReleaseLogStream()    m_pLogFile->flush();}

#endif

