/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "CECFileVar.h"
#include "CyrException.h"
#include <Solutions.h>
#include "windows.h"

#ifndef CYRANO_TP_STANDALONE
#   include "RemoteRepository.h"
extern CRemoteRepository* Repos;
#endif

CECFileVar::CECFileVar(const string& name,int size,int options,const string& fid,const string& file)
        :CECProcessVar(options,name)
{
    string strFullFileName = file + ".fvr";
    _size=size;
    if(_options & VTINT)
        _size=32;

#ifndef CYRANO_TP_STANDALONE
    if(!Repos->CheckoutVariableFile(strFullFileName))
        throw CyrException("CECFileVar::CECFileVar","Unable to retrieve file '"
            + strFullFileName + "' for file variable " + get_name(),-1);
    _filename =  getenv("OPENSTA_HOME");
    _filename += "Engines\\Temp\\" + strFullFileName;
#else
    // Get Repository path.
    HKEY hKey;
    DWORD dwKeyValLen;
    unsigned char* szRepositoryPath = NULL;
    RegOpenKeyEx( HKEY_LOCAL_MACHINE, CURRENT_PROJECT_KEY, 0, KEY_QUERY_VALUE, &hKey );
    RegQueryValueEx(hKey, "Path", NULL, NULL, NULL, &dwKeyValLen);
    szRepositoryPath = new unsigned char[dwKeyValLen];
    RegQueryValueEx(hKey, "Path", NULL, NULL, szRepositoryPath, &dwKeyValLen);
    RegCloseKey(hKey);
    std::string strRepositoryPath ((char *) szRepositoryPath);
    delete [] szRepositoryPath;
	_filename = strRepositoryPath + "Repository\\Data\\" + strFullFileName;
#endif
    _ifile=new ifstream();
    _ifile->open(_filename.c_str());
    if(!_ifile->good())
    {
        delete _ifile;
        throw CyrException("CECFileVar::CECFileVar","Unable to open file '"
            + strFullFileName + "' for file variable " + get_name(),-1);
    }
}

CECFileVar::~CECFileVar()
{
    _ifile->close();
    delete _ifile;
}
void CECFileVar::next_value()
{
    char* buf=new char [_size];
    _ifile->getline(buf,_size);
    _cur_value=buf;
    delete buf;
    if(_ifile->eof())
        reset_seed();//if we've got to the end go to the beginning
}

void CECFileVar::reset_seed(unsigned long seed)
{
    //start from beginning of file
#ifndef HPUX
    if(_ifile->is_open())
#endif
    {
        _ifile->close();
        delete _ifile;
        _ifile=new ifstream();
    }
    _ifile->clear();
    _ifile->open(_filename.c_str());
    if(!_ifile->good())
    {
        if(_ifile->eof())
			throw CyrException("CECFileVar::reset_seed","Already at end of file" + _filename,-1);
        throw CyrException("CECFileVar::reset_seed","Unable to open file for file variable " + _filename,-1);


    }
}
