/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "FileTransferNode_i.h"
#include <ctime.hpp>
#include <CECInetInfo.h>
#include "../Factory/Factory.h"
#include "CyrWrapLib.h"
#include "..\..\XMLRelay\xmlrpc\xmlrpc.h"
#include <fcntl.h>
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>

const long BYTESEQ_SIZE = 1024;
#include "..\..\XMLRelay\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;


FileTransferNodeImpl::FileTransferNodeImpl(CORBAServer* corba)
{
	m_nFileDesc = -1;
	m_bFileOpen = false;
	m_nSize = -1;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "Registering FileTransferNode interface");
    m_corba=corba;
	m_RelayMap = XMLRelayMap::_nil();

    CECInetInfo inet;
    m_registered_name=inet.GetIPAddressAsString();
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),FileTransferNode::CLASS_NAME,m_registered_name.c_str());

}

FileTransferNodeImpl::~FileTransferNodeImpl()
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "DeRegistering FileTransferNode interface");
    m_corba->UnNameObject( FileTransferNode::CLASS_NAME,m_registered_name.c_str());
}

CORBA::Boolean FileTransferNodeImpl::GetRootFile(const char* strReposHost, const char* strFile)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "GetRootFile: "+ string(strFile));
    CECInetInfo inet;
    string underscore_ip_name=inet.GetIPAddressAsString(strReposHost);
	if (IsReposOnLAN(strReposHost))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetRootFile ON LAN ");
		CECOrbServer orbServer(m_corba, underscore_ip_name.c_str(), "", "");
		CECOrbProject orbProject(&orbServer, "Repository");
		CECOrbFolder orbRootFolder(&orbProject);
	    CECOrbFile trcSetFile(&orbRootFolder, strFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\Temp\\";
#else
		strDestPath += "Engines/Temp/";
#endif
		strDestPath += strFile;

		trcSetFile.CheckOut(strDestPath.c_str(), false);

		return true;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetRootFile ON WEB");
		string strURL = GetRemoteFileURL(strReposHost, REPOS_ROOT, "TraceSettings.txt");
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetRootFile: URL="+strURL);
		if (strURL == "")
			return false;

		return TransferFileToLocal(strURL, "TraceSettings.txt");

	}

	return false;
}

CORBA::Boolean FileTransferNodeImpl::GetObjectCodeFile(const char* strReposHost, const char* strFile)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "GetObjectCodeFile: "+string(strFile));
    CECInetInfo inet;
    string underscore_ip_name=inet.GetIPAddressAsString(strReposHost);
	if (IsReposOnLAN(strReposHost))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetObjectCodeFile ON LAN");
		CECOrbServer orbServer(m_corba, underscore_ip_name.c_str(), "", "");
		CECOrbProject orbProject(&orbServer, "Repository");
		CECOrbFolder orbRootFolder(&orbProject);
		CECOrbFolder orbCodeFolder(&orbRootFolder, "ObjectCode");
	    CECOrbFile objFile(&orbCodeFolder, strFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\Temp\\";
#else
		strDestPath += "Engines/Temp/";
#endif
		strDestPath += strFile;

		objFile.CheckOut(strDestPath.c_str(), false);

		return true;
	}
	else
	{

		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetObjectCodeFile ON WEB");
		string strURL = GetRemoteFileURL(strReposHost, REPOS_SCRIPT, strFile);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetObjectCodeFile: URL="+strURL);
		if (strURL == "")
			return false;

		return TransferFileToLocal(strURL, strFile);
	}

	return false;
}

CORBA::Boolean FileTransferNodeImpl::GetCollectorFile(const char* strReposHost, const char* strFile)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "GetCollectorFile: "+string(strFile));
    CECInetInfo inet;
    string underscore_ip_name=inet.GetIPAddressAsString(strReposHost);
	if (IsReposOnLAN(strReposHost))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetCollectorFile ON LAN");
		CECOrbServer orbServer(m_corba, underscore_ip_name.c_str(), "", "");
		CECOrbProject orbProject(&orbServer, "Repository");
		CECOrbFolder orbRootFolder(&orbProject);
		CECOrbFolder orbProfileFolder(&orbRootFolder, "Profiles");
	    CECOrbFile profileFile(&orbProfileFolder, strFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\Temp\\";
#else
		strDestPath += "Engines/Temp/";
#endif
		strDestPath += strFile;

		profileFile.CheckOut(strDestPath.c_str(), false);

		return true;
	}
	else
	{

		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetCollectorFile ON WEB");
		string strURL = GetRemoteFileURL(strReposHost, REPOS_COLLECTOR, strFile);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetCollectorFile: URL="+strURL);
		if (strURL == "")
			return false;

		return TransferFileToLocal(strURL, strFile);
	}

	return false;
}

CORBA::Boolean FileTransferNodeImpl::GetDataFile(const char* strReposHost, const char* strFile)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "GetDataFile: "+string(strFile));
    CECInetInfo inet;
    string underscore_ip_name=inet.GetIPAddressAsString(strReposHost);
	if (IsReposOnLAN(strReposHost))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetDataFile ON LAN");
		CECOrbServer orbServer(m_corba, underscore_ip_name.c_str(), "", "");
		CECOrbProject orbProject(&orbServer, "Repository");
		CECOrbFolder orbRootFolder(&orbProject);
		CECOrbFolder orbDataFolder(&orbRootFolder, "Data");
	    CECOrbFile dataFile(&orbDataFolder, strFile);
		string strDestPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
		strDestPath += "Engines\\Temp\\";
#else
		strDestPath += "Engines/Temp/";
#endif
		strDestPath += strFile;

		dataFile.CheckOut(strDestPath.c_str(), false);

		return true;
	}
	else
	{

		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetDataFile ON WEB");
		string strURL = GetRemoteFileURL(strReposHost, REPOS_DATA, strFile);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetDataFile: URL="+strURL);
		if (strURL == "")
			return false;

		return TransferFileToLocal(strURL, strFile);
	}

	return false;
}

bool FileTransferNodeImpl::IsReposOnLAN(string strReposHost)
{
    CECInetInfo info;
    string underscore_ip_name=info.GetIPAddressAsString(strReposHost);
	
    string factory=underscore_ip_name+".CyranoFactory";
    string dataservername=underscore_ip_name+"_CyrStProvider_001";

	// Attempt to connect to factory on the repository host machine
    try
    {
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(Provider::CLASS_NAME,dataservername.c_str());
		if (!CORBA::is_nil(obj))
			return true;

    }
    catch(...)
    {
    }

	return false;
}

string FileTransferNodeImpl::GetRemoteFileURL(string strReposHost, int nType, string strFile)
{
	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "GetRemoteFileURL: "+strFile);
		if (CORBA::is_nil(m_RelayMap))
				Connect();

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strReposHost.c_str(), def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "No entry in relay map for "+strReposHost);
			return "";
		}
		g_Logger->WriteLogMsg(TRCLEV_LOW, "Relay = "+string(def.strHost));

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass),def.bUseSSL);
		CXMLRequest* req = new CXMLRequest();

		switch (nType)
		{
			case REPOS_ROOT:
			{
				req->SetMethodName("FILECONTROL.CheckOutReposTraceSettings");
				req->AddParameter(new CXMLString(strReposHost));
				g_Logger->WriteLogMsg(TRCLEV_LOW, "Create FILECONTROL.CheckOutReposTraceSettings request");
				break;
			}
			case REPOS_COLLECTOR:
			{
				req->SetMethodName("FILECONTROL.CheckOutReposProfile");
				req->AddParameter(new CXMLString(strReposHost));
				req->AddParameter(new CXMLString(strFile));
				g_Logger->WriteLogMsg(TRCLEV_LOW, "Create FILECONTROL.CheckOutReposProfile request");
				break;
			}
			case REPOS_SCRIPT:
			{
				req->SetMethodName("FILECONTROL.CheckOutReposScript");
				req->AddParameter(new CXMLString(strReposHost));
				req->AddParameter(new CXMLString(strFile));
				break;
			}
			case REPOS_DATA:
			{
				req->SetMethodName("FILECONTROL.CheckOutReposData");
				req->AddParameter(new CXMLString(strReposHost));
				req->AddParameter(new CXMLString(strFile));
				break;
			}
		}
		req->SetTransport(transport);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "Send request");
		CXMLResponse* resp = NULL;
		try
		{
			resp = req->SendRequest();
		}
		catch(...)
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "EXCEPTION");
			return "";
		}
		g_Logger->WriteLogMsg(TRCLEV_LOW, "Got a response!!");
		if (!resp || resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, resp->PrintError());
			delete req;
			delete resp;
			return false;
		}
	

		string strURL = "";
		if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "base64")
		{
			CXMLBase64* pVal = (CXMLBase64*)resp->m_params.GetAt(0);
			strURL = pVal->GetValue();
		}
	
		g_Logger->WriteLogMsg(TRCLEV_LOW, "URL = "+strURL);

		delete req;
		delete resp;

		return strURL;
	}
	catch(...)
	{
	}
	return "";
}

bool FileTransferNodeImpl::TransferFileToLocal(string strURL, string strFile)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: "+strURL);
	if (strURL.substr(0,3) == "ftp")
	{
		string strServer, strUser, strPass, strRemotePath;
		int nPort;

		if(ParseFTPURL(strURL, strServer, nPort, strUser, strPass, strRemotePath))
		{
			string strLocal = getenv("OPENSTA_HOME");
			strLocal += "Engines\\Temp\\"+string(strFile);

			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Open FTP connection");
			CFTPFileTransfer ftp;
			ftp.OpenConnection(strServer, nPort, strUser, strPass);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Get file by FTP");
			return ftp.GetFile(strRemotePath, strLocal);
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Parse FTP URL failed");
		}
	}
	else if (strURL.substr(0,4) == "http")
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: Use HTTP");
		CHTTPTransport t(strURL);

		string strLocal = getenv("OPENSTA_HOME");
		strLocal += "Engines\\Temp\\"+string(strFile);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: CALL GetRemoteFile");
		bool bRet = t.GetRemoteFile(strLocal);
		if (bRet)
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: TRANSFERRED");
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "TransferFileToLocal: FAILED");
		}

		return bRet;
	}

	return false;
}

CORBA::Boolean FileTransferNodeImpl::OpenResultsFile(const char* strFileName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "OpenResultsFile: "+string(strFileName));

	if (m_bFileOpen)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "OpenResultsFile: File Already Open");
		return false;
	}

	string strPath = getenv("OPENSTA_HOME");
	strPath += "Engines\\Temp\\"+string(strFileName);

    //Determine if file exists and check size
    struct stat st;
	g_Logger->WriteLogMsg(TRCLEV_LOW, "OpenResultsFile: Check File Exists");
    int result=::stat(strPath.c_str(),&st);
    // Determine if file exists
    if ( result != 0)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "OpenResultsFile: File Does Not Exist!");
        return false;
	}
	g_Logger->WriteLogMsg(TRCLEV_LOW, "OpenResultsFile: It Exists!");

    // get size of file
    m_nSize = st.st_size;

    // Open the file
	g_Logger->WriteLogMsg(TRCLEV_LOW, "OpenResultsFile: Open The File");
#ifdef WIN32
    m_nFileDesc = open( strPath.c_str(), _O_BINARY|_O_RDONLY);
#else
    m_nFileDesc = open( strPath, O_RDONLY);
#endif
    if (m_nFileDesc==-1)
        return false;

	m_bFileOpen = true;
	return true;
}

ByteSequence* FileTransferNodeImpl::ReadBytes()
{
    // Determine size of file to read - limiting it to the maximum we
    // have specified.  If we have read to the end of the file then we
    // return an empy sequence so that the iteration can be stopped.
    int count = m_nSize - tell(m_nFileDesc);
    if ( count == 0 )
    {
        // Create and return empty sequence
        ByteSequence_var bySeq = new ByteSequence();
        return bySeq._retn();
    }

    if ( count > BYTESEQ_SIZE )
        count = BYTESEQ_SIZE;

    // Allocate new buffer for bytes
    CORBA::Octet* buf = ByteSequence::allocbuf( count );

    // Read characters from file
    read(m_nFileDesc, buf, count );

    // Create sequence of bytes for transmission
    ByteSequence_var bySeq = new ByteSequence( count, count, buf, 1 );

    // Return reference and inform client that it has responsiblity for
    // deallocating memory.  Code copied from Henning and Vinoski pg 423.
    return bySeq._retn();
}

CORBA::Boolean FileTransferNodeImpl::CloseResultsFile()
{
	if (m_bFileOpen)
	{
	    close(m_nFileDesc);
		m_bFileOpen = false;
		m_nSize = -1;
		m_nFileDesc = -1;
		return true;
	}

	return false;
}

CORBA::Boolean FileTransferNodeImpl::StoreResultsFile(const char* strReposHost, const char* strTest, const char* strTestRun, const char* strFileName)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "StoreResultsFile: "+string(strFileName));

    CECInetInfo inet;
    string underscore_ip_name=inet.GetIPAddressAsString(strReposHost);
	if (IsReposOnLAN(strReposHost))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "StoreResultsFile: ON LAN");
		bool bRet = false;
		try
		{
			// Construct path of local results file
			string strLocalPath = getenv("OPENSTA_HOME");
#ifdef _WIN32
			strLocalPath  += "Engines\\Temp\\";
#else
			strLocalPath  += "Engines/Temp/";
#endif
			strLocalPath += strFileName;

			CECOrbServer orbServer(m_corba, underscore_ip_name.c_str(), "", "");
			CECOrbProject orbProject(&orbServer, "Repository");
			CECOrbFolder orbRootFolder(&orbProject);
			CECOrbFolder orbTestsFolder(&orbRootFolder, "Tests");
			CECOrbFolder orbTestFolder(&orbTestsFolder, strTest);
			CECOrbFolder orbTestRunFolder(&orbTestFolder, strTestRun);
	
			// Check in file to the test run folder
			CECOrbFile resFile(&orbTestRunFolder, strFileName, "desc");
			resFile.CheckIn(strLocalPath.c_str(), false);
#ifdef _WIN32
			::DeleteFile( strLocalPath.c_str() );
#else
			unlink(strLocalPath.c_str() );
#endif
			bRet = true;
		}
		catch(...)
		{
		}

		return bRet;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "StoreResultsFile: ON WEB");
		RelayDef def;	
		if (!m_RelayMap->GetLocalRelayDef(def))
			return false;
		g_Logger->WriteLogMsg(TRCLEV_LOW, string("StoreResultsFile: LocalRelay="+def.strHost).c_str());

		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass),def.bUseSSL);
		CXMLRequest* req = new CXMLRequest();
		req->SetMethodName("FILECONTROL.StoreResultsFile");
		req->AddParameter(new CXMLString(strReposHost));
		req->AddParameter(new CXMLString(strFileName));
		req->AddParameter(new CXMLString(strTest));
		req->AddParameter(new CXMLString(strTestRun));
		req->AddParameter(new CXMLString(inet.GetHostName()));
		req->SetTransport(transport);
		CXMLResponse* resp = req->SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "StoreResultsFile: "+resp->PrintError());
			delete req;
			delete resp;
			return false;
		}

		if (resp->m_params.GetSize() == 1)
		{
			CXMLBool* pVal = (CXMLBool*)resp->m_params.GetAt(0);
			return pVal->GetValue();
		}

	}

	return false;
}

void FileTransferNodeImpl::Connect()
{
	try
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, "WebRelay");
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}
	catch(...)
	{
	}
}

bool FileTransferNodeImpl::ParseFTPURL(string strURL, string& strServer, int& nPort, string& strUser,
				string& strPass, string& strRemotePath)
{
	// FTP URL in the form ftp://user:pass@server:port/path

	string strTmp = strURL;

	// Remove ftp://
	if (strURL.substr(0,6) != "ftp://")
		return false;	
	strTmp.erase(0,6);


	// search for '@' and parse username/password
	int nPos = strTmp.find_first_of('@');
	if (nPos > 0)
	{
		int nColonPos = strTmp.find_first_of(':');
		if (nColonPos <= 0)
			return false;

		strUser = strTmp.substr(0,nColonPos);
		strPass = strTmp.substr(nColonPos+1, nPos-nColonPos-1);

		strTmp.erase(0, nPos+1);
	}

	// Next find the server address and port
	nPos = strTmp.find_first_of('/');
	if (nPos <= 0)
		return false;

	int nColonPos = strTmp.find_first_of(':');
	if (nColonPos > 0)
	{
		strServer = strTmp.substr(0, nColonPos);
		nPort = atoi(strTmp.substr(nColonPos+1, nPos).c_str());
	}
	else
	{
		strServer = strTmp.substr(0,nPos);
		nPort = 21;
	}
	strTmp.erase(0,nPos);

	// The rest is the remote path.
	strRemotePath = strTmp;

	return true;
}
