/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrFFApi
// @module  Folder Implementation | This module contains the implementation of
//          the CECFFFolder class.  This class is simply a C++ wrapper around the
//          OpenSTARep COM interface IStFolder.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "CECFFFolder.h"
#include "CECFFFile.h"
#include <Repository/CyrStHelper.inl>
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <direct.h>
#pragma warning (disable:4786)
#pragma warning (disable:4503)
///////////////////////////////////////////////////////////////////////////////
//*KH 7/2/2000 ST Folder deleteion problem workaraoud //
#define CYR_DELETED             "CYR_DELETED"

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor creates a Folder item based on the root Folder of
//          a Project.  The name of the Folder is the same as the Project.
//          From this Folder it is possible to retrieve all further Folders.
// @parm    A pointer to a project object.
CECFFFolder::CECFFFolder( CECFFProject* project )
{
    const char* where = "CECFFFolder::CECFFFolder";
    const char* whatA = "Instantiating IProject";
    long status = CYR_FOLDER_NOT_FOUND;
    m_project = project;
    m_parent = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor attempts to retrieve a Folder object from a parent
//          Folder.  All child Folders of the parent are searched and their
//          names are compared with the name requested.
// @parm    A pointer to the parent folder object.
// @parm    The name of the Folder to retrieve.
CECFFFolder::CECFFFolder( CECFFFolder* parent, const char* name )
{
    // Error information
    const char* where = "CECFFFolder::CECFFFolder";
    long status = CYR_FOLDER_NOT_FOUND;

    // Try to retrieve folder
    if ( RetrieveFolder( parent, name ) == false )
        throw CyrRepositoryException( "Cannot locate Folder", where, status );

    // Create physical folder on disk
    string fullPath( GetRootDirectory().c_str() );
    fullPath += parent->GetFolderHierarchy();
    fullPath += name;
    _mkdir( fullPath.c_str() );
    m_name=name;
}

bool CECFFFolder::RetrieveFolder( CECFFFolder* parent, const char* name )
{
    // Loop through all folders until we find the one we require
    m_project=NULL;
    m_parent = parent;
    m_name=name;
    return true;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor creates a new child folder beneath a specified
//          parent.  This method also physically creates the Folder if one does
//          not already exist.
// @parm    A pointer to parent Folder object.
// @parm    The requested name of the new folder.
// @parm    The description for this new folder.
CECFFFolder::CECFFFolder( CECFFFolder* parent, const char* name, const char* desc )
{
    const char* where = "CECFFFolder::CECFFFolder";
    m_parent = parent;
    m_name=name;
    m_project = NULL;
    m_desc=desc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor destroys the underlying COM interface as part of its
//          tidying up.
CECFFFolder::~CECFFFolder()
{}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns a list of child Folders directly beneath this
//          Folder.  The list will be separated by '\|' characters and
//          terminated by two '\|' characters.
// @rdesc   The separated list of folder names.
// @parm    Will contain the number of Folders.

string CECFFFolder::GetFolders( long& count ) const
{
    return "";
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc  This method returns a list of Files contained by this Folder.  The
//         list will be '\|' separated and terminated by two '\|' characters.
// @rdesc  The separated list of file names.
// @parm   Will contain the number of Files within this Folder.
string CECFFFolder::GetFiles( long& count ) const
{
    string str( "" );
    return str;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the name of the Folder as recognised by
//          OpenSTARep .  It should be the same name as the one passed into the
//          constructor and therefore should not be needed.  However, it has
//          been added here for completeness.
// @rdesc   The name of the Folder.
string CECFFFolder::GetName() const
{
    return m_name;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the current description for this Folder.
// @rdesc   The current Folder description.
string CECFFFolder::GetDescription() const
{
    return m_desc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method sets a new description for this Folder.
// @parm    The new description.
void CECFFFolder::SetDescription( const char* desc )
{
    m_desc=desc;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the full path name for this Folder.
// @parm    The full path name of the folder.
string CECFFFolder::GetFullPath() const
{
    string fullpath = GetRootDirectory().c_str();
    fullpath += GetFolderHierarchy();
    string path( fullpath );
    return path;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Unless this method is called, all changes to the Folder object will
//          be discarded.  This method persists the changes into the OpenSTARep
//          database.
void CECFFFolder::Update()
{}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method removes the Folder entry from the OpenSTARep database.
//          Note that this action is irreversible and any checking or
//          confirmations should be done by the client before this method is
//          invoked.

void CECFFFolder::Remove()
{}


void CECFFFolder::Refresh()
{}


CECFFServer* CECFFFolder::GetParentServer()
{
    if ( m_project != NULL )
    {
        return m_project->GetParentServer();
    }
    else
    {
        return GetParentFolder()->GetParentProject()->GetParentServer();
    }
}

CECFFProject* CECFFFolder::GetParentProject()
{
    if ( m_project != NULL )
        return m_project;
    else
        return GetParentFolder()->GetParentProject();
}

CECFFFolder* CECFFFolder::GetParentFolder()
{
    return m_parent;
}

string CECFFFolder::GetFolderHierarchy()const
{
    string path("");
    if ( m_project != NULL )
    {
        path+=m_project->GetName()+"\\";
    }
    if(m_parent)path+=m_parent->GetFolderHierarchy();
    path+=m_name;
    if(m_parent)path+="\\";
    return path;
}
