/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrFFApi
// @module  CECFFResultsFlusher Implementation | This module contains the implementation
//          of the CECFFResultsFlusher class
//
///////////////////////////////////////////////////////////////////////////////
#ifdef _WIN32
#   pragma warning (disable : 4503)
#endif

#include "CECFFResultsFlusher.h"
#include "CECFFTestRun.h"
#include "CECFFFile.h"
#include <Repository/CyrStHelper.inl>
#include <iostream>
#include <mutex.hpp>
extern CTMutex OutListMutex;

void* CECFFResultsFlusher_Run(void* args)
{
    CECFFResultsFlusher* TRF=reinterpret_cast<CECFFResultsFlusher*>(args);

	while(TRF->IsRunning())
	{
	    for ( OutTbl::iterator i = TRF->m_pOutput->begin(); i != TRF->m_pOutput->end(); i++ )
		{
			TRF->flushList(i->first, i->second);			
		}
		::Sleep(1000);
	}
    return NULL;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Standard Constructor
// @parm   The current Test Run Folder.
CECFFResultsFlusher::CECFFResultsFlusher(string testRunPath, OutTbl* pOutput)
{
	m_pOutput = pOutput;
	m_bRunning = false;
	m_strTestRunPath = testRunPath;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Destructor - Perform any cleanup here
CECFFResultsFlusher::~CECFFResultsFlusher()
{
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method saves the entire contents of the output list to
//          several files within the Test Output folder.  Each string is saved
//          one per line in a standard ASCII file.
//          Currently this purge procedure does not affect the OTA headers
//          file which remains in memory. The reason for this is to ensure
//          that TestCommander (or any similar monitoring tool may dynamically
//          know what stats are being collected.
// @parm    The filename of the current set of output.
// @parm    The list of output strings.
void CECFFResultsFlusher::flushList(string key, OutL* outList)
{
    CTLockGuard<CTMutex> lock(OutListMutex);

    // As a sanity check we ensure that we actually do have something in the list
    if ( outList->empty() )
    {
        return;
    }

    // Get correct filename for file
    string strFullpath = m_strTestRunPath+"\\"+ key;

    // Open file for output and write contents to file
    std::ofstream* pFile = new std::ofstream( strFullpath.c_str(), std::ios::binary|std::ios::app|std::ios::out);
    if( pFile->good() )
    {
        // And loop round for the defined chunk of data we wish to flush
        while (!outList->empty())
        {
            string strList = *(outList->begin());
            outList->pop_front();

            // And write to the file
            pFile->write( strList.c_str(), strList.length() );
            pFile->write( "\r\n", 2 );
        }

        // Delete file - flushing contents to disk
        delete pFile;
    }
}

void CECFFResultsFlusher::Start()
{
    try
    {
        // Create Flusher thread and start it running.
        flusher = new omni_thread(&CECFFResultsFlusher_Run ,this);
		m_bRunning = true;
        flusher->start();

    }
    catch( ... )
    {}
	
}

void CECFFResultsFlusher::Stop()
{
	m_bRunning = false;

	void* retval = NULL;
	flusher->join(&retval);	
}

BOOL CECFFResultsFlusher::IsRunning()
{
	return m_bRunning;
}