/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStProvider
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStProv_h
#define _CyrStProv_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Factory/Provider_i.h>
#include <CyrffApi/CyrStLib.h>

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <functional>
#include <map>
using std::map;

///////////////////////////////////////////////////////////////////////////////
// @type BeanArray | This list contains a collection of COM wrapper objects.
typedef map<string,CECFFBean*,std::less<string> >  BeanArray;

///////////////////////////////////////////////////////////////////////////////
// @class   CyrStProvider | This Provider is responsible for creating OpenSTARep
//          objects.  This class relies on COM and should only be instantiated
//          on a compliant Windows platform.
// @base    public | ProviderImpl
class CyrStProvider : public ProviderImpl
{
    // @access Public Members
public:
    // @cmember Constructor
    CyrStProvider( CORBAServer* pServer, const char* name, const char* owner );
    virtual void Shutdown();

    // @cmember Create a new instance of a class
    CORBA::Object_ptr New( ParamList_ptr params );

    // @cmember Store COM object in array
    string   StorePtr( CECFFBean* bean, const char* name, const char* user );
    CECFFBean* GetPtr( string key );    // @cmember Get COM wrapper from array
    void        RemovePtr( string key ); // @cmember Remove COM wrapper from array

    // @cmember,mfunc Returns a pointer to a CORBA helper class.
    CORBAServer* GetHelper()
    {
        return m_pServer;
    }

    // @access Private Members
private:
    BeanArray*  m_beans;
};

#endif
