/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __CYRTOOLS_BINARY__
#define __CYRTOOLS_BINARY__

//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __MTSAFE__
#  include "mtsafe.hpp"
#endif  // __MTSAFE__

#include <iostream>
#include <fstream>


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CTBinary //
///////////////////////////////////////////////////////////////////////////////
class CTBinary : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                             // Constructor(s) and destructor.
    CTBinary ()
: pBuffer_ (NULL), iLen_ (0)
    { }

    CTBinary (const CTBinary& binary)
    {
        copy (binary) ;
    }
    CTBinary (void* pBuffer, int iLen) ;
    ~CTBinary ()
    {
        reset () ;
    }

    //-----------------------------------------------------------------------------
public:                             // Control methods.
    friend CTBinary  operator+ (const CTBinary& binary1, const CTBinary& binary2) ;
    CTBinary&                       operator+= (const CTBinary& binary)
    {
        append (binary) ;
        return *this ;
    }
    CTBinary&                       operator= (const CTBinary& binary)
    {
        reset () ;
        copy (binary) ;
        return *this ;
    }
    char&                           operator[] (const size_t i)
    {
        return ( (char*) pBuffer_)[i] ;
    }
    void                            reset (void) ;

private:
    CTBinary&                       append (const CTBinary& binary) ;
    CTBinary&                       copy (const CTBinary& binary) ;

    //-----------------------------------------------------------------------------
public:                             // Get* methods.
    void*                           data (void) const
    {
        return pBuffer_ ;
    }
    int                             length (void) const
    {
        return iLen_ ;
    }
    int                             index (const CTBinary& binary, int iStart = 0)
    {
        return index (binary.pBuffer_, binary.iLen_, iStart) ;
    }
    int                             index (void* pBuffer, int iLen, int iStart = 0) ;

    //-----------------------------------------------------------------------------
private:                            // Members.
    void*                           pBuffer_ ;
    int                             iLen_ ;

    //-----------------------------------------------------------------------------
}

; // CTBinary

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CTBinary //
///////////////////////////////////////////////////////////////////////////////

#endif // __CYRTOOLS_BINARY__
