/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*            **** C O M M O N ****                  */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_GET_NODENAME_WIN32                  */
/*          ============                         */
/*                                       */
/*  Description:                                 */
/*  ============                                 */
/*                                       */
/*  Routine to retrieve the current nodename on a WIN32 system.           */
/*                                       */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  WIN32-Specific Module.
 */
#ifdef WIN32

/*
 *  Standard ANSI header files.
 */
#include <string.h>
#include <windows.h>

/*
 *  Function Prototypes
 */
#include psl_common_file (psl_rtndef.h)    /* PSL Routines */

/*
 *  Structure Definitions.
 */
#include psl_common_file (shrmsg.h)     /* PSL Messages */
#include psl_common_file (pslerr_prmdef.h)  /* Error Trapping */
#include psl_common_file (psldiag_prmdef.h) /* Diagnostics Constants */


/************************************************************************/
/*                                  */
/*  PSL_GET_NODENAME - Retrieve current nodename                */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     iret = PSL_GET_NODENAME (diag_ptr, nodename_ptr, max_size)   */
/*                                  */
/*  Where:                              */
/*                                  */
/*     iret             = Unsigned Long PSL Routine Return Status   */
/*     diag_ptr         = Pointer to Diagnostics Stack      */
/*     nodename_ptr     = Pointer to nodename (returned)        */
/*     max_size         = Size of Output Buffer                 */
/*                                  */
/*  Return the current nodename.                                */
/*                                  */
/************************************************************************/

unsigned long   PSL_GET_NODENAME (struct diagstk *diag_ptr,
                                  char *nodename_ptr,
                                  unsigned long max_size)
{
    /*
     *  Local Data
     */
    unsigned long    iret;          /* Routine Return Status */
    unsigned long    start_diag;        /* Start of Diagnostics Stack */
    unsigned long    namelen;       /* Name length */


    /*
     *  Maintain the diagnostics stack.
     */
    start_diag = PSL_ADD_STATDIAG (diag_ptr, "During PSL_GET_NODENAME");

    namelen = max_size;         /* Set max length */
    if (!GetComputerName (nodename_ptr, &namelen))
        /* Get name */
    {
        (void) strcpy (nodename_ptr, "UNKNOWN");
    }
    /*
     *  Unwind the diagnostics stack.
     */
    PSL_REMOVE_DIAGNOSTICS (diag_ptr, start_diag);

    /*
     *  Return success.
     */
    iret = PSL_NORMAL;

    return iret;
}

#endif
