/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_TPARSE                  */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This module contains a modified version of the LIB$TPARSE        */
/*  routine                                                      */
/*                                       */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  Standard ANSI header files
 */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL Function Prototypes */

static long GET_STRING (struct tpaarg *argblk_ptr,
                        long type);

/*
 *  Structure definitions
 */
#include <psl_tpaargdef.h>      /* TPARSE Argument Block */
#include <psl_tpatabdef.h>      /* TPARSE Table Entry */
#include <shrmsg.h>         /* PSL Messages */

/*
 * Symbolic names for string type in GET_STRING
 */
#define STRTYP_SPACES       0
#define STRTYP_NUMERIC      1
#define STRTYP_HEXNUMERIC   2
#define STRTYP_ALPHANUMERIC 3
#define STRTYP_SYMBOL       4
#define STRTYP_QUOTED       5
#define STRTYP_DATANAME     6
#define STRTYP_EXPRESSION   7
#define STRTYP_OCTNUMERIC   8
#define STRTYP_KEYWORD      9
#define STRTYP_LONGDATANAME 10



/************************************************************************/
/*                                  */
/*  PSL_TPARSE - Parse a user-defined syntax            */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     iret = PSL_TPARSE (argblk_ptr, istate_ptr);          */
/*                                  */
/*  Where:                              */
/*                                  */
/*     argblk_ptr = Pointer to PSL_TPARSE argument block        */
/*     istate_ptr = Pointer to initial state table          */
/*                                  */
/*    On return:                            */
/*  Contents of argblk are updated, e.g. to indicate        */
/*     string not processed by the parser               */
/*  iret   = Routine status                     */
/*         = 1 if successful parse                  */
/*         = PSL_SYNTAXERR if unsuccessful parse            */
/*         = PSL_BADPARAM if state table is invalid         */
/*         = value of action routine rejecting transition       */
/*                                  */
/************************************************************************/
unsigned long PSL_TPARSE (
    struct tpaarg *argblk_ptr,  /* Pointer to argument block */
    struct tpatab *istate_ptr)  /* Pointer to initial state */
{
    /*
     *  Local Data.
     */
    struct   tpatab     *state_ptr;     /* Pointer to current state */
    long       iret;       /* Return status */
    long       action_sts;     /* Return status from action routine */
    long       spcoff;     /* Offset of last space */
    long       match_len;      /* Match length */
    char       *string_ptr;    /* String pointer */
    char       *saved_string_ptr;  /* Saved string pointer */
    long       saved_stringcnt;    /* Saved string count */
    struct   tpatab     *saved_trans_ptr;   /* Saved transition pointer */
    char       *token_ptr;     /* Token pointer */
    char       *keyword_ptr;   /* Keyword pointer */
    void       (*tracertn_ptr) (struct tpaarg *, short int);
    /* Trace routine */
    typedef  void       (*PTRACE) (struct tpaarg *, short);
    /* Type for trace routine */
    unsigned long       (*action_ptr) (struct tpaarg *);
    /* Action routine */
    unsigned long       *data_ptr;      /* Data pointer */
    char       string[256];    /* Working string */



    /*
     *  Variables for PSL_ASCBIN.
     */
    unsigned char       numchr;     /* Number of characters to convert */
    unsigned char       radix;      /* Conversion radix */
    unsigned short      chrcnt;     /* Unconverted characters */



    /*
     *    Initialize some flags
     */
    (*argblk_ptr).optionsw &= ~TPAARG_AMBIGM;
    /* Initialize Ambiguity Flag */
    action_sts = 0;         /* Initialize Action Status */
    state_ptr = istate_ptr;     /* Initialize state */
    /*
     *    MAIN LOOP:
     */
loop:
    /*
     *    Check next type code and compare it against current string
     */
    (*argblk_ptr).trans_ptrps = state_ptr;
    /* Save current transition address */
    /*
     *    If spaces/tabs are to be ignored, and this is not a PSLTPA_LAMBDA
     *    match, then skip any spaces and tabs
     */
    spcoff = 0;         /* Initialize Last Space flag */
    if ((((*argblk_ptr).optionsw  & TPAARG_BLANKSM) == 0) &&

            ((*state_ptr).token_typeb != PSLTPA_LAMBDA))
    {
        spcoff = GET_STRING (argblk_ptr, STRTYP_SPACES);
        /* Flush all spaces/tabs */
        (*argblk_ptr).stringcntl -= spcoff;
        /* Discount spaces from string length */
        (*argblk_ptr).string_ptrpc += spcoff;
        /* And add to string address */
    }
    /*
     *   Initialize pointers for parsing next token
     */
    match_len = 0;          /* Initialize character count */
    (*argblk_ptr).token_ptrpc = (*argblk_ptr).string_ptrpc;
    /* Set token pointer = string pointer */
    token_ptr = (*argblk_ptr).token_ptrpc;
    /* Set working token pointer */
    string_ptr = (*argblk_ptr).string_ptrpc;
    /* Set working string pointer */
    /*
     *   For all types except PSLTPA_LAMBDA, PSLTPA_EOS and PSLTPA_SUBXPR
     *   check to see if the string is (now) null and, if so, exit with
     *   'no match' status.
     */
    if (((*state_ptr).token_typeb != PSLTPA_LAMBDA) &&
            ((*state_ptr).token_typeb != PSLTPA_EOS) &&
            ((*state_ptr).token_typeb != PSLTPA_SUBXPR) &&
            ((*argblk_ptr).stringcntl == 0))
    {
        goto tkn_failed;        /* Exit with no match */
    }
    /*
     *   Now case according to the token type
     */
    switch ((*state_ptr).token_typeb)
    {
    case PSLTPA_CHAR:
        {
            /*
             *   PSLTPA_CHAR - single ASCII character
             */
            if ((*state_ptr).token_charb == (unsigned char) *string_ptr)
            {
                match_len = 1;      /* Set match length */
                (*argblk_ptr).charb = (*state_ptr).token_charb;
                /* Set up char for action routine */
                goto matched;       /* And check for action routine */
            }
            goto tkn_failed;        /* Exit with error */
        }
    case PSLTPA_KEYWORD:
        {
            /*
             *   PSLTPA_KEYWORD - multi-character keyword
             *
             *   A keyword token may be either:
             *
             *   1. An exact match, or
             *   2. Abbreviated to some number of characters fixed for the call, or
             *   3. Arbitrarily abbreviated (so that the first match wins), or
             *   4. Abbreviated to the minimum which is locally unambiguous.
             *
             *   Condition 4 is tested for ambiguity by finding the next entry in
             *   the keyword table and matching it against the token string.  The
             *   keyword strings for each state are padded with a filler to prevent
             *   false matches across states.
             */
            if (((*argblk_ptr).optionsw & TPAARG_AMBIGM) != 0)
            {
                goto tkn_failed;    /* If we already have ambiguity flag */
                /* set then reject this keyword. */
            }
            match_len = GET_STRING (argblk_ptr, STRTYP_KEYWORD);
            /* Get a symbol */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            keyword_ptr = (*state_ptr).keyword_ptrpc;
            /* Get keyword pointer */
            if (memcmp (token_ptr,keyword_ptr,match_len) != 0)
            {
                goto tkn_failed;    /* Error if no match */
            }
            if ((*(keyword_ptr+match_len) == 0) ||
                    /* If all keyword matched */
                    (((*argblk_ptr).mcountb != 0) &&
                     /* Or we have a match count */
                     ((*argblk_ptr).mcountb <= match_len)) ||
                    /* and we have satisfied it */
                    (((*argblk_ptr).optionsw & TPAARG_ABBRFMM) != 0))
                /* Or first match wanted */
            {
                goto matched;       /* Then success */
            }
            if (((*argblk_ptr).optionsw & TPAARG_ABBREVM) == 0)
            {
                goto tkn_failed;    /* Error if abbreviation not allowed */
            }
            keyword_ptr = (char *) memchr (token_ptr+match_len,0xff,65535) + 1;
            /* Point to next keyword */
            if (memcmp (token_ptr,keyword_ptr,match_len) != 0)
            {
                goto matched;       /* Success if no match */
            }
            (*argblk_ptr).optionsw |= TPAARG_AMBIGM;
            /* Else flag as ambiguous */
            goto tkn_failed;        /* And exit with error */
        }
    case PSLTPA_EOS:
        {
            /*
             *   PSLTPA_EOS - Match End of String
             */
            if ((*argblk_ptr).stringcntl == 0)
            {
                goto matched;       /* We are at the end of the string */
            }
            else
            {
                goto tkn_failed;    /* Else exit with error */
            }
        }
    case PSLTPA_ANY:
        {
            /*
             *   PSLTPA_ANY - Match any single character
             */
            (*argblk_ptr).charb = *string_ptr;
            /* Set up char for action routine */
            match_len = 1;      /* And set length */
            goto matched;       /* And return it */
        }
    case PSLTPA_ALPHA:
        {
            /*
             *   PSLTPA_ALPHA - Any alphabetic character
             *  'A' - 'Z', 'a' - 'z', ^XC0 - ^XCF, ^XD1 - ^XDD,
             *  ^XDF - ^XEF, ^XF1 - ^XFD
             */
            if ((isalpha(*string_ptr))  /* A-Z or a-z */
                    || (((*string_ptr) >= 0xC0) &&
                        ((*string_ptr) <= 0xCF))
                    /* C0 - CF */
                    || (((*string_ptr) >= 0xD1) &&
                        ((*string_ptr) <= 0xDD))
                    /* D1 - DD */
                    || (((*string_ptr) >= 0xDF) &&
                        ((*string_ptr) <= 0xEF))
                    /* DF - EF */
                    || (((*string_ptr) >= 0xF1) &&
                        ((*string_ptr) <= 0xFD)))
                /* F1 - FD */
            {
                (*argblk_ptr).charb = *string_ptr;
                /* Return Matching Character */
                match_len = 1;      /* And set length */
                goto matched;       /* And return it */
            }
            else
            {
                goto tkn_failed;    /* Else exit with error */
            }
        }
    case PSLTPA_DIGIT:
        {
            /*
             *   PSLTPA_DIGIT - Any Digit ('0' - '9')
             */
            if (isdigit(*string_ptr))   /* 0 - 9 */
            {
                (*argblk_ptr).charb = *string_ptr;
                /* Return Matching Character */
                match_len = 1;      /* And set length */
                goto matched;       /* And return it */
            }
            else
            {
                goto tkn_failed;    /* Else exit with error */
            }
        }
    case PSLTPA_STRING:
        {
            /*
             *   PSLTPA_STRING - Match any alphanumeric string
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_ALPHANUMERIC);
            /* Get a string */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    case PSLTPA_SYMBOL:
        {
            /*
             *   PSLTPA_SYMBOL - Match any symbol
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_SYMBOL);
            /* Get a symbol */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    case PSLTPA_BLANK:
        {
            /*
             *   PSLTPA_BLANK - Any sequence of blanks and tabs
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_SPACES);
            /* Get blanks */
            if (match_len == 0)     /* Did we find them? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    case PSLTPA_DECIMAL:
        {
            /*
             *   PSLTPA_DECIMAL - Any decimal number
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_NUMERIC);
            /* Get numeric string */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                (void) strncpy(string,token_ptr,match_len);
                /* Copy to work string */
                string[match_len] = '\0';
                /* Null-terminate */


                numchr = (unsigned char) match_len;
                /* Number of characters to convert */
                radix = 10;     /* Conversion Radix */

                iret = PSL_ASCBIN ( /* Convert string */
                           &numchr,    /* Number of characters to convert */
                           string,     /* Buffer to convert */
                           &radix,     /* Conversion Radix */
                           &(*argblk_ptr).numberl,
                           /* Converted number */
                           &chrcnt);   /* Unconverted characters */
                if (!(psl_test_status (iret))) goto tkn_failed;
                /* Error if not valid number */


                goto matched;       /* Return value */
            }
        }
    case PSLTPA_OCTAL:
        {
            /*
             *   PSLTPA_OCTAL - Any octal number
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_OCTNUMERIC);
            /* Get numeric string */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                (void) strncpy(string,token_ptr,match_len);
                /* Copy to work string */
                string[match_len] = '\0';
                /* Null-terminate */


                numchr = (unsigned char) match_len;
                /* Number of characters to convert */
                radix = 8;      /* Conversion Radix */

                iret = PSL_ASCBIN ( /* Convert string */
                           &numchr,    /* Number of characters to convert */
                           string,     /* Buffer to convert */
                           &radix,     /* Conversion Radix */
                           &(*argblk_ptr).numberl,
                           /* Converted number */
                           &chrcnt);   /* Unconverted characters */
                if (!(psl_test_status (iret))) goto tkn_failed;
                /* Error if not valid number */


                goto matched;       /* Return value */
            }
        }
    case PSLTPA_HEX:
        {
            /*
             *   PSLTPA_HEX - Any hexadecimal number
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_HEXNUMERIC);
            /* Get hex numeric string */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                (void) strncpy(string,token_ptr,match_len);
                /* Copy to work string */
                string[match_len] = '\0';
                /* Null-terminate */

                numchr = (unsigned char) match_len;
                /* Number of characters to convert */
                radix = 16;     /* Conversion Radix */

                iret = PSL_ASCBIN ( /* Convert string */
                           &numchr,    /* Number of characters to convert */
                           string,     /* Buffer to convert */
                           &radix,     /* Conversion Radix */
                           &(*argblk_ptr).numberl,
                           /* Converted number */
                           &chrcnt);   /* Unconverted characters */
                if (!(psl_test_status (iret))) goto tkn_failed;
                /* Error if not valid number */


                goto matched;       /* Return value */
            }
        }
    case PSLTPA_SUBXPR:
        {
            /*
             *   PSLTPA_SUBXPR - Subexpression
             */
            saved_stringcnt = (*argblk_ptr).stringcntl;
            /* Save current string count */
            saved_string_ptr = (*argblk_ptr).string_ptrpc;
            /* Save current string pointer */
            (*argblk_ptr).tracelvll -= 1;
            /* Decrement Trace Level */
            tracertn_ptr = (PTRACE) (*argblk_ptr).tracertn_ptrpf;
            /* Get trace routine pointer */
            if (((*argblk_ptr).tracelvll > 0)
                    /* If tracing */
                    && (tracertn_ptr != NULL)) /* And there is a Trace Routine */
            {
                (*tracertn_ptr)(argblk_ptr,5);
                /* Call the trace routine */
            }
            saved_trans_ptr = (*argblk_ptr).trans_ptrps;
            /* Save current transition address */
            action_sts = PSL_TPARSE (argblk_ptr,(*state_ptr).subexpr_ptrps);

            /* Call ourselves recursively */
            (*argblk_ptr).trans_ptrps = saved_trans_ptr;
            /* Restore current transition address */
            (*argblk_ptr).tracelvll += 1;
            /* Reset Trace Level */
            (*argblk_ptr).token_ptrpc = saved_string_ptr;
            /* Reset token pointer */
            match_len = (*argblk_ptr).string_ptrpc - saved_string_ptr;
            /* Calculate character count */
            if (psl_test_status (action_sts))
                /* If parse was successful */
            {
                (*argblk_ptr).stringcntl += match_len;
                /* Add char count to string count */
                (*argblk_ptr).string_ptrpc = saved_string_ptr;
                /* And reset string pointer */
                goto matched;       /* Exit with success */
            }
            else
            {
                (*argblk_ptr).stringcntl = saved_stringcnt;
                /* Error - reset saved string count */
                (*argblk_ptr).string_ptrpc = saved_string_ptr;
                /* And reset string pointer */
                goto tkn_failed;    /* Exit with error */
            }
        }
    case PSLTPA_LAMBDA:
        {
            /*
             *   PSLTPA_LAMBDA - always matches
             */
            goto matched;       /* Exit with success */
        }
    case PSLTPA_QUOTED:
        {
            /*
             *   PSLTPA_QUOTED - Match any quoted string
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_QUOTED);
            /* Get a quoted string */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    case PSLTPA_DATANAME:
        {
            /*
             *   PSLTPA_DATANAME - Match any dataname
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_DATANAME);
            /* Get a dataname */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    case PSLTPA_LONGDATANAME:
        {
            /*
             *   PSLTPA_LONGDATANAME - Match any long dataname
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_LONGDATANAME);
            /* Get a dataname */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    case PSLTPA_EXPRESSION:
        {
            /*
             *   PSLTPA_EXPRESSION - Match any expression
             */
            match_len = GET_STRING (argblk_ptr, STRTYP_EXPRESSION);
            /* Get an expression */
            if (match_len == 0)     /* Did we find one? */
            {
                goto tkn_failed;    /* Error if not */
            }
            else
            {
                goto matched;       /* Else return it */
            }
        }
    default:
        {
            return PSL_BADPARAM;
        }
    }

    /*
     *   The transition has failed because the token did not match -
     *   call the trace routine if one is specified and join common
     *   failure path.
     */
tkn_failed:
    tracertn_ptr = (PTRACE) (*argblk_ptr).tracertn_ptrpf;
    /* Get trace routine pointer */
    if (((*argblk_ptr).tracelvll > 0)
            /* If tracing */
            && (tracertn_ptr != NULL)) /* And there is a Trace Routine */
    {
        (*tracertn_ptr)(argblk_ptr,1);
        /* Call the trace routine */
    }
    goto failed;            /* Exit with error */


    /*
     *   We had a successful match - update string and token pointers
     */
matched:
    (*argblk_ptr).stringcntl -= match_len;
    /* Update string count */
    (*argblk_ptr).string_ptrpc += match_len;
    /* And string pointer */
    (*argblk_ptr).tokencntl = match_len;
    /* Set up token count */
    /*
     *   Copy parameter into the TPARSE Block
     */
    (*argblk_ptr).paraml = (*state_ptr).action_paraml;
    /* Copy parameter */
    /*
     *   See if the user has specified an Action Routine and, if so, call it.
     */
    action_ptr = (*state_ptr).action_ptrpf;
    /* Get action routine pointer */
    if (action_ptr != NULL)     /* If there is an action routine */
    {
        iret = 1;           /* Initialize status */
        iret = (*action_ptr)(argblk_ptr);
        /* Call the action routine */
        if (iret != 0)      /* If there is a meaningful status */
        {
            action_sts = iret;  /* Then store it */
        }

        /*
         *   If the action routine has rejected the transition - pretend it
         *    never matched. Call the trace routine if one is specified and
         *   join common failure path.
         *
         *   N.B.: Strictly, an action routine should return either 0 (error) or
         *   1 (success). However, they actually return 0 (error), PSL_NORMAL
         *   (a windows message number indicating success) or some other windows
         *   message number ('probably' indicating an error). Therefore to test
         *   for a failure status from an action routine we check that the status
         *   is NOT PSL_NORMAL below. This is NOT bullet-proof, as an action
         *   routine could return a 'success' status that is NOT PSL_NORMAL. The
         *   solution to this is to modify all action routines to return 0 or 1.
         */
        if (iret != PSL_NORMAL) /* If action routine returned error */
        {
            (*argblk_ptr).stringcntl += match_len;
            /* Reset string count */
            (*argblk_ptr).string_ptrpc -= match_len;
            /* And string pointer */
            tracertn_ptr = (PTRACE) (*argblk_ptr).tracertn_ptrpf;
            /* Get trace routine pointer */
            if (((*argblk_ptr).tracelvll > 0)
                    /* If tracing */
                    && (tracertn_ptr != NULL))
                /* And there is a Trace Routine */
            {
                (*tracertn_ptr)(argblk_ptr,2);
                /* Call the trace routine */
            }
            goto failed;        /* Exit with no match */
        }
    }
    /*
     *   Either there was no action routine, or it returned success - we take the
     *   transition.  First we call the trace routine if one is specified.
     */
    tracertn_ptr = (PTRACE) (*argblk_ptr).tracertn_ptrpf;
    /* Get trace routine pointer */
    if (((*argblk_ptr).tracelvll > 0)
            /* If tracing */
            && (tracertn_ptr != NULL)) /* And there is a Trace Routine */
    {
        (*tracertn_ptr)(argblk_ptr,3);
        /* Call the trace routine */
    }
    /*
     *   Next we check to see if we need to store any data and, if so, do so.
     */
    data_ptr = (unsigned long *) (*state_ptr).data_ptrp;
    /* Get data pointer */
    if (data_ptr != NULL)
    {
        if ((*state_ptr).maskl != 0)
            /* If there is a mask value */
        {
            *data_ptr |= (*state_ptr).maskl;
            /* Then set the mask */
        }
        else
        {
            /*
             *   Data address specified, but not a mask value.  Set up the data
             *   according to the token type.
             */
            switch ((*state_ptr).token_typeb)
            {
            case PSLTPA_STRING:
            case PSLTPA_SYMBOL:
            case PSLTPA_BLANK:
            case PSLTPA_SUBXPR:
            case PSLTPA_QUOTED:
            case PSLTPA_DATANAME:
            case PSLTPA_LONGDATANAME:
            case PSLTPA_EXPRESSION:
                {
                    /*
                     *   Allocate memory for the string, copy the string to it, and return
                     *   a pointer to it
                     */
                    string_ptr = (char *) calloc (1, (*argblk_ptr).tokencntl + 1);

                    /* Allocate memory */
                    (void) strncpy (string_ptr, (*argblk_ptr).token_ptrpc,
                                    (*argblk_ptr).tokencntl);
                    /* Copy string */
                    *((char **) data_ptr) = string_ptr;
                    /* Return string pointer */
                    break;
                }
            case PSLTPA_CHAR:
            case PSLTPA_ANY:
            case PSLTPA_ALPHA:
            case PSLTPA_DIGIT:
                {
                    /*
                     *   Return matching character
                     */
                    *(char *)data_ptr = (*argblk_ptr).charb;
                    /* Return matching character */
                    break;
                }
            case PSLTPA_DECIMAL:
            case PSLTPA_OCTAL:
            case PSLTPA_HEX:
                {
                    /*
                     *   Return converted value
                     */
                    *data_ptr = (*argblk_ptr).numberl;
                    /* Return converted value */
                    break;
                }
            }
        }
    }

    /*
     *   Take the transition.  If an explicit target exists, follow it.
     *   PSLTPA_EXIT means exit with success, PSLTPA_FAIL means exit with
     *   failure.  If no explicit target exists skip transitions until we
     *   reach the beginning of the next state.
     */
    (*argblk_ptr).optionsw &= ~TPAARG_AMBIGM;
    /* Reset Ambiguity Flag */
    action_sts = 0;         /* Clear Action Status */
    if ((*state_ptr).quitb == PSLTPA_EXIT)
    {
        return PSL_NORMAL;      /* Exit with success */
    }
    else if ((*state_ptr).quitb == PSLTPA_FAIL)
    {
        return PSL_SYNTAXERR;   /* Exit with standard error */
    }
    else if ((*state_ptr).target_ptrps != NULL)
    {
        state_ptr = (*state_ptr).target_ptrps;
        /* Reset state pointer to target */
        goto loop;          /* Go to new state */
    }
    else
    {
        while (!(*state_ptr).end_stateb)
            /* While not at end of state */
        {
            state_ptr += 1;     /* Step to next entry */
        }
        state_ptr += 1;     /* Step to next state */
        goto loop;          /* Go to new state */
    }


    /*
     *   Transition does not match.  Skip the current transition and set up
     *   for the next one.  If this was last transition in the state then
     *   return an error.
     */
failed:
    if (!(*state_ptr).end_stateb)   /* If not the last transition */
    {
        state_ptr += 1;     /* Step to next entry */
        (*argblk_ptr).stringcntl += spcoff;
        /* Return flushed spaces */
        (*argblk_ptr).string_ptrpc -= spcoff;
        /* And subtract from string address */
        spcoff = GET_STRING (argblk_ptr, STRTYP_SPACES);
        /* Reset space count */
        goto loop;          /* And try next transition */
    }
    /*
     *   There is no next transition so we must exit.  First see if tracing
     *   is active and, if so, trace the event.
     */
    tracertn_ptr = (PTRACE) (*argblk_ptr).tracertn_ptrpf;
    /* Get trace routine pointer */
    if (((*argblk_ptr).tracelvll > 0)
            /* If tracing */
            && (tracertn_ptr != NULL)) /* And there is a Trace Routine */
    {
        (*tracertn_ptr)(argblk_ptr,4);
        /* Call the trace routine */
    }
    /*
     *   Next locate the next symbol so that we can blame it for the error
     *   and exit with an error status (either ours or the one from the
     *   Action Routine.
    */
    (*argblk_ptr).tokencntl = GET_STRING (argblk_ptr, STRTYP_SPACES);

    /* Step over spaces/tabs */
    (*argblk_ptr).tokencntl = GET_STRING (argblk_ptr, STRTYP_SYMBOL);

    /* Locate next symbol */
    if ((*argblk_ptr).tokencntl == 0)
        /* If none */
    {
        if ((*argblk_ptr).stringcntl != 0)
            /* If something in string */
        {
            (*argblk_ptr).tokencntl += 1;
            /* then reset to 1 */
        }
    }
    if (action_sts == 0)        /* If no action status */
    {
        action_sts = PSL_SYNTAXERR; /* Return standard error */
    }
    return action_sts;      /* Return with status */
}


static  long    GET_STRING (struct tpaarg *argblk_ptr,
                            long type)
{
    /*
     *  GET_STRING - Extract String of specified type
     *
     *   This routine looks for a string of the specified type at
     *   the front of the string being parsed.
     *
     *   Calling Sequence:
     *
     *  iret = GET_STRING (argblk_ptr, type)
     *
     *   Where, on Input:
     *
     *  argblk_ptr = Pointer to PSL_TPARSE argument block       
     *  type = String Type Code.
     *      STRTYP_SPACES   = Spaces
     *      STRTYP_NUMERIC  = Numeric
     *      STRTYP_HEXNUMERIC   = Hexnumeric
     *      STRTYP_ALPHANUMERIC = Alphanumeric
     *      STRTYP_SYMBOL   = Symbol
     *      STRTYP_QUOTED   = Quoted string
     *      STRTYP_DATANAME = Dataname
     *      STRTYP_LONGDATANAME = Long Dataname
     *      STRTYP_EXPRESSION   = Expression (terminated by comma or white space)
     *      STRTYP_OCTNUMERIC   = Octnumeric
     *      STRTYP_KEYWORD  = Keyword
     *
     *   And on Output:
     *
     *  iret = character count of found string
     *
     *   The routine simply scans through the input string until it finds 
     *   a character that is not in the class, counting the characters.
     */
    /*
     *  Local Data.
     */
    long int    count = 0;  /* String count */
    char        *char_ptr;  /* Pointer to character */
    char        quote;      /* Quote character */
    char        hyph_und = 0;   /* Hyphen or underscore */

    char_ptr = (*argblk_ptr).string_ptrpc;
    /* Point to start of string */
    quote = 0;          /* No quote yet */
    while (count < (*argblk_ptr).stringcntl)
        /* While there are characters left */
    {
        switch (type)
        {
        case STRTYP_SPACES:
            {
                /*
                 *   Tabs or spaces 
                 */
                if ((*char_ptr != ' ')
                        && (*char_ptr != '\t')) goto end;
                /* If character is not space or tab */
                break;
            }
        case STRTYP_NUMERIC:
            {
                /*
                 *   Decimal
                 *
                 *   The first character may be a '+' or '-' for decimal numbers.
                 */


                if (count == 0)     /* If this is the first character */
                {
                    if ((*char_ptr == '+')
                            || (*char_ptr == '-')) break;
                }


                if (!isdigit(*char_ptr)) goto end;
                /* If not decimal */
                break;
            }
        case STRTYP_HEXNUMERIC:
            {
                /*
                 *   Hexadecimal
                 */
                if (!isxdigit(*char_ptr)) goto end;
                /* If not hex */
                break;
            }
        case STRTYP_ALPHANUMERIC:
            {
                /*
                 *   Alphanumeric
                 */
                if (!isalnum(*char_ptr)) goto end;
                /* If not alphanumeric */
                break;
            }
        case STRTYP_SYMBOL:
            {
                /*
                 *   Symbol (alphanumeric plus '$' and '_'
                 */
                if (!isalnum(*char_ptr)
                        && (*char_ptr != '$')
                        && (*char_ptr != '_')) goto end;
                /* If not alphanumeric, $ or _ */
                break;
            }
        case STRTYP_QUOTED:
            {
                /*
                 *   Quoted string
                 */
                if (count == 0)     /* If this is the first character */
                {
                    if ((*char_ptr != '"')
                            && (*char_ptr != '\'')) goto end;
                    /* If not quote then no match */
                    else quote = *char_ptr;
                    /* Save quote character */
                }
                else            /* We are looking for end quote */
                {
                    if (*char_ptr == quote)
                        /* If this is final quote */
                    {
                        count += 1; /* Count the quote */
                        quote = 0;  /* Show end quote read */
                        goto end;
                    }
                }
                break;
            }
        case STRTYP_DATANAME:
        case STRTYP_LONGDATANAME:
            {
                /*
                 *   Dataname
                 */
                if (count == 0)     /* If this is the first character */
                {
                    if (!isalpha(*char_ptr)) goto end;
                    /* It must be alphabetic */
                }
                /*
                 * Removed code that only read in up to 16 characters
                 */
                else
                {
                    if (isalnum(*char_ptr)) hyph_und = 0;
                    /* Show not hyphen or underscore */
                    else if ((*char_ptr == '-')
                             || (*char_ptr == '_'))
                        /* Hyphen or underscore */
                    {
                        if (hyph_und) goto end_dataname;
                        /* If two consecutive - or _ */
                        hyph_und = 1;   /* Show we've had - or _ */
                    }
                    else        /* Not a valid character */
                    {
end_dataname:
                        if (hyph_und) count -= 1;
                        /* If last character was - or _ */
                        /* then drop it */
                        goto end;
                    }
                }
                break;
            }
        case STRTYP_EXPRESSION:
            {
                /*
                 *   Expression
                 */
                if ((*char_ptr == ' ')
                        || (*char_ptr == '\t')
                        || (*char_ptr == ',')) goto end;
                /* Terminate on white space or comma */
                break;
            }
        case STRTYP_OCTNUMERIC:
            {
                /*
                 *   Octal
                 */
                if ( (!isdigit(*char_ptr)) ||
                        (*char_ptr == '8') ||
                        (*char_ptr == '9') ) goto end;
                /* If not octal */
                break;
            }

        case STRTYP_KEYWORD:
            {
                /*
                 *   Keyword (alphanumeric plus '$' and '_' & embedded '-'
                 */
                if (!isalnum(*char_ptr)
                        && (*char_ptr != '$')
                        && (*char_ptr != '_')
                        && ((*char_ptr != '-')
                            ||  (count == 0)
                            ||  (count >= ((*argblk_ptr).stringcntl -1))
                            ||  (!isalnum(*(char_ptr-1)))
                            ||  (!isalnum(*(char_ptr+1))))) goto end;
                /* If not alphanumeric, $ or _ or */
                /* embedded hyphen */
                break;
            }

        default:
            {
                goto end;
            }
        }
        /*
         *   The character matched - count it and try the next
         */
        count += 1;         /* Increment character count */
        char_ptr += 1;      /* Point to next character */
    }
    /*
     *   The character did not match - just exit
     */
end:
    if (quote != 0) count = 0;  /* No match if quoted string */
    /* not terminated */

    /*
     * If dataname being parsed and its length is greater than 16 
     * or  along dataname is being parsed and its length is greater
     * 64 themn this is an error.
     *
     * Set count length as = 0 which causes an error.
     */
    if (((type == STRTYP_DATANAME) && (count > 16)) ||
        ((type == STRTYP_LONGDATANAME) && (count > 64)))
    {
        count = 0;
    }

    return count;           /* Return the match count */
}

