#ifndef PSL__SCL_TLODEF_LOADED
#define PSL__SCL_TLODEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 *  scl_tlodef.h:
 *  =============
 *
 *  This module describes the layout of a Test Layout Definition.
 *
 *  The structures in the file are used by SCL to store test and
 *  thread definition data during parsing. This data is written
 *  to a TOF file in ASCII format and would more appropraitely be held
 *  in structures defined for the purpose - i.e. with more meaningful
 *  field names and with no redundant fields.
 */

/* */
/*    TEST LAYOUT HEADER RECORD */
/*    ========================= */
/* */
/*    Define the layout of the Header record for the Test Layout definition */
/* */
struct tlh
{
    /* */
    char                    tlh_ftypc4[4];    /* File Type */
    PSL_LONGWORD_T          tlh_fvrnl;    /* Load Generator Version */
#define            TLH_SCP_FVRN 0X54    /* File version     */
    /* */
    /*  Creation/Insertion Date/Time */
    /* */
    PSL_U_LONGWORD_T        tlh_ctimq[2]; /* Compiled Date/Time    */
    PSL_U_LONGWORD_T        tlh_ltimq[2]; /* Inserted Date/Time    */
    PSL_LONGWORD_T          tlh_testll;   /* Test length */
    char                    tlh_testnamec16[16];  /* Test Name */
    char                    tlh_testdescc50[50];  /* Test Description */
    char                    tlh_testtypeb;    /* Type of Test Definitions */
#define            TLH_TESTTYPE_TEST 1  /* Type = Tests in file */
#define            TLH_TESTTYPE_TERMINALS 2
    /* Type = Terminal definitions */
    char                    tlh_seqtypeb; /* Type of sequencing */
#define            TLH_SEQUENCE_SEQUENTIAL 1
    /* Sequential sequencing */
#define            TLH_SEQUENCE_CONCURRENT 2
    /* Concurrent sequencing */
    PSL_LONGWORD_T          tlh_testcntl; /* Count of following items */
    /* */
    /*  This field defines the sub-version number of this test. */
    /* */
    PSL_LONGWORD_T          tlh_subverl;  /* Subversion number */
#define            TLH_TST_SVRN 0X0 /* File sub-version */
    /* */
    /*  Flags. */
    /* */
    PSL_U_LONGWORD_T        tlh_flagsl;   /* Flags         */
#define            TLFL_NOTLSTITM 0X1   /* NOT Last Item in file */
    /* */
    /*  Note that the security fields that follow MUST be at the */
    /*  same offset as the corresponding fields in SCPDEF for the */
    /*  benefit of the SLB_CHK_PROTECTION routine. */
    /* */
    char                    tlh_fillc24[24];
    /* */
    /*  This field defines the username of the owner of the test. */
    /*  It will be filled in by SLB, TDL or LGN to the username of */
    /*  the user who adds the test to the script library. */
    /* */
    char                    tlh_ownusrnmc12[12];
    /* */
    /*  This field defines the protection mask to be set for the test. */
    /*  It will be filled in by SLB, TDL or LGN to the default protection */
    /*  mask of the user who adds the test to the script library unless */
    /*  over-ridden by the /PROTECTION qualifier. */
    /* */
    /*  Note that the individual bits are defined in SCPDEF. */
    /* */
    PSL_LONGWORD_T          tlh_sprotl;
    /* */
    /*  This field defines the UIC of the owner of the test. */
    /*  It will be filled in by SLB, TDL or LGN to the UIC of the user */
    /*  who adds the test to the script library. */
    /* */
    PSL_LONGWORD_T          tlh_ownuicl;
    /* */
    /*  This field is UNIX specific. It defines the User Id of the owner */
    /*  of the test. It is set by SLB, TDL or LGN to the effective user id. */
    /*  of the user who adds the test to the script library. */
    /* */
    PSL_LONGWORD_T          tlh_ownuidl;
    /* */
    /*  This field is UNIX specific. It defines the Group Id of the owner */
    /*  of the test. It is set by SLB, TDL or LGN to the effective group id. */
    /*  of the user who adds the test to the script library. */
    /* */
    PSL_LONGWORD_T          tlh_owngidl;
    /* */
    /*      This field defines whether or not the Test Protection was defauled */
    /*      or not.  If so, the protection comes from the "$DEFAULT" Test Definition. */
    /* */
    Boolean                 tlh_prot_defaultedn;
    /* */
    /* */
    /*      The following fields are to indicate if there is any platform */
    /*      or operating system specific code in the test. It is stored in */
    /*      the header and then checked in LGR to check the validity of running */
    /*      the test on the current platform or OS. */
    /* */
    /* */
    /*      This field holds a longword and determines the platform the */
    /*      test was compiled on. */
    /* */
    PSL_LONGWORD_T          tlh_compile_platl;
#define            TLH_PLAT_NTI 1
#define            TLH_PLAT_VAX 3
#define            TLH_PLAT_ALPHA 4
#define            TLH_PLAT_SUN 5
#define            TLH_PLAT_HP 6
#define            TLH_PLAT_IBM 7
#define            TLH_PLAT_NCR 8
#define            TLH_PLAT_OSF 9
    /* */
    /*      This field holds a longword and determines the operating system the */
    /*      test was compiled on. */
    /* */
    PSL_LONGWORD_T          tlh_compile_opsysl;
#define            TLH_OS_UNIX 2
#define            TLH_OS_WIN32 3
    /* */
    /*      This field indicates if there was any platform specific */
    /*      code in the test. */
    /* */
    Boolean                 tlh_plat_spec_coden;
    /* */
    /*      This field indicates if there was any operting system specific */
    /*      code in the test. */
    /* */
    Boolean                 tlh_opsys_spec_coden;
#ifdef WIN32
    /* */
    /*      This field holds a longword and determines the operating system the */
    /*      test is destined to run on.  THIS FIELD IS ONLY USED BY COMMANDER, */
    /*  AND IS USED TO INDICATE WHICH FIELDS SHOULD BE DISPLAYED.  It takes */
    /*  the same parameters as the compile_opsys field. */
    /* */
    PSL_LONGWORD_T          tlh_executer_opsysl;
#endif

#ifdef WIN32
    char                    tlh_spareb328[328];   /* filler to 512 */
#endif
#ifdef UNIX
#ifdef OSF
    char tlh_spareb332[244];
#else
char tlh_spareb332[332];
#endif
#endif
} ;

/*    TEST THREAD DEFINITION RECORD
 *    =============================
 *
 *    Define structure to hold Test Thread definition data.
 */
struct tthr
{
    /*
     *    The Following field contains a bit-mask of the fields which are or are
     *    not defaulted. If the bit is set the corresponding field is defaulted.
     */
    PSL_U_LONGWORD_T        tthr_default_flagsl;

    /*** #define            TTHRD_??? 0X1 ***/

    /*
     *    This field is a bitmask of flags indicating the state of tracing
     *    for the various items that are traceable.
     */
    PSL_U_LONGWORD_T        tthr_trace_maskl;
#define            TTHRT_IPC 0X1
#define            TTHRT_DTC 0X2
#define            TTHRT_PIPE 0X4
#define            TTHRT_ERRORS 0X8
#define            TTHRT_MISC 0X10
#define            TTHRT_IPC2 0X20
#define            TTHRT_DTC2 0X40
#define            TTHRT_USER1 0X80
#define            TTHRT_USER2 0X100
#define            TTHRT_USER3 0X200

    /*
     *    This field defines the target type.
     */
    /***      PSL_LONGWORD_T          tthr_???; ***/

    /*
     *    This field is used to pad the structure to a quadword boundary.
     */
    char                    tthr_spareb8[8];
} ;

/* */
/*    TEST PARAMETER DEFINITION RECORD */
/*    ================================ */
/* */
/*    Define the layout of the Test Definition Parameter Record. */
/* */
struct prm
{
    /* */
    /*    This field defines the name of the audit file to be used by the test */
    /* */
    char                    prm_message_logc52[52];
    /* */
    /*    This field defines the total number of VCB slots which are to be */
    /*    allocated by the run-time component. */
    /* */
    PSL_U_LONGWORD_T        prm_max_threadsl;
    /* */
    /*    This field defines the name of the statistics file to be used by the */
    /*    test. */
    /* */
    char                    prm_statistics_filec52[52];
    /* */
    /*    This field defines a character representation of the default input rate */
    /*    to be used by the test. It is in the form nn.m */
    /* */
    char                    prm_input_ratec8[8];
    /* */
    /*    This field defines the working set quota for the test. */
    /* */
    PSL_U_LONGWORD_T        prm_wsquotal;
    /* */
    /*    This field defines the name of the environment under which this test will */
    /*    be executed. */
    /* */
    char                    prm_environmentc12[12];
    /* */
    /*    This field defines the scope of the environment under which this test will */
    /*    be executed. */
    /* */
    PSL_U_LONGWORD_T        prm_scopel;
    /* */
    /*    This field defines the name of any hardcopy log device for the test. */
    /* */
    char                    prm_hardcopy_logc52[52];
    /* */
    /*    This field defines the base priority at which the run-time process will */
    /*    be created. */
    /* */
    PSL_U_LONGWORD_T        prm_priorityl;
    /* */
    /*    This field defines the name of any history file for the test. */
    /* */
    char                    prm_history_filec52[52];
    /* */
    /*    This field defines the name of any report file for the test. */
    /* */
    char                    prm_report_filec52[52];

    /* */
    /*    This field defines whetehr a new or existing audit file will be used by */
    /*    the test. */
    /* */
    PSL_U_LONGWORD_T        prm_log_typel;
    /* */
    /*    This field defines whether or not a snap dump file is created when an */
    /*    exception error occurs. */
    /* */
    PSL_U_LONGWORD_T        prm_dump_stylel;
    /* */
    /*    This field defines whether or not a run-time component will be */
    /*    automatically created. */
    /* */
    PSL_U_LONGWORD_T        prm_auto_startl;
    /* */
    /*    This field defines whether or not the run-time component may be shared */
    /*    between multiple tests. */
    /* */
    PSL_U_LONGWORD_T        prm_sharedl;
    /* */
    /*    This field defines whether or not a run-time component will be */
    /*    automatically closed down when the test completes. */
    /* */
    PSL_U_LONGWORD_T        prm_autoclosel;
    /* */
    /*    This field defines whether or not the entire test should be aborted on */
    /*    failure of any thread within the test. */
    /* */
    PSL_U_LONGWORD_T        prm_abortfaill;
    /* */
    /*    This field defines the maximum number of terminals which may be active on */
    /*    the run-time component at any one time. */
    /* */
    PSL_U_LONGWORD_T        prm_max_concurrentl;
    /* */
    /*    This field is a flag which signifies whether or not DEBUG is to be enabled */
    /*    for the run-time component. It is a static parameter. */
    /* */
    PSL_U_LONGWORD_T        prm_debugl;
    /* */
    /*    The Following field contains a bit-mask of the fields which are or are */
    /*    not defaulted. If the bit is set the corresponding field is defaulted. */
    /* */
    PSL_U_LONGWORD_T        prm_default_flagsl;
#define PRM_TSTDF_MESSAGE_LOG 0X1
#define PRM_TSTDF_MAX_THREADS 0X2
#define PRM_TSTDF_STATISTICS_FILE 0X4
#define PRM_TSTDF_INPUT_RATE 0X8
#define PRM_TSTDF_WSQUOTA 0X10
#define PRM_TSTDF_ENVIRONMENT 0X20
#define PRM_TSTDF_SCOPE 0X40
#define PRM_TSTDF_HARDCOPY_LOG 0X80
#define PRM_TSTDF_PRIORITY 0X100
#define PRM_TSTDF_HISTORY_FILE 0X200
#define PRM_TSTDF_REPORT_FILE 0X400
#define PRM_TSTDF_LOG_TYPE 0X800
#define PRM_TSTDF_DUMP_STYLE 0X1000
#define PRM_TSTDF_AUTO_START 0X2000
#define PRM_TSTDF_SHARED 0X4000
#define PRM_TSTDF_AUTOCLOSE 0X8000
#define PRM_TSTDF_ABORTFAIL 0X10000
#define PRM_TSTDF_MAX_CONCURRENT 0X20000
#define PRM_TSTDF_DEBUG 0X40000
#define PRM_TSTDF_LOGICAL_NODE 0X80000
#define PRM_TSTDF_TRACE_MASK 0X100000
#define PRM_TSTDF_FAILTC 0X200000
#define PRM_TSTDF_PASSTC 0X400000
#define PRM_TSTDF_COMPTEST 0X800000
#define PRM_TSTDF_ABRTTEST 0X1000000
#define PRM_TSTDF_SCP_PROT 0X2000000
#define PRM_TSTDF_TST_PROT 0X4000000
#define PRM_TSTDF_PRC_TYPE 0X8000000
#define PRM_TSTDF_HISTTEM 0X10000000
#define PRM_TSTDF_REPTEM 0X20000000
#define PRM_TSTDF_IGNRINF 0X40000000
#define PRM_TSTDF_DSC 0X80000000
    /* */
    /*    Define the character pointers to hold the expanded filenames after */
    /*    the parameter substitution has been applied. */
    /* */
    char                    *prm_msglog_ptrpc;
    char                    *prm_stafil_ptrpc;
    char                    *prm_hisfil_ptrpc;
    char                    *prm_repfil_ptrpc;
    /* */
    /*    This field is spare */
    /*   */
    char                    prm_spareb20[20];
    /* */
    /*    This field is a bitmask of flags indicating the state of tracing */
    /*    for the various items that are traceable. */
    /* */
    PSL_U_LONGWORD_T        prm_trace_maskl;
#define            PRMT_IPC 0X1
#define            PRMT_DTC 0X2
#define            PRMT_PIPE 0X4
#define            PRMT_ERRORS 0X8
#define            PRMT_MISC 0X10
#define            PRMT_IPC2 0X20
#define            PRMT_DTC2 0X40
#define            PRMT_USER1 0X80
#define            PRMT_USER2 0X100
#define            PRMT_USER3 0X200
    /* */
    /*    This field stores the Fail Test-case message template. */
    /* */
    char                    prm_failtcc84[84];
    /* */
    /*    This field stores the Pass Test-case message template. */
    /* */
    char                    prm_passtcc84[84];
    /* */
    /*    This field stores the Complete Test message template. */
    /* */
    char                    prm_comptestc84[84];
    /* */
    /*    This field stores the Abort Test message template. */
    /* */
    char                    prm_abrttestc84[84];
    /* */
    /*    These fields are used to store pointers to the message templates. */
    /* */
    char                    *prm_failtc_ptrpc;
    char                    *prm_passtc_ptrpc;
    char                    *prm_comptest_ptrpc;
    char                    *prm_abrttest_ptrpc;
    /* */
    /* */
    /*    These fields define the default Script and Test Protection. */
    /* */
    PSL_LONGWORD_T          prm_scp_protl;
    PSL_LONGWORD_T          prm_tst_protl;
    /* */
    /*    This field defines the process type: SPAWNED or DETACHED.  */
    /* */
    PSL_LONGWORD_T          prm_process_typel;
#define            PRTYP_DETACHED 1
#define            PRTYP_SPAWNED 2
    /* */
    /* */

    /*    These fields hold the template strings for the history and report files */
    /* */
    char                    prm_histtemc84[84];
    char                    prm_reptemc84[84];
    /*    This field defines whether particular information messages should be */
    /*    inhibited. */
    /* */
    PSL_U_LONGWORD_T        prm_ignrinfl;

    /* */
    /*    This field is spare and is to pad the structure to 1024 bytes. */
    /*    Note that the PRM structure size should be kept to a multiple of 512 */
    /*    bytes. */
    /* */
#ifdef WIN32
    char                    prm_spareb116[116];
#endif
#ifdef UNIX
#ifdef OSF
    char prm_spareb116[500];
    /* Pad to 1536 */
#else
    char prm_spareb116[116];
#endif
#endif
} ;


/*                   */
/*    Define the layout of the Test layout Definition Structure containing */
/*    pointers to further Test layout definitions */
/* */
struct tlv
{
    /* */
    /*    This field defines the name of the sub-level of this test */
    /* */
    char                    tlv_subnamec20[20];
    /* */
    /*    This field defines the sequence number of the sub-level. */
    /* */
    PSL_U_LONGWORD_T        tlv_sequencel;
    /* */
    /*    This field holds a number of flags which signify whether or not the */
    /*    corresponding field in the TLV record is set to default or not. */
    /* */
    PSL_U_LONGWORD_T        tlv_default_flagsl;
#define            TLVD_SEQUENCE 0X1
#define            TLVD_SUBNAME 0X2
#define            TLVD_LOGICAL_NODE 0X4
    /* */
    /*    This field defines the logical node name on which the Test Executor */
    /*    process will be created. */
    /*   */
    char                    tlv_logical_nodec20[20];
    /* */
    /*    These are filler/alignment bytes. */
    /* */
#ifdef WIN32
    char                    tlv_spareb16[16];
#endif
#ifdef UNIX
#ifdef OSF
    /***%literal "    char tlv_spareb16[0];"; */
#else
    char tlv_spareb16[16];
#endif
#endif
    /* */
} ;

/* */
/*    Define the layout of the Test Layout Definition structure containing */
/*    terminal definitions. First define the parameter record which is */
/*    required in the terminal definition */
/* */
struct tdp
{
    /* */
    /*    This field defines the actual value entered for a CHARACTER parameter */
    /* */
    char                    tdp_char_paramc52[52];
    /* */
    /*    This field defines the actual value entered for an INTEGER parameter */
    /* */
    PSL_LONGWORD_T          tdp_int_paraml;
    /* */
    /*    This field defines the prompt string entered for the parameter. */
    /* */
    char                    tdp_promptc52[52];
    /* */
    /*    This field defines whether or not the parameter value will be echoed */
    /*    when entered. */
    /* */
    PSL_LONGWORD_T          tdp_paramechol;
    /* */
    /*    This field defines the data type of the parameter. */
    /* */
    PSL_LONGWORD_T          tdp_paramtypel;
#define            TDP_PRMTYP_INT 1
#define            TDP_PRMTYP_CHR 2
    /* */
    /*    This field holds a bitmask indicating whether or not the fields in the */
    /*    TDP record have been defaulted. */
    /* */
    /*    Note that the PARAMTYPE may only be defaulted if we have no VALUE. */
    /* */
    PSL_U_LONGWORD_T        tdp_default_flagsl;
#define            TDPD_PROMPT 0X1
#define            TDPD_PARAMECHO 0X2
#define            TDPD_PARAMUPCASE 0X4
#define            TDPD_PARAMTYPE 0X8
#define            TDPD_PARAMVAL 0X10
    /* */
    /* */
    /*    This field defines the script parameter name. */
    /* */
    char                    tdp_namec20[20];
    /* */
    /*    This field defines whether or not the parameter value will be converted */
    /*    to uppercase when entered. */
    /* */
    PSL_LONGWORD_T          tdp_paramupcasel;
    /* */
    char                    tdp_spare1b8[8];
    /* */
    /* */
} ;

/* */
/*    Now actually define the TDF record itself */
/* */
struct tdf
{
    /* */
    /*    This field defines the name of the terminal which is to be created. It */
    /*    is a V-TEST dataname used for internal and user representation of the */
    /*    physical device. */
    /* */
    char                    tdf_trm_namec20[20];
    /* */
    /*    This field defines the username under which this script is to be logged */
    /*    in if 'Auto' login has been specified. */
    /* */
    char                    tdf_usernamec36[36];
    /* */
    /*    This field defines the name of the primary script which is to be executed */
    /*    by this terminal. */
    /* */
    char                    tdf_scriptc20[20];
    /* */
    /*    For remote LAT terminals this field defines the name of the LAT service */
    /*    to which this terminal is to attach. */
    /* */
    char                    tdf_ltservicec20[20];
    /* */
    /*    For remote LAT terminals this field defines the name of the LAT service */
    /*    to which this terminal is to attach. */
    /* */
    char                    tdf_ltnodec8[8];
    /* */
    /*    This field defines the number of terminals which are to be created by */
    /*    this terminal definition. It is in the range 1-999 */
    /* */
    PSL_U_LONGWORD_T        tdf_numberl;
    /* */
    /*    This field defines the name of the physical device which is to be attached */
    /*    if the device type field is 'Physical'. */
    /* */
    char                    tdf_physical_devicec36[36];
    /* */
    /*    This field defines the sequence number of the definition. It will be in */
    /*    the range 0-99. */
    /* */
    PSL_U_LONGWORD_T        tdf_seq_nol;
    /* */
    /*    This field defines the number of seconds to delay between this terminal */
    /*    definition being eligible to execute its script and actually starting to */
    /*    execute the script. */
    /* */
    PSL_U_LONGWORD_T        tdf_startl;
    /* */
    /*    This field defines whether or not the terminal is to recognise 8-bit */
    /*    sequences. */
    /* */
    PSL_U_LONGWORD_T        tdf_character_sizel;
    /* */
    /*    This field defines the inter-terminal delay period. */
    /* */
    PSL_U_LONGWORD_T        tdf_delayl;
    /* */
    /*    This field defines the base priority of the created terminal. */
    /* */
    PSL_U_LONGWORD_T        tdf_priorityl;
    /* */
    /*    This field defines the width of the terminal to be created in columns. */
    /* */
    PSL_U_LONGWORD_T        tdf_widthl;
    /* */
    /*    This field defines the page size in lines of the terminal */
    /* */
    PSL_U_LONGWORD_T        tdf_pagel;
    /* */
    /*    This field defines the number of parameters defined for the terminal. */
    /* */
    PSL_U_LONGWORD_T        tdf_scpprm_countl;
    /* */
    /*    This field is a toggle field which defines the type of the device which */
    /*    is to be created. */
    /* */
    PSL_LONGWORD_T          tdf_device_typel;
    /* */
    /*    This field defines the baud rate of the terminal to be created. */
    /* */
    PSL_LONGWORD_T          tdf_baud_ratel;
    /* */
    /*   This field defines the terminal type to be created. */
    /* */
    PSL_LONGWORD_T          tdf_terminal_typel;
    /* */
    /*    This field defines the terminal ID of the terminal to be created. This */
    /*    defines the response which the run-time component sends in response to */
    /*    a terminal inquiry sequence. */
    /* */
    PSL_LONGWORD_T          tdf_terminal_idl;
    /* */
    /*    This field defines the type of login to be performed for the terminal. */
    /* */
    PSL_LONGWORD_T          tdf_login_typel;
    /* */
    /*    This field defines whether or not script tracing is to be enabled. */
    /* */
    PSL_LONGWORD_T          tdf_traceflagl;
    /* */
    /*    These fields contain any values specified for the script parameters. */
    /* */
    struct tdp              tdf_paramsr8[8];
    /* */
    /*    This field defines any name specified as the trace file specification. */
    /* */
    char                    tdf_tracefilec52[52];
    /* */
    /*    This field defines whether or not this terminal is to be recorded. */
    /* */
    PSL_LONGWORD_T          tdf_recordl;
    /* */
    /*    This field determines the type of recording to be created. */
    /* */
    PSL_LONGWORD_T          tdf_record_typel;
    /* */
    /*    This field defines the index of the parent of this terminal. */
    /* */
    struct bdt              *tdf_parent_posps;
    /* */
    /*    This field is a sideways pointer to the next terminal definition on the */
    /*    same level as ourselves. */
    /* */
    PSL_LONGWORD_T          tdf_next_idxl;
    /* */
    /*    This field defines whether or not typeahead is to be enabled initially. */
    /* */
    PSL_LONGWORD_T          tdf_typeaheadl;
    /* */
    /*    This field defines the name of the recording file. */
    /* */
    char                    tdf_recordfilec52[52];
    /* */
    /*    This field defines whether the timing of input characters fed to the */
    /*    script will be 'fine' or 'coarse'. */
    /* */
    PSL_LONGWORD_T          tdf_input_timingl;
    /* */
    /*    This field defines the startup delay for this terminal in seconds. */
    /* */
    PSL_LONGWORD_T          tdf_startdelayl;
    /* */
    /*    This field defines the terminal's answerback sequence */
    /* */
    char                    tdf_answerbackc32[32];
    /* */
    /*    This field determines whether or not the terminal is to be automatically */
    /*    detached from LGR on exit from the primary script. */
    /* */
    PSL_LONGWORD_T          tdf_detach_exitl;
    /* */
    /*    This field defines whether or not this terminal is to have the device */
    /*    hungup when the device is detached. */
    /* */
    PSL_LONGWORD_T          tdf_hangup_detachl;
    /* */
    /*    This field defines whether or not this terminal is to stop recording when */
    /*    the terminal is detached. */
    /* */
    PSL_LONGWORD_T          tdf_stoprecdetl;
    /* */
    /*    This field defines whether or not this terminal is to send out 7 or 8 */
    /*    bit sequences when inputting SCL function key constants. */
    /* */
    PSL_LONGWORD_T          tdf_fkey_controll;
    /* */
    /*    This field holds the TERM_INFO value for the thread when running under */
    /*    UNIX. */
    /* */
    char                    tdf_terminfo_namec36[36];
    /* */
    /*    This field holds the value specified for the 'Auto Sync' field when */
    /*    running on the HP. */
    /* */
    PSL_LONGWORD_T          tdf_auto_syncl;
    /* */
    /*    This field holds the value specified for the 'Enable IOM' field when */
    /*    running on the HP. */
    /* */
    PSL_LONGWORD_T          tdf_enable_ioml;
    /* */
    /*    This field holds a number of flags which signify whether or not the */
    /*    corresponding field in the TDF record is set to default or not. */
    /* */
    PSL_U_LONGWORD_T        tdf_default_flags1l;
#define TDF_THRDF1_TRM_NAME 0X1
#define TDF_THRDF1_USERNAME 0X2
#define TDF_THRDF1_DEVICE_TYPE 0X4
#define TDF_THRDF1_LTNODE 0X8
#define TDF_THRDF1_NUMBER 0X10
#define TDF_THRDF1_PHYSICAL_DEVICE 0X20
#define TDF_THRDF1_SEQ_NO 0X40
#define TDF_THRDF1_START 0X80
#define TDF_THRDF1_CHARACTER_SIZE 0X100
#define TDF_THRDF1_DELAY 0X200
#define TDF_THRDF1_WIDTH 0X400
#define TDF_THRDF1_PAGE 0X800
#define TDF_THRDF1_BAUD_RATE 0X1000
#define TDF_THRDF1_TERMINAL_TYPE 0X2000
#define TDF_THRDF1_TERMINAL_ID 0X4000
#define TDF_THRDF1_LOGIN_TYPE 0X8000
#define TDF_THRDF1_TRACEFLAG 0X10000
#define TDF_THRDF1_TRACEFILE 0X20000
#define TDF_THRDF1_RECORD 0X40000
#define TDF_THRDF1_RECORD_TYPE 0X80000
#define TDF_THRDF1_TYPEAHEAD 0X100000
#define TDF_THRDF1_RECORDFILE 0X200000
#define TDF_THRDF1_INPUT_TIMING 0X400000
#define TDF_THRDF1_STARTDELAY 0X800000
#define TDF_THRDF1_ANSWERBACK 0X1000000
#define TDF_THRDF1_DETACH_EXIT 0X2000000
#define TDF_THRDF1_HANGUP_DETACH 0X4000000
#define TDF_THRDF1_STOPRECDET 0X8000000
#define TDF_THRDF1_FKEY_CONTROL 0X10000000
#define TDF_THRDF1_LTSERVICE 0X20000000
    PSL_U_LONGWORD_T        tdf_default_flags2l;
#define TDF_THRDF2_AUTO_SYNC 0X1
#define TDF_THRDF2_ENABLE_IOM 0X2
#define TDF_THRDF2_TCPIPADDR 0X4
#define TDF_THRDF2_TRMINFO 0X8
#define TDF_THRDF2_TST_UNIT 0X10
#define TDF_THRDF2_TCPIP_PORT 0X20
#define TDF_THRDF2_USE_TCP_SOCKET 0X40
#define TDF_THRDF2_EMULMOD 0X80
#define TDF_THRDF2_EMULTYP 0X100
#define TDF_THRDF2_RECORD_OPTS 0X200
    /* */
    /*    This is an old V5.3 field which used to hold the TCP/IP address. This */
    /*    field has now been extended and is held as FQTCPIP_ADDRESSC128 */
    /* */
    char                    tdf_54x007_tcpipaddrc20[20];
    /* */
    /*    This field defines the test unit name of this definition. */
    /* */
    char                    tdf_unit_namec20[20];
    /* */
    /*    Define the character pointers to hold the expanded filenames after */
    /*    the parameter substitution has been applied. */
    /* */
    char                    *tdf_recfil_ptrpc;
    char                    *tdf_trcfil_ptrpc;

    /*    This field is used to hold the number of the TCP/IP port to connect to */
    /*    if the terminal type is Remote TCP/IP and the 'Direct Socket Connection' */
    /*    flag is TRUE */
    /* */
    PSL_LONGWORD_T          tdf_tcpip_portl;
    /* */
    /*    This field is a flag which indicates whether or not the terminal created */
    /*    (if a remote TCP/IP type) is to use the direct socket interface rather */
    /*    than create a terminal and spawn a Telnet connection. */
    /* */
    PSL_LONGWORD_T          tdf_use_tcp_socketl;

    /* */
    /*    add fields for the EMULATOR TYPE and MODEL used if the terminal type */
    /*    (thread type) is set to EMULATOR. */
    /* */
    char                    tdf_emulator_typec16[16];
#define            EMULTYPE_VT52 "VT52"
#define            EMULTYPE_VT100 "VT100"
#define            EMULTYPE_VT3007BIT "VT300 7-Bit"
#define            EMULTYPE_VT3008BIT "VT300 8-Bit"
#define            EMULTYPE_VIEWDATA40 "Viewdata 40"
#define            EMULTYPE_VIEWDATA80 "Viewdata 80"
#define            EMULTYPE_VIEWDATASPLIT "Viewdata Split"
#define            EMULTYPE_DG410 "DG410"
#define            EMULTYPE_HP70092 "HP70092"
#define            EMULTYPE_ANSIBBS "Ansi BBS"
#define            EMULTYPE_PT250 "PT250"
#define            EMULTYPE_TA6530 "TA6530"
#define            EMULTYPE_IBM3270 "IBM3270"
#define            EMULTYPE_IBM3151 "IBM3151"
#define            EMULTYPE_WY50 "WY50"
#define            EMULTYPE_WY50PLUS "WY50+"
#define            EMULTYPE_WY60 "WY60"
#define            EMULTYPE_TVI910 "TVI910"
#define            EMULTYPE_TVI920 "TVI920"
#define            EMULTYPE_TVI925 "TVI925"
#define            EMULTYPE_ADDSA2 "ADDSA2"
#define            EMULTYPE_HZ1500 "HZ1500"
#define            EMULTYPE_ATT4410 "ATT4410"
#define            EMULTYPE_STRATUSV102 "Stratus V102"
#define            EMULTYPE_ICL7561 "ICL7561"
#define            EMULTYPE_IBM5250 "IBM5250"
    char                    tdf_emulator_modelc16[16];
#define            EMULMODEL1_32782 "3278-2"
#define            EMULMODEL1_32783 "3278-3"
#define            EMULMODEL1_32784 "3278-4"
#define            EMULMODEL1_32785 "3278-5"
#define            EMULMODEL1_32782E "3278-2-E"
#define            EMULMODEL1_32783E "3278-3-E"
#define            EMULMODEL1_32784E "3278-4-E"
#define            EMULMODEL1_32785E "3278-5-E"
#define            EMULMODEL1_32792 "3279-2"
#define            EMULMODEL1_32793 "3279-3"
#define            EMULMODEL1_32794 "3279-4"
#define            EMULMODEL1_32795 "3279-5"
#define            EMULMODEL2_52911 "5291-1"
#define            EMULMODEL2_52922 "5292-2"
#define            EMULMODEL2_525111 "5251-11"
#define            EMULMODEL2_31792 "3179-2"
#define            EMULMODEL2_3196A1 "3196-A1"
#define            EMULMODEL2_31802 "3180-2"
#define            EMULMODEL2_3477FC "3477-FC"
#define            EMULMODEL2_3477FG "3477-FG"
#define            EMULMODEL2_3486BA "3486-BA"
#define            EMULMODEL2_3487HA "3487-HA"
#define            EMULMODEL2_3487HC "3487-HC"
    /* */
    /*    For TCP/IP terminals this field defines the fully qualified name of the */
    /*    TCP/IP Address to which this terminal is to attach. */
    /* */
    char                    tdf_fqtcpip_addressc128[128];
    /*
     *    Target type.
     */
    PSL_LONGWORD_T          tdf_targetl;

    /*
     *    Source File Type (Script/TOF)
     */
    PSL_LONGWORD_T      tdf_srctypel;

#ifdef WIN32
    char                    tdf_spare3b180[176];
#endif
#ifdef UNIX
#ifdef OSF
    char tdf_spare3b324[320];
    /* Pad to 2560 */
#else
    char tdf_spare3b212[208];
#endif
#endif
} ;
/* */
/**/
/*  End of C Include File */
/**/
#endif  /* PSL__SCL_TLODEF_LOADED */
