#ifndef PSL__SCL_TOF_LOADED
#define PSL__SCL_TOF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 * scl_tof.h:
 * ==========
 *
 * This module defines the structures used by scl to store 'tof' data, which
 * it uses to create cript object ('tof') files.
 */

/*
 *  SCL_TOFH
 *
 *  This structure is used to store data that is required in the script object
 *   ('tof') file header record.
 */
struct scl_tofh
{

    /*
     *  This field contains up to 50 characters of descriptive text for the
     *  script.
     */
    char            tofh_sdsc_c52[52];

    /*
     *  This field defines the mode of operation of the script.
     */
    char            tofh_mode_b;

#define                 MODE_SCREEN 0X1
#define                 MODE_DEBUG 0X2
#define                 MODE_CHR_CELL 0X4
#define                 MODE_IMPACT 0X8
#define                 MODE_TEST 0X10
#define                 MODE_HTTP 0X20

    /*
     *  Pad to word boundary.
     */
    char            tofh_spare1;

    /*
     *  This field indicates whether there is any operating specific code in
     *  the script. The codes are those written to the TOF file header record.
     */
    PSL_WORD_T      tofh_osspcl;

    /*
     *  This field indicates whether there is any platform specific code in
     *  the script.The codes are those written to the TOF file header record.
     */
    PSL_WORD_T      tofh_pltspcl;

    /*
     *  This field defines the maximum number of lines which should be allocated
     *  for the screen image. It is set from the value specified (or defaulted)
     *  for the MODE statement.
     */
    PSL_WORD_T      tofh_sline_w;

    /*
     *  This field defines the maximum number of columns which should be allocated
     *  for the screen image. It is set from the value specified (or defaulted)
     *  for the MODE statement.
     */
    PSL_WORD_T      tofh_scols_w;

    /*
     *  Unused bytes for expansion and padding to quadword boundary.
     */
    char            tofh_spare2_c216[216];

} ;

/*
 *  SCL_TOFL
 *
 *  This structure is used to store a set of 'tof' lines in a linked list.
 */
struct scl_tofl
{

    /*
     *  Pointer to 'tof' line buffer.
     */
    char            *tofl_linptrpc;
} ;

/*
 *  The following item defines the length that should be used for all TOF
 *  item buffers.
 */
#define     TOFBUFLEN 1024      /* TOF item buffer length */

/*
 *  The following is an arbitrary value used by SCL to limit the length of 'TOF'
 *  lines in order to produce reasonably 'neat' TOF output. TOF records or sub-records
 *  that can be split over multiple lines (mostly 'ltxt' fields) should be 'broken',
 *  where possible, to keep line length within this limit, e.g. value lists. The value is not
 *  an absolute limit - SCL can write beyond this column, e.g. where a single
 *  value in a value list is too long to fit or where an 'stxt' field extends
 *  beyond the limit.
 */
#define     TOF_RIGHT_MARGIN 72 /* TOF right margin */

/*
 *  End of C Include File.
 */
#endif  /* PSL__SCL_TOF_LOADED */
