/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/**
 * Define the methods for the classes that implement the AtvfRecRule
 * fully.
 **/
#include <string>
#ifndef __VAX
#include <typeinfo>
#endif

#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
#pragma warning(disable:4786)
using namespace std;
#else //unknown
#error Unknown Platform
#endif // unknown

// Local includes
#include "AtvfRealRecRules.h"

// AtvfNoValRule methods
AtvfNoValRule::AtvfNoValRule(const string& token)
        :   AtvfRecRule(token, false, false)
{
}


string AtvfNoValRule::type() const
{
    return "No Value";
}

// AtvfSimplRule methods
AtvfSimplRule::AtvfSimplRule(const string& token)
        :   AtvfRecRule(token, true, false)
{}


string AtvfSimplRule::type() const
{
    return "String";
}

// AtvfDelimRule methods
AtvfDelimRule::AtvfDelimRule(const string& token)
        :   AtvfRecRule(token, true, true)
{}


string AtvfDelimRule::type() const
{
    return "Delimited";
}

string* AtvfDelimRule::parseInput(AtvfInput& in) const
{
    string* strP = in.getLine();
    string::size_type startOfDelim = strP->find_first_not_of(" \t");

    // find last character in delimiter
    string::size_type endOfPos;
    for (endOfPos = strP->size();
            (strP->at(endOfPos-1)==' ') || (strP->at(endOfPos-1)=='\t');
            endOfPos--);

    string delim = strP->substr(startOfDelim,int(endOfPos-startOfDelim));
    delete strP;
    return in.getUntil(delim);
}

void AtvfDelimRule::saveValue(const string& value, ostream& out) const
{
    static string delimchr("\"\'`?#");
    string::size_type dpos = 0;
    string delim(1, delimchr[dpos++]);
    while(value.find(delim) != string::npos)
    {
        if(dpos < delimchr.size())
        {
            delim.resize(delim.size() - 1);
        }
        else
        {
            dpos = 0;
        }
        delim.append(1, delimchr[dpos++]);
    }
    out << " " << delim << endl;
    out << value << delim;
}
