/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TCommonRcrd.inl

#include "TCommonRcrd.h"

// Define RTTI keys for each record type ...

// VARIABLE MANIPULATION
const AtvfKey TRcrdSetvar::			KEY("Setvar");
const AtvfKey TRcrdFormat::			KEY("Format");
const AtvfKey TRcrdGenvar::			KEY("Genvar");
const AtvfKey TRcrdSeed::			KEY("Seed");
const AtvfKey TRcrdConvert::		KEY("Convert");

// DIAGNOSTIC / MESSAGING
const AtvfKey TRcrdLogmsg::			KEY("Logmsg");

// SYNCHRONISATION
const AtvfKey TRcrdWait::			KEY("Wait");

// FLOW CONTROL
const AtvfKey TRcrdLabel::			KEY("Label");
const AtvfKey TRcrdCond::			KEY("Conditional");
const AtvfKey TRcrdIF_Then::		KEY("IfThen");
const AtvfKey TRcrdElse::			KEY("Else");
const AtvfKey TRcrdEndIF::			KEY("Eif");
const AtvfKey TRcrdLoop::			KEY("Loop");
const AtvfKey TRcrdEndLoop::		KEY("Eloop");
const AtvfKey TRcrdBreakContinue::	KEY("BreakContinue");
const AtvfKey TRcrdGoto::			KEY("Goto");

// CALL SUBROUTINE
const AtvfKey TRcrdCsub::			KEY("Csub");
const AtvfKey TRcrdSubroutine::		KEY("Subroutine");
const AtvfKey TRcrdReturn::			KEY("Return");


//ERROR HANDLING/TERMINATION COMMANDS
const AtvfKey TRcrdAbort::  		KEY("Abort");
const AtvfKey TRcrdGoError::		KEY("Goerror");
const AtvfKey TRcrdDisable::		KEY("Disable");
const AtvfKey TRcrdExit::			KEY("Exit");


// STATE
const AtvfKey TRcrdLoad::			KEY("Load");

// FILE HANDLING
const AtvfKey TRcrdOpen::			KEY("Open");
const AtvfKey TRcrdRead::			KEY("Read");
const AtvfKey TRcrdWrite::			KEY("Write");
const AtvfKey TRcrdFcntl::			KEY("Fcntl");

// FORMAL TEST CONTROL
const AtvfKey TRcrdTcstart::		KEY("Tcstart");
const AtvfKey TRcrdTcterm::			KEY("Tcterm");
const AtvfKey TRcrdTcend::			KEY("Tcend");




// The following sections follow the headings detailed in the document
// 'Compiling SCL to TOF'

// *****************************************************************************
// * VARIABLE MANIPULATION
// *****************************************************************************

bool TRcrdSetvar::_registeredCreator = false;
const bool TRcrdSetvar::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdSetvar::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdSetvar>());
    _registeredCreator = true;
    return true;
}


bool TRcrdFormat::_registeredCreator = false;
const bool TRcrdFormat::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdFormat::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdFormat>());
    _registeredCreator = true;
    return true;
}

bool TRcrdGenvar::_registeredCreator = false;
const bool TRcrdGenvar::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdGenvar::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdGenvar>());
    _registeredCreator = true;
    return true;
}

bool TRcrdSeed::_registeredCreator = false;
const bool TRcrdSeed::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdSeed::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdSeed>());
    _registeredCreator = true;
    return true;
}

bool TRcrdConvert::_registeredCreator = false;
const bool TRcrdConvert::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdConvert::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdConvert>());
    _registeredCreator = true;
    return true;
}

// *****************************************************************************
// * DIAGNOSTIC / MESSAGING
// *****************************************************************************

bool TRcrdLogmsg::_registeredCreator = false;
const bool TRcrdLogmsg::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdLogmsg::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdLogmsg>());
    _registeredCreator = true;
    return true;
}

// *****************************************************************************
// * SYNCHRONISATION
// *****************************************************************************

bool TRcrdWait::_registeredCreator = false;
const bool TRcrdWait::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdWait::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdWait>());
    _registeredCreator = true;
    return true;
}

bool TRcrdLabel::_registeredCreator = false;
const bool TRcrdLabel::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdLabel::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdLabel>());
    _registeredCreator = true;
    return true;
}


// *****************************************************************************
// * FLOW CONTROL
// *****************************************************************************
bool TRcrdCond::_registeredCreator = false;
const bool TRcrdCond::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdCond::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdCond>());
    _registeredCreator = true;
    return true;
}

bool TRcrdIF_Then::_registeredCreator = false;
const bool TRcrdIF_Then::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdIF_Then::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdIF_Then>());
    _registeredCreator = true;
    return true;
}

bool TRcrdElse::_registeredCreator = false;
const bool TRcrdElse::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdElse::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdElse>());
    _registeredCreator = true;
    return true;
}

bool TRcrdEndIF::_registeredCreator = false;
const bool TRcrdEndIF::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdEndIF::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdEndIF>());
    _registeredCreator = true;
    return true;
}

bool TRcrdLoop::_registeredCreator = false;
const bool TRcrdLoop::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdLoop::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdLoop>());
    _registeredCreator = true;
    return true;
}

bool TRcrdEndLoop::_registeredCreator = false;
const bool TRcrdEndLoop::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdEndLoop::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdEndLoop>());
    _registeredCreator = true;
    return true;
}

bool TRcrdBreakContinue::_registeredCreator = false;
const bool TRcrdBreakContinue::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdBreakContinue::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdBreakContinue>());
    _registeredCreator = true;
    return true;
}


bool TRcrdGoto::_registeredCreator = false;
const bool TRcrdGoto::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdGoto::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdGoto>());
    _registeredCreator = true;
    return true;
}

bool TRcrdSubroutine::_registeredCreator = false;
const bool TRcrdSubroutine::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdSubroutine::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdSubroutine>());
    _registeredCreator = true;
    return true;
}

bool TRcrdReturn::_registeredCreator = false;
const bool TRcrdReturn::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdReturn::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdReturn>());
    _registeredCreator = true;
    return true;
}

bool TRcrdCsub::_registeredCreator = false;
const bool TRcrdCsub::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdCsub::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdCsub>());
    _registeredCreator = true;
    return true;
}
///////////////////////////////////////////////////////////////////////////
//  ERROR HANDLING/TERMINATION COMMANDS
///////////////////////////////////////////////////////////////////////////
bool TRcrdAbort::_registeredCreator = false;
const bool TRcrdAbort::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdAbort::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdAbort>());
    _registeredCreator = true;
    return true;
}
bool TRcrdGoError::_registeredCreator = false;
const bool TRcrdGoError::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdGoError::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdGoError>());
    _registeredCreator = true;
    return true;
}
bool TRcrdDisable::_registeredCreator = false;
const bool TRcrdDisable::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdDisable::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdDisable>());
    _registeredCreator = true;
    return true;
}
bool TRcrdExit::_registeredCreator = false;
const bool TRcrdExit::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdExit::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdExit>());
    _registeredCreator = true;
    return true;
}

// *****************************************************************************
// * STATE
// *****************************************************************************

bool TRcrdLoad::_registeredCreator = false;
const bool TRcrdLoad::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdLoad::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdLoad>());
    _registeredCreator = true;
    return true;
}

// *****************************************************************************
// * FILE HANDLING
// *****************************************************************************
bool TRcrdOpen::_registeredCreator = false;
const bool TRcrdOpen::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdOpen::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdOpen>());
    _registeredCreator = true;
    return true;
}

bool TRcrdRead::_registeredCreator = false;
const bool TRcrdRead::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdRead::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdRead>());
    _registeredCreator = true;
    return true;
}

bool TRcrdWrite::_registeredCreator = false;
const bool TRcrdWrite::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdWrite::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdWrite>());
    _registeredCreator = true;
    return true;
}

bool TRcrdFcntl::_registeredCreator = false;
const bool TRcrdFcntl::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdFcntl::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdFcntl>());
    _registeredCreator = true;
    return true;
}

// *****************************************************************************
// * FORMAL TEST CONTROL
// *****************************************************************************
bool TRcrdTcstart::_registeredCreator = false;
const bool TRcrdTcstart::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdTcstart::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdTcstart>());
    _registeredCreator = true;
    return true;
}

bool TRcrdTcterm::_registeredCreator = false;
const bool TRcrdTcterm::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdTcterm::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdTcterm>());
    _registeredCreator = true;
    return true;
}

bool TRcrdTcend::_registeredCreator = false;
const bool TRcrdTcend::_RegisterCreator()
{
    if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    (*recCreatorMap)[TRcrdTcend::KEY] = SharedPtr<TRecCreator>(new TRecMake<TRcrdTcend>());
    _registeredCreator = true;
    return true;
}

// Lookup the TRecCreator template for a Transaction Record given its KEY.

//static RecCreator_map_t* recCreatorMap = NULL;

SharedPtr<TRecCreator> getTRecCreator(const AtvfKey& key)
{
	//if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
    RecCreator_map_t::const_iterator i = recCreatorMap->find(key);
    return (
                (i == recCreatorMap->end()) ?
                    SharedPtr<TRecCreator>(new TRecMake<TRecord>())
                                            :
                    (*i).second
           );
}
