/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc	    TInterRecords.h
// @module  TInterRecords | This module defines the interface to all classes
//          derived from the TRecord class. These classes handle the processing
//	    of TOF records.
///////////////////////////////////////////////////////////////////////////////
#ifndef CYR_TINTERRECORDS
#define CYR_TINTERRECORDS

#ifdef WIN32
#pragma warning(disable:4786)
#endif

#include <map>
#include <list>
#include <string>

#ifdef unix
#elif defined (WIN32)
using namespace std;
#else //unknown
#error Unknown Platform
#endif // unknown

/**
 * Definition of Start Timer Record class
 **/
class TRcrdStimer : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdStimer(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdStimer(*this);
    }
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdStimer

/**
 * Definition of End Timer Record class
 **/
class TRcrdEtimer : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdEtimer(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdEtimer(*this);
    };
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdEtimer


#ifndef __DLLSA__
//
// Definition of Aquire mutex Record class
class TRcrdAcmutex : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdAcmutex(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}

    // Accessors
    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdAcmutex(*this);
    }
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdAcmutex

//
// Definition of Release mutex Record class
class TRcrdRlmutex : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdRlmutex(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdRlmutex(*this);
    }
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdRlmutex

//
// Definition of Set semaphore Record class
class TRcrdStsemaphore : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdStsemaphore(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdStsemaphore(*this);
    }
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdStsemaphore

//
// Definition of Wait for semaphore Record class
class TRcrdWtsemaphore : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdWtsemaphore(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdWtsemaphore(*this);
    }
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdWtsemaphore

//
// Definition of clear semaphore Record class
class TRcrdClsemaphore : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
    TRcrdClsemaphore(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}

    // Accessors
    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdClsemaphore(*this);
    }
    static const bool _RegisterCreator();

private:
    static bool _registeredCreator;
}
; // TRcrdClsemaphore

#endif __DLLSA__
#endif
