/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// mdbdelay.cxx - sleep for "ms" milliseconds scaled by
// "timefac" percentage ie. 100 = 100% of ms, 0 = 0% (no delay)

#include "mdbdelay.h"

extern "C"
{
    // For FD_ZERO
#include <memory.h>
#ifdef __VMS
#include <time.h>
#endif
}


void mdb_delay(unsigned int ms, unsigned int timefac)
{
    // Short or no delay, immediate return
    if((timefac == 0) || (ms < 100))
        return;

    // Scale delay if requested
    if(timefac != 100)
        ms = ms / (100/timefac);

    // select() appears not to wait on WIN32, use Sleep().
#if defined(WIN32)
    Sleep(ms);

#else
    MDBFDMASK fds;
    struct timeval to;

    to.tv_sec  = ms/1000;
    to.tv_usec = (ms*1000) - (to.tv_sec * 1000000);

    FD_ZERO(&fds);
    select(0, &fds, &fds, &fds, &to);

#endif
    return;
}
