/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "StdAfx.h"

#include "CECMutex.h"
#include "TEMainStructure.hpp"

extern CORBAServer*		pCServer;//This has become a Global variable due to
extern CMainExecuterStructure *g_pTExecuter;

//compilation problems if included in some headers
CECMutex::CECMutex(const string& name,int scope)
#ifndef CYRANO_TP_STANDALONE
    :   m_key(0)
#endif
{
	m_bOnLan = false;
	m_bOnWeb = false;

#ifndef CYRANO_TP_STANDALONE

    m_strName=name;
    switch(scope)
    {
    case TOF_MTXSCP_TESTWIDE:
        
        m_strName=g_pTExecuter->GetTestRun()+"."+name+".TEST-WIDE";
        break;
    case TOF_MTXSCP_LOCAL:
    default:
        m_strName=g_pTExecuter->GetTestRun()+"."+name+".LOCAL";
        break;
    }

	//Does the mutex already exist on LAN?
	try
    {
		CORBA::Object_var obj;
        obj=pCServer->FindObjectWithName(TPMutex::CLASS_NAME,m_strName.c_str());
		if(!(CORBA::is_nil(obj))) 
		{
			m_mutex=TPMutex::_narrow(obj);
			m_bOnLan = true;
			return;
		}
    }
    catch(...)
    {
	}

	//can we connect to the repository var factory on tihs LAN?
	try
	{
		CECInetInfo inet;
		CORBA::Object_var Ofac = pCServer->FindObjectWithName(VarFactory::CLASS_NAME,
        inet.GetIPAddressAsString(g_pTExecuter->GetRepositoryHost()).c_str());
		if(!(CORBA::is_nil(Ofac)))
		{
			m_bOnLan= true;
			try
			{
				VarFactory_var VF=VarFactory::_narrow(Ofac.inout());
				if(!CORBA::is_nil(VF))
				{
					//CREATING NEW MUTEX ON LAN
					Ofac=VF->CreateMutex(m_strName.c_str());//Create CYRANO Test type semaphore
					m_mutex=TPMutex::_narrow(Ofac);
					return;
				}
			}
			catch(...)
			{
				return;
			}
		}
	}
	catch(...)
	{
		m_bOnLan = false;
	}

	//Does it already exist on WEB ?
	try
	{
		CORBA::Object_var obj;
	    obj=pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
		if(!(CORBA::is_nil(obj)))
		{
			//narrowed to XML relay var factory
			XMLRelayVarFactory_var VFonWEB=XMLRelayVarFactory::_narrow(obj);
			if(!(CORBA::is_nil(VFonWEB)))
			{
				m_bOnWeb = true;
				try
				{
					if(VFonWEB->DoesMutexExist((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str()))
					{
						//FOUND EXISTING MUTEX ON WEB
						return;
					}
				}
				catch(...)
				{
				}
			}
		}
	}
	catch(...)
	{
		m_bOnWeb = false;
	}
		
	//create a new one on the WEB
	try
	{
		CORBA::Object_var obj;
		obj=pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			
		//narrowed to XML relay var factory
		XMLRelayVarFactory_var VFonWEB=XMLRelayVarFactory::_narrow(obj);
		if(!(CORBA::is_nil(VFonWEB)))
		{
			m_bOnWeb=VFonWEB->CreateMutex((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str());//Create CYRANO Test type semaphore
			return;
		}
	}
	catch(...)
	{
		m_bOnWeb = false;
	}

#endif
}
CECMutex::~CECMutex()
{}

bool CECMutex::lock()
{
#ifndef CYRANO_TP_STANDALONE
	
	if(m_bOnLan)
	{
		m_key=m_mutex->lock();
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			m_key=VFonWeb->TPMutex_lock((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str());
		}
		catch(...)
		{
			return false;
		}
	}
#else
    m_mutex.Lock();
#endif
    return true;
}

bool CECMutex::unlock()
{
#ifndef CYRANO_TP_STANDALONE
	if(m_bOnLan)
	{
		m_mutex->unlock(m_key);
		return true;
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			if(VFonWeb->TPMutex_unlock((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str(),m_key))
			{
				return true;
			}
		}
		catch(...)
		{
			return false;
		}
	}
#else
    m_mutex.Unlock();
#endif
    return true;
}

#ifndef CYRANO_TP_STANDALONE

bool CECMutex::trylock()
{
	if(m_bOnLan)
	{
		m_key=m_mutex->trylock();
		return m_key != -1; //failed to lock
	}
	else if(m_bOnWeb)
	{
		try
		{
			CORBA::Object_var obj2 = pCServer->FindObjectWithName(XMLRelayVarFactory::CLASS_NAME,"WebRelay");
			XMLRelayVarFactory_var VFonWeb = XMLRelayVarFactory::_narrow(obj2);
			m_key = VFonWeb->TPMutex_trylock((g_pTExecuter->GetRepositoryHost()).c_str(),m_strName.c_str());
			return m_key != -1; //failed to lock
		}
		catch(...)
		{
		}
	}
	return false;	
}

#endif
