/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*  tpg_subs.c:
 *  ===========
 *
 *  This module contains general subroutines used by TPG.
 */

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 *  Function Prototypes for External Routines.
 */
#include <psl_rtndef.h>         /* PSL Routines */
#include "tpg_rtndef.h"         /* TPG Routines */

/*
 *  Structure Definitions.
 */
#include <shrmsg.h>         /* PSL messages */
#include "tpg_comdef.h"         /* Common Data Structure */
#include "tpg_cmd.h"            /* Command Line Data Structure */

/************************************************************************/
/*                                  */
/*  TPG_GETNXT_FILE - Get next file specified on command line   */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     iret = TPG_GETNXT_FILE (tpg_com_ptr, outbuf_ptr);        */
/*                                  */
/*  Where:                              */
/*                                  */
/*     iret         = Unsigned Long PSL Routine Return Status   */
/*          = PSL_NORMAL - if success           */
/*          = PSL_ENDMATCH - if no more files match     */
/*     tpg_com_ptr  = Pointer to Common Data Structure      */
/*     outbuf_ptr   = Pointer to buffer to hold returned info.  */
/*                                  */
/*  This routine is called to pick up the file name of the next     */
/*  file to compile. It is based on the name specified on the   */
/*  command line.                           */
/*                                  */
/************************************************************************/

unsigned long TPG_GETNXT_FILE (struct tpg_com *tpg_com_ptr,
                               char *outbuf_ptr)
{
    /*
     *  Local data
     */
    unsigned long       iret;       /* Status Return variable */
    char       *src_name_ptr;  /* Pointer to Source File Name */
    struct   tpg_cmd    *tpg_cmd_ptr;   /* Pointer to Command Line Data */

    static Boolean      first_time = PTRUE; /* First time through flag */
    static   char       **src_ptr_ptr;  /* Pointer to pointer to string */
    static unsigned long src_count;     /* Source file count */

    /*
     *  Copy pointers for ease of reference.
     */
    tpg_cmd_ptr = (*tpg_com_ptr).cmd_ptrps;

    /*
     *  If first time, initialize pointer to the source file list and
     *  clear count.
     */
    if (first_time)
    {
        src_ptr_ptr = (*tpg_cmd_ptr).tpg_value_ptrppc;
        src_count = 0;
        first_time = PFALSE;
    }

    /*
     *  If there are no more files return status to indicate thsi.
     */
    if (src_count == (*tpg_cmd_ptr).tpg_value_countl)
    {
        iret = PSL_ENDMATCH;
        goto end;
    }

    /*
     *  Copy source file name to buffers in Common Data Structure.
     *  Let's hope these buffers are big enough!
     */
    src_name_ptr = *src_ptr_ptr;

    (void) strcpy (outbuf_ptr, src_name_ptr);

    /*
     *  Increment counter and point to next pointer.
     */
    src_count += 1;
    src_ptr_ptr += 1;

    iret = PSL_NORMAL;      /* Signal Success */

    /*
     *  END:
     *
     *  Routine exit point.
     */
end:
    return iret;
}
