/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayMap_i.h"
#include <CECInetInfo.h>
#include "..\include\Logger.h"
#include "OstaTraceDef.h"
#include "xmlrpc.h"

extern CLogger* g_Logger;
const string RelayMapFile = "relaymap.dat";

XMLRelayMapImpl::XMLRelayMapImpl(CORBAServer* corba, CServerSettings settings)
{
    m_registered_name="WebRelay";
	m_settings = settings;

    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayMap: Registering XMLRelayMap interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayMap::CLASS_NAME,m_registered_name.c_str());
	LoadRelyMap();
}

XMLRelayMapImpl::~XMLRelayMapImpl()
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayMap: DeRegistering XMLRelayMap interface");
    m_corba->UnNameObject(XMLRelayMap::CLASS_NAME,m_registered_name.c_str());
}

CORBA::Boolean XMLRelayMapImpl::GetRelayForAlias(const char* strAlias, RelayDef& def)
{
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayMap: GetRelayForAlias: Lookup Alias "+string(strAlias));
	for(RelayMap::iterator iter = m_relayMap.begin(); iter != m_relayMap.end(); iter++)
	{
		if (iter->first == string(strAlias))
		{
			def = iter->second;
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayMap: GetRelayForAlias: Found Alias "+string(strAlias));
			return true;
		}
	}
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayMap: GetRelayForAlias: Cannot Find Alias "+string(strAlias));
	return false;
}

void XMLRelayMapImpl::AddRelayForAlias(const RelayDef& def)
{
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayMap: AddRelayForAlias: Adding Alias "+string(def.strHost));
	m_relayMap.insert(RelayMap::value_type(string(def.strAlias), def));
}

void XMLRelayMapImpl::GetRelayList(RelayList& rList)
{
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayMap: GetRelayList: Sending relay list");

	rList.length(m_relayMap.size());
	int i=0;
	for(RelayMap::iterator iter = m_relayMap.begin(); iter != m_relayMap.end(); iter++)
	{
		rList[i] = iter->second;
		i++;
	}

	return;
}


void XMLRelayMapImpl::LoadRelyMap()
{
	g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayMap: LoadRelyMap");
	string strPath = getenv("OPENSTA_HOME");
	strPath+="Server\\"+RelayMapFile;

	ifstream f;
	f.open(strPath.c_str(), ios::in);
	if (f)
	{
	    char* pLine=new char[256];
		while(f.getline( pLine, 256))
		{
			RelayDef def;
			string strLine = pLine;

			// Read Alias
			int nComma = strLine.find_first_of(',');
			string strAlias = strLine.substr(0, nComma);
			def.strAlias = strAlias.c_str();
			strLine.erase(0,nComma+1);

			// Read Host IP Address / Fully Qualified Domain Name
			nComma = strLine.find_first_of(',');
			def.strHost = strLine.substr(0, nComma).c_str();
			strLine.erase(0,nComma+1);

			// Read Path to the XMLRPC ISAPI Extension
			nComma = strLine.find_first_of(',');
			def.strExtPath = strLine.substr(0, nComma).c_str();
			strLine.erase(0,nComma+1);

			// Read Port
			nComma = strLine.find_first_of(',');
			def.lPort = atoi(strLine.substr(0, nComma).c_str());
			strLine.erase(0,nComma+1);

			// Read Username
			nComma = strLine.find_first_of(',');
			def.strUser = strLine.substr(0, nComma).c_str();
			strLine.erase(0,nComma+1);

			// Read Password
			nComma = strLine.find_first_of(',');
			def.strPass = strLine.substr(0, nComma).c_str();
			strLine.erase(0,nComma+1);

			// Read Use SSL
			nComma = strLine.find_first_of(',');
			def.bUseSSL = atoi(strLine.substr(0, nComma).c_str()) != FALSE;

			m_relayMap.insert(RelayMap::value_type(strAlias, def));
		}
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_NONE, "RelayMap: LoadRelyMap: Failed to open relay map file");
	}
}

CORBA::Boolean XMLRelayMapImpl::GetLocalRelayDef(RelayDef& def)
{
	g_Logger->WriteLogMsg(TRCLEV_NONE, "RelayMap: GetLocalRelayDef");

	CECInetInfo info;
	def.strAlias = info.GetHostName().c_str();

	string strAddr = info.GetIPAddressAsString();
    string::size_type p = 0;
    while((p = strAddr.find_first_of('_', p)) != strAddr.npos)
        strAddr.replace(p++, 1, 1, '.');

	def.strHost = strAddr.c_str();
	def.strExtPath = m_settings.m_strExtensionPath.c_str();
	def.lPort = m_settings.m_nWebPort;
	def.strUser = "";
	def.strPass = "";
	def.bUseSSL = m_settings.m_bUseSSL;

	return true;
}
