/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IS_HTTPExecuterSnap.cpp: implementation of the CXML_IS_HTTPExecuterSnap class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IS_HTTPExecuterSnap.h"
#include "XML_IS_HTTPTaskGroupSnap.h"
#include "XML_IS_HTTPTaskSnap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IS_HTTPExecuterSnap::CXML_IS_HTTPExecuterSnap()
{
	m_bSet = false;
}

CXML_IS_HTTPExecuterSnap::CXML_IS_HTTPExecuterSnap(HTTP_TestExecuterSnapshot_var HTTPTeSnapshot_var)
{
	SetValue(HTTPTeSnapshot_var);
}

CXML_IS_HTTPExecuterSnap::CXML_IS_HTTPExecuterSnap(HTTP_TestExecuterSnapshot HTTPTeSnapshot)
{
	SetValue(HTTPTeSnapshot);
}


CXML_IS_HTTPExecuterSnap::~CXML_IS_HTTPExecuterSnap()
{

}

void CXML_IS_HTTPExecuterSnap::SetValue(HTTP_TestExecuterSnapshot HTTPTeSnapshot)
{
	m_HTTPTeSnapshot = HTTPTeSnapshot;
	m_bSet = true;
}

void CXML_IS_HTTPExecuterSnap::SetValue(HTTP_TestExecuterSnapshot_var HTTPTeSnapshot_var)
{
	//copy from the var type into the member variable

	m_HTTPTeSnapshot.lAvgConnectionTime=HTTPTeSnapshot_var->lAvgConnectionTime;
	m_HTTPTeSnapshot.strTestExecuter=HTTPTeSnapshot_var->strTestExecuter;
					
	long numberOfTaskGroupSnaps=HTTPTeSnapshot_var->seqTaskGroupSnaps.length();
	m_HTTPTeSnapshot.seqTaskGroupSnaps.length(numberOfTaskGroupSnaps);

	//now copy all the task group snaps

	if(numberOfTaskGroupSnaps)
	{
		HTTP_TaskGroupSnap taskGroupSnap;

		for(long a= 0; a < numberOfTaskGroupSnaps; a++)
		{
		    taskGroupSnap.lTaskGroupId         = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lTaskGroupId;
			taskGroupSnap.lCompletedIterations = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lCompletedIterations;
			taskGroupSnap.lRunTime             = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lRunTime;
			taskGroupSnap.lTotalUsers          = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lTotalUsers;
			taskGroupSnap.lRequests            = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lRequests;
			taskGroupSnap.lFailedRequests      = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lFailedRequests;
			taskGroupSnap.dBytesIn             = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].dBytesIn;
			taskGroupSnap.dBytesOut            = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].dBytesOut;
			taskGroupSnap.lMinRequestLatency   = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lMinRequestLatency;
			taskGroupSnap.lMaxRequestLatency   = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lMaxRequestLatency;
			taskGroupSnap.lAvgRequestLatency   = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].lAvgRequestLatency;

			long numberOfTaskSnaps = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].seqTaskSnaps.length();
			taskGroupSnap.seqTaskSnaps.length(numberOfTaskSnaps);

			if(numberOfTaskSnaps)
			{
				HTTP_TaskSnap ts;
							
				for(long b = 0; b < numberOfTaskSnaps; b++)
				{
					ts.lTaskId      = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].seqTaskSnaps[b].lTaskId;
				    ts.lActiveUsers = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].seqTaskSnaps[b].lActiveUsers;
				    ts.lIterations  = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].seqTaskSnaps[b].lIterations;
				    ts.lRuntime     = HTTPTeSnapshot_var->seqTaskGroupSnaps[a].seqTaskSnaps[b].lRuntime;
					taskGroupSnap.seqTaskSnaps[b]=ts;
			   }
			}
			m_HTTPTeSnapshot.seqTaskGroupSnaps[a]=taskGroupSnap;				
		}					
	}
	m_bSet = true;
}

HTTP_TestExecuterSnapshot CXML_IS_HTTPExecuterSnap::GetValue()
{
	return m_HTTPTeSnapshot;
}

string CXML_IS_HTTPExecuterSnap::GetType()
{
	return "HTTPExecuterSnapshot";
}

string CXML_IS_HTTPExecuterSnap::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}


bool CXML_IS_HTTPExecuterSnap::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;


	strXML += "<value>";
	strXML += "<HTTPExecuterSnapshot>";
	strXML += AddMember("strTestExecuter",&CXMLString(string(m_HTTPTeSnapshot.strTestExecuter)));
	strXML += AddMember("lAvgConnectionTime",&CXMLLong(m_HTTPTeSnapshot.lAvgConnectionTime));


	CXMLArray arrTasks;
	for (int i = 0; i<(int)m_HTTPTeSnapshot.seqTaskGroupSnaps.length(); i++)
	{
		HTTP_TaskGroupSnap temp = m_HTTPTeSnapshot.seqTaskGroupSnaps[i];
		CXML_IS_HTTPTaskGroupSnap* task = new CXML_IS_HTTPTaskGroupSnap(temp);
		arrTasks.AddElement(task);
	}

	strXML += AddMember("seqTaskGroupSnaps",&arrTasks);

	strXML += "</HTTPExecuterSnapshot>";
	strXML += "</value>";


	return true;
}

bool CXML_IS_HTTPExecuterSnap::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pMember = pNode->GetFirstChild();
	while(pMember)
	{
		CXMLNodeElement* pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = pMember->GetChildElementByName("value");
		CXMLNodeElement* pType = pValue->GetFirstChild();

		
		if (strName == "lAvgConnectionTime")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTPTeSnapshot.lAvgConnectionTime=pVal->GetValue();
		}
		else if (strName == "strTestExecuter")
		{
			CXMLString* pVal = new CXMLString();
			pVal->FromXML(pType);
			m_HTTPTeSnapshot.strTestExecuter=pVal->GetValue().c_str();
		}
		else if (strName == "seqTaskGroupSnaps")
		{
			CXMLArray* pVal = new CXMLArray();
			pVal->FromXML(pType);
			m_HTTPTeSnapshot.seqTaskGroupSnaps.length(pVal->GetSize());
			for (int i =0; i<pVal->GetSize(); i++)
			{
				CXML_IS_HTTPTaskGroupSnap* pSnap = (CXML_IS_HTTPTaskGroupSnap*)pVal->GetAt(i);
				m_HTTPTeSnapshot.seqTaskGroupSnaps[i] = pSnap->GetValue();
			}
		}
		pMember = pMember->GetNextSibling();
	}

	return true;
}

