/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IS_HTTPTaskGroupSnap.cpp: implementation of the CXML_IS_HTTPTaskGroupSnap class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IS_HTTPTaskGroupSnap.h"
#include "XML_IS_HTTPTaskSnap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IS_HTTPTaskGroupSnap::CXML_IS_HTTPTaskGroupSnap()
{
	m_bSet = false;
}

CXML_IS_HTTPTaskGroupSnap::~CXML_IS_HTTPTaskGroupSnap()
{

}


CXML_IS_HTTPTaskGroupSnap::CXML_IS_HTTPTaskGroupSnap(HTTP_TaskGroupSnap TGSnap)
{
	SetValue(TGSnap);
}


void CXML_IS_HTTPTaskGroupSnap::SetValue(HTTP_TaskGroupSnap TGSnap)
{
	m_HTTP_TaskGroupSnap = TGSnap;
	m_bSet = true;
}


HTTP_TaskGroupSnap CXML_IS_HTTPTaskGroupSnap::GetValue()
{
	return m_HTTP_TaskGroupSnap;
}

string CXML_IS_HTTPTaskGroupSnap::GetType()
{
	return "HTTPTaskGroupSnap";
}

string CXML_IS_HTTPTaskGroupSnap::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}


bool CXML_IS_HTTPTaskGroupSnap::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;


	strXML += "<value>";
	strXML += "<HTTPTaskGroupSnap>";

	strXML += AddMember("lTaskGroupId",&CXMLLong(m_HTTP_TaskGroupSnap.lTaskGroupId));
	strXML += AddMember("lCompletedIterations",&CXMLLong(m_HTTP_TaskGroupSnap.lCompletedIterations));
	strXML += AddMember("lRunTime",&CXMLLong(m_HTTP_TaskGroupSnap.lRunTime));
	strXML += AddMember("lTotalUsers",&CXMLLong(m_HTTP_TaskGroupSnap.lTotalUsers));
	strXML += AddMember("lRequests",&CXMLLong(m_HTTP_TaskGroupSnap.lRequests));
	strXML += AddMember("lFailedRequests",&CXMLLong(m_HTTP_TaskGroupSnap.lFailedRequests));
	strXML += AddMember("dBytesIn",&CXMLDouble(m_HTTP_TaskGroupSnap.dBytesIn));
	strXML += AddMember("dBytesOut",&CXMLDouble(m_HTTP_TaskGroupSnap.dBytesOut));
	strXML += AddMember("lMinRequestLatency",&CXMLLong(m_HTTP_TaskGroupSnap.lMinRequestLatency));
	strXML += AddMember("lMaxRequestLatency",&CXMLLong(m_HTTP_TaskGroupSnap.lMaxRequestLatency));
	strXML += AddMember("lAvgRequestLatency",&CXMLLong(m_HTTP_TaskGroupSnap.lAvgRequestLatency));

	
	CXMLArray arrTasks;
	for (int i = 0; i<(int)m_HTTP_TaskGroupSnap.seqTaskSnaps.length(); i++)
	{
		HTTP_TaskSnap temp = m_HTTP_TaskGroupSnap.seqTaskSnaps[i];
		CXML_IS_HTTPTaskSnap* task = new CXML_IS_HTTPTaskSnap(temp);
		arrTasks.AddElement(task);
	}
	strXML += AddMember("seqTaskSnaps",&arrTasks);

	strXML += "</HTTPTaskGroupSnap>";
	strXML += "</value>";


	return true;
}

bool CXML_IS_HTTPTaskGroupSnap::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pMember = pNode->GetFirstChild();
	while(pMember)
	{
		CXMLNodeElement* pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = pMember->GetChildElementByName("value");
		CXMLNodeElement* pType = pValue->GetFirstChild();

		
		if (strName == "lTaskGroupId")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lTaskGroupId=pVal->GetValue();
		}
		else if (strName == "lCompletedIterations")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lCompletedIterations=pVal->GetValue();
		}
		else if (strName == "lRunTime")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lRunTime=pVal->GetValue();
		}
		else if (strName == "lTotalUsers")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lTotalUsers=pVal->GetValue();
		}
		else if (strName == "lRequests")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lRequests=pVal->GetValue();
		}
		else if (strName == "lFailedRequests")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lFailedRequests=pVal->GetValue();
		}
		else if (strName == "dBytesIn")
		{
			CXMLDouble* pVal = new CXMLDouble();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.dBytesIn=pVal->GetValue();
		}
		else if (strName == "dBytesOut")
		{
			CXMLDouble* pVal = new CXMLDouble();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.dBytesOut=pVal->GetValue();
		}
		else if (strName == "lMinRequestLatency")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lMinRequestLatency=pVal->GetValue();
		}
		else if (strName == "lMaxRequestLatency")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lMaxRequestLatency=pVal->GetValue();
		}
		else if (strName == "lAvgRequestLatency")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.lAvgRequestLatency=pVal->GetValue();
		}
		else if (strName == "seqTaskSnaps")
		{
			CXMLArray* pVal = new CXMLArray();
			pVal->FromXML(pType);
			m_HTTP_TaskGroupSnap.seqTaskSnaps.length(pVal->GetSize());
			for (int i =0; i<pVal->GetSize(); i++)
			{
				CXML_IS_HTTPTaskSnap* pSnap = (CXML_IS_HTTPTaskSnap*)pVal->GetAt(i);
				m_HTTP_TaskGroupSnap.seqTaskSnaps[i] = pSnap->GetValue();
			}
		}
		pMember = pMember->GetNextSibling();
	}

	return true;
}

