/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IS_TaskGroupSnap.cpp: implementation of the CXML_IS_TaskGroupSnap class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IS_TaskGroupSnap.h"
#include "XML_IS_TaskSnap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IS_TaskGroupSnap::CXML_IS_TaskGroupSnap()
{
	m_bSet = false;
}

CXML_IS_TaskGroupSnap::CXML_IS_TaskGroupSnap(TaskGroupSnap tgSnap)
{
	SetValue(tgSnap);
}

CXML_IS_TaskGroupSnap::~CXML_IS_TaskGroupSnap()
{

}

void CXML_IS_TaskGroupSnap::SetValue(TaskGroupSnap tgSnap)
{
	m_tgSnap = tgSnap;
	m_bSet = true;
}

TaskGroupSnap CXML_IS_TaskGroupSnap::GetValue()
{
	return m_tgSnap;
}

string CXML_IS_TaskGroupSnap::GetType()
{
	return "TaskGroupSnap";
}

bool CXML_IS_TaskGroupSnap::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;

	strXML += "<value>";
	strXML += "<TaskGroupSnap>";
	strXML += AddMember("lTaskGroupId",&CXMLLong(m_tgSnap.lTaskGroupId));
	strXML += AddMember("lCompletedIterations",&CXMLLong(m_tgSnap.lCompletedIterations));
	strXML += AddMember("lRunTime",&CXMLLong(m_tgSnap.lRunTime));
	strXML += AddMember("lActiveUsers",&CXMLLong(m_tgSnap.lActiveUsers));
	strXML += AddMember("lRequests",&CXMLLong(m_tgSnap.lRequests));
	strXML += AddMember("dBytes",&CXMLDouble(m_tgSnap.dBytes));

	
	CXMLArray arrTasks;
	for (int i = 0; i<(int)m_tgSnap.seqTaskSnaps.length(); i++)
	{
		TaskSnap snap = m_tgSnap.seqTaskSnaps[i];
		CXML_IS_TaskSnap* tSnap = new CXML_IS_TaskSnap(snap);
		arrTasks.AddElement(tSnap);
	}
	strXML += AddMember("seqTaskSnaps",&arrTasks);
	
	
	strXML += "</TaskGroupSnap>";
	strXML += "</value>";

	return true;
}

bool CXML_IS_TaskGroupSnap::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pMember = pNode->GetFirstChild();
	while(pMember)
	{
		CXMLNodeElement* pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = pMember->GetChildElementByName("value");
		CXMLNodeElement* pType = pValue->GetFirstChild();

		if (strName == "lTaskGroupId")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgSnap.lTaskGroupId=pVal->GetValue();
		}
		else if (strName == "lCompletedIterations")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgSnap.lCompletedIterations=pVal->GetValue();
		}
		else if (strName == "lRunTime")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgSnap.lRunTime=pVal->GetValue();
		}
		else if (strName == "lActiveUsers")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgSnap.lActiveUsers=pVal->GetValue();
		}
		else if (strName == "lRequests")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_tgSnap.lRequests=pVal->GetValue();
		}
		else if (strName == "dBytes")
		{
			CXMLDouble* pVal = new CXMLDouble();
			pVal->FromXML(pType);
			m_tgSnap.dBytes=pVal->GetValue();
		}
		else if (strName == "seqTaskSnaps")
		{
			CXMLArray* pVal = new CXMLArray();
			pVal->FromXML(pType);
			m_tgSnap.seqTaskSnaps.length(pVal->GetSize());
			for (int i =0; i<pVal->GetSize(); i++)
			{
				CXML_IS_TaskSnap* pSnap = (CXML_IS_TaskSnap*)pVal->GetAt(i);
				m_tgSnap.seqTaskSnaps[i] = pSnap->GetValue();
			}
		}
		pMember = pMember->GetNextSibling();
	}

	return true;
}

string CXML_IS_TaskGroupSnap::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}
