/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XML_IS_VUSnap.cpp: implementation of the CXML_IS_VUSnap class.
//
//////////////////////////////////////////////////////////////////////

#include "XML_IS_VUSnap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXML_IS_VUSnap::CXML_IS_VUSnap()
{
	m_bSet = false;
}

CXML_IS_VUSnap::CXML_IS_VUSnap(VUSnap snap)
{
	SetValue(snap);
}

CXML_IS_VUSnap::~CXML_IS_VUSnap()
{

}

void CXML_IS_VUSnap::SetValue(VUSnap snap)
{
	m_snap = snap;
	m_bSet = true;
}

VUSnap CXML_IS_VUSnap::GetValue()
{
	return m_snap;
}

string CXML_IS_VUSnap::GetType()
{
	return "VUSnap";
}

bool CXML_IS_VUSnap::ToXML(string& strXML)
{
	if (!m_bSet)
		return false;

	strXML += "<value>";
	strXML += "<VUSnap>";
	strXML += AddMember("lVUId",&CXMLLong(m_snap.lVUId));
	strXML += AddMember("lStartTime",&CXMLLong(m_snap.lStartTime));
	strXML += AddMember("lRunTime",&CXMLLong(m_snap.lRunTime));
	strXML += AddMember("lCompletedIterations",&CXMLLong(m_snap.lCompletedIterations));
	strXML += AddMember("strNoteText",&CXMLString(string(m_snap.strNoteText)));
	strXML += "</VUSnap>";
	strXML += "</value>";

	return true;
}

bool CXML_IS_VUSnap::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pMember = pNode->GetFirstChild();
	while(pMember)
	{
		CXMLNodeElement* pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = pMember->GetChildElementByName("value");
		CXMLNodeElement* pType = pValue->GetFirstChild();

		if (strName == "strNoteText")
		{
			CXMLString* pVal = new CXMLString();
			pVal->FromXML(pType);
			m_snap.strNoteText=pVal->GetValue().c_str();
		}
		else if (strName == "lVUId")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_snap.lVUId=pVal->GetValue();
		}
		else if (strName == "lStartTime")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_snap.lStartTime=pVal->GetValue();
		}
		else if (strName == "lRunTime")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_snap.lRunTime=pVal->GetValue();
		}
		else if (strName == "lCompletedIterations")
		{
			CXMLLong* pVal = new CXMLLong();
			pVal->FromXML(pType);
			m_snap.lCompletedIterations=pVal->GetValue();
		}
		pMember = pMember->GetNextSibling();
	}

	return true;
}

string CXML_IS_VUSnap::AddMember(string name, CXMLParameter* param)
{
	string strXML = "";
	strXML += "<member>";
	strXML += "<name>"+name+"</name>";
	string strTmp;
	param->ToXML(strTmp);
	strXML += strTmp;
	strXML += "</member>";
	return strXML;
}
