/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayArchMgr_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;

XMLRelayArchMgrImpl::XMLRelayArchMgrImpl(CORBAServer* corba)
{
    // Get IP address (with dots replaced by underscores) of daemon host.
    m_registered_name="WebRelay";
    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Registering XMLRelayArchMgrImpl interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayArchMgr::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayArchMgrImpl::~XMLRelayArchMgrImpl()
{
    m_corba->UnNameObject( XMLRelayArchMgr::CLASS_NAME,m_registered_name.c_str());
}

char* XMLRelayArchMgrImpl::FileHostProperties(const char* strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: FileHostProperties: Relaying FileHostProperties request for host "+string(strTargetHost));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetHost, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: FileHostProperties: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayAM: FileHostProperties: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("ARCHMGR.FileHostProperties");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetHost));
	CXMLResponse* resp = req->SendRequest();
	g_Logger->WriteLogMsg(TRCLEV_HIGH, resp->m_strXMLResponse);
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Relay Error: "+resp->PrintError());
		return false;
	}


	string strRet = "";
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "base64")
	{
		CXMLBase64* pStringVal = (CXMLBase64*)resp->m_params.GetAt(0);
		strRet = pStringVal->GetValue();
	}
	g_Logger->WriteLogMsg(TRCLEV_HIGH, strRet);

	delete req;
	delete resp;

    return CORBA::string_dup(strRet.c_str());
}

CORBA::Boolean XMLRelayArchMgrImpl::StartOSTADaemon(const char* strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: StartOSTADaemon: Relaying StartOSTADaemon request for host "+string(strTargetHost));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetHost, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: StartOSTADaemon: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayAM: StartOSTADaemon: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("ARCHMGR.StartOSTADaemon");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetHost));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Relay Error: "+resp->PrintError());
		return false;
	}


	bool bRet = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pBoolVal = (CXMLBool*)resp->m_params.GetAt(0);
		bRet = pBoolVal->GetValue();
	}

	delete req;
	delete resp;

    return bRet;
}

CORBA::Boolean XMLRelayArchMgrImpl::StopOSTADaemon(const char* strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: StopOSTADaemon: Relaying StopOSTADaemon request for host "+string(strTargetHost));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetHost, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: StopOSTADaemon: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayAM: StopOSTADaemon: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("ARCHMGR.StopOSTADaemon");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetHost));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Relay Error: "+resp->PrintError());
		return false;
	}


	bool bRet = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pBoolVal = (CXMLBool*)resp->m_params.GetAt(0);
		bRet = pBoolVal->GetValue();
	}

	delete req;
	delete resp;

    return bRet;
}

CORBA::Boolean XMLRelayArchMgrImpl::RestartOSTADaemon(const char* strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: RestartOSTADaemon: Relaying RestartOSTADaemon request for host "+string(strTargetHost));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetHost, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: RestartOSTADaemon: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayAM: RestartOSTADaemon: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("ARCHMGR.RestartOSTADaemon");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetHost));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Relay Error: "+resp->PrintError());
		return false;
	}


	bool bRet = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pBoolVal = (CXMLBool*)resp->m_params.GetAt(0);
		bRet = pBoolVal->GetValue();
	}

	delete req;
	delete resp;

    return bRet;
}

CORBA::Boolean XMLRelayArchMgrImpl::Stop(const char* strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Stop: Relaying Stop request for host "+string(strTargetHost));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetHost, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Stop: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayAM: Stop: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("ARCHMGR.Stop");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetHost));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Relay Error: "+resp->PrintError());
		return false;
	}


	bool bRet = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pBoolVal = (CXMLBool*)resp->m_params.GetAt(0);
		bRet = pBoolVal->GetValue();
	}

	delete req;
	delete resp;

    return bRet;
}

CORBA::Boolean XMLRelayArchMgrImpl::IsDmnRunning(const char* strTargetHost)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: IsDmnRunning: Relaying IsDmnRunning request for host "+string(strTargetHost));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetHost, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: IsDmnRunning: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayAM: IsDmnRunning: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("ARCHMGR.IsDmnRunning");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetHost));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayAM: Relay Error: "+resp->PrintError());
		return false;
	}


	bool bRet = false;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "boolean")
	{
		CXMLBool* pBoolVal = (CXMLBool*)resp->m_params.GetAt(0);
		bRet = pBoolVal->GetValue();
	}

	delete req;
	delete resp;

    return bRet;
}
