/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayTimeService_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

extern CLogger* g_Logger;

XMLRelayTimeServiceImpl::XMLRelayTimeServiceImpl(CORBAServer* corba)
{
    // Get IP address (with dots replaced by underscores) of daemon host.
    m_registered_name="WebRelay";
    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTS: Registering XMLRelayTimeService interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayTimeService::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayTimeServiceImpl::~XMLRelayTimeServiceImpl()
{
    m_corba->UnNameObject( XMLRelayTimeService::CLASS_NAME,m_registered_name.c_str());
}

CORBA::ULong  XMLRelayTimeServiceImpl::GetTime(const char* strTargetMachine)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTS: GetTime: Relaying GetTime request for host "+string(strTargetMachine));

	if (CORBA::is_nil(m_RelayMap))
	{
		CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
		m_RelayMap = XMLRelayMap::_narrow(obj);
	}

	RelayDef def;	
	if (!m_RelayMap->GetRelayForAlias(strTargetMachine, def))
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTS: GetTime: No relay server defined for this host");
		return false;
	}
	else
	{
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayTS: GetTime: Relay server is "+string(def.strHost));
	}

	
	CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);

	CXMLRequest* req = new CXMLRequest();
	req->SetMethodName("TIMESERVICE.GetTime");
	req->SetTransport(transport);
	req->AddParameter(new CXMLString(strTargetMachine));
	CXMLResponse* resp = req->SendRequest();
	if (resp->IsError())
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTS: Relay Error: "+resp->PrintError());
		return -1;
	}


	long lTime = -1;
	if (resp->m_params.GetSize() == 1 && resp->m_params.GetAt(0)->GetType() == "i4")
	{
		CXMLLong* pLongVal = (CXMLLong*)resp->m_params.GetAt(0);
		lTime = pLongVal->GetValue();
	}

	delete req;
	delete resp;

    return lTime;
}
