/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// FileControl.cpp: implementation of the CFileControl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileControl.h"
#include "CyrWrapLib.h"
#include "factory.h"
#include "CECInetInfo.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"
#include "FileTransferNode.h"

#include "..\RelayMapLib\XMLRelayMap.h"

extern CLogger* g_Logger;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileControl::CFileControl(CORBAServer* pServer, CServerSettings& settings)
{
	// Start Winsock
	WSADATA wsd;
	WSAStartup(0x0101, &wsd);
	m_pCorba = pServer;
	m_settings = settings;
}

CFileControl::~CFileControl()
{
    WSACleanup();
}

CXMLResponse* CFileControl::ProcessMethodCall(CXMLRequest* request)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "FC: ProcessMethodCall "+request->m_strFullMethodName);
	if (request->m_strMethod == "CheckOutReposTraceSettings")
	{
		if (request->GetNumParams() == 1)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			return CheckOutReposTraceSettings(pParam1->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 1)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 1)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "CheckOutReposData")
	{
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			return CheckOutReposData(pParam1->GetValue(),pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "CheckOutReposProfile")
	{
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			return CheckOutReposProfile(pParam1->GetValue(),pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "CheckOutReposScript")
	{
		if (request->GetNumParams() == 2)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			return CheckOutReposScript(pParam1->GetValue(),pParam2->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 2)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 2)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "CheckInReposResults")
	{
		if (request->GetNumParams() == 4)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			CXMLString* pParam3 = (CXMLString*)request->GetParam(2);
			CXMLString* pParam4 = (CXMLString*)request->GetParam(3);
			return CheckInReposResults(pParam1->GetValue(),pParam2->GetValue(),
									   pParam3->GetValue(),pParam4->GetValue());
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 4)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 4)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}
	else if (request->m_strMethod == "StoreResultsFile")
	{
		if (request->GetNumParams() == 5)
		{
			CXMLString* pParam1 = (CXMLString*)request->GetParam(0);
			CXMLString* pParam2 = (CXMLString*)request->GetParam(1);
			CXMLString* pParam3 = (CXMLString*)request->GetParam(2);
			CXMLString* pParam4 = (CXMLString*)request->GetParam(3);
			CXMLString* pParam5 = (CXMLString*)request->GetParam(4);
			CXMLResponse* resp =  StoreResultsFile(pParam1->GetValue(),pParam2->GetValue(),
									pParam3->GetValue(),pParam4->GetValue(),
									pParam5->GetValue());

			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Response = "+resp->ToXML());
			return resp;
		}
		else
		{
			CXMLResponse* response = new CXMLResponse();
			if (request->GetNumParams() < 5)
				response->SetError(CXMLResponse::INSUFFICIENT_PARAMS);
			if (request->GetNumParams() > 5)
				response->SetError(CXMLResponse::TOO_MANY_PARAMS);
			return response;
		}

	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: ProcessMethodCall: Error: Method Not Found");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::FUNC_NOT_FOUND);
	return response;
}

CXMLResponse* CFileControl::CheckOutReposTraceSettings(string strReposHost)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "FC: CheckOutReposTraceSettings: Repos = "+strReposHost);

	try
	{
		// Connect to the repository
	    CECInetInfo info;
	    string underscore_ip_name=info.GetIPAddressAsString(strReposHost);

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposTraceSettings: Get OrbServer");
		CECOrbServer ReposServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposTraceSettings: Get OrbProject");
		CECOrbProject Project(&ReposServer, "Repository");
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposTraceSettings: Get RootOrbFolder");
		CECOrbFolder RootFolder(&Project);

		CECOrbFile File(&RootFolder, "TraceSettings.txt");

		string strPath = "";
		if (m_settings.m_bUseFTP)
			strPath = m_settings.m_strFtpRoot + "TraceSettings.txt";
		else
			strPath = m_settings.m_strWebRoot + m_settings.m_strCachePath + "TraceSettings.txt";
		
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposTraceSettings: Check out to localPath = "+strPath);
		File.CheckOut(strPath.c_str(), false);

		string strURL = "";
		string strAddr = info.GetIPAddressAsString();
	    string::size_type p = 0;
	    while((p = strAddr.find_first_of('_', p)) != strAddr.npos)
	        // Replace continuation with space
	        strAddr.replace(p++, 1, 1, '.');

		if (m_settings.m_bUseFTP)
			strURL = "ftp://"+m_settings.m_strFtpUser+":"+m_settings.m_strFtpPass+"@"+strAddr+"/"+"TraceSettings.txt";
		else
			strURL = "http://"+ strAddr+m_settings.m_strCachePath+"TraceSettings.txt";

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposTraceSettings: URL = "+strURL);
		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBase64(strURL));
		return response;

	}
	catch(...)
	{
	}

	g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposTraceSettings: UNKNOWN_ERROR");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CFileControl::CheckOutReposProfile(string strReposHost, string strProfile)
{
	g_Logger->WriteLogMsg(TRCLEV_MEDIUM, "FC: CheckOutReposProfile: Repos = "+strReposHost+" Profile = "+strProfile);

	try
	{
		// Connect to the repository
	    CECInetInfo info;
	    string underscore_ip_name=info.GetIPAddressAsString(strReposHost);

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposProfile: Get OrbServer");
		CECOrbServer ReposServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposProfile: Get OrbProject");
		CECOrbProject Project(&ReposServer, "Repository");
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposProfile: Get RootFolder");
		CECOrbFolder RootFolder(&Project);

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposProfile: Get Profiles Folder");
		CECOrbFolder ProfilesFolder(&RootFolder, "Profiles");


		CECOrbFile File(&ProfilesFolder, strProfile.c_str());

		string strPath = "";
		if (m_settings.m_bUseFTP)
			strPath = m_settings.m_strFtpRoot + strProfile;
		else
			strPath = m_settings.m_strWebRoot + m_settings.m_strCachePath + strProfile;

		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposProfile: Check out to localPath = "+strPath);
		File.CheckOut(strPath.c_str(), false);

		string strURL = "";
		string strAddr = info.GetIPAddressAsString();
	    string::size_type p = 0;
	    while((p = strAddr.find_first_of('_', p)) != strAddr.npos)
	        // Replace continuation with space
	        strAddr.replace(p++, 1, 1, '.');

		if (m_settings.m_bUseFTP)
			strURL = "ftp://"+m_settings.m_strFtpUser+":"+m_settings.m_strFtpPass+"@"+strAddr+"/"+strProfile;
		else
			strURL = "http://"+ strAddr+m_settings.m_strCachePath+strProfile;
		g_Logger->WriteLogMsg(TRCLEV_HIGH, "FC: CheckOutReposProfile: URL = "+strURL);
		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBase64(strURL));
		return response;

	}
	catch(...)
	{
	}

	g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckOutReposProfile: UNKNOWN_ERROR");
	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CFileControl::CheckOutReposScript(string strReposHost, string strScript)
{
	try
	{
		// Connect to the repository
	    CECInetInfo info;
	    string underscore_ip_name=info.GetIPAddressAsString(strReposHost);

		CECOrbServer ReposServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		CECOrbProject Project(&ReposServer, "Repository");
		CECOrbFolder RootFolder(&Project);

		CECOrbFolder TofsFolder(&RootFolder, "ObjectCode");

		CECOrbFile File(&TofsFolder, strScript.c_str());

		string strPath = "";
		if (m_settings.m_bUseFTP)
			strPath = m_settings.m_strFtpRoot + strScript;
		else
			strPath = m_settings.m_strWebRoot + m_settings.m_strCachePath + strScript;


		File.CheckOut(strPath.c_str(), false);

		string strURL = "";
		string strAddr = info.GetIPAddressAsString();
	    string::size_type p = 0;
	    while((p = strAddr.find_first_of('_', p)) != strAddr.npos)
	        // Replace continuation with space
	        strAddr.replace(p++, 1, 1, '.');

		if (m_settings.m_bUseFTP)
			strURL = "ftp://"+m_settings.m_strFtpUser+":"+m_settings.m_strFtpPass+"@"+strAddr+"/"+strScript;
		else
			strURL = "http://"+ strAddr+m_settings.m_strCachePath+strScript;

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBase64(strURL));
		return response;

	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

CXMLResponse* CFileControl::CheckOutReposData(string strReposHost, string strData)
{
	try
	{
		// Connect to the repository
	    CECInetInfo info;
	    string underscore_ip_name=info.GetIPAddressAsString(strReposHost);

		CECOrbServer ReposServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		CECOrbProject Project(&ReposServer, "Repository");
		CECOrbFolder RootFolder(&Project);

		CECOrbFolder DataFolder(&RootFolder, "Data");


		CECOrbFile File(&DataFolder, strData.c_str());

		string strPath = "";
		if (m_settings.m_bUseFTP)
			strPath = m_settings.m_strFtpRoot + strData;
		else
			strPath = m_settings.m_strWebRoot + m_settings.m_strCachePath + strData;

		File.CheckOut(strPath.c_str(), false);

		string strURL = "";
		string strAddr = info.GetIPAddressAsString();
	    string::size_type p = 0;
	    while((p = strAddr.find_first_of('_', p)) != strAddr.npos)
	        // Replace continuation with space
	        strAddr.replace(p++, 1, 1, '.');

		if (m_settings.m_bUseFTP)
			strURL = "ftp://"+m_settings.m_strFtpUser+":"+m_settings.m_strFtpPass+"@"+strAddr+"/"+strData;
		else
			strURL = "http://"+ strAddr+m_settings.m_strCachePath+strData;
		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBase64(strURL));
		return response;

	}
	catch(...)
	{
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}


CXMLResponse* CFileControl::CheckInReposResults(string strReposHost, 
						string strFileURL, string strTest, string strTestRun)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: "+strFileURL);

	// Retreieve the file
	int nPos = strFileURL.find_last_of('/');
	string strFileName = strFileURL.substr(nPos+1,strFileURL.length()-nPos-1);
	string strLocalPath = ".\\" + strFileName;

	if (strFileURL.substr(0,4) == "http")
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: Retrieve the file to the local machine by HTTP.");
		CHTTPTransport transport(strFileURL);
		if (!transport.GetRemoteFile(strLocalPath))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: Retrieve FAILED!");
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(false));
			return response;
		}
	}
	else if (strFileURL.substr(0,3) == "ftp")
	{
		string strServer, strUser, strPass, strRemotePath;
		int nPort;

		if(ParseFTPURL(strFileURL, strServer, nPort, strUser, strPass, strRemotePath))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: Retrieve the file to the local machine by FTP.");
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: strServer = "+strServer);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: strUser = "+strUser);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: strPass = "+strPass);
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: strRemotePath = "+strRemotePath);
			CFTPFileTransfer ftp;
			ftp.OpenConnection(strServer, nPort, strUser, strPass);
			if (!ftp.GetFile(strRemotePath, strLocalPath))
			{
				g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: GetFile FAILED!");
				CXMLResponse* response = new CXMLResponse();
				response->m_params.Add(new CXMLBool(false));
				return response;
			}
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: Retrieve FAILED!");
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(false));
			return response;
		}
	}

	try
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: Connect to the repository.");
		// Connect to the repository
	    CECInetInfo info;
	    string underscore_ip_name=info.GetIPAddressAsString(strReposHost);

		CECOrbServer ReposServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		CECOrbProject Project(&ReposServer, "Repository");
		CECOrbFolder RootFolder(&Project);
		CECOrbFolder TestsFolder(&RootFolder, "Tests");

		CECOrbFolder TestFolder(&TestsFolder, strTest.c_str());
		CECOrbFolder RunFolder(&TestFolder, strTestRun.c_str());

		CECOrbFile File(&RunFolder, strFileName.c_str(), "");
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: Check in the file.");
		File.CheckIn(strLocalPath.c_str(), false);

		CXMLResponse* response = new CXMLResponse();
		CXMLBool* pBool = new CXMLBool();
		pBool->SetValue(true);
		response->m_params.Add(new CXMLBool(true));
		return response;
	
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: CheckInReposResults: EXCEPTION.");
	}

	CXMLResponse* response = new CXMLResponse();
	response->m_params.Add(new CXMLBool(false));
	return response;
}

CXMLResponse* CFileControl::StoreResultsFile(string strReposHost, string strFileName, string strTest, string strTestRun, string strInjector)
{
	g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: "+strFileName+" Injector = "+strInjector);

	try
	{
		// First get the file from the injector machine to this (Relay) machine
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Transfer file from injector to the local (RELAY) machine.");
		CECInetInfo info;
		string strIP = info.GetIPAddressAsString(strInjector);
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Narrow FileTransferNode on "+strIP);
		CORBA::Object_var obj=m_pCorba->FindObjectWithName(FileTransferNode::CLASS_NAME, strIP.c_str()); 
		FileTransferNode_ptr FileTransfer = FileTransferNode::_narrow(obj);
		if (CORBA::is_nil(FileTransfer))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: FileTransferNode NULL!!!");
		}


		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: OpenResultsFile "+strFileName);
		if (!FileTransfer->OpenResultsFile(strFileName.c_str()))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Failed to open the file.");
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(false));
			return response;
		}

	    // Open a new file in the web/ftp directory

		string destPath = "";
		if (m_settings.m_bUseFTP)
			destPath = m_settings.m_strFtpRoot + strFileName;
		else
			destPath = m_settings.m_strWebRoot + m_settings.m_strCachePath + strFileName;

	    fstream* f = new fstream( destPath.c_str(), std::ios::binary|std::ios::out );

	    // Loop through iterator
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Transferring file...");
		ByteSequence_var byteSeq = FileTransfer->ReadBytes();
	    while ( byteSeq->length() != 0 )
	    {
	        // Write contents to file and get next sequence
	        f->write((const char*) byteSeq->get_buffer(), byteSeq->length() );
	        byteSeq = FileTransfer->ReadBytes();
	    }

	    // Close file and flush contents
	    delete f;

		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Transferred. Closing file...");
		FileTransfer->CloseResultsFile();


		// Next Call the remote relay to transfer and check in the file
		// to the repository

		CORBA::Object_var rmObj = m_pCorba->FindObjectWithName(XMLRelayMap::CLASS_NAME, "WebRelay");
		XMLRelayMap_var pRelayMap = XMLRelayMap::_narrow(rmObj);

		RelayDef def;
		if (!pRelayMap->GetRelayForAlias(strReposHost.c_str(), def))
		{
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(false));
			return response;
		}

		// ... form the URL
		string strURL = "";
		string strAddr = info.GetIPAddressAsString();
	    string::size_type p = 0;
	    while((p = strAddr.find_first_of('_', p)) != strAddr.npos)
	        // Replace continuation with space
	        strAddr.replace(p++, 1, 1, '.');
	
		if (m_settings.m_bUseFTP)
			strURL = "ftp://"+m_settings.m_strFtpUser+":"+m_settings.m_strFtpPass+"@"+strAddr+"/"+strFileName;
		else
			strURL = "http://"+ strAddr+m_settings.m_strCachePath+strFileName;

		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Ask the remote relay machine to check in the file.");
		CHTTPTransport trans(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);
		CXMLRequest req;
		req.SetMethodName("FILECONTROL.CheckInReposResults");
		req.AddParameter(new CXMLString(strReposHost));
		req.AddParameter(new CXMLString(strURL));
		req.AddParameter(new CXMLString(strTest));
		req.AddParameter(new CXMLString(strTestRun));
		req.SetTransport(trans);
		CXMLResponse* resp = req.SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: Check-in ERROR.");
			CXMLResponse* response = new CXMLResponse();
			response->m_params.Add(new CXMLBool(false));
			return response;
		}

		CXMLBool* pRetVal = (CXMLBool*)resp->m_params.GetAt(0);

		CXMLResponse* response = new CXMLResponse();
		response->m_params.Add(new CXMLBool(pRetVal->GetValue()));
		return response;
	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "FC: StoreResultsFile: EXCEPTION");
	}

	CXMLResponse* response = new CXMLResponse();
	response->SetError(CXMLResponse::UNKNOWN_ERROR);
	return response;
}

bool CFileControl::ParseFTPURL(string strURL, string& strServer, int& nPort, string& strUser,
				string& strPass, string& strRemotePath)
{
	// FTP URL in the form ftp://user:pass@server:port/path

	string strTmp = strURL;

	// Remove ftp://
	if (strURL.substr(0,6) != "ftp://")
		return false;	
	strTmp.erase(0,6);


	// search for '@' and parse username/password
	int nPos = strTmp.find_first_of('@');
	if (nPos > 0)
	{
		int nColonPos = strTmp.find_first_of(':');
		if (nColonPos <= 0)
			return false;

		strUser = strTmp.substr(0,nColonPos);
		strPass = strTmp.substr(nColonPos+1, nPos-nColonPos-1);

		strTmp.erase(0, nPos+1);
	}

	// Next find the server address and port
	nPos = strTmp.find_first_of('/');
	if (nPos <= 0)
		return false;

	int nColonPos = strTmp.find_first_of(':');
	if (nColonPos > 0)
	{
		strServer = strTmp.substr(0, nColonPos);
		nPort = atoi(strTmp.substr(nColonPos+1, nPos).c_str());
	}
	else
	{
		strServer = strTmp.substr(0,nPos);
		nPort = 21;
	}
	strTmp.erase(0,nPos);

	// The rest is the remote path.
	strRemotePath = strTmp;

	return true;
}