/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "XMLRelayTestRun_i.h"
#include <CECInetInfo.h>
#include "xmlrpc.h"
#include "..\include\Logger.h"
#include "OstaTraceDef.h"

#include <string>
#include <iostream>
#include <fstream>
using namespace std;

extern CLogger* g_Logger;

XMLRelayTestRunImpl::XMLRelayTestRunImpl(CORBAServer* corba)
{
    m_registered_name="WebRelay";
    m_corba=corba;

	g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTM: Registering XMLRelayTestRun interface");
	m_corba->ActivateObject(this);
    m_corba->NameObject( this->_this(),XMLRelayTestRun::CLASS_NAME,m_registered_name.c_str());

	m_RelayMap = XMLRelayMap::_nil();
}

XMLRelayTestRunImpl::~XMLRelayTestRunImpl()
{
    m_corba->UnNameObject( XMLRelayTestRun::CLASS_NAME,m_registered_name.c_str());
}

void XMLRelayTestRunImpl::AddTestOutput(const char* strReposHost, const char* strTestRunReg, const char* strType, const char* strMessage)
{
	try
	{

		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTR: AddTestOutput: Relaying AddTestOutput request for TestRun "+string(strTestRunReg));
		if (CORBA::is_nil(m_RelayMap))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTR: AddTestOutput: Narrow RelayMap");
			CORBA::Object_ptr obj = m_corba->FindObjectWithName(XMLRelayMap::CLASS_NAME, m_registered_name.c_str());
			m_RelayMap = XMLRelayMap::_narrow(obj);
		}

		RelayDef def;	
		if (!m_RelayMap->GetRelayForAlias(strReposHost, def))
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTR: AddTestOutput: No relay server defined for this host");
			return;
		}
		else
		{
			g_Logger->WriteLogMsg(TRCLEV_HIGH, "RelayTR: AddTestOutput: Relay server is "+string(def.strHost));
		}

	
		CHTTPTransport transport(string(def.strHost),string(def.strExtPath),def.lPort,string(def.strUser),string(def.strPass), def.bUseSSL);
	
		CXMLRequest* req = new CXMLRequest();
		req->SetMethodName("TESTRUN.AddTestOutput");

		req->SetTransport(transport);
		req->AddParameter(new CXMLString(strTestRunReg));
		req->AddParameter(new CXMLString(strType));
		req->AddParameter(new CXMLString(strMessage));
		CXMLResponse* resp = req->SendRequest();
		if (resp->IsError())
		{
			g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTR: AddTestOutput: Relay Error:"+resp->PrintError());
			return;
		}

		delete resp;
		delete req;

		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTR: AddTestOutput: COMPLETED");

	}
	catch(...)
	{
		g_Logger->WriteLogMsg(TRCLEV_LOW, "RelayTR: AddTestOutput: EXCEPTION");
		return;
	}
}

