/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarFactoryHandler.h: interface for the CVarFactoryHandler class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TIMESERVICEHANDLER_H__DBA2FBE4_6962_46EF_933F_BFB76C8C4331__INCLUDED_)
#define AFX_TIMESERVICEHANDLER_H__DBA2FBE4_6962_46EF_933F_BFB76C8C4331__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <OSTAORB/OSTAORB.h>
#include "xmlrpc.h"
#include "OSTAORB.h"
#include "TPCYRSemaphore.h"
#include "TPMutex.h"
#include "CECVar.h"
#include "VarFactory.h"
#include <string>
#include <GlobalVariable.h>
#include <mutex.hpp>

using namespace std;

class CVarFactoryHandler : public CCallHandler  
{
	struct HandlerVarList
	{
		long lVarId;
		GlobalVariable_var  variable;	
	};

public:


	CVarFactoryHandler(CORBAServer* pServer);
	virtual ~CVarFactoryHandler();

	CXMLResponse* ProcessMethodCall(CXMLRequest* request);

	CXMLResponse* ClearAll(string strTargetMachine);
	CXMLResponse* DeregisterScript(string strTargetMachine,string strScript, bool bKeepAlive);
	CXMLResponse* RegisterScript(string strTargetMachine,string strScript);

	CXMLResponse* CreateMutex(string strTargetMachine, string strName);
	CXMLResponse* CreateSimpleSemaphore(string strTargetMachine, string strName);

	CXMLResponse* GetVariable(string strTargetMachine, string strName, long lSize, string strFile, string strFileid,
					  string strDimensions, string strValues, long lOptions, long lSeed, string strScriptname);

	CXMLResponse* TPMutex_unlock(string strTargetMachine, string strMutexName, long lMutexid);
	CXMLResponse* TPMutex_trylock(string strTargetMachine, string strMutexName);
	CXMLResponse* TPMutex_lock(string strTargetMachine, string strMutexName);
	CXMLResponse* TPSemaphore_acquire(string strTargetMachine, string strSemName);
	CXMLResponse* TPSemaphore_release(string strTargetMachine, string strSemName);
	CXMLResponse* TPSemaphore_tryacquire(string strTargetMachine, string strSemName);

	CXMLResponse* GlobalVariable_get_value(string strTargetMachine, string strVarName, string strExpression, long lVarId);
	CXMLResponse* GlobalVariable_set_value(string strTargetMachine, string strVarName, string strNewval, string strExpression, long lVarId);
	CXMLResponse* GlobalVariable_next_value(string strTargetMachine, string strVarName, long lVarId);
	CXMLResponse* GlobalVariable_generate_value(string strTargetMachine, string strVarName, long lVarId);
	CXMLResponse* GlobalVariable_reset_seed(string strTargetMachine, string strVarName, long lVarId);

	CXMLResponse* DoesSimpleSemaphoreExist(string strTargetMachine, string strVarName);
	CXMLResponse* DoesMutexExist(string strTargetMachine, string strVarName);

private:
	CORBAServer* m_pCorba;
	CArray <HandlerVarList,HandlerVarList> m_VariableList;
	CTMutex m_VarListMutex;

};

#endif // !defined(AFX_TIMESERVICEHANDLER_H__DBA2FBE4_6962_46EF_933F_BFB76C8C4331__INCLUDED_)
