/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Logger.h: interface for the CLogger class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOGGER_H__16077784_C4F8_41EB_A9E8_B31C6A2FFB8D__INCLUDED_)
#define AFX_LOGGER_H__16077784_C4F8_41EB_A9E8_B31C6A2FFB8D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include <ipMutex.hpp>
#include <ctime.hpp>

#include <string>
#include <fstream>
using namespace std;

class CLogger  
{
public:
	CLogger(string strFileName)
	{
		m_nTraceLevel = 10; // default trace level;

		char * pOpenStaHome = getenv("OPENSTA_HOME") ;
		string strLocalPath = pOpenStaHome;
		strLocalPath += "Server\\Logs\\" + strFileName;

		m_pLogFile = new ofstream( strLocalPath.c_str(), std::ios::binary|std::ios::trunc|std::ios::out);

	};

	virtual ~CLogger()
	{
		delete m_pLogFile;
	};

	void WriteLogMsg(int nLevel, string strMsg)
	{
	    static CTMutex mutex;
	    CTLockGuard<CTMutex> lock(mutex);

		if(nLevel>m_nTraceLevel)
		{
			//it's too minor
			return;
		}

	    if(m_pLogFile->good())
	    {
			CTTime tm;
			tm.now();
			string strTmp = tm.Format("%d-%m-%Y %H-%M-%S") + ",";
			m_pLogFile->write( strTmp.c_str(), strTmp.length() );
	        m_pLogFile->write( strMsg.c_str(), strMsg.length() );
	        m_pLogFile->write( "\r\n", 2 );
	        m_pLogFile->flush();
	    }
	};

	void SetLevel(int nLevel)
	{
		m_nTraceLevel = nLevel;
	};

private:
	int m_nTraceLevel;
    std::ofstream*      m_pLogFile;
};

#endif // !defined(AFX_LOGGER_H__16077784_C4F8_41EB_A9E8_B31C6A2FFB8D__INCLUDED_)
