/*
Module : SOCMFC.H
Purpose: Interface for an MFC wrapper class for sockets
Created: PJN / 05-08-1998

Copyright (c) 2001 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/

#ifndef __SOCMFC_H__
#define __SOCMFC_H__


////// Includes //////////////////////////////////////////////////////////

#ifndef __AFXPRIV_H__
#pragma message("To avoid this message please put afxpriv.h in your PCH")
#include <afxpriv.h>
#endif




////// Socket exception class ////////////////////////////////////////////

void AfxThrowWSocketException(int nError = 0);

class CWSocketException : public CException
{
public:
//Constructors / Destructors
	CWSocketException(int nError);
	~CWSocketException();

//Methods
#ifdef _DEBUG
	virtual void Dump(CDumpContext& dc) const;
#endif
	virtual BOOL GetErrorMessage(LPTSTR lpstrError, UINT nMaxError,	PUINT pnHelpContext = NULL);
	CString GetErrorMessage();

//Data members
	int m_nError;

protected:
	DECLARE_DYNAMIC(CWSocketException)
};



////// Simple socket wrapper class ////////////////////////////////////////////

class CWSocket
{
public:
//Constructors / Destructors
  CWSocket();
  ~CWSocket();

//General Methods
  void   Create();
  BOOL   IsCreated() const; 
  void   Connect(LPCTSTR pszHostAddress, int nPort, LPCTSTR pszLocalBoundAddress = NULL);
  void   Bind(int nPort, LPCTSTR pszLocalBoundAddress = NULL);
  void   Listen(int nBacklog = SOMAXCONN);
  void   Accept(CWSocket& clientSocket, sockaddr_in& clientAddress);
  void   Close();
  void   Send(void* pBuffer, int nBuf);
  int    Receive(void* pBuffer, int nBuf);
  BOOL   IsReadible(DWORD dwTimeout);
  void   Attach(SOCKET socket);
  SOCKET Detach(); 

//Operators
  operator SOCKET();

protected:
  void   Connect(const SOCKADDR* lpSockAddr, int nSockAddrLen);
  SOCKET m_hSocket;
};

#endif //__SOCHANDLER_H__