/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLArray.cpp: implementation of the CXMLArray class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "xmlrpc.h"

extern CCallManager* g_pCallManager;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXMLArray::CXMLArray()
{

}

CXMLArray::~CXMLArray()
{
	while (m_elements.GetSize() > 0)
	{
		delete m_elements.GetAt(0);
		m_elements.RemoveAt(0);
	}
}

bool CXMLArray::ToXML(string& strXML)
{
	if (GetSize() == 0)
		return false;

	strXML += "<value>";
	strXML += "<array>";
	strXML += "<data>";
	for (int i = 0; i<GetSize(); i++)
	{
		string strVal;
		m_elements.GetAt(i)->ToXML(strVal);
		strXML += strVal;
	}
	strXML += "</data>";
	strXML += "</array>";
	strXML += "</value>";

	return true;
}

bool CXMLArray::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

	// pNode is the <array> node of the xml.
    CXMLNodeElement* pData = NULL;
	pData = pNode->GetChildElementByName("data");

	// Each child is an element of the array. Parse the type and value and construct
	// the element.
    CXMLNodeElement* pChild = pData->GetFirstChild();
	while(pChild)
	{
		CXMLNodeElement* pParamType = pChild->GetFirstChild();

		string strType = pParamType->GetNodeName();
		if (strType == "string")
		{
			CXMLString* strXML = new CXMLString();
			strXML->FromXML(pParamType);
			m_elements.Add(strXML);
		}
		else if (strType == "int")
		{
			CXMLInt* iXML = new CXMLInt();
			iXML->FromXML(pParamType);
			m_elements.Add(iXML);
		}
		else if (strType == "i4")
		{
			CXMLLong* lXML = new CXMLLong();
			lXML->FromXML(pParamType);
			m_elements.Add(lXML);
		}
		else if (strType == "boolean")
		{
			CXMLBool* bXML = new CXMLBool();
			bXML->FromXML(pParamType);
			m_elements.Add(bXML);
		}
		else if (strType == "double")
		{
			CXMLDouble* dXML = new CXMLDouble();
			dXML->FromXML(pParamType);
			m_elements.Add(dXML);
		}
		else if (strType == "array")
		{
			CXMLArray* arrXML = new CXMLArray();
			arrXML->FromXML(pParamType);
			m_elements.Add(arrXML);
		}
		else if (strType == "struct")
		{
			CXMLStruct* colXML = new CXMLStruct();
			colXML->FromXML(pParamType);
			m_elements.Add(colXML);
		}
		else if (strType == "base64")
		{
			CXMLBase64* b64XML = new CXMLBase64();
			b64XML->FromXML(pParamType);
			m_elements.Add(b64XML);
		}
		else if (strType == "dateTime.iso8601")
		{
			CXMLDate* dateXML = new CXMLDate();
			dateXML->FromXML(pParamType);
			m_elements.Add(dateXML);
		}
		else
		{
			CCustomDataProvider* pProvider = m_pCallManager->GetProviderForType(strType);
			if (pProvider != NULL)
			{
				CXMLParameter* pParam = pProvider->CreateInstance();
				pParam->SetCallManager(m_pCallManager);
				pParam->FromXML(pParamType);
				m_elements.Add(pParam);
			}
			else
			{
				cout<<"Unable to find handler for custom type: "<<strType<<endl;
			}
		}
		delete pParamType;
		pChild = pChild->GetNextSibling();
	}

	return true;
}

void CXMLArray::AddElement(CXMLParameter* element)
{
	m_elements.Add(element);
}

int CXMLArray::GetSize()
{
	return m_elements.GetSize();
}

CXMLParameter* CXMLArray::GetAt(int nPos)
{
	return m_elements.GetAt(nPos);
}
