/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLStruct.cpp: implementation of the CXMLStruct class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "xmlrpc.h"

extern CCallManager* g_pCallManager;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CXMLStruct::CXMLStruct()
{

}

CXMLStruct::~CXMLStruct()
{
	while (m_elements.GetSize() > 0)
	{
		delete m_elements.GetAt(0).pValue;
		m_elements.RemoveAt(0);
	}

}

bool CXMLStruct::ToXML(string& strXML)
{
	if (GetSize() == 0)
		return false;

	strXML += "<value>";
	strXML += "<struct>";
	for (int i = 0; i<GetSize(); i++)
	{
		strXML += "<member>";
		strXML += "<name>"+m_elements.GetAt(i).strName+"</name>";
		string strVal;
		m_elements.GetAt(i).pValue->ToXML(strVal);
		strXML += strVal;
		strXML += "</member>";
	}
	strXML += "</struct>";
	strXML += "</value>";
	return true;
}

bool CXMLStruct::FromXML(CXMLNodeElement* pNode)
{
/* Kyle Harrison 11/8/01 : Modified to use new CXMLParser Wrapper Class */ 

    CXMLNodeElement* pChild = pNode->GetFirstChild();
	while(pChild)
	{
		CXMLNodeElement* pMember = pChild->GetFirstChild();

		CXMLNodeElement* pName = NULL;
		pName = pMember->GetChildElementByName("name");
		string strName = pName->GetText();

		CXMLNodeElement* pValue = NULL;
		pValue = pMember->GetChildElementByName("value");

		CXMLNodeElement* pType = pValue->GetFirstChild();
		string strType = pType->GetNodeName();

		if (strType == "string")
		{
			CXMLString* strXML = new CXMLString();
			strXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = strXML;
			m_elements.Add(el);
		}
		else if (strType == "int")
		{
			CXMLInt* iXML = new CXMLInt;
			iXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = iXML;
			m_elements.Add(el);
		}
		else if (strType == "i4")
		{
			CXMLLong* lXML = new CXMLLong();
			lXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = lXML;
			m_elements.Add(el);
		}
		else if (strType == "boolean")
		{
			CXMLBool* bXML = new CXMLBool();
			bXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = bXML;
			m_elements.Add(el);
		}
		else if (strType == "double")
		{
			CXMLDouble* dXML = new CXMLDouble();
			dXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = dXML;
			m_elements.Add(el);
		}
		else if (strType == "array")
		{
			CXMLArray* arrXML = new CXMLArray();
			arrXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = arrXML;
			m_elements.Add(el);
		}
		else if (strType == "struct")
		{
			CXMLStruct* colXML = new CXMLStruct();
			colXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = colXML;
			m_elements.Add(el);
		}
		else if (strType == "base64")
		{
			CXMLBase64* b64XML = new CXMLBase64();
			b64XML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = b64XML;
			m_elements.Add(el);
		}
		else if (strType == "dateTime.iso8601")
		{
			CXMLDate* dateXML = new CXMLDate();
			dateXML->FromXML(pType);

			StructElement el;
			el.strName = strName;
			el.pValue = dateXML;
			m_elements.Add(el);
		}
		else
		{
			CCustomDataProvider* pProvider = g_pCallManager->GetProviderForType(strType);
			if (pProvider)
			{
				CXMLParameter* pParam = pProvider->CreateInstance();
				pParam->FromXML(pType);
				StructElement el;
				el.strName = strName;
				el.pValue = pParam;
				m_elements.Add(el);
			}
			else
			{
				cout<<"Unable to find handler for custom type: "<<strType<<endl;
			}
		}

		pChild = pChild->GetNextSibling();
	}

	return true;
}

void CXMLStruct::AddElement(StructElement element)
{
	m_elements.Add(element);
}

int CXMLStruct::GetSize()
{
	return m_elements.GetSize();
}

CXMLStruct::StructElement CXMLStruct::GetAt(int nPos)
{
	return m_elements.GetAt(nPos);
}

CXMLParameter* CXMLStruct::GetValueForName(string strName)
{
	for (int i = 0; i<GetSize(); i++)
	{
		if(m_elements.GetAt(i).strName == strName)
			return m_elements.GetAt(i).pValue;
	}

	return NULL;
}
