/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*            **** C O M M O N ****                  */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_METHODS                 */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This module contains macros that determine which methods         */
/*  are used each of the platforms.                              */
/*                                       */
/*                                       */
/*===========================================================================*/

/*
 *    This file is used to determine which methods are used on each platform.
 *    It is intended to make switching between methods (for a platform) as 
 *    simple as possible, by simply changing the macros in this file and then
 *    performing a full build.  This requirement has become apparent
 *    during the introduction of OSF, since to get things moving we need to use
 *    the DPID method (because it's flexible), but are likely to want to use 
 *    PSLVPM when we get the time to implement it.
 */
#ifndef PSL_METHODS
#define PSL_METHODS 1

/*
 *  SUN methods
 *  ===========
 */
#ifdef SUN
/*
 *  Use pslvpm STREAMS module with lgr
 */
#   define USE_METHOD_PSLVPM 1

/*
 *  SUN has support for driving physical terminals
 */
#   define PHYSICAL_TERMINAL_SUPPORT 1

/*
 *  NCR methods
 *  ===========
 */
#elif defined(NCR)
/*
*  Use pslvpm STREAMS module with lgr
*/
#   define USE_METHOD_PSLVPM 1

/*
*  CTM technology requires dpid
*/
#   define CTM_REQUIRES_DPID 1

/*
*  OSF methods
*  ===========
*/
#elif defined(OSF)
/*
*  Use pslvpm STREAMS module with lgr
*/
#   define USE_METHOD_PSLVPM 1

/*
*  If OSF is using PSLVPM then the following macros must be defined
*/
#   ifdef USE_METHOD_PSLVPM
/*
*  The following comment was true but DEC have given us an unofficial
*      patch which fixes the problem and unfortunately introduces another
*      problem. The macro is nolonger required but is left in for reference
*      as the C source still contains code for it.
*      (See the comments in LGR_GETMSG_PSEUDO())
*
*  In OSF we do not appear to obtain M_PROTO messages from the streamhead
*  with getmsg(). As a result we use a different means of obtaining
*  M_READs and M_IOCTLs.
*/
/*# define NO_GETMSG_MPROTOS 1*/
/*
*  The following comment was true but DEC have given us a patch which
*  partly fixes the problem and so the macro is nolonger required.
*  In OSF the pslvpm_open() function in the pslvpm streams module does
*  not get passed a dev_t and so pslvpm does not know on which device
*  it is pushed. As a result, anything using pslvpm must pass the dev_t
*  to it via a streams message.
*/
/*# define PSLVPM_REQUIRES_DEVT 1*/
#   endif /* OSF and USE_METHOD_PSLVPM */
/*
*  CTM technology requires dpid
*/
#   define CTM_REQUIRES_DPID 1

/*
*  HP methods
*  ==========
*/
#elif defined(HP)
#   if defined(HPV9) || defined(HPV10)
/*
*      HP (9.x and 10.0)  uses IOM with lgr
*/
#       define USE_METHOD_IOM 1

/*
*      HP-UX 10.20 uses STREAMS.
*/
#   else /* !HP9 and !HP10.0 */
/*
*      Use pslvpm STREAMS module with lgr
*/
#       define USE_METHOD_PSLVPM 1

/*
*      CTM technology requires dpid
*/
#       define CTM_REQUIRES_DPID 1
/*
*      HP-UX 10.20 requires both ctm_servers to be running
*/
#       define TWIN_CTM_SERVERS  1
#    endif /* HP9 and HP10.0 */

/*
*  IBM methods
*  ===========
*/
#elif defined(IBM)
/*
*  AIX 4.1.4 uses the pslvpm STREAMS module with lgr
*/
#   ifdef AIXV41
/*
*      Use pslvpm STREAMS module with lgr
*/
#       define USE_METHOD_PSLVPM 1
/*
*  We define MUST_RECORD_BEFORE_PROCESS_CREATION in AIX 4.1.4 because
*      any terminal attribute sets/gets occuring at the same time in the shell
*      and dpid (record from startup) cause the shell and dpid to block
*      forever. So, we start recordings after creating the terminal and doing
*      the attach delay code but before starting the shell/telnet.
*/
#       define MUST_RECORD_BEFORE_PROCESS_CREATION 1

/*
*      CTM technology requires dpid
*/
#       define CTM_REQUIRES_DPID 1

#   else
/*
*  AIX 3.2.5 uses dpid with lgr
*/
#       define USE_METHOD_DPID 1
#   endif

/*
*  VMS methods
*  ===========
*/
#elif defined(VMS)
#elif defined(WIN32)
#else
#error Platform unsupported at this line
#endif

/*
 *  Methods based on the choices made above
 */
#ifdef USE_METHOD_PSLVPM
#   if defined(SVR4) || defined(OSF) || defined(AIXV41)
#       define MIR_THROTTLE_REQUIRED 1
#   endif
#endif
#endif /* PSL_METHODS */
