#ifndef PSL__PSL_TRCDEF_LOADED
#define PSL__PSL_TRCDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: PSL_TRCDEF                  */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  PSL Trace Structure.                             */
/*                                       */
/*                                       */
/*===========================================================================*/

/*** MODULE PSL_TRCDEF ***/

/*  This structure contains all the information that the PSL Tracing */
/*  Routines require in order to control Tracing. */

struct psl_trc
{

    /*  Pointer to Trace File. This is NULL if Tracing is not taking */
    /*  place to a file. */

    FILE                    *psl_trc_file_ptrpf;

    /*  General Trace Flags. */

    PSL_U_LONGWORD_T        psl_trc_genflgl;
#define            PTGN_COMMAND 0X1 /* Command Line */
#define            PTGN_TIMING 0X2  /* Timing Information */
#define            PTGN_ALIGN 0X4   /* Alignment Faults */
#define            PTGN_ERRORS 0X8  /* Errors */
#define            PTGN_TPARSE 0X10 /* Tparse */
#define            PTGN_MISC 0X20   /* Miscellaneous */
#define            PTGN_MEM 0X40    /* Memory allocation */
#define            PTGN_PIPES 0X80  /* Pipe Operations */

    /*  Form Driver Trace Flags. */

    PSL_U_LONGWORD_T        psl_trc_fdrflgl;
#define            PTFD_ENABLED 0X1 /* Facility Enabled */
#define            PTFD_MONITOR 0X2 /* PROCDMP/MONITOR */
#define            PTFD_REPEAT 0X4  /* Repeat Groups */

    /*  Screen Image Trace Flags. */

    PSL_U_LONGWORD_T        psl_trc_scrflgl;
#define            PTSC_ENABLED 0X1 /* Facility Enabled */
#define            PTSC_IMGRTNS 0X2 /* Image Routines */
#define            PTSC_FILRTNS 0X4 /* File I/O Routines */
#define            PTSC_TRMRTNS 0X8 /* Terminal I/O Routines */
#define            PTSC_UPDRTNS 0X10    /* Upadet Routines */
#define            PTSC_CVTRTNS 0X20    /* Convert Routines */
#define            PTSC_INTRTNS 0X40    /* Internal Routines */
#define            PTSC_FILSTRC 0X80    /* File Structures */
#define            PTSC_HDRSTRC 0X100   /* Header Structures */
#define            PTSC_IFISTRC 0X200   /* Internal File Structures */
#define            PTSC_IMGSTRC 0X400   /* Image Structures */
#define            PTSC_OLDSTRC 0X800   /* Old Image Structures */
#define            PTSC_RECSTRC 0X1000  /* Record Structures */
#define            PTSC_WRKSTRC 0X2000  /* Work Structures */

    /*  User Trace Flags. */
    /*  Note that 32 user Trace Flags could be defined, but there is */
    /*  no need for that many at the moment. */

    PSL_U_LONGWORD_T        psl_trc_usrflgl;
#define            PTUS_ENABLED 0X1 /* Facility Enabled */
#define            PTUS_USER1 0X2   /* User 1 */
#define            PTUS_USER2 0X4   /* User 2 */
#define            PTUS_USER3 0X8   /* User 3 */
#define            PTUS_USER4 0X10  /* User 4 */
#define            PTUS_USER5 0X20  /* User 5 */
#define            PTUS_USER6 0X40  /* User 6 */
#define            PTUS_USER7 0X80  /* User 7 */
#define            PTUS_USER8 0X100 /* User 8 */



    /*  TRM Routine Trace Flags. */

    PSL_U_LONGWORD_T        psl_trc_trmflgl;
#define            PTTR_ENABLED 0X1 /* Facility Enabled */
#define            PTTR_GENRTNS 0X2 /* General Routines */
#define            PTTR_CLEAR 0X4   /* TRM_CLEAR */
#define            PTTR_OUTPUT 0X8  /* TRM_OUTPUT */
#define            PTTR_INPUT 0X10  /* TRM_INPUT */
#define            PTTR_DRWRTNS 0X20    /* TRM_DRWxxx */
#define            PTTR_HLPRTNS 0X40    /* Help/Error Routines */
#define            PTTR_SCROLL 0X80 /* TRM_SCROLL */
#define            PTTR_SPCRTNS 0X100   /* Special Routines */
#define            PTTR_ODDRTNS 0X200   /* Odd Routines */



    /*  IPC Trace Flags. */

    PSL_U_LONGWORD_T        psl_trc_ipcflgl;
#define            PTIP_IPC 0X1 /* IPC routines */
#define            PTIP_DTC 0X2 /* DTC Information */
#define            PTIP_IPC2 0X4    /* IPC messages */
#define            PTIP_DTC2 0X8    /* DTC Messages */

    /*    Spare alignment bytes to quadword boundary. */

    PSL_U_LONGWORD_T        psl_trc_spareflagl;



} ;


/*  End of C Include File */

#endif  /* PSL__PSL_TRCDEF_LOADED */
