/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__E247E9E3_42A1_4647_9C54_EA2EDE11397A__INCLUDED_)
#define AFX_MAINFRM_H__E247E9E3_42A1_4647_9C54_EA2EDE11397A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "WorkspaceBar.h"
class CArchitectureManager;

class CMainFrame : public CFrameWnd
{
	
protected: // create from serialization only
	CMainFrame();
	DECLARE_DYNCREATE(CMainFrame)

// Attributes
public:
	BOOL OpenFile(const CString& strPath);
	void ShowCloseBtn(BOOL bShow = TRUE);
    virtual void GetMessageString(UINT nID, CString& rMessage) const;

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMainFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMainFrame();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	typedef enum{enNewTestMenuPos,enNewProfileMenuPos,enNewScriptMenuPos} FileMenuPositions;

public:  // control bar embedded members
	CStatusBar  m_wndStatusBar;
	CToolBar    m_wndToolBar;
	CWorkspaceBar m_wndWorkspace;
	CMenu m_MenuIP;

	int m_nMenuItemsAdded;

    HACCEL GetAccelTable() { return m_hAccelTable; } 

// Generated message map functions
protected:
	//{{AFX_MSG(CMainFrame)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnTestsMonitor();
	afx_msg void OnToolsRepositorypath();
	afx_msg void OnToolsTracesettings();
	afx_msg void OnHelpContents();
	afx_msg void OnHelpIndex();
	afx_msg void OnHelpSearch();
	afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
	afx_msg void OnViewRepository();
	afx_msg void OnUpdateViewRepository(CCmdUI* pCmdUI);
	afx_msg void OnNcPaint();
	afx_msg void OnNcLButtonDown(UINT nHitTest, CPoint point);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnCaptureChanged(CWnd *pWnd);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnPaint();
	afx_msg void OnArchitectureManager();
	afx_msg void OnUpdateArchitectureManager(CCmdUI* pCmdUI);
	afx_msg void OnSettingChange(UINT uFlags, LPCTSTR lpszSection);
	afx_msg BOOL OnNcActivate(BOOL bActive);
	//}}AFX_MSG
	afx_msg void OnCustomMenuCommand(UINT nID);
	afx_msg void OnRefresh();
    afx_msg void OnExternalTool(UINT nID);
    afx_msg void OnUpdateExternalTool(CCmdUI* pCmdUI);
	DECLARE_MESSAGE_MAP()
public:
	typedef enum{enScript,enProfile,enTest} PluginType;

	typedef struct {
		CString strName;
		CString strExtension;
		CLSID clsid;
		PluginType nType;
		UINT uiNewCommandId;
	}PluginInfo;
	CList <PluginInfo,PluginInfo&> m_listPlugins;
protected:
	CDocument *m_pCurrentDoc;

	int LoadPluginType(PluginType nType);

	void LoadTestPlugins();
	void LoadScriptPlugins();
	void LoadProfilePlugins();

	BOOL InvokeModeller(const CString& strScript);
	
	void CreateRepository(const CString& strRespositoryPath);
private:

    // External tools:
    struct ExternalTool
    {
        CString m_strLabel;         // The label to put on the menu item
        CString m_strInfoMessage;   // The information message to display in the status bar for this tool
        CString m_strFile;          // The executable or document file
        CString m_strArguments;     // The optional arguments
        CString m_strInitialDir;    // The optional initial directory
        CString m_strBitmapFile;    // The path to the 16*15 bitmap file
        UINT    m_uiCommanID;       // The associated command ID
    };
    CArray<ExternalTool, ExternalTool&> m_externalTools;
    UINT m_uiNextExternalToolCmdID;
    void AddToolbarButtonForTool(CToolBar& tb, LPCTSTR zPath);

	CString m_strHelpFilePath;
	CString m_strOpenItem;
    CArchitectureManager* m_pArchitectureManager;

	BOOL m_bCapturedMouse;
	BOOL m_bCloseBtnDown;
	CRect m_rcCloseBtn;
	void DrawCloseBtn();
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__E247E9E3_42A1_4647_9C54_EA2EDE11397A__INCLUDED_)
