// HttpSingleStepFile.cpp: implementation of the CHttpSingleStepFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HttpSingleStepFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHttpSingleStepFile::CHttpSingleStepFile()
{
	m_hFileMutex = INVALID_HANDLE_VALUE;
	m_posRead = NULL;
	m_bReading = true;
	m_dwFilePosition = 0;
}

CHttpSingleStepFile::~CHttpSingleStepFile()
{
	CloseHandle(m_hFileMutex);
	
	POSITION pos = m_listSteps.GetHeadPosition();
	while(pos)
		delete (LPVOID)m_listSteps.GetNext(pos);
	m_listSteps.RemoveAll();
}

BOOL CHttpSingleStepFile::Open(LPCTSTR szRunFolder,BOOL bReading)
{
	m_File.Close();
	if(m_hFileMutex != INVALID_HANDLE_VALUE)
		CloseHandle(m_hFileMutex);

	m_posRead = NULL;
	m_dwFilePosition = 0;
	POSITION pos = m_listSteps.GetHeadPosition();
	while(pos)
		delete (LPVOID)m_listSteps.GetNext(pos);
	m_listSteps.RemoveAll();
	
	//Generate a mutext name for this run file
	CString strMtxName(szRunFolder);
	strMtxName.TrimRight("\\");
	int nPos = strMtxName.ReverseFind('\\');
	strMtxName = strMtxName.Mid(nPos+1);
	strMtxName.MakeUpper();
	strMtxName += "_SBS_RESULTS_FILE";

	m_hFileMutex = CreateMutex(NULL,FALSE,strMtxName);
	m_bReading = bReading;
	
	return m_File.Open(CString(szRunFolder) + "\\HTTP_SBS.log",
		(bReading?(CFile::modeRead|CFile::shareDenyNone):(CFile::modeWrite|CFile::modeCreate|CFile::modeNoTruncate|CFile::shareDenyWrite)));
}

BOOL CHttpSingleStepFile::Write(CBaseSingleStepData* pData)
{
	if(m_bReading)
		return FALSE;

	if(WaitForSingleObject(m_hFileMutex,INFINITE) != WAIT_OBJECT_0)
		return false;

	int nType = pData->GetType();
	m_File.Write(&nType,sizeof(int));

	DWORD dwLen=0;
	LPBYTE pBuffer = pData->SaveToBuffer(dwLen);
	m_File.Write(pBuffer,dwLen);
	delete []pBuffer;
		
	ReleaseMutex(m_hFileMutex);

	return true;
}

CBaseSingleStepData* CHttpSingleStepFile::Read()
{
	if(!m_bReading)
		return NULL;

	if(!m_posRead && !RefreshData())
		return NULL;

	if(m_posRead)
		return m_listSteps.GetNext(m_posRead);
	else
		return NULL;
}

BOOL CHttpSingleStepFile::RefreshData()
{
	DWORD dwResult = WaitForSingleObject(m_hFileMutex,20000);

	if((dwResult != WAIT_ABANDONED) && (dwResult != WAIT_OBJECT_0))
		return false;

	DWORD dwLen = m_File.GetLength();
	DWORD dwToRead = dwLen -m_dwFilePosition;
	if(dwToRead <=0)
	{
		ReleaseMutex(m_hFileMutex);
		return FALSE;
	}

	POSITION posTail = m_listSteps.GetTailPosition();

	HANDLE hMap = CreateFileMapping((HANDLE)(m_File.m_hFile),NULL,PAGE_READONLY,0,0,"SBSResultsData");
	if(hMap == NULL)
	{
		ReleaseMutex(m_hFileMutex);
		return false;
	}

	LPBYTE pBuffer = (LPBYTE)MapViewOfFile(hMap,FILE_MAP_READ,0,0,0);
	if(!pBuffer)
	{
		CloseHandle(hMap);
		ReleaseMutex(m_hFileMutex);
		return false;
	}

	LPBYTE pBufPos = pBuffer + m_dwFilePosition;
	
	while(m_dwFilePosition < dwLen)
	{
		int nType;
		memcpy(&nType,pBufPos,sizeof(int));		pBufPos += sizeof(int);

		CBaseSingleStepData *pData = NULL;
		switch(nType)
		{
		case CBaseSingleStepData::enNextRequest:
			pData = new CNextRequestData;
		break;
		case CBaseSingleStepData::enRequestComplete:
			pData = new CRequestCompleteData;
		break;
		}
		if(pData)
		{
			DWORD dwLen = pData->LoadFromBuffer(pBufPos);
			pBufPos += dwLen;
			m_dwFilePosition += dwLen+sizeof(int);
			m_listSteps.AddTail(pData);
		}
	}

	//If we have reached end of file, set file position to first new entry
	if(!m_posRead)
	{
		if(!posTail)
			m_posRead = m_listSteps.GetHeadPosition();
		else
		{
			m_posRead = posTail;
			m_listSteps.GetNext(m_posRead);
		}
	}
	
    UnmapViewOfFile(pBuffer);
    CloseHandle(hMap);
    
	ReleaseMutex(m_hFileMutex);

	return true;
}

void CHttpSingleStepFile::SeekToBegin()
{
	m_posRead = m_listSteps.GetHeadPosition();
}

CNextRequestData* CHttpSingleStepFile::FindNextRequestFromId(int nRequestId)
{
	if(!IsOpen())
		return NULL;

	RefreshData();
	POSITION pos = m_listSteps.GetHeadPosition();
	while(pos)
	{
		CBaseSingleStepData *pData = m_listSteps.GetNext(pos);
		if(!pData || (pData->GetType() != CBaseSingleStepData::enNextRequest))
			continue;

		if(((CNextRequestData*)pData)->GetRequestId() == nRequestId)
			return (CNextRequestData*)pData;
	}
	return NULL;
}
