// HttpSingleStepFile.h: interface for the CHttpSingleStepFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HTTPSINGLESTEPFILE_H__300C78BB_F75A_4E0F_B090_C3C703FFAE02__INCLUDED_)
#define AFX_HTTPSINGLESTEPFILE_H__300C78BB_F75A_4E0F_B090_C3C703FFAE02__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CBaseSingleStepData;
class CNextRequestData;
class CHttpSingleStepFile  
{
public:
	CHttpSingleStepFile();
	virtual ~CHttpSingleStepFile();

	BOOL Open(LPCTSTR,BOOL bReading=TRUE);
	BOOL IsOpen(){return (m_File.m_hFile!=CFile::hFileNull);}
	
	BOOL Write(CBaseSingleStepData*);
	CBaseSingleStepData* Read();
	void SeekToBegin();
		
	CNextRequestData* FindNextRequestFromId(int nRequestId);
		
private:
	CList <CBaseSingleStepData*,CBaseSingleStepData*> m_listSteps;
	CFile m_File;
	HANDLE m_hFileMutex;
	POSITION m_posRead;

	BOOL m_bReading;

	DWORD m_dwFilePosition;
	BOOL RefreshData();
};

class CBaseSingleStepData
{
public:
	typedef enum {enNextRequest,enRequestComplete} DataType;
	
	virtual DataType GetType()const =0;
	
	virtual DWORD LoadFromBuffer(LPBYTE pData)=0;
	virtual LPBYTE SaveToBuffer(DWORD& dwBufLen)const =0;
	
protected:
	CBaseSingleStepData()
	{
	}
	~CBaseSingleStepData()
	{
	}
};

class CNextRequestData : public CBaseSingleStepData
{
public:
	CNextRequestData()
	{
		m_szRequest = NULL;
		m_szScript = NULL;
		m_nRequestId = -1;
	}
	~CNextRequestData()
	{
		delete []m_szRequest;
		delete []m_szScript;
	}

	DataType GetType() const {return enNextRequest;}
	int GetTaskGroup() const {return m_nTaskGroup;}
	int GetVUId() const {return m_nVUId;}
	int GetConId() const {return m_nConId;}
	LPCTSTR GetRequest() const {return m_szRequest;}
	LPCTSTR GetScript() const {return m_szScript;}
	int GetScriptLine() const {return m_nScriptLine;}
	int GetRequestId() const {return m_nRequestId;}
	LPCTSTR GetUrl() const 
	{
		CString strRequest(m_szRequest);
		return strRequest.SpanExcluding("\r\n");
	}
		
	void SetRequestId(int nID){m_nRequestId = nID;}
	void SetTaskGroup(int nGroup){m_nTaskGroup = nGroup;}
	void SetVUId(int nID){m_nVUId = nID;}
	void SetConId(int nID){m_nConId = nID;}
	void SetRequest(LPCTSTR szRequest)
	{
		if(m_szRequest)
			delete []m_szRequest;
		if(!szRequest)
			return;
		int nLen = strlen(szRequest)+1;
		m_szRequest = new char[nLen];
		memcpy(m_szRequest,szRequest,nLen);
	}

	void SetScript(LPCTSTR szScript)
	{
		if(m_szScript)
			delete []m_szScript;
		if(!szScript)
			return;
		int nLen = strlen(szScript)+1;
		m_szScript = new char[nLen];
		memcpy(m_szScript,szScript,nLen);
	}
	void SetScriptLine(int nLine){m_nScriptLine = nLine;}

	LPBYTE SaveToBuffer(DWORD& dwBufLen) const 
	{
		int nReqLen = strlen(m_szRequest)+1;
		int nScrLen = strlen(m_szScript)+1;
		dwBufLen = sizeof(int)*5 + nReqLen + nScrLen;
		LPBYTE pBuf = new BYTE[dwBufLen];
		LPBYTE pBufPos = pBuf;
	
		memcpy(pBufPos,&m_nTaskGroup,sizeof(int));		pBufPos += sizeof(int);
		memcpy(pBufPos,&m_nVUId,sizeof(int));			pBufPos += sizeof(int);
		memcpy(pBufPos,&m_nConId,sizeof(int));			pBufPos += sizeof(int);
		memcpy(pBufPos,m_szRequest,nReqLen);			pBufPos += nReqLen;
		memcpy(pBufPos,m_szScript,nScrLen);				pBufPos += nScrLen;
		memcpy(pBufPos,&m_nScriptLine,sizeof(int));		pBufPos += sizeof(int);
		memcpy(pBufPos,&m_nRequestId,sizeof(int));		//pBufPos += sizeof(int);
		return pBuf;
	}

	DWORD LoadFromBuffer(LPBYTE pData)
	{
		delete []m_szScript;
		delete []m_szRequest;
		LPBYTE pBufPos = pData;
		memcpy(&m_nTaskGroup,pBufPos,sizeof(int));		pBufPos += sizeof(int);
		memcpy(&m_nVUId,pBufPos,sizeof(int));			pBufPos += sizeof(int);
		memcpy(&m_nConId,pBufPos,sizeof(int));			pBufPos += sizeof(int);
		int nReqLen = strlen((char*)pBufPos)+1;
		m_szRequest = new char[nReqLen];
		memcpy(m_szRequest,pBufPos,nReqLen);			pBufPos += nReqLen;
		int nScrLen = strlen((char*)pBufPos)+1;
		m_szScript = new char[nScrLen];
		memcpy(m_szScript,pBufPos,nScrLen);				pBufPos += nScrLen;
		memcpy(&m_nScriptLine,pBufPos,sizeof(int));		pBufPos += sizeof(int);
		memcpy(&m_nRequestId,pBufPos,sizeof(int));		pBufPos += sizeof(int);
		return pBufPos-pData;
	}

	CNextRequestData& operator=(const CNextRequestData& NRD)
	{
		m_nTaskGroup = NRD.m_nTaskGroup;
		m_nVUId = NRD.m_nVUId;
		m_nConId = NRD.m_nConId;
		int nLen = strlen(NRD.m_szRequest);
		if(nLen)
		{
			m_szRequest = new char[nLen+1];
			memcpy(m_szRequest,NRD.m_szRequest,nLen+1);
		}
		else
			m_szRequest = NULL;

		nLen = strlen(NRD.m_szScript);
		if(nLen)
		{
			m_szScript = new char[nLen+1];
			memcpy(m_szScript,NRD.m_szScript,nLen+1);
		}
		else
			m_szScript = NULL;

		m_nScriptLine = NRD.m_nScriptLine;
		m_nRequestId = NRD.m_nRequestId;

		return *this;
	}
private:
	int m_nTaskGroup;
	int m_nVUId;
	int m_nConId;
	char* m_szRequest;
	char* m_szScript;
	int m_nScriptLine;
	int m_nRequestId;
};

class CRequestCompleteData : public CBaseSingleStepData
{
public:
	CRequestCompleteData()
	{
		m_nTaskGroup =-1;
		m_nVUId = -1;
		m_nConId = -1;
		m_nBodyLength=0;
		m_bIsBinaryData = false;
		m_szHeaders=NULL;
		m_szBody = NULL;
	}
	~CRequestCompleteData()
	{
		if(m_szHeaders)
			delete []m_szHeaders;

		if(m_szBody)
			delete []m_szBody;
	}

	DataType GetType() const {return enRequestComplete;}

	int GetTaskGroup() const {return m_nTaskGroup;}
	int GetVUId() const {return m_nVUId;}
	int GetConId() const {return m_nConId;}
	BOOL IsBinaryData() const {return m_bIsBinaryData;}
	LPCTSTR GetHeaders() const {return m_szHeaders;}
	const char* GetBody() const {return m_szBody;}
	int GetBodyLength() const {return m_nBodyLength;}

	void SetTaskGroup(int nGroup){m_nTaskGroup = nGroup;}
	void SetVUId(int nID){m_nVUId = nID;}
	void SetConId(int nID){m_nConId = nID;}
	void SetHeaders(LPCTSTR szHeaders)
	{
		if(m_szHeaders)
			delete []m_szHeaders;
		if(!szHeaders)
			return;
		int nLen = strlen(szHeaders)+1;
		m_szHeaders = new char[nLen];
		memcpy(m_szHeaders,szHeaders,nLen);
	}
	void SetBody(LPCTSTR szBody,int nLength,BOOL bBinary)
	{
		if(m_szBody)
		{
			delete []m_szBody;
			m_szBody = NULL;
			m_nBodyLength = 0;
		}
		m_bIsBinaryData = bBinary;
		m_nBodyLength = nLength;
		if(szBody && m_nBodyLength && !bBinary)
		{
			m_szBody = new char[nLength];
			memcpy(m_szBody,szBody,nLength);
		}
	}

	LPBYTE SaveToBuffer(DWORD& dwBufLen) const
	{
		int nHeadersLen = strlen(m_szHeaders)+1;

		dwBufLen = sizeof(int)*4 + nHeadersLen + sizeof(BOOL);
		if(!m_bIsBinaryData)
			dwBufLen += m_nBodyLength;
		LPBYTE pBuf = new BYTE[dwBufLen];
		LPBYTE pBufPos = pBuf;
		
		memcpy(pBufPos,&m_nTaskGroup,sizeof(int));		pBufPos += sizeof(int);
		memcpy(pBufPos,&m_nVUId,sizeof(int));			pBufPos += sizeof(int);
		memcpy(pBufPos,&m_nConId,sizeof(int));			pBufPos += sizeof(int);
		memcpy(pBufPos,&m_bIsBinaryData,sizeof(BOOL));	pBufPos += sizeof(BOOL);
		memcpy(pBufPos,m_szHeaders,nHeadersLen);		pBufPos += nHeadersLen;
		memcpy(pBufPos,&m_nBodyLength,sizeof(int));		pBufPos += sizeof(int);
		if(!m_bIsBinaryData)
			memcpy(pBufPos,m_szBody,m_nBodyLength);			//pBufPos += m_nBodyLength;
		
		return pBuf;
	}

	DWORD LoadFromBuffer(LPBYTE pData)
	{
		delete [] m_szHeaders;
		m_szHeaders = NULL;
		delete [] m_szBody;
		m_szBody = NULL;

		LPBYTE pBufPos = pData;
		memcpy(&m_nTaskGroup,pBufPos,sizeof(int));		pBufPos += sizeof(int);
		memcpy(&m_nVUId,pBufPos,sizeof(int));			pBufPos += sizeof(int);
		memcpy(&m_nConId,pBufPos,sizeof(int));			pBufPos += sizeof(int);
		memcpy(&m_bIsBinaryData,pBufPos,sizeof(BOOL));	pBufPos += sizeof(BOOL);
		int nHeadersLen = strlen((char*)pBufPos)+1;
		m_szHeaders = new char[nHeadersLen];
		memcpy(m_szHeaders,pBufPos,nHeadersLen);		pBufPos += nHeadersLen;
		memcpy(&m_nBodyLength,pBufPos,sizeof(int));		pBufPos += sizeof(int);
		
		if(!m_bIsBinaryData)
		{
			m_szBody = new char[m_nBodyLength];
			memcpy(m_szBody,pBufPos,m_nBodyLength);			pBufPos += m_nBodyLength;
		}

		return pBufPos-pData;
	}

	CRequestCompleteData& operator=(const CRequestCompleteData& RCD)
	{
		m_nTaskGroup = RCD.m_nTaskGroup;
		m_nVUId = RCD.m_nVUId;
		m_nConId = RCD.m_nConId;
		m_bIsBinaryData = RCD.m_bIsBinaryData;
		int nLen = strlen(RCD.m_szHeaders);
		if(nLen)
		{
			m_szHeaders = new char[nLen+1];
			memcpy(m_szHeaders,RCD.m_szHeaders,nLen+1);
		}
		else
			m_szHeaders = NULL;
		if(RCD.m_nBodyLength && !m_bIsBinaryData)
		{
			m_szBody = new char[RCD.m_nBodyLength];
			memcpy(m_szBody,RCD.m_szBody,RCD.m_nBodyLength);
		}
		else
			m_szBody = NULL;
		m_nBodyLength = RCD.m_nBodyLength;
		
		return *this;
	}
		
private:
	int m_nTaskGroup;
	int m_nVUId;
	int m_nConId;
	BOOL m_bIsBinaryData;
	char *m_szHeaders;
	char *m_szBody;
	int m_nBodyLength;
};

#endif // !defined(AFX_HTTPSINGLESTEPFILE_H__300C78BB_F75A_4E0F_B090_C3C703FFAE02__INCLUDED_)
