/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// PluginManager.cpp: implementation of the CPluginManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PluginManager.h"
#include "PluginTypes.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

typedef BOOL  ( * LPREGTEMPLFUNC)            (CDocManager* pDocManager);
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPluginManager::CPluginManager(CString plugPath)
{
	CFileFind finder;

	// build a string with wildcards
	CString strWildcard(plugPath);
	strWildcard += "\\*.dll";

	// start working for files
	BOOL bWorking = finder.FindFile(strWildcard);

	while (bWorking)
	{
		bWorking = finder.FindNextFile();
		LoadPlugin(finder.GetFilePath());	
	}

	finder.Close();	
}

CPluginManager::~CPluginManager()
{
    for(int nDllNdx=0;nDllNdx<m_PluginDLLs.GetSize();nDllNdx++)
    {
		AfxFreeLibrary(m_PluginDLLs[nDllNdx]->GetHandle());
		delete m_PluginDLLs[nDllNdx];
	}
    m_PluginDLLs.RemoveAll();
}

void CPluginManager::LoadPlugin(CString szPath)
{
    HINSTANCE hDLL = AfxLoadLibrary(szPath);
    if (hDLL != NULL)
    {
		// Make sure it's a valid OpenSTA plugin
		FARPROC verifyProc = ::GetProcAddress(hDLL ,"OpenSTAVerify");
		if (verifyProc && verifyProc())
		{
			LPGETTITLEFUNC getTitleProc = (LPGETTITLEFUNC)::GetProcAddress(hDLL ,"GetTitle");
			if (!getTitleProc)
				return;

			CString szTitle = "";
			getTitleProc(szTitle);

			LPGETPACKAGEFUNC getPackageProc = (LPGETPACKAGEFUNC)::GetProcAddress(hDLL ,"GetPackage");
			if (!getPackageProc)
				return;

			CString szPackage = "";
			getPackageProc(szPackage);

			FARPROC getTypeProc = ::GetProcAddress(hDLL ,"GetType");
			if (!getTypeProc)
				return;
			int nType = getTypeProc();			

			CPlugin* pPlug = new CPlugin(hDLL, nType, szTitle.GetBuffer(0), szPackage.GetBuffer(0));
			m_PluginDLLs.Add(pPlug);			
		}
		else
			AfxFreeLibrary(hDLL);

    }
}

CPlugin*	CPluginManager::GetPlugin(int nType, CString szTitle)
{
    for(int nDllNdx=0;nDllNdx<m_PluginDLLs.GetSize();nDllNdx++)
	{
		if (m_PluginDLLs[nDllNdx]->GetType() == nType && 
			m_PluginDLLs[nDllNdx]->GetTitle() == szTitle)
			return m_PluginDLLs[nDllNdx];
	}

	return NULL;
}

void	CPluginManager::ListPluginsByType(int nType, CStringArray& arrSzTitles)
{
	for(int nDllNdx=0;nDllNdx<m_PluginDLLs.GetSize();nDllNdx++)
	{
		CPlugin* plug = (CPlugin*)m_PluginDLLs[nDllNdx];
		if (plug->GetType() == nType)
		{
			arrSzTitles.Add(plug->GetTitle());
		}
	}
}

void	CPluginManager::ListPluginsByPackage(CString szPackage, CStringArray& arrSzTitles)
{
	for(int nDllNdx=0;nDllNdx<m_PluginDLLs.GetSize();nDllNdx++)
	{
		CPlugin* plug = (CPlugin*)m_PluginDLLs[nDllNdx];
		if (plug->GetPackage() == szPackage)
		{
			arrSzTitles.Add(plug->GetTitle());
		}
	}

}
