/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ServerSettingsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "relaycfg.h"
#include "ServerSettingsDlg.h"
#include "mainfrm.h"
#include <direct.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include <string>
using namespace std;

/////////////////////////////////////////////////////////////////////////////
// CServerSettingsDlg dialog


CServerSettingsDlg::CServerSettingsDlg(BOOL bRestart, CWnd* pParent /*=NULL*/)
	: CDialog(CServerSettingsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CServerSettingsDlg)
	m_nWebPort = 0;
	m_strWebRoot = _T("");
	m_bUseFTP = FALSE;
	m_strFTPRoot = _T("");
	m_nFTPPort = 0;
	m_bUseSSL = FALSE;
	m_strCache = _T("");
	m_strExtPath = _T("");
	m_strFtpUser = _T("");
	m_strFtpPass = _T("");
	//}}AFX_DATA_INIT

	m_bRestart = bRestart;
}


void CServerSettingsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CServerSettingsDlg)
	DDX_Text(pDX, IDC_PORT, m_nWebPort);
	DDV_MinMaxInt(pDX, m_nWebPort, 0, 99999999);
	DDX_Text(pDX, IDC_WEBROOT, m_strWebRoot);
	DDX_Check(pDX, IDC_USERFTP, m_bUseFTP);
	DDX_Text(pDX, IDC_FTPROOT, m_strFTPRoot);
	DDX_Text(pDX, IDC_FTPPORT, m_nFTPPort);
	DDV_MinMaxInt(pDX, m_nFTPPort, 0, 99999999);
	DDX_Check(pDX, IDC_SSL, m_bUseSSL);
	DDX_Text(pDX, IDC_CACHE, m_strCache);
	DDX_Text(pDX, IDC_EXTENSION, m_strExtPath);
	DDX_Text(pDX, IDC_FTP_USER, m_strFtpUser);
	DDX_Text(pDX, IDC_FTP_PASS, m_strFtpPass);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CServerSettingsDlg, CDialog)
	//{{AFX_MSG_MAP(CServerSettingsDlg)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_USERFTP, OnUserftp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CServerSettingsDlg message handlers

BOOL CServerSettingsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	string strPath = getenv("OPENSTA_HOME");
	strPath += "SERVER\\WebSvr.ini";
	m_settings.ReadSettings(strPath);
	
	m_nWebPort = m_settings.m_nWebPort;
	m_strWebRoot = m_settings.m_strWebRoot.c_str();
	m_bUseSSL = m_settings.m_bUseSSL;
	m_strExtPath = m_settings.m_strExtensionPath.c_str();
	m_strCache = m_settings.m_strCachePath.c_str();


	m_bUseFTP = m_settings.m_bUseFTP;
	m_nFTPPort = m_settings.m_nFtpPort;
	m_strFTPRoot = m_settings.m_strFtpRoot.c_str();
	m_strFtpUser = m_settings.m_strFtpUser.c_str();
	m_strFtpPass = m_settings.m_strFtpPass.c_str();

	if (!m_bUseFTP)
	{
		CWnd* pWnd = GetDlgItem(IDC_FTPPORT);
		pWnd->EnableWindow(FALSE);

		CWnd* pWnd1 = GetDlgItem(IDC_FTPROOT);
		pWnd1->EnableWindow(FALSE);
	}

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CServerSettingsDlg::OnSave() 
{
	UpdateData(TRUE);

	if (m_strWebRoot.IsEmpty())
	{
		AfxMessageBox("Please enter the root directory of your web server.");
		return;
	}

	// Remove trailing '\' if there is one
	m_strWebRoot.TrimRight("\\");

	// make sure the root dir exists
	CFileStatus fs;
	if (!CFile::GetStatus(m_strWebRoot, fs))
	{
		AfxMessageBox("Please enter the root directory of your web server.");
		return;
	}

	if (m_strExtPath.IsEmpty())
	{
		AfxMessageBox("Please enter web server path where the ISAPI Extenstension will be installed.");
		return;
	}

	// Copy the ISAPI Extension into the web directory
	CString strFullPath = m_strWebRoot+m_strExtPath;
	CString strDllPath = getenv("OPENSTA_HOME");
	strDllPath+="SERVER\\xmlpost.dll";

	CString strTmp = strFullPath;
	int nPos = strTmp.Find('/',3);
	while(nPos > 0)
	{
		CString strDir = strTmp.Left(nPos);
		CreateDirectory(strDir, NULL);
		nPos = strTmp.Find('/',nPos+1);
	}

	if (!CopyFile(strDllPath, strFullPath, false))
	{
		AfxMessageBox("Failed to install the ISAPI Extension.");
		return;
	}

	CopyCommonDlls();

	// Create cache directory
	strFullPath = m_strWebRoot+m_strCache;

	strTmp = strFullPath;
	nPos = strTmp.Find('/',3);
	while(nPos > 0)
	{
		CString strDir = strTmp.Left(nPos);
		CreateDirectory(strDir, NULL);
		nPos = strTmp.Find('/',nPos+1);
	}


	SaveSettings();

	if(m_bRestart)
	{
		CMainFrame* pMainFrame = (CMainFrame*)AfxGetMainWnd();
		int nRet = AfxMessageBox("The Relay Daemon needs to be restarted for\nthe changes to take effect.\nRestart now?", MB_YESNO);
		if (nRet == IDYES)
		{
			pMainFrame->StopDmn();
			pMainFrame->StartDmn();
		}
	}

	EndDialog(IDOK);
}

void CServerSettingsDlg::OnUserftp() 
{
	UpdateData(TRUE);

	CWnd* pWnd = GetDlgItem(IDC_FTPPORT);
	CWnd* pWnd1 = GetDlgItem(IDC_FTPROOT);

	if (!m_bUseFTP)
	{
		pWnd->EnableWindow(FALSE);
		pWnd1->EnableWindow(FALSE);
	}
	else
	{
		pWnd->EnableWindow(TRUE);
		pWnd1->EnableWindow(TRUE);
	}
}

void CServerSettingsDlg::SaveSettings()
{
	UpdateData(TRUE);
	
	m_settings.m_nWebPort = m_nWebPort;
	m_settings.m_strWebRoot = m_strWebRoot;
	m_settings.m_bUseSSL = m_bUseSSL != FALSE;
	m_settings.m_strExtensionPath = m_strExtPath;
	m_settings.m_strCachePath = m_strCache;


	m_settings.m_bUseFTP = m_bUseFTP != FALSE;
	m_settings.m_nFtpPort = m_nFTPPort;
	m_settings.m_strFtpRoot = m_strFTPRoot;
	m_settings.m_strFtpUser = m_strFtpUser;
	m_settings.m_strFtpPass = m_strFtpPass;

	string strPath = getenv("OPENSTA_HOME");
	strPath += "SERVER\\WebSvr.ini";
	m_settings.SaveSettings(strPath);
}

void CServerSettingsDlg::CopyCommonDlls()
{
	CString strExtDir = m_strWebRoot+m_strExtPath;
	int nPos = strExtDir.ReverseFind('/');
	strExtDir = strExtDir.Left(nPos+1);

	CString strCommonDir = getenv("OPENSTA_HOME");
	strCommonDir += "COMMON\\";

	CFileFind ff;
    int bWorking = ff.FindFile(strCommonDir+"omni*.*");
    while (bWorking)
    {
        bWorking = ff.FindNextFile();
		CopyFile(ff.GetFilePath(), strExtDir+ff.GetFileName(), false);
	}

    bWorking = ff.FindFile(strCommonDir+"stlport*.*");
    while (bWorking)
    {
        bWorking = ff.FindNextFile();

		CopyFile(ff.GetFilePath(), strExtDir+ff.GetFileName(), false);
	}
}
