/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "RELAYCFG.h"

#include "MainFrm.h"
#include "ServerSettingsDlg.h"
#include "RelayMapDlg.h"
#include "OutputLog.h"
#include "TraceLevelDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	WM_ICON_NOTIFY			WM_USER+10

UINT ExecuteThreadProc(LPVOID pParam);
UINT CheckerThreadProc(LPVOID pParam);

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_POPUP_SERVER_CONFIG, OnPopupServerConfig)
	ON_COMMAND(ID_POPUP_START, OnPopupStart)
	ON_COMMAND(ID_POPUP_STOP, OnPopupStop)
	ON_COMMAND(ID_POPUP_LOG, OnPopupLog)
	ON_UPDATE_COMMAND_UI(ID_POPUP_START, OnUpdatePopupStart)
	ON_UPDATE_COMMAND_UI(ID_POPUP_STOP, OnUpdatePopupStop)
	ON_UPDATE_COMMAND_UI(ID_POPUP_LOG, OnUpdatePopupLog)
	ON_COMMAND(ID_POPUP_EXIT, OnPopupExit)
	ON_COMMAND(ID_POPUP_RELAY_MAP, OnPopupRelayMap)
	ON_COMMAND(ID_POPUP_SETTRACELEVEL, OnPopupSettracelevel)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_ICON_NOTIFY, OnTrayNotification)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pExec = NULL;
	m_bRunning = FALSE;

	ReadTraceLevel();	

	CECGridCtrl::GridCtrlInit(AfxGetInstanceHandle());
}

CMainFrame::~CMainFrame()
{
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create the tray icon
	if (!m_TrayIcon.Create(this, WM_ICON_NOTIFY, _T("OpenSTA Relay Daemon Config"), NULL, IDR_POPUP_MENU))
		return -1;

	m_TrayIcon.SetIcon(IDI_STOPPED);
	m_pCheckerThread = AfxBeginThread(&CheckerThreadProc, this);

	// Attempt to start automatically
	StartDmn();
	return 0;
}

LRESULT CMainFrame::OnTrayNotification(WPARAM wParam, LPARAM lParam)
{
	// Delegate all the work back to the default implementation in
	// CTrayIcon.
	return m_TrayIcon.OnTrayNotification(wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame/CTrayIcon menu message handlers

void CMainFrame::OnPopupServerConfig() 
{
	CServerSettingsDlg dlg(m_bRunning);
	dlg.DoModal();
}

void CMainFrame::OnPopupStart() 
{
	StartDmn();
}

void CMainFrame::StartDmn()
{
	// Check that the server settings have been configued
	string strPath = getenv("OPENSTA_HOME");
	strPath += "SERVER\\WebSvr.ini";
	CFileStatus fs;
	if (!CFile::GetStatus(strPath.c_str(), fs))
	{
		CServerSettingsDlg dlg(FALSE);
		if (dlg.DoModal() != IDOK)
			return;
	}

	m_pExecThread = AfxBeginThread(&ExecuteThreadProc, this);
}

void CMainFrame::OnPopupStop() 
{
	StopDmn();
}

void CMainFrame::StopDmn()
{
	if (m_pExec)
	{
		m_pExec->KillProcess();
		delete m_pExec;
		m_pExec = NULL;

		m_TrayIcon.SetIcon(IDI_STOPPED);
	}
}


UINT ExecuteThreadProc(LPVOID pParam)
{
	CMainFrame* pMainFrame = (CMainFrame*)pParam;

	CString strCmd;
	strCmd.Format("%sSERVER\\RelayDmn.exe -T %d", getenv("OPENSTA_HOME"), pMainFrame->m_nTraceLevel);
	pMainFrame->m_pExec = new CCmdExec(strCmd);

	pMainFrame->m_pExec->Execute(pMainFrame->m_strOutLog);

	return 0;
}

UINT CheckerThreadProc(LPVOID pParam)
{
	CMainFrame* pMainFrame = (CMainFrame*)pParam;
	while(true)
	{
		if (pMainFrame->m_pExec && pMainFrame->m_pExec->IsActive())
		{
			pMainFrame->m_TrayIcon.SetIcon(IDI_RUNNING);
			pMainFrame->m_bRunning = true;
		}
		else
		{
			pMainFrame->m_TrayIcon.SetIcon(IDI_STOPPED);
			pMainFrame->m_bRunning = false;
		}
		Sleep(1000);
	}
}

void CMainFrame::OnPopupLog() 
{
	COutputLog log;
	log.DoModal();
}

void CMainFrame::OnUpdatePopupStart(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_bRunning);	
}

void CMainFrame::OnUpdatePopupStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bRunning);		
}

void CMainFrame::OnUpdatePopupLog(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bRunning);			
}

void CMainFrame::OnPopupExit() 
{
	OnPopupStop();
	m_TrayIcon.HideIcon();
	m_TrayIcon.RemoveIcon();

	OnClose();
}

void CMainFrame::OnPopupRelayMap() 
{
	CRelayMapDlg dlg;
	dlg.DoModal();	
}

void CMainFrame::OnPopupSettracelevel() 
{
	CTraceLevelDlg dlg(m_nTraceLevel);
	if (dlg.DoModal() == IDOK)
	{
		m_nTraceLevel = dlg.GetTraceLevel();
		StoreTraceLevel();
		if(m_bRunning)
		{
			int nRet = AfxMessageBox("The Relay Daemon needs to be restarted for\nthe changes to take effect.\nRestart now?", MB_YESNO);
			if (nRet == IDYES)
			{
				StopDmn();
				StartDmn();
			}
		}
	}
}

void CMainFrame::ReadTraceLevel()
{
	m_nTraceLevel = 0;

    HKEY    hRegKey = NULL;
    char    szKeyValueData[256];
    DWORD   dwKeyLen = sizeof(szKeyValueData);

    //
    // First lets open the key
    if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "SOFTWARE\\OpenSTA\\RelayCfg", 0, KEY_READ, &hRegKey) == ERROR_SUCCESS)
    {
        //
        // And query the Requested Field
        if (RegQueryValueEx(hRegKey, "Tracing", 0, NULL, (unsigned char *) &szKeyValueData, &dwKeyLen) == ERROR_SUCCESS)
        {
            m_nTraceLevel = atoi(szKeyValueData);
        }

        // then close the key again
        RegCloseKey(hRegKey);
    }

}

void CMainFrame::StoreTraceLevel()
{
    HKEY    hRegKey;

    char value[4];
	sprintf(value, "%d", m_nTraceLevel);

    if (RegCreateKeyEx (HKEY_LOCAL_MACHINE
                        ,"SOFTWARE\\OpenSTA\\RelayCfg"
                        ,0
                        ,NULL
                        ,REG_OPTION_NON_VOLATILE
                        ,KEY_WRITE
                        ,NULL
                        ,&hRegKey
                        ,NULL) == ERROR_SUCCESS)
    {

        RegSetValueEx(hRegKey ,"Tracing", 0, REG_SZ ,(const unsigned char*) value, strlen(value) + 1);
        RegCloseKey(hRegKey);
    }

}

BOOL CMainFrame::IsRunning()
{
	return m_bRunning;
}
