#include "stdafx.h"
#include <iostream>
#include "ctime.hpp"
#include "SummaryTask.hpp"
#include <malloc.h>

//
// CSummaryTask class
//
CSummaryTask::CSummaryTask()
:bScript_(false),		
iDuration_(0),
bIsDuration_(false),
iNumIterations_(0),
iMaxIterDelay_(0),
iMinIterDelay_(0),
iFixedIterDelay_(0),
bConstantDelay_(false)
{
}

CSummaryTask::CSummaryTask(	bool bScript, 
						   	string name,
							string target,
							string description,
							int iDuration, 
							bool bIsDuration, 
							int iNumIterations,
							int iMaxIterDelay,
							int iMinIterDelay,
							int iFixedIterDelay,
							bool bConstantDelay)
:bScript_(bScript),		
name_(name),
target_(target),
description_(description),
iDuration_(iDuration),
bIsDuration_(bIsDuration),
iNumIterations_(iNumIterations),
iMaxIterDelay_(iMaxIterDelay),
iMinIterDelay_(iMinIterDelay),
iFixedIterDelay_(iFixedIterDelay),
bConstantDelay_(bConstantDelay)
{
}

void CSummaryTask::display()
{
	std::cout << "Type           : " << (bScript_?"Script Task":"Collector Task") << std::endl;
	std::cout << "Name           : " << name_ << std::endl;
	std::cout << "Target         : " << target_ << std::endl;
	std::cout << "Description    : " << description_ << std::endl;
	std::cout << "IsDuration     : " << (bIsDuration_?"true":"false") << std::endl;
	std::cout << "Duration       : " << iDuration_<< std::endl;
	std::cout << "NumIterations  : " << iNumIterations_<< std::endl;
	std::cout << "MaxIterDelay   : " << iMaxIterDelay_<< std::endl;
	std::cout << "MinIterDelay   : " << iMinIterDelay_<< std::endl;
	std::cout << "FixedIterDelay : " << iFixedIterDelay_ << std::endl;
	std::cout << "ConstantDelay  : " << (bConstantDelay_?"true":"false") << std::endl;
}

//
// CSummaryTaskGroup class
//
CSummaryTaskGroup::CSummaryTaskGroup()
:bIsDelayed_(false),
bStopOnComplete_(false),
iVUsPerBatch_(0),
iInterBatchDelay_(0),
iBatchRampUpTime_(0),
bShowAdvancedVUInfo_(false),
iTaskGroupIterations_(0),
iTaskGroupDuration_(0),
iStartInterval_(0),
iNumVUs_(0),
iNumVUsTimRes_(0),
iNumVUsHttpRes_(0),
iVUPerInterval_(0),
iStartDelay_(0),
bDuration_(false),
bScheduled_(false)
{
}

CSummaryTaskGroup::CSummaryTaskGroup(	string		hostname,
										string		name,
										string		description,
										bool		bIsDelayed,
										bool		bStopOnComplete,
										int			iVUsPerBatch,
										int			iInterBatchDelay,
										int			iBatchRampUpTime,
										bool		bShowAdvancedVUInfo,
										int			iTaskGroupIterations,
										int			iTaskGroupDuration,
										int			iStartInterval,
										int			iNumVUs,
										int			iNumVUsTimRes,
										int			iNumVUsHttpRes,
										int			iVUPerInterval,
										int			iStartDelay,
										bool		bDuration,
										bool		bScheduled,
										string		scheduledTime)
:iTimestamp_(0),
hostname_(hostname),
name_(name),
description_(description),
bIsDelayed_(bIsDelayed),
bStopOnComplete_(bStopOnComplete),
iVUsPerBatch_(iVUsPerBatch),
iInterBatchDelay_(iInterBatchDelay),
iBatchRampUpTime_(iBatchRampUpTime),
bShowAdvancedVUInfo_(bShowAdvancedVUInfo),
iTaskGroupIterations_(iTaskGroupIterations),
iTaskGroupDuration_(iTaskGroupDuration),
iStartInterval_(iStartInterval),
iNumVUs_(iNumVUs),
iNumVUsTimRes_(iNumVUsTimRes),
iNumVUsHttpRes_(iNumVUsHttpRes),
iVUPerInterval_(iVUPerInterval),
iStartDelay_(iStartDelay),
bDuration_(bDuration),
bScheduled_(bScheduled),
scheduledTime_(scheduledTime)
{
}

void CSummaryTaskGroup::initialize(	string		hostname,
									string		name,
									string		description,
									bool		bIsDelayed,
									bool		bStopOnComplete,
									int			iVUsPerBatch,
									int			iInterBatchDelay,
									int			iBatchRampUpTime,
									bool		bShowAdvancedVUInfo,
									int			iTaskGroupIterations,
									int			iTaskGroupDuration,
									int			iStartInterval,
									int			iNumVUs,
									int			iNumVUsTimRes,
									int			iNumVUsHttpRes,
									int			iVUPerInterval,
									int			iStartDelay,
									bool		bDuration,
									bool		bScheduled,
									string		scheduledTime)
{
	hostname_ = hostname ;
	name_ = name ;
	description_ = description ;
	bIsDelayed_ = bIsDelayed ;
	bStopOnComplete_ = bStopOnComplete ;
	iVUsPerBatch_ = iVUsPerBatch ;
	iInterBatchDelay_ = iInterBatchDelay ;
	iBatchRampUpTime_ = iBatchRampUpTime ;
	bShowAdvancedVUInfo_ = bShowAdvancedVUInfo ;
	iTaskGroupIterations_ = iTaskGroupIterations ;
	iTaskGroupDuration_ = iTaskGroupDuration ;
	iStartInterval_ = iStartInterval ;
	iNumVUs_ = iNumVUs ;
	iNumVUsTimRes_ = iNumVUsTimRes ;
	iNumVUsHttpRes_ = iNumVUsHttpRes ;
	iVUPerInterval_ = iVUPerInterval ;
	iStartDelay_ = iStartDelay ;
	bDuration_ = bDuration ;
	bScheduled_ = bScheduled ;
	scheduledTime_ = scheduledTime ;
}

void CSummaryTaskGroup::display()
{
	CTime tm(iTimestamp_);

	std::cout << "Time                : " << tm.Format("%d/%m/%Y %H:%M:%S") << std::endl; 
	std::cout << "hostname            : " << hostname_ << std::endl;
	std::cout << "name                : " << name_ << std::endl;
	std::cout << "description         : " << description_ << std::endl;
	std::cout << "Delayed             : " << (bIsDelayed_?"true":"false") << std::endl; 
	std::cout << "StopOnComplete      : " << (bStopOnComplete_?"true":"false") << std::endl; 
	std::cout << "VUs per batch       : " << iVUsPerBatch_ << std::endl;
	std::cout << "InterBatchDelay     : " << iInterBatchDelay_ << std::endl;
	std::cout << "BatchRampUpTime     : " << iBatchRampUpTime_ << std::endl;
	std::cout << "ShowAdvancedVUInfo  : " << (bShowAdvancedVUInfo_?"true":"false") << std::endl; 
	std::cout << "TaskGroupIterations : " << iTaskGroupIterations_ << std::endl;
	std::cout << "TaskGroupDuration   : " << iTaskGroupDuration_ << std::endl;
	std::cout << "StartInterval       : " << iStartInterval_ << std::endl;
	std::cout << "NumVUs              : " << iNumVUs_ << std::endl;
	std::cout << "NumVUsTimRes        : " << iNumVUsTimRes_ << std::endl;
	std::cout << "NumVUsHttpRes       : " << iNumVUsHttpRes_ << std::endl;
	std::cout << "StartDelay          : " << iStartDelay_ << std::endl;
	std::cout << "Duration            : " << (bDuration_?"true":"false") << std::endl; 
	std::cout << "Scheduled           : " << (bScheduled_?"true":"false") << std::endl; 
	std::cout << "scheduledTime       : " << scheduledTime_ << std::endl;

	for (CSummaryTaskVectorType::iterator itr = taskVector_.begin(); itr != taskVector_.end(); itr++)
	{
		(*itr)->display ();
		std::cout << std::endl ;
	}
}

CSummaryTaskGroup::~CSummaryTaskGroup()
{
	for (CSummaryTaskVectorType::iterator itr = taskVector_.begin(); itr != taskVector_.end(); itr++)
	{
		CSummaryTask * pst = (*itr) ;
		if (pst)
			delete pst ;
	}
}

void CSummaryTaskGroup::clear()
{
	iTimestamp_ = 0;
	hostname_ = "";
	name_ = "" ;
	description_ = "" ;
	bIsDelayed_ = false ;
	bStopOnComplete_ = false ;
	iVUsPerBatch_ = 0 ;
	iInterBatchDelay_ = 0 ;
	iBatchRampUpTime_ = 0 ;
	bShowAdvancedVUInfo_ = 0 ;
	iTaskGroupIterations_ = 0 ;
	iTaskGroupDuration_ = 0 ;
	iStartInterval_ = 0 ;
	iNumVUs_ = 0 ;
	iNumVUsTimRes_ = 0 ;
	iNumVUsHttpRes_ = 0 ;
	iVUPerInterval_ = 0 ;
	iStartDelay_ = 0 ;
	bDuration_ = false ;
	bScheduled_ = false ;
	scheduledTime_ = "" ;

	for (CSummaryTaskVectorType::iterator itr = taskVector_.begin(); itr != taskVector_.end(); itr++)
	{
		CSummaryTask * pst = (*itr) ;
		if (pst)
			delete pst ;
	}
}

//
// CSummaryFile class
//
bool CSummaryFile::openForAppend()
{
	CFileException e;
	if (!fd_.Open(fileName_.c_str(),CFile::modeCreate | CFile::modeNoTruncate | CFile::modeReadWrite |CFile::shareDenyWrite |CFile::typeBinary, &e))
	{
		error_ = e.m_cause ;
		return false;
	}

	bFileWrite_ = true ;

	fd_.SeekToBegin(); 
	if (!readInt(iVersion_))
		return false;
	if (!readInt(iNbTaskGroup_))
		return false;
	if (!readString(testName_))
		return false;
	if (!readString(testDescription_))
		return false;
	fd_.SeekToEnd(); 
	return true ; 
}

bool CSummaryFile::openForWrite(string testName, string testDescription)
{
	CFileException e;
	if (!fd_.Open(fileName_.c_str(),CFile::modeCreate | CFile::modeReadWrite |CFile::shareDenyWrite |CFile::typeBinary, &e))
	{
		error_ = e.m_cause ;
		return false;
	}

	bFileWrite_ = true ;

	// Write the current version
	iVersion_ = TASKSUMMARY_VERSION ;
	writeInt(iVersion_);
	writeInt(iNbTaskGroup_);

	writeString(testName);
	writeString(testDescription);

	return true ; 
}

bool CSummaryFile::openForRead()
{
	CFileException e;
	if (!fd_.Open(fileName_.c_str(),CFile::modeRead | CFile::shareDenyWrite |CFile::typeBinary, &e))
	{
		error_ = e.m_cause ;
		return false;
	}
	bFileWrite_ = false ;

	if (!readInt(iVersion_))
		return false;
	if (!readInt(iNbTaskGroup_))
		return false;
	if (!readString(testName_))
		return false;
	if (!readString(testDescription_))
		return false;
	return true ; 
}

bool CSummaryFile::close()
{
	bool bRet = true ;
	if (bFileWrite_)
	{
		try
		{
			fd_.Seek(sizeof(int), CFile::begin);
		}
		catch(...)
		{
			error_ = "CSummaryFile::close : seek failed" ;
			bRet = false;
		}
		if (!writeInt(iNbTaskGroup_))
			bRet = false;
	}
	fd_.Close() ;
	iNbTaskGroup_ = 0 ;
	bFileWrite_ = false ;
	iVersion_ = -1; 
	return bRet ;
}


bool CSummaryFile::writeString(const string & str)
{
	try
	{
		int iLen = str.size() ;
		fd_.Write(&iLen, sizeof(int));
		fd_.Write(str.c_str(), iLen);
	}
	catch(...)
	{
		return false;
	}
	return true ;
}

bool CSummaryFile::writeInt(int i)
{
	try
	{
		fd_.Write(&i, sizeof(int));
	}
	catch(...)
	{
		return false;
	}
	return true ;
}

bool CSummaryFile::writeBool(bool b)
{
	try
	{
		fd_.Write(&b, sizeof(bool));
	}
	catch(...)
	{
		return false;
	}
	return true ;
}


bool CSummaryFile::readString(string & str)
{
	unsigned int uiLen = 0 ;
	readInt((int&)uiLen);
	char * zBuf = (char *)_alloca(uiLen + 1);
	if (fd_.Read(zBuf, uiLen) < uiLen)
		return false;
	zBuf[uiLen] = '\0';
	str = zBuf ; 
	return true ;
}

bool CSummaryFile::readInt(int & i)
{
	if (fd_.Read(&i, sizeof(int)) < sizeof(int))
		return false;
	return true ;
}

bool CSummaryFile::readBool(bool & b)
{
	if (fd_.Read(&b, sizeof(bool)) < sizeof(bool))
		return false;
	return true ;
}


bool CSummaryFile::writeTaskGroup(int iTimestamp, CSummaryTaskGroup & stg)
{
	stg.iTimestamp_ = iTimestamp ;
	if (!writeInt(iTimestamp)) return false;
	if (!writeString(stg.hostname_)) return false;
	if (!writeString(stg.name_)) return false;
	if (!writeString(stg.description_) ) return false;
	if (!writeBool(stg.bIsDelayed_)) return false;
	if (!writeBool(stg.bStopOnComplete_) ) return false;
	if (!writeInt(stg.iVUsPerBatch_)) return false;	
	if (!writeInt(stg.iInterBatchDelay_)) return false;	
	if (!writeInt(stg.iBatchRampUpTime_)) return false;	
	if (!writeBool(stg.bShowAdvancedVUInfo_) ) return false;
	if (!writeInt(stg.iTaskGroupIterations_)) return false;	
	if (!writeInt(stg.iTaskGroupDuration_)) return false;	
	if (!writeInt(stg.iStartInterval_)) return false;	
	if (!writeInt(stg.iNumVUs_)) return false;	
	if (!writeInt(stg.iNumVUsTimRes_)) return false;	
	if (!writeInt(stg.iNumVUsHttpRes_)) return false;	
	if (!writeInt(stg.iVUPerInterval_)) return false;	
	if (!writeInt(stg.iStartDelay_)) return false;	
	if (!writeBool(stg.bDuration_)) return false;	
	if (!writeBool(stg.bScheduled_)) return false;
	if (!writeString(stg.scheduledTime_)) return false;

	// if (!write the number of Tasks
	if (!writeInt(stg.GetNbTasks ()))
		return false;

	for (CSummaryTaskVectorType::iterator itr = stg.taskVector_.begin(); itr != stg.taskVector_.end(); itr++)
	{
		CSummaryTask * pst = (*itr) ;
		if (!writeBool(pst->bScript_)) return false;			
		if (!writeString(pst->name_)) return false;		
		if (!writeString(pst->target_)) return false;
		if (!writeString(pst->description_)) return false;
		if (!writeInt(pst->iDuration_)) return false;
		if (!writeBool(pst->bIsDuration_)) return false;
		if (!writeInt(pst->iNumIterations_)) return false;
		if (!writeInt(pst->iMaxIterDelay_)) return false;
		if (!writeInt(pst->iMinIterDelay_)) return false;
		if (!writeInt(pst->iFixedIterDelay_)) return false;
		if (!writeBool(pst->bConstantDelay_)) return false;
	}

	iNbTaskGroup_++ ;
	fd_.Flush();
	return true ;
}


bool CSummaryFile::readTaskGroup(CSummaryTaskGroup & stg)
{
	stg.clear();

	readInt(stg.iTimestamp_);
	readString(stg.hostname_);
	readString(stg.name_);
	readString(stg.description_) ;
	readBool(stg.bIsDelayed_);
	readBool(stg.bStopOnComplete_) ;
	readInt(stg.iVUsPerBatch_);	
	readInt(stg.iInterBatchDelay_);	
	readInt(stg.iBatchRampUpTime_);	
	readBool(stg.bShowAdvancedVUInfo_) ;
	readInt(stg.iTaskGroupIterations_);	
	readInt(stg.iTaskGroupDuration_);	
	readInt(stg.iStartInterval_);	
	readInt(stg.iNumVUs_);	
	readInt(stg.iNumVUsTimRes_);	
	readInt(stg.iNumVUsHttpRes_);	
	readInt(stg.iVUPerInterval_);	
	readInt(stg.iStartDelay_);	
	readBool(stg.bDuration_);	
	readBool(stg.bScheduled_);
	readString(stg.scheduledTime_);

	// read the number of Tasks
	int iNbTasks = 0;
	readInt(iNbTasks);

	for (int i = 0; i<iNbTasks; i++)
	{
		CSummaryTask * pst = new CSummaryTask ;
		if (!pst)
			return false;
		readBool(pst->bScript_);			
		readString(pst->name_);		
		readString(pst->target_);
		readString(pst->description_);
		readInt(pst->iDuration_);
		readBool(pst->bIsDuration_);
		readInt(pst->iNumIterations_);
		readInt(pst->iMaxIterDelay_);
		readInt(pst->iMinIterDelay_);
		readInt(pst->iFixedIterDelay_);
		readBool(pst->bConstantDelay_);
		stg.taskVector_.push_back(pst) ;
	}

	return true ;
}