/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestPluginView.cpp : implementation of the CTestPluginView class
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "TestExecuteDlg.h"
#include "SCLCompiler.h"
#include "CompileDlg.h"
#include "RegFunctions.h"

#include "TestPluginDoc.h"
#include "TestPluginView.h"
#include "PluginTypes.h"
#include "TestTabPlugin.h"
#include "SummaryTask.hpp"
#include "ctime.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern BOOL bFirstLoad;

#define IDC_TABS 0x1000
static CString LOGFILE = "Active.log";
#define IDT_FIND_TEST 1
#define IDT_INITIALISE 2
#define IDT_EXECUTE_TEST 3
const int TIMETOSTARTTEST=5000;
/////////////////////////////////////////////////////////////////////////////
// CTestPluginView

UINT CTestPluginView::FindTest(LPVOID pParam)
{
	CTestPluginView* pView = (CTestPluginView*)pParam;
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CCORBAInfo* pInfo = pApp->GetInfo();

	try
	{
		while(WaitForSingleObject(pView->m_hEndThreadEvent,2000) == WAIT_TIMEOUT)
		{
			if((pInfo->HasServerBeenCreated())==FALSE)
			{	
				//the daemon might have started now
				pInfo->CreateServer();
			}
		
			if ((pInfo->HasServerBeenCreated())==TRUE)
			{
				switch (pInfo->IntIsTestActive(pView->m_strTest, pView->m_strManagerReg))
				{	
					case 3:
					{
						pView->m_bTestActive = true;
						pView->m_bGotStatus=true;
						pView->m_bStopTest = TRUE ;
						break;
					}
					case 2:
					{
						pView->m_bTestActive = true;
						pView->m_bGotStatus=true;
						pView->m_bStopTest = FALSE ;
						pView->m_bKillTest = FALSE ;
						//AfxMessageBox("test active");
						break;
					}
					case 1:
					{
						pView->m_bTestActive = false;
						pView->m_bGotStatus=true;
						pView->m_bStopTest= FALSE ;
						pView->m_bKillTest = FALSE ;
						//AfxMessageBox("test inactive");
						break;
					}
					case 0:
					{
						pView->m_bGotStatus=false;	//no daemon
						//AfxMessageBox("no daemon");
						break;
					}
					default:
					{
						pView->m_bGotStatus=false;	//no daemon?
						//AfxMessageBox("defaulted");
						break;
					}
				}
			}
			else
			{
				pView->m_bGotStatus=false;
				//AfxMessageBox("no corba server yet constructed");
			}
		}
	}
	catch(...)
	{
	}
	return 0;
}


IMPLEMENT_DYNCREATE(CTestPluginView, CView)

BEGIN_MESSAGE_MAP(CTestPluginView, CView)
	//{{AFX_MSG_MAP(CTestPluginView)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
	ON_WM_ERASEBKGND()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(ID_EXECUTE_TEST, OnExecuteTest)
	ON_COMMAND(ID_COMPILE, OnCompile)
	ON_COMMAND(ID_STOP_TEST, OnStopTest)
	ON_UPDATE_COMMAND_UI(ID_EXECUTE_TEST, OnUpdateExecuteTest)
	ON_UPDATE_COMMAND_UI(ID_STOP_TEST, OnUpdateStopTest)
	ON_WM_TIMER()
	ON_COMMAND(ID_KILL_TEST, OnKillTest)
	ON_UPDATE_COMMAND_UI(ID_KILL_TEST, OnUpdateKillTest)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	ON_NOTIFY(TCN_SELCHANGING,IDC_TABS,OnTabsSelChanging)
	ON_MESSAGE(WMU_SETSTEPBYSTEPMODE,OnPluginSetStepByStepMode)
	ON_MESSAGE(WMU_START_TEST,OnPluginStartTest)
	ON_MESSAGE(WMU_STOP_TEST,OnPluginStopTest)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView construction/destruction

CTestPluginView::CTestPluginView()
{
	m_bTestActive = false;
	m_bActivated = false;
	m_bStopTest = FALSE ;
	m_bKillTest = FALSE ;
	m_pTestFindThread = NULL;
	m_bTestExecuteButtonPressed=false;
	m_bGotStatus=false;	//dont display any buttons until Corba gets m_bTestActive
}

CTestPluginView::~CTestPluginView()
{
	if(m_pTestFindThread)
	{
		SetEvent(m_hEndThreadEvent);
		WaitForSingleObject(m_pTestFindThread->m_hThread,5000);
	}
	CloseHandle(m_hEndThreadEvent);
}

BOOL CTestPluginView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView drawing

void CTestPluginView::OnDraw(CDC* pDC)
{
	CTestPluginDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView printing

BOOL CTestPluginView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTestPluginView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CTestPluginView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CTestPluginView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView diagnostics

#ifdef _DEBUG
void CTestPluginView::AssertValid() const
{
	CView::AssertValid();
}

void CTestPluginView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTestPluginDoc* CTestPluginView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTestPluginDoc)));
	return (CTestPluginDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView message handlers

BOOL CTestPluginView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;	
//	return CView::OnEraseBkgnd(pDC);
}

int CTestPluginView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if(!m_ctlTabs.Create(WS_CHILD|WS_VISIBLE,CRect(0,0,0,0),this,IDC_TABS))
		return -1;

	m_TabImages.Create(16,16,ILC_COLOR8|ILC_MASK,0,4);
	m_ctlTabs.SetImageList(&m_TabImages);

	CFont *pFont=CFont::FromHandle((HFONT)GetStockObject(ANSI_VAR_FONT));
	m_ctlTabs.SetFont(pFont);

	return 0;
}

void CTestPluginView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	if(m_ctlTabs.GetSafeHwnd())
		m_ctlTabs.MoveWindow(0,0,cx,cy);
}

void CTestPluginView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	SetTimer(IDT_INITIALISE,50,NULL);
}

void CTestPluginView::Initialise()
{
	static bool bFirstTime = true;

	if(bFirstLoad || !bFirstTime)
		return;

	bFirstTime = false;

	CTestFile *pTestFile = GetDocument()->GetTestFile();
	int nGroup = pTestFile->GetStepByStepTaskGroup();
	BOOL bStepByStepMode = (nGroup!=-1);

	//Get Test Name
	CString strPath(pTestFile->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath.Delete(0,nSlashPos+1);

	int nPos = strPath.ReverseFind('.');
	m_strTest = strPath.Left(nPos);
	CString strExtension = (strPath.Mid(nPos+1));

	GetTestPluginTabsFromExtension(strExtension,m_arstrTabs);

	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CPluginManager* pPluginManager = pApp->GetPluginManager();

	int nCount=0;
	int nTabNdx=0;
	for(int nPlugin =0;nPlugin<m_arstrTabs.GetSize();nPlugin++)
	{
		CTestTabPlugin plug(m_arstrTabs[nPlugin], pPluginManager);
		plug.SetTestFile(GetDocument()->GetTestFile());
		plug.SetDocument(GetDocument());
		plug.SetCorbaInfo(pApp->GetInfo());
		plug.SetPluginManager(pPluginManager);
		plug.SetNotifyWindow(m_hWnd);
	
		int nNumTabs = plug.GetNumTabs();
		if(!nNumTabs)
			continue;

		CBitmap bmpTabs;
		plug.GetTabImages(bmpTabs);
		if(bmpTabs.GetSafeHandle())
			m_TabImages.Add(&bmpTabs,RGB(0,128,128));

		CString strTabName;
		for(int nTab=0;nTab<nNumTabs;nTab++)
		{
			CDialog *pDialog = plug.CreateTab(nTab,&m_ctlTabs,strTabName);
			if(pDialog)
			{
				TabData TD;
				TD.pTab = pDialog;
				TD.strName = strTabName;
				TD.nImage = nCount;
				if(bStepByStepMode)
					TD.bHidden = !plug.DisplayInStepByStepMode(nTab);
				else
					TD.bHidden = false;
				m_arTabs.Add(TD);
				if(!TD.bHidden)
					m_ctlTabs.InsertItem(nTabNdx++,strTabName,nCount,pDialog);
				else
					pDialog->ShowWindow(SW_HIDE);
				nCount++;
			}
		}
	}

	m_ctlTabs.SetCurSel(0);

	m_hEndThreadEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	m_pTestFindThread = AfxBeginThread(&FindTest,this);

	SetTimer(IDT_FIND_TEST, 1000, NULL);
}

void CTestPluginView::OnExecuteTest() 
{
	StartTest();
}

BOOL CTestPluginView::StartTest()
{
	if(m_bTestExecuteButtonPressed)
	{
		//can't start another test manager
		return FALSE;
	}

	if(!CompileScripts())
		return FALSE;

	m_bTestExecuteButtonPressed=true;//simple lock

	//find out what tests are already running and warn the user
	CCORBAInfo CurrentInfo;
	CStringArray arstrTests;
	CString theList="The following tests are currently running\n";
	CString testname;
	CurrentInfo.GetActiveTests(arstrTests);
		
	for(int nTest=0;nTest<arstrTests.GetSize();nTest++)
	{	
		testname="\n" + arstrTests[nTest];
		theList=theList+testname;
	}

	if(arstrTests.GetSize()>0)
	{
		theList=theList+"\n\nYour test will not behave correctly if one of the hosts is already in use\nDo you still wish to start this test?";
		if(MessageBox(theList,"OpenSTA",MB_YESNO|MB_DEFBUTTON2|MB_ICONEXCLAMATION )!=IDYES)
		{
			//reset as if button wasn't pressed and stop
			m_bTestExecuteButtonPressed=false;
			return FALSE;	
		}
	}
	//end of warning

	CTestExecuteDlg init(m_strTest);
	if(init.DoModal()!=IDOK)
		return false;

	m_bStopTest = FALSE ;
	m_bKillTest = FALSE ;
	m_bSummaryWritten = false;

	if (WriteSummary())
		m_bSummaryWritten = true;
	
	SetTimer(IDT_EXECUTE_TEST,TIMETOSTARTTEST, NULL);
	return TRUE;
}

BOOL CTestPluginView::CompileScripts()
{
	//Save test before compiling
	GetDocument()->SaveEmbedding();

	//Get the output directory
	CString strOutputDir = GetCompilerOutputDir();

	CTestFile *pTestFile = GetDocument()->GetTestFile();

	//Get the name of this test
	CString strTestPath(pTestFile->GetTestFilePath());

	//Get the test name without its extension
	CString strTestNoExt;
	int nDotPos = strTestPath.ReverseFind('.');
	if(nDotPos != -1)
		strTestNoExt = strTestPath.Left(nDotPos);
	
	int nNumGroups = pTestFile->GetNumTaskGroups();
	for(int nGroup=0;nGroup<nNumGroups;nGroup++)
	{
		int nNumTasks = pTestFile->GetNumTasks(nGroup);	
		for(int nTask=0;nTask<nNumTasks;nTask++)
		{
			CTestFile::TaskType nType = pTestFile->GetTaskType(nGroup,nTask);
			CString strPath = pTestFile->GetTaskPath(nGroup,nTask);

			switch(nType)
			{
			case CTestFile::enScript:
			{	
				//Compile the script
				if(!Compile(strPath))
					return false;

			}
			break;
			default:
				ASSERT(FALSE);
			break;
			}
		}
	}
	return true;
}

BOOL CTestPluginView::Compile(const CString& strPath)
{
	CSCLCompiler compiler;
	if(!compiler.CompileScript(strPath,TRUE))
	{
		CCompileDlg dlg(strPath,compiler.GetErrorMessage());
		dlg.DoModal();
		return false;
	}
	return true;
}

void CTestPluginView::OnCompile() 
{
	if(CompileScripts())
		AfxMessageBox("Compiled successfully");
}

BOOL CTestPluginView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	if(m_ctlTabs.GetCurrentTab() && 
		m_ctlTabs.GetCurrentTab()->OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
		return true;

	return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CTestPluginView::OnStopTest() 
{
	//Get Test Name
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath.Delete(0,nSlashPos+1);

	int nPos = strPath.ReverseFind('.');
	if(nPos==-1)
	{
		return;
	}
	CWaitCursor wc;	
	try
	{
		m_strTest = strPath.Left(nPos);
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CCORBAInfo* pInfo = pApp->GetInfo();
		pInfo->StopTest(m_strTest);
		m_bStopTest = TRUE ;
	}
	catch(...)
	{
	}
}

BOOL CTestPluginView::PreTranslateMessage(MSG* pMsg) 
{
	if(m_ctlTabs.GetCurrentTab() && 
		m_ctlTabs.GetCurrentTab()->PreTranslateMessage(pMsg))
		return true;
	
	return CView::PreTranslateMessage(pMsg);
}

void CTestPluginView::OnTabsSelChanging(NMHDR* pNMHDR,LRESULT* pResult)
{
	//Save file when change tabs
	GetDocument()->SaveEmbedding();
	*pResult =0;
}


void CTestPluginView::OnUpdateExecuteTest(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_bTestActive&&!m_bTestExecuteButtonPressed&&m_bGotStatus);
}

void CTestPluginView::OnUpdateStopTest(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bTestActive && m_bGotStatus&& !m_bStopTest);	
	pCmdUI->SetCheck(m_bStopTest);
}

void CTestPluginView::OnTimer(UINT nIDEvent) 
{
	switch(nIDEvent)
	{
	case IDT_EXECUTE_TEST:
		//timeout on starting the test
		KillTimer(IDT_EXECUTE_TEST);
		m_bTestExecuteButtonPressed=false; 
	break;
	case IDT_INITIALISE:
		KillTimer(IDT_INITIALISE);
		Initialise();
	break;
	case IDT_FIND_TEST:
	{
		if (m_bTestActive && !m_bActivated)
		{
			CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
			CPluginManager* pPluginManager = pApp->GetPluginManager();
			for (int nPlugIn = 0; nPlugIn<m_arstrTabs.GetSize();nPlugIn++)
			{
				CTestTabPlugin plug(m_arstrTabs[nPlugIn], pPluginManager);
				plug.SetTestActive(TRUE,m_strManagerReg);
			}

			m_bActivated = true;
		}
		else if (!m_bTestActive && m_bActivated)
		{
			CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
			CPluginManager* pPluginManager = pApp->GetPluginManager();
			for (int nPlugIn = 0; nPlugIn<m_arstrTabs.GetSize();nPlugIn++)
			{
				CTestTabPlugin plug(m_arstrTabs[nPlugIn], pPluginManager);
				plug.SetTestActive(FALSE,"");
			}

			m_bSummaryWritten = false;
			m_bActivated = false;
		}

		if (m_bTestActive && !m_bSummaryWritten)
			WriteSummary();
	}
	break;
	}

	CView::OnTimer(nIDEvent);
}

void CTestPluginView::OnKillTest() 
{
	if (AfxMessageBox("You are about to kill the test " + GetDocument()->GetTestFile()->GetTestName() + ".\nAre you sure that is what you want to do ?", MB_OKCANCEL) == IDCANCEL)
		return ;
	//Get Test Name
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath.Delete(0,nSlashPos+1);

	int nPos = strPath.ReverseFind('.');
	if(nPos==-1)
	{
		return;
	}

	CWaitCursor wc;			
	try
	{
		m_strTest = strPath.Left(nPos);
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CCORBAInfo* pInfo = pApp->GetInfo();
		pInfo->KillTest(m_strTest);
		m_bStopTest = m_bKillTest = TRUE ;
	}
	catch(...)
	{
	}	
}

void CTestPluginView::OnUpdateKillTest(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bTestActive&&m_bGotStatus && !m_bKillTest);		
	pCmdUI->SetCheck(m_bKillTest);
}

BOOL CTestPluginView::WriteSummary()
{
	CTLockGuard<CTMutex> guard(m_writeSummaryMutex_);
	if (m_bSummaryWritten)
		return TRUE ;
	//Create a Test summary file in the test run folder which 
	//has just been created...
	//...Look for the newest run in this test folder
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath = strPath.Left(nSlashPos+1);

	CString strRunFolder="";
	CString strReg="";

	//corba info class
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CCORBAInfo* pInfo = pApp->GetInfo();

	try
	{
		if (pInfo->IsTestActive(m_strTest, strReg) && !strReg.IsEmpty())
		{
			if(pInfo->GetRunFolderName(strRunFolder,strReg) && !strRunFolder.IsEmpty())
			{
				CString	strFolder;
				CTTime tm;
				tm.now();
				strFolder+=strPath+strRunFolder;

				CSummaryFile sf(LPCTSTR(strFolder + "\\QTSummary.bin"));
				CString strSummary;
				if (!GetDocument()->GetTestFile()->GetSummary(strSummary, tm.getSeconds(), sf))
				{
					return FALSE;
				}

				CStdioFile f;
				try
				{
					if(f.Open(strFolder + "\\" + strSummaryFile,CFile::modeCreate|CFile::modeWrite))
					{
						f.WriteString(strSummary);
						f.Close();
					}
				}
				catch(...)
				{
				}
//				GetDocument()->GetTestFile()->FillSummaryFile (tm.getSeconds(), sf);
				
				//Copy configuration file to results folder
				m_bSummaryWritten = true;
				CopyFile(GetDocument()->GetTestFile()->GetTestFilePath(),strFolder+"\\"+strResultsTestFile,TRUE);
				return true;
			}
		}
	}	
	catch(...)
	{
	}

	return false;
}

LRESULT CTestPluginView::OnPluginSetStepByStepMode(WPARAM wParam,LPARAM lEnable)
{
	GetDocument()->GetTestFile()->SetStepByStepMode((int)wParam);
	GetDocument()->SetModifiedFlag();

	if(lEnable)
	{
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CPluginManager* pPluginManager = pApp->GetPluginManager();

		int nTabNdx=0;
		for(int nPlugin =0;nPlugin<m_arstrTabs.GetSize();nPlugin++)
		{
			CTestTabPlugin plug(m_arstrTabs[nPlugin], pPluginManager);

			int nNumTabs = plug.GetNumTabs();
			for(int nTab=0;nTab<nNumTabs;nTab++)
			{
				if(!plug.DisplayInStepByStepMode(nTab))
				{
					m_ctlTabs.DeleteItem(nTabNdx);
					m_arTabs[nTabNdx].bHidden = true;
				}
				else
					nTabNdx++;
			}
		}
	}
	else
	{
		for(int nTab = 0;nTab<m_arTabs.GetSize();nTab++)
		{
			TabData &TD = m_arTabs[nTab];
			if(TD.bHidden)
			{
				m_ctlTabs.InsertItem(nTab,TD.strName,TD.nImage,TD.pTab);
				TD.bHidden = false;
			}
		}
	}

	//Pass on to framewnd to disable toolbars
	return AfxGetMainWnd()->SendMessage(WMU_SETSTEPBYSTEPMODE,wParam,lEnable);
}

LRESULT CTestPluginView::OnPluginStartTest(WPARAM wParam,LPARAM lParam)
{
	return StartTest();
}

LRESULT CTestPluginView::OnPluginStopTest(WPARAM wParam,LPARAM lParam)
{
	OnStopTest();
	return 0;
}