/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ConfigurationDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ConfigurationDlg.h"
#include "OstaDefs.h"
#include "TestFile.h"
#include "RegFunctions.h"
#include "CECGridCtrl.h"
#include "TestCtrl.h" //for the enum enHostCol etc...
#include "PropertiesBar.h"
#include "BaseUIHelpID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CTestFile* m_pTestFile;
extern CPropertiesBar *m_pPropertiesBar;
extern CDocument *m_pDocument;
extern CToolBar *m_pToolbar;
extern HINSTANCE m_hInstance;
/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg dialog
#define IDT_ACTIVATE 1

#pragma warning(disable:4355)
CConfigurationDlg::CConfigurationDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CConfigurationDlg::IDD, pParent),
	m_DropTarget(this)
{
	CECGridCtrl::GridCtrlInit(m_hInstance);
	//{{AFX_DATA_INIT(CConfigurationDlg)
	m_strDescription = _T("");
	//}}AFX_DATA_INIT
	m_bTestActive = false;
	m_bPropBarVisible = true;
}
#pragma warning(default:4355)

void CConfigurationDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigurationDlg)
	DDX_Control(pDX, IDC_DESCRIPTION, m_ctlTestDescription);
	DDX_Control(pDX, IDC_TESTS_LIST, m_ctlTestList);
	DDX_Text(pDX, IDC_DESCRIPTION, m_strDescription);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigurationDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigurationDlg)
	ON_WM_SIZE()
	ON_EN_CHANGE(IDC_DESCRIPTION, OnChangeDescription)
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_TOGGLE_SHOWRUNS, OnToggleShowruns)
	ON_UPDATE_COMMAND_UI(ID_TOGGLE_SHOWRUNS, OnUpdateToggleShowruns)
	ON_UPDATE_COMMAND_UI(ID_DELETESELECTION, OnUpdateDeleteselection)
	ON_COMMAND(ID_DELETESELECTION, OnDeleteselection)
	ON_WM_TIMER()
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
	ON_NOTIFY(NM_CLICK,IDC_TESTS_LIST, OnSglclkTestList)
	ON_MESSAGE(WMU_CELL_BUTTON_PRESSED, OnButtonChange)
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg message handlers

void CConfigurationDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	
	if(m_ctlTestDescription.GetSafeHwnd() && m_ctlTestList.GetSafeHwnd())
	{
		CRect rcDesc;
		m_ctlTestDescription.GetWindowRect(&rcDesc);
		ScreenToClient(&rcDesc);

		m_ctlTestDescription.SetWindowPos(NULL,0,0,cx-rcDesc.left-5,rcDesc.Height(),SWP_NOZORDER|SWP_NOMOVE);
		m_ctlTestList.MoveWindow(0,rcDesc.bottom + 5 ,cx,cy-rcDesc.Height()-15);
	}
}

BOOL CConfigurationDlg::OnEraseBkgnd(CDC *pDC)
{
	return CDialog::OnEraseBkgnd(pDC);
}

BOOL CConfigurationDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_DropTarget.Register(this);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

DROPEFFECT CConfigurationDlg::CDropTarget::OnDragOver( CWnd* pWnd, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point )
{
	CRect rcTestList;
	m_pDlg->m_ctlTestList.GetWindowRect(&rcTestList);
	m_pDlg->ScreenToClient(&rcTestList);

	if(rcTestList.PtInRect(point))
	{
		CPoint pt(point);
		pt.y -= rcTestList.top;

		return m_pDlg->m_ctlTestList.OnDragOver(pDataObject,dwKeyState,pt);
	}

	return DROPEFFECT_NONE;
}

BOOL CConfigurationDlg::CDropTarget::OnDrop( CWnd* pWnd,COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	CRect rcTestList;
	m_pDlg->m_ctlTestList.GetWindowRect(&rcTestList);
	m_pDlg->ScreenToClient(&rcTestList);

	if(rcTestList.PtInRect(point))
	{
		CPoint pt(point);
		pt.y -= rcTestList.top;
		
		return m_pDlg->m_ctlTestList.OnDrop(pDataObject,dropEffect,pt);
	}

	return FALSE;
}

BOOL CConfigurationDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_DELETE:
		{
			if (m_ctlTestList.m_hWnd == pMsg->hwnd)
			{
				m_ctlTestList.DeleteSelection();
				return true;
			}
			break;
		}
		case VK_F5:
		{
			if (m_ctlTestList.m_hWnd == pMsg->hwnd)
			{
				m_ctlTestList.Refresh();
				m_ctlTestList.RedrawWindow();
				//redraw propbar too?
				return true;
			}
			break;
		}
		case VK_F6:
		{
			if (m_ctlTestList.m_hWnd == pMsg->hwnd)
			{
				m_ctlTestList.OutputTestFileAsText();
				return true;
			}
			break;
		}

		}
	break;
	}
	
	return false;//CDialog::PreTranslateMessage(pMsg);
	//if cdialog::pretrans is called the escape is removed?

}

void CConfigurationDlg::OnChangeDescription() 
{
	UpdateData(TRUE);
	m_pTestFile->SetDescription(m_strDescription);	
	m_pDocument->SetModifiedFlag();
}

void CConfigurationDlg::OnOK() 
{
}

void CConfigurationDlg::OnCancel() 
{
}

void CConfigurationDlg::OnButtonChange(int nRow, int nCol)
{	
	//button pressed in the test control window
	if(nCol==CTestCtrl::enStartStopStatusCol)
	{
		m_ctlTestList.StartStopButtonPressed(nRow);
	}
	else if(nCol==CTestCtrl::enNameCol)
	{
		m_ctlTestList.DisableButtonPressed(nRow);
	}
	return;
}

void CConfigurationDlg::SetTestActive(CString strReg)
{
	m_bTestActive=TRUE;
	m_strReg=strReg;
	if(m_pPropertiesBar)
	{
		m_pPropertiesBar->Activate(m_pTestFile,m_ctlTestList.GetSafeHwnd(),m_strReg);
		m_pPropertiesBar->SetTestActive();
	}
	m_ctlTestList.SetTestActive(m_strReg);
	
	return;
}

void CConfigurationDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);
	
	if(bShow)
	{
		if(m_bPropBarVisible && m_pPropertiesBar)
			m_pPropertiesBar->m_pDockSite->ShowControlBar(m_pPropertiesBar,TRUE,TRUE);
	}
	else
	if(m_pPropertiesBar)
	{
		m_bPropBarVisible = m_pPropertiesBar->IsWindowVisible();
		m_pPropertiesBar->m_pDockSite->ShowControlBar(m_pPropertiesBar,FALSE,TRUE);
	}

	if(m_pToolbar)
		m_pToolbar->m_pDockSite->ShowControlBar(m_pToolbar,bShow,TRUE);

	((CFrameWnd*)AfxGetMainWnd())->RecalcLayout(FALSE);
}

void CConfigurationDlg::StopActiveTest()
{
	m_bTestActive=FALSE;
	m_strReg=""; 
	if(m_pPropertiesBar)
	{
		m_pPropertiesBar->StopActiveTest();
	}

	m_ctlTestList.SetTestInactive();
	
	m_ctlTestList.Refresh();
	m_ctlTestList.RedrawWindow();
	return;
}

void CConfigurationDlg::OnSglclkTestList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_GRIDVIEW *pNMGV = (NM_GRIDVIEW*)pNMHDR;
	*pResult = 0;
	
	if(m_pPropertiesBar)
		m_pPropertiesBar->Edit(pNMGV->iRow,pNMGV->iColumn);
	return;
}

CString CConfigurationDlg::GetStrReg()
{
	return m_strReg;
}

void CConfigurationDlg::OnToggleShowruns() 
{
	if(!m_pPropertiesBar)
		return;

	BOOL bShow = !m_pPropertiesBar->IsVisible();
	if(m_pPropertiesBar)
		m_pPropertiesBar->m_pDockSite->ShowControlBar(m_pPropertiesBar,bShow,TRUE);

	((CFrameWnd*)AfxGetMainWnd())->RecalcLayout();//was false
}

void CConfigurationDlg::OnUpdateToggleShowruns(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( IsWindowVisible());
	if(m_pPropertiesBar)
		pCmdUI->SetCheck(m_pPropertiesBar->IsWindowVisible() );
}

void CConfigurationDlg::OnUpdateDeleteselection(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_ctlTestList.GetSelectedCount()?TRUE:FALSE);	
}

void CConfigurationDlg::Activate()
{
	SetTimer(IDT_ACTIVATE,50,NULL);
}

void CConfigurationDlg::OnDeleteselection() 
{
	m_ctlTestList.DeleteSelection();
}

void CConfigurationDlg::OnTimer(UINT nIDEvent) 
{
	switch(nIDEvent)
	{
	case IDT_ACTIVATE:
	{
		KillTimer(IDT_ACTIVATE);
		m_ctlTestList.SetDocument(m_pDocument);
		m_ctlTestList.SetTestFile(m_pTestFile);
		m_strDescription = m_pTestFile->GetDescription();
		m_ctlTestList.Refresh();

		m_pPropertiesBar->Activate(m_pTestFile,m_ctlTestList.GetSafeHwnd(),m_strReg);
		UpdateData(FALSE);

		if(m_bPropBarVisible && m_pPropertiesBar && IsWindowVisible())
			m_pPropertiesBar->m_pDockSite->ShowControlBar(m_pPropertiesBar,TRUE,TRUE);

		if(m_pToolbar && IsWindowVisible())
			m_pToolbar->m_pDockSite->ShowControlBar(m_pToolbar,TRUE,TRUE);

	}
	break;
	}
	
	CDialog::OnTimer(nIDEvent);
}

BOOL CConfigurationDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{	
	pHelpInfo->dwContextId = HIDD_CONFIGURATION;
	
	return CDialog::OnHelpInfo(pHelpInfo);
}

BOOL CConfigurationDlg::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	return CWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}
