/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// GraphWnd.cpp : implementation file
//

#include "stdafx.h"
#include "GraphWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphWnd

IMPLEMENT_DYNCREATE(CGraphWnd, CMDIChildWnd)

CGraphWnd::CGraphWnd()
{
	m_pGraph = NULL;
	m_bActive = false;
	m_bCloseEnabled = true;
}

CGraphWnd::~CGraphWnd()
{
	delete m_pGraph;
	m_pGraph = NULL;
}

BEGIN_MESSAGE_MAP(CGraphWnd, CMDIChildWnd)
	//{{AFX_MSG_MAP(CGraphWnd)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_SYSCOMMAND()
	ON_WM_ERASEBKGND()
	ON_WM_NCPAINT()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_INITMENU()
	//}}AFX_MSG_MAP
	ON_WM_CHILDACTIVATE()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphWnd message handlers
void CGraphWnd::SetGraph(CWnd *pGraph)
{
	m_pGraph = pGraph;
}

void CGraphWnd::OnSize(UINT nType, int cx, int cy) 
{
	CMDIChildWnd::OnSize(nType, cx, cy);
	if(m_pGraph)
		m_pGraph->MoveWindow(0,0,cx,cy);
}


void CGraphWnd::OnDestroy() 
{
	::SendMessage(m_hWndNotify,WMU_MDISTATECHANGED,(WPARAM)m_hWnd,enDestroyed);
	
	CMDIChildWnd::OnDestroy();
		
	if(m_pGraph)
	{
		m_pGraph->DestroyWindow();
		delete m_pGraph;
		m_pGraph = NULL;
	}
}

void CGraphWnd::OnSysCommand(UINT nID, LPARAM lParam) 
{
	UINT nItemID = (nID & 0xFFF0);

	switch (nItemID)
	{
	case SC_MAXIMIZE:
	{
		if(GetWindowLong(m_hWnd,GWL_STYLE) & WS_MINIMIZE)
		{
			MDIRestore();
		}

		CRect rcClient;
		GetParent()->GetClientRect(&rcClient);
		MoveWindow(rcClient);
	}
	return;
	case SC_CLOSE:
	{
		if(!m_bCloseEnabled)
			return;
	}
	break;
	}
	
	CMDIChildWnd::OnSysCommand(nID, lParam);
}

BOOL CGraphWnd::OnEraseBkgnd(CDC* pDC) 
{
	return true;	
//	return CMDIChildWnd::OnEraseBkgnd(pDC);
}

void CGraphWnd::OnNcPaint() 
{
	CMDIChildWnd::OnNcPaint();

	if(m_bActive)
	{
		CRect rcWnd;
		GetWindowRect(&rcWnd);
		rcWnd.OffsetRect(-rcWnd.left,-rcWnd.top);
	
		//Redraw the caption in as active
		
		BOOL bMinimized = (GetWindowLong(m_hWnd,GWL_STYLE) & WS_MINIMIZE);
		
		CRect rcCaption(GetSystemMetrics(SM_CXFRAME),
						GetSystemMetrics(SM_CYFRAME),
						rcWnd.right-(GetSystemMetrics(SM_CXFRAME)+GetSystemMetrics(SM_CXSIZE)*3)+(bMinimized ? 3:2),
						GetSystemMetrics(SM_CYFRAME) + GetSystemMetrics(SM_CYCAPTION)-(bMinimized ? 2:1));

		CWindowDC dc(this);
		DrawCaption(m_hWnd,(HDC)dc,&rcCaption,DC_ACTIVE|DC_TEXT);

		//Fill in around the system buttons
		CRect rc(rcCaption.right,
				 rcCaption.top,
				 rcWnd.right-GetSystemMetrics(SM_CXFRAME),
				 rcCaption.top + (bMinimized ? 1:2));
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.bottom-2;
		rc.bottom = rcCaption.bottom;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.top;
		rc.left = rc.right - 2;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));
		
		rc.right = rcWnd.right-GetSystemMetrics(SM_CXFRAME)-GetSystemMetrics(SM_CXSIZE);
		rc.left = rc.right-2;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		//Fill in around the icon
		rc.SetRect(rcWnd.left+GetSystemMetrics(SM_CXFRAME),
				 rcCaption.top,
				 rcWnd.left+GetSystemMetrics(SM_CXFRAME)+GetSystemMetrics(SM_CXSMICON)+1,
				 rcCaption.top + (bMinimized ? 2:3));
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.bottom-4;
		rc.bottom = rcCaption.bottom;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.top = rcCaption.top;
		rc.right = rc.left + (bMinimized ? 2:3);
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));

		rc.left = rcWnd.left+GetSystemMetrics(SM_CXFRAME)+GetSystemMetrics(SM_CXSMICON)+(bMinimized ? 0:1);
		rc.right = rc.left + 2;
		dc.FillSolidRect(rc,GetSysColor(COLOR_ACTIVECAPTION));
	}
}

void CGraphWnd::OnNcLButtonDown(UINT nHitTest, CPoint point) 
{
//	PostMessage(WM_NCPAINT);
	CMDIChildWnd::OnNcLButtonDown(nHitTest, point);
}

void CGraphWnd::OnChildActivate()
{
	CWnd *pParent = GetMDIFrame()->GetWindow(GW_CHILD);
	CWnd *pChild = pParent->GetWindow(GW_CHILD);
	CWnd *pChildFrame = pChild;
	while(pChild)
	{
		if(pChild != this)
		{
			if(((CGraphWnd*)pChild)->m_bActive)
			{
				((CGraphWnd*)pChild)->m_bActive = false;
				pChild->RedrawWindow(NULL,NULL,RDW_FRAME|RDW_INVALIDATE|RDW_NOERASE);

			}
		}
		pChild = pChild->GetNextWindow();
	}

	if(!m_bActive)
	{
		m_bActive = true;
		::SendMessage(m_hWndNotify,WMU_MDISTATECHANGED,(WPARAM)m_hWnd,enActive);
		pChildFrame->RedrawWindow(NULL,NULL,RDW_FRAME|RDW_INVALIDATE|RDW_NOERASE);//|RDW_ALLCHILDREN);
	}
}

void CGraphWnd::OnInitMenu(CMenu* pMenu) 
{
	CMDIChildWnd::OnInitMenu(pMenu);

	//Disable close button if necessary
	if(pMenu && !m_bCloseEnabled)
	{
		pMenu->EnableMenuItem(SC_CLOSE,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
	}
}
