/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// MonitoringBar.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "MonitoringBar.h"
#include "TestFile.h"
#include "Plugintypes.h"
#include "TestMonitorPlugin.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMonitoringBar dialog

extern HINSTANCE m_hInstance;
extern CPluginManager* m_pPluginManager;
#define IDC_MONITOR_TREE 1050

CMonitoringBar::CMonitoringBar(CWnd* pParent /*=NULL*/):
	m_hWndNotify(NULL),m_pTestFile(NULL),m_bEnabled(FALSE)
{
	//{{AFX_DATA_INIT(CMonitoringBar)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CMonitoringBar::DoDataExchange(CDataExchange* pDX)
{
	CSizingControlBarG::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitoringBar)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitoringBar, CSizingControlBarG)
	//{{AFX_MSG_MAP(CMonitoringBar)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_NOTIFY(NM_CLICK, IDC_MONITOR_TREE, OnClickMonitorTree)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitoringBar message handlers

int CMonitoringBar::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSizingControlBarG::OnCreate(lpCreateStruct) == -1)
		return -1;
		
	m_fontStatus.CreateStockObject(DEFAULT_GUI_FONT );


	ModifyStyle(WS_CLIPCHILDREN,0);
	
	m_ctlTestStatusBox.Create("Test Status",WS_CHILD|WS_VISIBLE|BS_GROUPBOX,CRect(0,0,10,10),this,0);
	m_ctlTestStatusBox.SetFont(&m_fontStatus);
	
	m_ctlTestStatus.Create("INACTIVE",WS_CHILD|WS_VISIBLE,CRect(0,0,0,0),this,0);
	m_ctlTestStatus.SetFont(&m_fontStatus);

	AfxSetResourceHandle(m_hInstance);
	m_ImageList.Create(IDB_TRTREE_IMAGES,16,0,RGB(0,128,128));
	AfxSetResourceHandle(AfxGetInstanceHandle());

	m_ctlMonitorTree.Create(WS_CHILD|WS_VISIBLE|WS_BORDER|TVS_HASLINES|TVS_LINESATROOT|TVS_HASBUTTONS,CRect(0,0,0,0),this,IDC_MONITOR_TREE);
	m_ctlMonitorTree.SetImageList(&m_ImageList,TVSIL_STATE);
	m_ctlMonitorTree.SetImageList(&m_ImageList,TVSIL_NORMAL);
	
	m_ImageList.SetOverlayImage(1,1);
	m_ImageList.SetOverlayImage(2,2);

	return 0;
}

void CMonitoringBar::OnSize(UINT nType, int cx, int cy) 
{
	CSizingControlBarG::OnSize(nType, cx, cy);
	
	if(m_ctlTestStatusBox.GetSafeHwnd())
		m_ctlTestStatusBox.MoveWindow(0,0,cx,40);

	if(m_ctlTestStatus.GetSafeHwnd())
		m_ctlTestStatus.MoveWindow(10,18,cx-20,20);

	if(m_ctlMonitorTree.GetSafeHwnd())
		m_ctlMonitorTree.MoveWindow(0,43,cx,cy-43);
}

void CMonitoringBar::SaveBranchState(HTREEITEM hBranch,const CString& strFolder,CMap <CString,LPCTSTR,UINT,UINT>& mapItemToState)
{
	CString strRun = strFolder + m_ctlMonitorTree.GetItemText(hBranch);
	UINT uiState = m_ctlMonitorTree.GetItemState(hBranch,0xffffffff);
	mapItemToState.SetAt(strRun,uiState);

	if(m_ctlMonitorTree.ItemHasChildren(hBranch))
	{
		HTREEITEM hNextItem;
		HTREEITEM hChildItem = m_ctlMonitorTree.GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = m_ctlMonitorTree.GetNextItem(hChildItem, TVGN_NEXT);
			SaveBranchState(hChildItem,strRun,mapItemToState);
			hChildItem = hNextItem;
		}
	}
}

void CMonitoringBar::RestoreBranchState(HTREEITEM hBranch,const CString& strFolder,CMap <CString,LPCTSTR,UINT,UINT>& mapItemToState)
{
	UINT uiState;
	CString strRun = strFolder + m_ctlMonitorTree.GetItemText(hBranch);
	if(mapItemToState.Lookup(strRun,uiState))
	{
		if(uiState & TVIS_EXPANDED)
			m_ctlMonitorTree.Expand(hBranch,TVE_EXPAND);
		if(uiState & TVIS_SELECTED)
			m_ctlMonitorTree.Select(hBranch,TVGN_CARET);

		m_ctlMonitorTree.SetItemState(hBranch,uiState,TVIS_OVERLAYMASK);
	}
		
	if(m_ctlMonitorTree.ItemHasChildren(hBranch))
	{
		HTREEITEM hNextItem;
		HTREEITEM hChildItem = m_ctlMonitorTree.GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = m_ctlMonitorTree.GetNextItem(hChildItem, TVGN_NEXT);
			RestoreBranchState(hChildItem,strRun,mapItemToState);
			hChildItem = hNextItem ;
		}
	}
}

void CMonitoringBar::OnClickMonitorTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;

	if(!m_bEnabled)
		return;

	CPoint ptCur;
	GetCursorPos(&ptCur);
	m_ctlMonitorTree.ScreenToClient(&ptCur);

	UINT uiFlags;
	HTREEITEM hHitItem = m_ctlMonitorTree.HitTest(ptCur,&uiFlags);
	if(!hHitItem || !(uiFlags & TVHT_ONITEM))
		return;

	NM_MONITOR mon;
	if(!m_mapItemToType.Lookup(hHitItem,mon))
		return;

	//Crude locks stops a second click occuring while
	//first is being processed
	static bool bLocked = false;
	if(bLocked)
		return;
	bLocked = true;

	UINT uiState = m_ctlMonitorTree.GetItemState(hHitItem,TVIS_OVERLAYMASK);
	BOOL bChecked = (uiState & INDEXTOOVERLAYMASK(2));
	if(bChecked)
		m_ctlMonitorTree.SetItemState(hHitItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
	else
		m_ctlMonitorTree.SetItemState(hHitItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);

	::SendMessage(m_hWndNotify,WMU_MONITOR_SELCHANGED,(WPARAM)&mon,!bChecked);

	bLocked = false;
}

void CMonitoringBar::SetTestFile(CTestFile *pTestFile)
{
	m_pTestFile = pTestFile;
}

void CMonitoringBar::Refresh()
{
	if(!m_pTestFile)
		return;

	if(!m_bEnabled)
	{
		//if tree is not enabled hide it
		m_ctlMonitorTree.DeleteAllItems();
		return;
	}

	CMap <CString,LPCTSTR,UINT,UINT> mapItemToState;
	
	HTREEITEM hItem = m_ctlMonitorTree.GetRootItem();
	while(hItem)
	{
		SaveBranchState(hItem,"",mapItemToState);
		hItem = m_ctlMonitorTree.GetNextSiblingItem(hItem);
	}

	m_ctlMonitorTree.DeleteAllItems();
	m_mapItemToType.RemoveAll();

	NM_MONITOR mon;

	hItem = m_ctlMonitorTree.InsertItem("Summary",1,1);
	mon.nTaskGroup = -1;
	mon.strScreenName = "Summary";
	mon.strTargetType = "";
	m_mapItemToType.SetAt(hItem,mon);

	hItem = m_ctlMonitorTree.InsertItem("Total Active Users",1,1);
	mon.nTaskGroup = -1;
	mon.strScreenName = "Total Active Users";
	mon.strTargetType = "";
	m_mapItemToType.SetAt(hItem,mon);

	hItem = m_ctlMonitorTree.InsertItem("Error Log",1,1);
	mon.nTaskGroup = -1;
	mon.strScreenName = "Error Log";
	mon.strTargetType = "";
	m_mapItemToType.SetAt(hItem,mon);


	int nSBSGroup = m_pTestFile->GetStepByStepTaskGroup();

	for (int nGroup =0; nGroup<m_pTestFile->GetNumTaskGroups(); nGroup++)
	{
		if((nSBSGroup != -1) && ((nGroup != nSBSGroup)&&(m_pTestFile->GetTaskType(nGroup,0)==CTestFile::enScript)))
			continue;

		HTREEITEM hParent = m_ctlMonitorTree.InsertItem(m_pTestFile->GetTaskGroupName(nGroup),0,0);

		if(!m_pTestFile->GetNumTasks(nGroup))
			continue;

		CString strActiveTarget = m_pTestFile->GetTaskTargetType(nGroup,0);

		CStringArray arrPlugs;
		m_pPluginManager->ListPluginsByType(TEST_MONITOR_PLUGIN, arrPlugs);

		CStringArray arrScreens;
		for(int nPlugin =0; nPlugin<arrPlugs.GetSize(); nPlugin++)
		{
			CTestMonitorPlugin plug(arrPlugs.GetAt(nPlugin), m_pPluginManager);
			if(plug.IsSupportedExtension(strActiveTarget))
			{
				plug.GetAvailableScreens(m_pTestFile, nGroup, arrScreens);
			}
		}

		HTREEITEM hItem;
		for (int nView = 0; nView<arrScreens.GetSize(); nView++)
		{
			hItem = m_ctlMonitorTree.InsertItem(arrScreens.GetAt(nView),1,1,hParent);

			mon.strTargetType = strActiveTarget;
			mon.strScreenName = arrScreens.GetAt(nView);
			mon.nTaskGroup = nGroup;
			m_mapItemToType.SetAt(hItem,mon);
		}
	}

	hItem = m_ctlMonitorTree.GetRootItem();
	while(hItem)
	{
		RestoreBranchState(hItem,"",mapItemToState);
		hItem = m_ctlMonitorTree.GetNextSiblingItem(hItem);
	}

}

void CMonitoringBar::SetCheck(NM_MONITOR& monInfo,BOOL bCheck)
{
	NM_MONITOR mon;
	HTREEITEM hItem;
	POSITION pos = m_mapItemToType.GetStartPosition();
	while(pos)
	{
		m_mapItemToType.GetNextAssoc(pos,hItem,mon);

		if(mon == monInfo)
		{
			if(bCheck)
				m_ctlMonitorTree.SetItemState(hItem,INDEXTOOVERLAYMASK(2),TVIS_OVERLAYMASK);
			else
				m_ctlMonitorTree.SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
			return;
		}
	}
}

void CMonitoringBar::ClearAllChecks()
{
	HTREEITEM hItem = m_ctlMonitorTree.GetRootItem();
	while(hItem)
	{
		ClearChecksOnBranch(hItem);
		hItem = m_ctlMonitorTree.GetNextSiblingItem(hItem);
	}
}

void CMonitoringBar::ClearChecksOnBranch(HTREEITEM hBranch)
{
	NM_MONITOR mon;
	if(m_mapItemToType.Lookup(hBranch,mon))
		m_ctlMonitorTree.SetItemState(hBranch,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
	
	HTREEITEM hItem = m_ctlMonitorTree.GetChildItem(hBranch);
	while(hItem)
	{
		ClearChecksOnBranch(hItem);
		hItem = m_ctlMonitorTree.GetNextSiblingItem(hItem);
	}
}

void CMonitoringBar::SetTestStatus(const CString& strStatus)
{
	if(m_ctlTestStatus.GetSafeHwnd())
		m_ctlTestStatus.SetWindowText(strStatus);
}



BOOL CMonitoringBar::OnEraseBkgnd(CDC* pDC) 
{
	CRect rc;
	m_ctlTestStatus.GetWindowRect(&rc);
	ScreenToClient(&rc);
	pDC->ExcludeClipRect(rc);

	m_ctlMonitorTree.GetWindowRect(&rc);
	ScreenToClient(&rc);
	pDC->ExcludeClipRect(rc);
	
	return CSizingControlBarG::OnEraseBkgnd(pDC);
}

void CMonitoringBar::EnableTree(BOOL bEnable)
{
	m_bEnabled = bEnable;
	Refresh();
}