

#include "stdafx.h"
#include <afxdllx.h>
#include "PluginTypes.h"
#include "resource.h"       // main symbols
#include "ResultsDlg.h"
#include "TestFile.h"
#include "PluginManager.h"
#include "TestRunBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CResultsDlg *m_pResultsDlg = NULL;

CToolBar *m_pToolbar = NULL;
CTestRunBar *m_pTestRunBar = NULL;

CTestFile *m_pTestFile = NULL;
CDocument *m_pDocument = NULL;
CPluginManager* m_pPluginManager = NULL;

static AFX_EXTENSION_MODULE CResultsTabDll = { NULL, NULL };

HINSTANCE m_hInstance = NULL;
extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	m_hInstance = hInstance;
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("ResultsTabDll.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CResultsTabDll, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.

		new CDynLinkLibrary(CResultsTabDll);

    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CResultsTabDll.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CResultsTabDll);

		delete m_pResultsDlg;
		m_pResultsDlg = NULL;
		delete m_pTestRunBar;
		m_pTestRunBar = NULL;
		delete m_pToolbar;
		m_pToolbar = NULL;
    }

    return 1;   // ok
}

BOOL IsInitialised()
{
	return (m_pResultsDlg && m_pTestRunBar && m_pTestFile);
}

extern "C" AFX_EXT_API
BOOL OpenSTAVerify()
{
	return true;
}

extern "C" AFX_EXT_API
void GetTitle(CString& strTitle)
{
	strTitle = "Results Display Tab";
}

extern "C" AFX_EXT_API
int GetType()
{
	return TEST_PLUGIN_TAB;
}

extern "C" AFX_EXT_API
void GetPackage(CString& strPackage)
{
	strPackage = "RESULTS TAB";
}

extern "C" AFX_EXT_API
void SetTestFile(CTestFile *pTestFile)
{
	m_pTestFile = pTestFile;

	if(IsInitialised())
		m_pResultsDlg->Activate();
}

extern "C" AFX_EXT_API
void SetDocument(CDocument *pDoc)
{
	m_pDocument = pDoc;
}

extern "C" AFX_EXT_API
int GetNumTabs()
{
	return 1;
}

extern "C" AFX_EXT_API
CDialog* CreateTab(int nTab,CWnd *pParent,CString& strTabName)
{
	UINT uiIDD = 0;
	CDialog *pDialog = NULL;
	switch(nTab)
	{
	case 0:
		if(!m_pResultsDlg)
			m_pResultsDlg = new CResultsDlg;
		pDialog = m_pResultsDlg;
		uiIDD = CResultsDlg::IDD;
		strTabName = "Results";
	break;
	default:
		ASSERT(FALSE);
	break;
	}

	AfxSetResourceHandle(m_hInstance);
	if(!pDialog || !pDialog->Create(uiIDD,pParent))
	{
		AfxSetResourceHandle(AfxGetInstanceHandle());
		return NULL;
	}
	AfxSetResourceHandle(AfxGetInstanceHandle());

	if(IsInitialised())
		m_pResultsDlg->Activate();
	return pDialog;
}

extern "C" AFX_EXT_API
void GetTabImages(CBitmap& bmpTabs)
{
	AfxSetResourceHandle(m_hInstance);
	bmpTabs.LoadBitmap(IDB_TAB_IMAGES);
	AfxSetResourceHandle(AfxGetInstanceHandle());
}

/*extern "C" AFX_EXT_API
void SetCorbaInfo(CCORBAInfo *pCorbaInfo)
{
}*/

extern "C" AFX_EXT_API
BOOL LoadToolbars(CFrameWnd* pWndFrame,CFrameWnd *pOwner)
{
	if(!m_pTestRunBar)
		m_pTestRunBar = new CTestRunBar;
	else
		m_pTestRunBar->DestroyWindow();

	m_pTestRunBar->SetOwner(pOwner);
	AfxSetResourceHandle(m_hInstance);
	if (!m_pTestRunBar->Create(_T("Test Runs"), pWndFrame, 132))
	{
	    TRACE0("Failed to create Test Run bar\n");
	    return false;
	}
	AfxSetResourceHandle(AfxGetInstanceHandle());
	m_pTestRunBar->SetBarStyle(m_pTestRunBar->GetBarStyle() |
	    CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_pTestRunBar->EnableDocking(CBRS_ALIGN_ANY);

	pWndFrame->DockControlBar(m_pTestRunBar,AFX_IDW_DOCKBAR_RIGHT);
	pWndFrame->ShowControlBar(m_pTestRunBar,false,false);
	
	m_pToolbar = new CToolBar;
	m_pToolbar->SetOwner(pOwner);

	AfxSetResourceHandle(m_hInstance);
	// Create toolbar on client's frame window
	if (!m_pToolbar->CreateEx(pWndFrame, TBSTYLE_FLAT,WS_CHILD | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_pToolbar->LoadToolBar(IDR_RESULTS_TOOLBAR))
	{
		TRACE0("Failed to create toolbar\n");
		return FALSE;
	}

	m_pToolbar->EnableDocking(CBRS_ALIGN_TOP);
	pWndFrame->DockControlBar(m_pToolbar,AFX_IDW_DOCKBAR_TOP);

	AfxSetResourceHandle(AfxGetInstanceHandle());

	//This code will dock the new tool bar to the right of the plugins ones
	CDockBar *pDockBar = (CDockBar*)pWndFrame->GetControlBar(AFX_IDW_DOCKBAR_TOP);
	if(pDockBar)
	{
		for(int nBar = 0;nBar<pDockBar->m_arrBars.GetSize();nBar++)
		{
			CToolBar *pToolBar = (CToolBar*)pDockBar->m_arrBars[nBar];
			
			if(pToolBar == m_pToolbar)
				pDockBar->m_arrBars.RemoveAt(nBar--);
			else
			if(pToolBar)
			{
				while(pDockBar->m_arrBars[++nBar]);
				pDockBar->m_arrBars.InsertAt(nBar,m_pToolbar);
			}
		}
	}

	if(IsInitialised())
		m_pResultsDlg->Activate();

	return true;
}	

extern "C" AFX_EXT_API
void SetPluginManager(CPluginManager* pPluginManager)
{
	m_pPluginManager = pPluginManager;
}

extern "C" AFX_EXT_API
void SetTestActive(BOOL bActive,const CString& strReg)
{

}

extern "C" AFX_EXT_API
void LoadMenus(HMENU hMenu)
{
	AfxSetResourceHandle(m_hInstance);

	CMenu *pMenu = CMenu::FromHandle(hMenu);
	
	UINT uiWindowPos = -1;
	UINT uiCount = pMenu->GetMenuItemCount();

	CString strItem;
	//Look for Window menu
	for(UINT uiItem=0;uiItem<uiCount;uiItem++)
	{
		pMenu->GetMenuString(uiItem,strItem,MF_BYPOSITION);
		if(strItem == "&Window")
		{
			uiWindowPos = uiItem;
			break;
		}
	}

	if(uiWindowPos == -1)
	{
		CMenu menu;
		menu.CreateMenu();
		menu.AppendMenu(MF_STRING,ID_CASCADE,"&Cascade");
		menu.AppendMenu(MF_STRING,ID_TILEHORIZONTALLY,"Tile &Horizontally");
		menu.AppendMenu(MF_STRING,ID_TILEVERTICALLY,"Tile &Vertically");
		menu.AppendMenu(MF_STRING,ID_CLOSEALL,"Close &All");

		pMenu->InsertMenu(uiCount-2,MF_BYPOSITION|MF_POPUP,(UINT)menu.Detach(),"&Window");
	}
	else
	{
		CMenu *pSubMenu = pMenu->GetSubMenu(uiWindowPos);
		
		if(pSubMenu->GetMenuState(ID_CASCADE,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_CASCADE,"&Cascade");
		if(pSubMenu->GetMenuState(ID_TILEHORIZONTALLY,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_TILEHORIZONTALLY,"Tile &Horizontally");
		if(pSubMenu->GetMenuState(ID_TILEVERTICALLY,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_TILEVERTICALLY,"Tile &Vertically");
		if(pSubMenu->GetMenuState(ID_CLOSEALL,MF_BYCOMMAND) == 0xFFFFFFFF)
			pSubMenu->AppendMenu(MF_STRING,ID_CLOSEALL,"Close &All");
		pSubMenu->Detach();
	}
	
	pMenu->Detach();

	AfxSetResourceHandle(AfxGetInstanceHandle());
}

extern "C" AFX_EXT_API
BOOL DisplayInStepByStepMode(int nTab)
{
	return true;
}