/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#ifndef __MAIN_STRUCTURE_HPP__
#define __MAIN_STRUCTURE_HPP__

#include <string>
#include <map>
#include "mdbgetopt.h"
#include "ConfigFile.hpp"
#include "TaskGroup.hpp"
#include "SharedPointer.hpp"
#ifndef CYRANO_TP_STANDALONE
#   include "InjectorControl.h"
#   include "TimeService.h"
#   include "TestExecuter.h"
#endif

#ifndef CYRANO_TP_STANDALONE
#   include "OSTAORB/OSTAORB.h"
#else
#   include <CECSACORBA.h>
#endif

#include "AsyncVirtualUserController.h"
#include "LogTrace.hpp"

#ifdef __DLLSA__
struct stUsersParams
{
    stUsersParams() : pThreadPoolController(NULL) {}
    string zOdl;
    string zTof;
    CAsyncVirtualUserController *pThreadPoolController;
private:
    //// Forbidden:
    stUsersParams(const stUsersParams&);
    operator=(const stUsersParams&);
};
#endif

typedef ObjectPool<CAsyncSecureSocket> SecureSocketPool;

class CMainExecuterStructure : public SecureSocketPool::PoolObjectEnumerator
{
public:
    CMainExecuterStructure();
    ~CMainExecuterStructure();

#ifdef __DLLSA__
    BOOL    Init(HWND hWnd, char *TofFile, char *OdlDir);
#else
    BOOL    Init(int argc, char* argv[]);
#endif

    BOOL    Run();
#ifndef CYRANO_TP_STANDALONE
    BOOL    IsStepByStepMode() const {return m_bStepByStep;}
    BOOL    StepByStep_NotifyNewRequest(const char* zScriptName, long lLineNumber, long lTaskGroupID,
                DWORD dwUser, long lConId, const char* zRequest);
    void    StepByStep_NotifyComplete(long lTaskGroupID, DWORD dwUser, long lConId,
                BOOL bIsBinaryData, const char* zHeaders, const char* zBody,
                long lBodyLength);
    std::string GetTestName() const {return m_zTestName;}
#endif
    static  DWORD WINAPI StartTaskGroupThreadFunction(LPVOID pContext);
    BOOL    StartTaskGroup(SharedPtr<TaskGroupDefinition>& pTaskGroup);
    void    StopTaskGroup(int iTaskGroupNumber);
#ifndef CYRANO_TP_STANDALONE
    BOOL    GetStatsSnapShot(CAsyncVirtualUserController::TestExecuterSnapShot& sn);
    BOOL    GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots);
#endif
    void    GetTestExecuterVUs(long & lActiveVUs, long & lTotalVUs) const;
    BOOL    Complete();
    void    Stop();
    void    GetCachedTestVUCount(long & lTestActiveVUCount, long &lTestTotalVUCount) const;
    void    SetCachedTestVUCount(long lTestActiveVUCount, long lTestTotalVUCount);

    DWORD   GetProcessId()              const   { return m_dwProcessId; }
    const std::string &GetCurrentDir()  const   { return m_zCurrentDir; }
    const std::string &GetExeName()     const   { return m_zExeName; }

#ifdef __DLLSA__
    const std::string &GetTofFileDir()          const   { return m_zTofFileDir; }
    const std::string &GetTofFileNameAndPath()  const   { return m_zTofFileNameAndPath; }
#else
    InjectorControl_ptr GetInjectorControl()    const;
    const char* GetRegisterName()               const;
    const Cmdbgetopt &GetCmdLineOptions()       const {return m_cmdLine;}
    long  GetTimeOffset()                       const {return m_lOffset;}
    void  SetTimeOffset(long lOffset);
    std::string GetRepositoryHost()             const {return m_strRepositoryNode;}
    std::string GetTestRun()                    const {return m_strTestRun;}
    void NotifyTEStopping();
#endif
    CAsyncVirtualUserController *GetThreadPoolController()  { return m_pThreadPoolController; }
    void    LogError(const char* zErrMsg) const;
    void    TraceMsg(const std::string &zMsg, const char* zLevel = NULL) const;

protected:
    BOOL    CommonInit();
    void    UnInit();
    void    CreateTrace(int nTraceVal, LPCSTR zTraceFileName);

#ifdef USE_PERF_DATA
    enum ePrfCtrsAction { PrfCtrsUse, PrfCtrsInstall, PrfCtrsUninstall };
    BOOL    InitPerfCounters(ePrfCtrsAction installType) const;
    void    ReleasePerfCounters() const;
#endif

    void    EvaluateTPParams(DWORD &dwThreadPoolSize, DWORD &dwNbConcurrentThreads, DWORD &dwInitialVUCount, DWORD &dwVUCountGrowBy, DWORD &dwBrowserParallelism);
    BOOL    RunThreadPoolController();
    BOOL    RegisterCurrentDebugExecuter();

#ifndef __DLLSA__
    BOOL    ParseCommandLine(int argc, char* argv[]);

    BOOL    InitialiseCORBA(int argc, char* argv[]);
    BOOL    ReleaseCORBA();
#endif

#ifdef CYRANO_TP_STANDALONE
    BOOL    StartUsersCreationThread();
    static  void CreateUsers(void *pVoidParams);
#endif

#ifdef __DLLSA__
    BOOL    StartThreadPoolHandlerThread();
#endif

    static  unsigned ThreadPoolHandler(void *pVoidParams);

protected:
    TScriptRule    *m_pRule;

    std::string m_zExeName;
#ifndef __DLLSA__
    BOOL        m_bStepByStep;
    BOOL        m_bFirstTaskGroup;          // TRUE if this is the first task group.
    long        m_lOffset;                  // Time offset with Repository host.
    std::string m_zTestName;
    std::string m_zUnderscoredHostIpAddress;
    std::string m_zStatBaseFileName;        // Name of the files that will store the statistics. Extension must be added.
    std::string m_strRepositoryNode;        // Name of repository host.
    std::string m_strTestRun;               // Test run string.
    Cmdbgetopt  m_cmdLine;                  // Command line options.

    std::vector<HANDLE> m_otherThreads;     // Spawned threads to wait for on completion.
    CFastLock   m_otherThreadsLock;         // Associated lock.
#else
    std::string m_zTofFileNameAndPath;
    std::string m_zTofFileDir;
    HWND        m_hWnd;
    HANDLE      m_hThreadPoolHandlerThread;
#endif

    CConfigFile m_configFile;

    DWORD       m_dwProcessId;

    std::string     m_zCurrentDir;

#ifdef __DLLSA__
    stUsersParams   m_usersStartParams;
#endif

    CORBAServer                    *m_pServer;
    CAsyncVirtualUserController    *m_pThreadPoolController;
    long                            m_lCachedTestActiveVUCount;
    long                            m_lCachedTestTotalVUCount;

    // Socket pool:
    SecureSocketPool                m_socketPool;
    HANDLE                          m_hSocketPoolThread;
    static DWORD WINAPI StaticSocketPoolThreadFunction(LPVOID pContext);
    DWORD SocketPoolThreadFunction();
    HANDLE                          m_hSocketPoolEvent; // To stop the thread.
    long                            m_lSocketThreadSleepMs;
    virtual BOOL NextObject(CAsyncSecureSocket& socket); // From SecureSocketPool::PoolObjectEnumerator
    void StopSocketPoolThread();

    CTrace *m_pTrace;
    LogTrace *m_pLogTrace;
    BOOL    m_bInitialised;
};

#endif
