/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"

#include "TaskGroup.hpp"


///////////////////// TaskGroupDefinition //////////////////////////

TaskGroupDefinition::TaskGroupDefinition()
    : m_iTaskGroupNumber(0), m_lTaskGroupDuration(0), m_lNumVirtualUsers(0),
      m_lNumTaskGroupIterations(0), m_iInterBatchDelay(0), m_lVUsPerBatch(0),
      m_iBatchRampUpTime(0), m_iMaxVUsHttpRes(0), m_iMaxVUsTimerRes(0),
      m_lCurVirtualUsers(0), m_lDoneVirtualUsers(0), m_qwEndTime(0),
      m_bTaskGroupEnabled(TRUE), m_bUsePageTimers(FALSE),
      m_lVirtualUserID(0),m_iOffsetVUsHttpRes(1), m_iOffsetVUsTimerRes(1)
{
}

TaskGroupDefinition::TaskGroupDefinition(
    int iTaskGroupNumber, long lTaskGroupDuration,
    long lNumVirtualUsers, long lNumTaskGroupIterations,
    int  iInterBatchDelay, long lVUsPerBatch, int iBatchRampUpTime,
    int  iMaxVUsHttpRes, int  iMaxVUsTimerRes, BOOL m_bUsePageTimers)
    : m_iTaskGroupNumber(iTaskGroupNumber), m_lTaskGroupDuration(lTaskGroupDuration),
      m_lNumVirtualUsers(lNumVirtualUsers), m_lNumTaskGroupIterations(lNumTaskGroupIterations),
      m_iInterBatchDelay(iInterBatchDelay), m_lVUsPerBatch(lVUsPerBatch),
      m_iBatchRampUpTime(iBatchRampUpTime), m_iMaxVUsHttpRes(iMaxVUsHttpRes), 
	  m_iMaxVUsTimerRes(iMaxVUsTimerRes),
      m_lCurVirtualUsers(0),                m_lDoneVirtualUsers(0),
      m_qwEndTime(0),                       m_bTaskGroupEnabled(TRUE),
      m_bUsePageTimers(m_bUsePageTimers),   m_lVirtualUserID(0),
      m_iOffsetVUsHttpRes(1), m_iOffsetVUsTimerRes(1)
{
    if (m_iMaxVUsHttpRes > 1 && lNumVirtualUsers > 1)
	    m_iOffsetVUsHttpRes = (lNumVirtualUsers - 1)/ (m_iMaxVUsHttpRes - 1);

    if (m_iMaxVUsTimerRes > 1 && lNumVirtualUsers > 1)
    	m_iOffsetVUsTimerRes = (lNumVirtualUsers - 1)/ (m_iMaxVUsTimerRes  - 1);
}

TaskGroupDefinition::TaskGroupDefinition(const TaskGroupDefinition& taskGroup)
    : m_iTaskGroupNumber(taskGroup.m_iTaskGroupNumber), m_lTaskGroupDuration(taskGroup.m_lTaskGroupDuration),
      m_lNumVirtualUsers(taskGroup.m_lNumVirtualUsers), m_lNumTaskGroupIterations(taskGroup.m_lNumTaskGroupIterations),
      m_iInterBatchDelay(taskGroup.m_iInterBatchDelay), m_lVUsPerBatch(taskGroup.m_lVUsPerBatch),
      m_iBatchRampUpTime(taskGroup.m_iBatchRampUpTime), m_iMaxVUsHttpRes(taskGroup.m_iMaxVUsHttpRes),
	  m_iMaxVUsTimerRes(taskGroup.m_iMaxVUsTimerRes),
      m_tasks(taskGroup.m_tasks),
      m_lCurVirtualUsers(taskGroup.m_lCurVirtualUsers), m_lDoneVirtualUsers(taskGroup.m_lDoneVirtualUsers),
      m_qwEndTime(taskGroup.m_qwEndTime),               m_bTaskGroupEnabled(taskGroup.m_bTaskGroupEnabled),
      m_bUsePageTimers(taskGroup.m_bUsePageTimers),     m_lVirtualUserID(taskGroup.m_lVirtualUserID),
      m_iOffsetVUsHttpRes(taskGroup.m_iOffsetVUsHttpRes), m_iOffsetVUsTimerRes(taskGroup.m_iOffsetVUsTimerRes)

{
}

TaskGroupDefinition& TaskGroupDefinition::operator=(const TaskGroupDefinition& taskGroup)
{
    m_iTaskGroupNumber          = taskGroup.m_iTaskGroupNumber;
    m_bTaskGroupEnabled         = taskGroup.m_bTaskGroupEnabled;
    m_lTaskGroupDuration        = taskGroup.m_lTaskGroupDuration;
    m_qwEndTime                 = taskGroup.m_qwEndTime;
    m_lNumVirtualUsers          = taskGroup.m_lNumVirtualUsers;
    m_lNumTaskGroupIterations   = taskGroup.m_lNumTaskGroupIterations;
    m_iInterBatchDelay          = taskGroup.m_iInterBatchDelay;
    m_lVUsPerBatch              = taskGroup.m_lVUsPerBatch;
    m_iBatchRampUpTime          = taskGroup.m_iBatchRampUpTime;
    m_iMaxVUsHttpRes            = taskGroup.m_iMaxVUsHttpRes;
    m_iMaxVUsTimerRes           = taskGroup.m_iMaxVUsTimerRes;
    m_lCurVirtualUsers          = taskGroup.m_lCurVirtualUsers;
    m_lDoneVirtualUsers         = taskGroup.m_lDoneVirtualUsers;
    m_bUsePageTimers            = taskGroup.m_bUsePageTimers;
    m_tasks                     = taskGroup.m_tasks;
    m_lVirtualUserID            = taskGroup.m_lVirtualUserID;
	m_iOffsetVUsHttpRes			= taskGroup.m_iOffsetVUsHttpRes ;
	m_iOffsetVUsTimerRes		= taskGroup.m_iOffsetVUsTimerRes ;
    return *this;
}


///////////////////// TaskGroupDefinition::TaskDefinition //////////////////////////

TaskGroupDefinition::TaskDefinition::TaskDefinition()
    : m_lIterations(0), m_bFixedDelay(TRUE), m_lDuration(0)
{
    ZeroMemory(&m_delay, sizeof(m_delay));
}

TaskGroupDefinition::TaskDefinition::TaskDefinition(
    const std::string& strScriptName, long lDuration, long lIterations, int iFixedDelay)
    : m_strScriptName(strScriptName), m_lDuration(lDuration), m_lIterations(lIterations),
      m_bFixedDelay(TRUE)
{
    m_delay.m_iFixedDelay = iFixedDelay;
}

TaskGroupDefinition::TaskDefinition::TaskDefinition(
    const std::string& strScriptName, long lDuration, long lIterations, int iMinDelay, int iMaxDelay)
    : m_strScriptName(strScriptName), m_lDuration(lDuration), m_lIterations(lIterations),
      m_bFixedDelay(FALSE)
{
    if(iMinDelay > iMaxDelay)   // Bounds check.
        iMinDelay = iMaxDelay = 0;
    m_delay.m_variableDelay.m_iMinDelay = iMinDelay;
    m_delay.m_variableDelay.m_iMaxDelay = iMaxDelay;
}

TaskGroupDefinition::TaskDefinition::TaskDefinition(
    const TaskGroupDefinition::TaskDefinition& task)
    : m_strScriptName(task.m_strScriptName), m_lDuration(task.m_lDuration),
      m_lIterations(task.m_lIterations), m_bFixedDelay(task.m_bFixedDelay)
{
    CopyMemory(&m_delay, &task.m_delay, sizeof(m_delay));
}

TaskGroupDefinition::TaskDefinition& TaskGroupDefinition::TaskDefinition::operator=(
    const TaskGroupDefinition::TaskDefinition& task)
{
    m_strScriptName = task.m_strScriptName;
    m_lDuration     = task.m_lDuration;
    m_lIterations   = task.m_lIterations;
    m_bFixedDelay   = task.m_bFixedDelay;
    CopyMemory(&m_delay, &task.m_delay, sizeof(m_delay));
    return *this;
}
