/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __SYMBOL_ATTRIBUTES__
#define __SYMBOL_ATTRIBUTES__

#include "TagParsing.hpp"


//// --------------------------------------------------------------------------------
//// Symbol attributes (used only for tokens).
//// --------------------------------------------------------------------------------


template <typename U>
class CSymbolAttributes
{
public:

    //// --------------------------------------------------------------------------------
    //// Construction / Destruction.
    //// --------------------------------------------------------------------------------
    CSymbolAttributes() : eTag_(TAG_NONE) {}
    CSymbolAttributes(const CSymbolAttributes& sa) : eTag_(sa.eTag_), label_(sa.label_) {}

    //// --------------------------------------------------------------------------------
    //// Operators.
    //// --------------------------------------------------------------------------------
    void operator=(const CSymbolAttributes& sa) {eTag_ = sa.eTag_; label_ = sa.label_;}

    //// --------------------------------------------------------------------------------
    //// Manipulation.
    //// --------------------------------------------------------------------------------
    void Clear() {eTag_ = TAG_NONE; label_.pcBegin_ = NULL; label_.ulSize_ = 0;}

    //// --------------------------------------------------------------------------------
    //// Data.
    //// --------------------------------------------------------------------------------
    Tags eTag_;     // If token is a tag.
    struct Label
    {
        // Construction, assignment.
        Label() : pcBegin_(U()), ulSize_(0) {}
        Label(const Label& l) : pcBegin_(l.pcBegin_), ulSize_(l.ulSize_) {}
        void operator=(const Label& l) {pcBegin_ = l.pcBegin_; ulSize_ = l.ulSize_;}

        // Data.
        U pcBegin_;
        unsigned long ulSize_;
    } label_;       // If token is an attribute name or value, a text, a script, a comment.
};

#endif  // __SYMBOL_ATTRIBUTES__
