/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __GATEWAY_LISTEN_CLIENT__
#define __GATEWAY_LISTEN_CLIENT__


//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __LIBRARY__
#  include <library.hpp>
#endif  // __LIBRARY__

#ifndef __GATEWAY__
#  include "gateway.hpp"
#endif  // __GATEWAY__

#ifndef __GATEWAY_PARSE__
#  include "gwparse.hpp"
#endif  // __GATEWAY_PARSE__


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CGGatewayListen //
///////////////////////////////////////////////////////////////////////////////
class CGGatewayListen : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Types.

public:                         // Constructor(s) and destructor.
    CGGatewayListen (CGTask& task) ;
    ~CGGatewayListen () ;

public:                         // Operators.

public:                         // Controls methodes.
    static void                 abort (void*) ;
    void                        init (void) ;
    void                        listenForConnections (void) ;
    void                        newClient (void) ;

public:                         // Get methodes.
    const CGTask&               getTask (void)
    {
        return task_ ;
    } ;
    CGGatewayInfo&              getGatewayInfo (void)
    {
        return gatewayInfo_ ;
    } ;
    CTSocket&                   getSocket (void)
    {
        return listen_ ;
    } ;
    const CTLibrary*            getLibrary (void)
    {
        return pLibrary_ ;
    } ;

    //-----------------------------------------------------------------------------
private:                        // Operators.
    CGGatewayListen&            operator = (const CGGatewayListen&) ;

private:                        // Members.
    CGTask&                     task_ ;
    CGGatewayInfo               gatewayInfo_ ;
    CTSocket                    listen_ ;
    CTLibrary*                  pLibrary_ ;

    //-----------------------------------------------------------------------------
}

; // CGGatewayListen

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CGGatewayListen //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CGGatewayClient //
///////////////////////////////////////////////////////////////////////////////
class CGGatewayClient : public CTMTSafe
{
    //-----------------------------------------------------------------------------
    // Define
#define MAXBUFFERSIZE           65535L
#define INITBUFFERSIZE          2048L

#define STEP_PRECONNECTION      1
#define STEP_CONNECTED          2
#define STEP_DISCONNECTED       3

public:                         // Constructor(s) and destructor.
    CGGatewayClient (CGTask& task) ;
    ~CGGatewayClient () ;

public:                         // Operators.

public:                         // Controls methodes.
    static void                 abort (void*) ;
    void                        init (void) ;
    void                        processConnection (void) ;

public:                         // Get methodes.
    const CGTask&               getTask (void)
    {
        return task_ ;
    } ;
    CGGatewayInfo&              getGatewayInfo (void)
    {
        return gatewayInfo_ ;
    } ;
    CTSocket*                   getClientSocket (void)
    {
        return pClient_ ;
    } ;
    CTSocket*                   getTargetSocket (void)
    {
        return pTarget_ ;
    } ;
    const CTLibrary*            getLibrary (void)
    {
        return pLibrary_ ;
    } ;

    //-----------------------------------------------------------------------------
private:                        // Operators.
    CGGatewayClient&            operator = (const CGGatewayClient&) ;

private:                        // Members.
    CGTask&                     task_ ;
    CGGatewayInfo               gatewayInfo_ ;
    CTSocket*                   pClient_ ;
    CTSocket*                   pTarget_ ;
    CTLibrary*                  pLibrary_ ;
    CGWParse::fnNewGatewayParse newGatewayParse_ ;
    CGWParse*                   pGWParse_ ;
    int                         iConnectionStep_ ;
    void*                       pC2TBuffer_ ;
    void*                       pT2CBuffer_ ;
    long                        lMaxBufferSize_ ;
    long                        lC2TBufferSize_ ;
    long                        lT2CBufferSize_ ;

    //-----------------------------------------------------------------------------
}

; // CGGatewayClient

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CGGatewayClient //
///////////////////////////////////////////////////////////////////////////////


#endif  // __GATEWAY_LISTEN_CLIENT__
