/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __CSTOF__
#define __CSTOF__

#ifndef _WINDOWS_
#include <windows.h>
#endif  /* _WINDOWS_ */

#include  "definition.h"
#include <map>
#include <list>
#include <set>
using std::list;
using std::set;
using std::map;

#include  <ctime.hpp>
#include  <logfile.hpp>
#include <mutex.hpp>
#include "gwhttp.hpp"
/*
 * Tof separator
 */
#define SEPARATOR   "~~~"

enum enumSelectStatus
{
    ESL_ERROR   =   0,
    ESL_RECORD  =   1,
    ESL_PAUSE   =   2,
    ESL_RESUME  =   3,
    ESL_STOP    =   4,
    ESL_COMMENT =   5,
    ESL_STARTTIMER = 6,
    ESL_ENDTIMER = 7
} ;

enum enumObjectType
{
    EOT_UNKNOWN,
    EOT_CONNECT,
    EOT_DISCONNECT,
    EOT_REQUEST,
    EOT_SETVAR
} ;

typedef list<string>		stringStlList ;
typedef map<string, string> stringStlMap ;
typedef set<string>			stringStlSet ;

class CTofReference
{
public:
    CTofReference()
        : oneTof_(TRUE), serverName_(""), iPort_(80), tofFileName_(""),
          selectStatus_(ESL_STOP), tofTitle_(""), spid_(0), bOutOfOrder_(false)
    {}


    ~CTofReference()
    {}

    bool   getOneTof(void) const
    {
        return oneTof_;
    }
    const string & getServerName(void) const
    {
        return serverName_;
    }
    int getPort(void) const
    {
        return iPort_ ;
    }
    const string & getTofFileName(void) const
    {
        return tofFileName_;
    }
    const string & getTofTimerName(void) const
    {
        return tofTimerName_;
    }
    enumSelectStatus getSelectStatus(void) const
    {
        return selectStatus_;
    }
    const string & getTofTitle(void) const
    {
        return tofTitle_;
    }
    const CContextList & getContextList(void) const
    {
        return contextList_;
    }
    unsigned long getSpid(void) const
    {
        return spid_;
    }
    enumObjectType  getObjectType(void) const
    {
        return eObjectType_ ;
    }
    const unsigned long getConnectId(void) const
    {
        return ulConnectId_ ;
    }
    bool isOutOfOrder() const
    {
        return bOutOfOrder_ ;
    }

    void setOneTof (bool oneTof)
    {
        oneTof_ = oneTof;
    }
    void setServerName (string & serverName)
    {
        serverName_ = serverName;
    }
    void setPort(int iPort)
    {
        iPort_ = iPort ;
    }
    void setTofFileName (string & tofFileName)
    {
        tofFileName_ = tofFileName;
    }
    void setTofTimerName (string & tofTimerName)
    {
        tofTimerName_ = tofTimerName;
    }
    void setSelectStatus (enumSelectStatus selectStatus)
    {
        selectStatus_ = selectStatus;
    }
    void setTofTitle (string & tofTitle)
    {
        tofTitle_ = tofTitle;
    }
    void setContextList(CContextList & contextList)
    {
        contextList_ = contextList;
    }
    void setSpid(unsigned long spid)
    {
        spid_ = spid;
    }
    void setObjectType(enumObjectType eObjectType)
    {
        eObjectType_ = eObjectType ;
    }
    void setConnectId(unsigned long ulConnectId)
    {
        ulConnectId_ = ulConnectId ;
    }
    void setOutOfOrder(bool bOutOfOrder)
    {
        bOutOfOrder_ = bOutOfOrder;
    }

    void addContext(CContext *context);
    void removeContext(CContext *context);

    bool analyse(const string & tofFileName, bool oneTof, enumSelectStatus selectStatus, const string & message);
    bool analyse(const string & conId,const string & tofFileName, const string & strMode, const string & strCommand, const string & strMessage);
    bool createTofFile(string & tofFileName);
    bool createTofFile(CContext *context, string & tofFileName, string & tofTimerName);
    void associateTofFile(CContext *context);
    void closeTofFile(CContext *context = NULL);
    void writeCommentInTofFile(CContext *context = NULL, const string & message = "");
	void writeStartTimerInTofFile(CContext *context, const string & message);
	void writeEndTimerInTofFile(CContext *context, const string & message);
    void writeConnectInTofFile(CContext *context);
    void writeDisconnectInTofFile(CContext *context, bool bFullDocument);
    void writeRequestInTofFile(CContext *context, const string & header, char * pBody, int iBodyLen);
    void writeLoadResponseHeader(CContext * context, string & header, string & variable_prefix);
    static string modifyStringAsScl(string input) ;
	string modelCookies(CContext *context, string header, bool & bAddQuotes) ;
	bool incTimePreviousWrite(unsigned long ulMillisecond) ;
	void NTLM_authentication_client_1(CContext *context, string & headScl, bool & bAddQuotes) ; 
	void NTLM_authentication_client_2(CContext *context, string & headScl, bool & bAddQuotes) ;
	void Negotiate_authentication_client_1(CContext *context, string & headScl, bool & bAddQuotes) ; 
	void Negotiate_authentication_client_2(CContext *context, string & headScl, bool & bAddQuotes) ;
	void BASIC_authentication(CContext *context, string & headScl, bool & bAddQuotes) ;
	BYTE* uudecode(const char* pcInputBuf, DWORD* pdwDecoded) ;

protected:
    bool                oneTof_;            // One tof per connection or one tof for all
    string              serverName_ ;
    int                 iPort_ ;
    ofstream            tofStream_;         // Descripteur du fichier tof
    string              tofFileName_;       // Tof File Name
    string              tofTimerName_;
    enumSelectStatus    selectStatus_;      // RECORD, STOP, PAUSE
    string              tofTitle_;          // Tof File title
    unsigned long       spid_;
    CContextList        contextList_;       // List of context objects
    CTMutex             tofMutex_;
    enumObjectType      eObjectType_ ;
    unsigned long       ulConnectId_ ;
    static CTTime       timePreviousWrite_ ;
    stringStlList       varList_ ;
	stringStlList       authVarList_ ;
	stringStlMap        authVarMap_ ;
	stringStlMap        timerVarMap_ ;
    bool                bOutOfOrder_ ;      // set to true to make the gateway
										    // stop capturing (if a direct ssl request
											// is received.
	static stringStlMap		scriptCookieMap_ ;	
											// Map that contains the script cookies.
											// Key = cookie value (name=value)
											// Value = variable name
	static stringStlMap		fakedCookieMap_	;
											// Map that contains the faked cookies.
											// associated with HTTP cookies
											// Will be used to remove the faked cookies.
											// Key = faked cookie name
											// Value = associated URL
public:
	static bool				bAutomaticModel_ ;
	static bool				bAddCookieComment_ ;
private:
    //// Forbid assignment and copy.
    CTofReference(const CTofReference&);
    operator=(const CTofReference&);
}
;



#endif
