/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// NS47Information.cpp: implementation of the CNS47Information class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <winsock2.h>
#include "CSerialization.h"
#include "concertlib.h"
#include "ConstantesLib.h"
#include "NS47Information.h"
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------

CNS47Information::CNS47Information(CWnd* _pWnd)
        :   CBrowserInformation(_pWnd)
{
    m_lProxyType = PROXY_CONF_NONE ;
}

//-------------------------------------------------------------------

CNS47Information::~CNS47Information()
{}

//-------------------------------------------------------------------


BOOL CNS47Information::GetDefaultParameters( _TPreferencesProxyData *pData )
{
    m_bSettingOk = TRUE ;

    InitVersion() ;
    m_bSettingOk = InitPrefFileFullPath() ;

    if(!m_bSettingOk )
        return false;

    m_bSettingOk = LoadPrefFile() ;

    if(!m_bSettingOk )
        return false;

    //--- Proxy HTTP
    if (m_lProxyType != PROXY_CONF_NONE)
        pData->m_strProxyAddress = m_strProxyHTTPUrl ;
    else
        pData->m_strProxyAddress.Empty();

    pData->m_lAddressPort    = m_lProxyHTTPPort ;

    //---Proxy SSL
    pData->m_strProxySecure = m_strProxySSLUrl;
    pData->m_lSecurePort    = m_lProxySSLPort ;

    //---Proxy Override Adresses
    pData->m_bByPassProxyLocal      = FALSE ;
    pData->m_strAddressesNoProxy    = m_strNoProxyUrl;
    return true;
}

//--------------------------------------------------------------------
//
//  Backup the defaults values of the browser
//
//--------------------------------------------------------------------

void CNS47Information::SaveDefaultRegistryInfo()
{
    TCHAR *pczTempDir[_MAX_PATH] ;
    CString strTmpFileName ;
    CString strPrefFileName ;
    //----
    ::GetTempPath( _MAX_PATH - 1 , (char *)pczTempDir ) ;
    strTmpFileName = (char*)pczTempDir ;
    strTmpFileName += STR_MODELLER_NS47 ;

    if ( !GetPrefFileFullPath( &strPrefFileName ) ) return ;

    CopyFile( strPrefFileName, strTmpFileName, TRUE ) ;
}

//--------------------------------------------------------------------

void CNS47Information::ModifyRegistryForCapture( long lPort )
{
    CString     strPrefFileName ;
    CString     strValue ;
    CString     strHostName ;

    if ( !GetPrefFileFullPath( &strPrefFileName ) ) return ;

    try
    {
        TCHAR *pczBuffer[ID_LOCALHOST_SIZE + 1 ] ;
        CStdioFile fPref( strPrefFileName, CFile::modeRead | CFile::modeWrite | CFile::typeText ) ;
        //-----

        fPref.SeekToEnd() ;

        // 1 - Start Page
        strValue.Format( "user_pref(\"browser.startup.page\", %d); \n", STARTPAGE_CONF_NONE ) ;
        fPref.WriteString( strValue ) ;

        // 2 - AutoConfig
        fPref.WriteString( "user_pref(\"network.proxy.autoconfig_url\", \"\"); \n" ) ;

        // 3 -  Proxy server Secure + Address + ...
        strValue.Format( "user_pref(\"network.proxy.type\", %d); \n", PROXY_CONF_MANUAL ) ;
        fPref.WriteString( strValue ) ;

        gethostname( (char*)pczBuffer , ID_LOCALHOST_SIZE ) ;
        strHostName = (char*)pczBuffer ;

        //HTTP
        strValue.Format( "user_pref(\"network.proxy.http\", \"%s\"); \n", strHostName ) ;
        fPref.WriteString( strValue ) ;
        strValue.Format( "user_pref(\"network.proxy.http_port\", %d); \n", lPort ) ;
        fPref.WriteString( strValue ) ;

        //SSL
        strValue.Format( "user_pref(\"network.proxy.ssl\", \"%s\"); \n", strHostName ) ;
        fPref.WriteString( strValue ) ;
        strValue.Format( "user_pref(\"network.proxy.ssl_port\", %d); \n", lPort ) ;
        fPref.WriteString( strValue ) ;

        //FTP
        strValue.Format( "user_pref(\"network.proxy.ftp\", \"%s\"); \n", strHostName ) ;
        fPref.WriteString( strValue ) ;
        strValue.Format( "user_pref(\"network.proxy.ftp_port\", %d); \n", lPort ) ;
        fPref.WriteString( strValue ) ;

        //SOCKS
        strValue.Format( "user_pref(\"network.proxy.socks_server\", \"%s\"); \n", strHostName ) ;
        fPref.WriteString( strValue ) ;
        strValue.Format( "user_pref(\"network.proxy.socks_server_port\", %d); \n", lPort ) ;
        fPref.WriteString( strValue ) ;

        //GOPHER
        strValue.Format( "user_pref(\"network.proxy.gopher\", \"%s\"); \n", strHostName ) ;
        fPref.WriteString( strValue ) ;
        strValue.Format( "user_pref(\"network.proxy.gopher_port\", %d); \n", lPort ) ;
        fPref.WriteString( strValue ) ;

        //WAIS
        strValue.Format( "user_pref(\"network.proxy.wais\", \"%s\"); \n", strHostName ) ;
        fPref.WriteString( strValue ) ;
        strValue.Format( "user_pref(\"network.proxy.wais_port\", %d); \n", lPort ) ;
        fPref.WriteString( strValue ) ;

        // 4 - machines who do not use proxy : local + other machines
        fPref.WriteString( "user_pref(\"network.proxy.no_proxies_on\", \"\"); \n" ) ;

        fPref.Close() ;
    }
    catch( CFileException e)
    {
        ASSERT(FALSE) ;
        return ;
    }

}

//--------------------------------------------------------------------
//
//  Reinitialize the defaults values of the browser
//
//--------------------------------------------------------------------

void CNS47Information::SetDefaultParametersInRegistry()
{
    TCHAR   *pczTempDir[_MAX_PATH] ;
    CString strTmpFileName ;
    CString strPrefFileName ;
    BOOL    bFileOp ;
    //----
    ::GetTempPath( _MAX_PATH - 1 , (char *)pczTempDir ) ;
    strTmpFileName = (char*)pczTempDir ;
    strTmpFileName += STR_MODELLER_NS47 ;

    if ( !GetPrefFileFullPath( &strPrefFileName ) ) return ;

    bFileOp = CopyFile( strTmpFileName, strPrefFileName, FALSE ) ;
    bFileOp = DeleteFile( strTmpFileName ) ;
}

//--------------------------------------------------------------------

BOOL CNS47Information::GetBrowserFullPath( CString &_strPath )
{
    CString strLongPath ;
    CString strTitle ;
    CString strMsg ;
    TCHAR pczShortPath[_MAX_PATH] ;
    CString strVersion ;
    CString strVersionKey ;
    CString strTmpPath ;

    //---
    //1 - Test the version
    if( !IsNS47Version() )
    {
        strTitle.LoadString( IDS_WARNING ) ;
        strMsg.LoadString( IDS_BROWSER_NOT_FOUND ) ;
        m_pWnd->MessageBox(strMsg, strTitle, MB_OK ) ;
    }

    strVersion  = GetVersion() ;
    strVersionKey.Format( _T("SOFTWARE\\Netscape\\Netscape Navigator\\%s\\Main"), strVersion ) ;

    strTmpPath = GetExtendedProfileString( HKEY_LOCAL_MACHINE , strVersionKey,
                                           _T("Install Directory") , _T("") ) ;

    //2 - Test the path
    if( strTmpPath.IsEmpty() )
    {
        return FALSE ;
    }

    strLongPath.Format("%s\\Program\\netscape.exe", strTmpPath) ;

    GetShortPathName( (LPCSTR)strLongPath, pczShortPath, _MAX_PATH );
    _strPath = pczShortPath ;

    return TRUE ;
}

//--------------------------------------------------------------------

BOOL CNS47Information::IsInternetSettingOK()
{
    CString strWarning ;
    CString strMessage ;

    strWarning.LoadString( IDS_WARNING ) ;

    if( !m_bSettingOk )
    {
        strMessage.LoadString( IDS_NO_SETTING_FOUND ) ;
        AfxGetMainWnd()->MessageBox( strMessage , strWarning , MB_OK ) ;
        return FALSE ;
    }

    if( m_lProxyType == PROXY_CONF_AUTO )
    {
        strMessage.LoadString( IDS_NOT_USE_AUTO_PROXY ) ;
        AfxGetMainWnd()->MessageBox( strMessage , strWarning , MB_OK ) ;
        return FALSE ;
    }

    if( m_bIsRoamingAccessOn )
    {
        strMessage.LoadString( IDS_NOT_USE_ROAMING_ACCESS ) ;
        AfxGetMainWnd()->MessageBox( strMessage , strWarning , MB_OK ) ;
        return FALSE ;
    }

    return TRUE ;
}

//--------------------------------------------------------------------

BOOL CNS47Information::IsNS47Version()
{
    CString strVersion ;

    strVersion = GetVersion() ;

    if( strncmp( strVersion, _T(STR_NS47_VERSION), 3 ) )
        return FALSE ;

    return TRUE ;
}

//--------------------------------------------------------------------

CString CNS47Information::GetVersion()
{
    return m_strVersion ;
}

//--------------------------------------------------------------------

void CNS47Information::InitVersion()
{
    CString strVersion ;

    m_strVersion = GetExtendedProfileString( HKEY_LOCAL_MACHINE , _T("SOFTWARE\\Netscape\\Netscape Navigator"),
                   _T("CurrentVersion") , _T("") ) ;
}

//--------------------------------------------------------------------

BOOL CNS47Information::LoadPrefFile()
{
    CString     strValue ;
    CString     strLastValue ;
    CStdioFile  PrefFile ;
    CString     strFileName ;
    BOOL        bFileOp ;

    m_bIsRoamingAccessOn = FALSE ;

    if( !GetPrefFileFullPath( &strFileName ) ) return FALSE ;

    try
    {
        //$$$
        bFileOp = PrefFile.Open( strFileName , CFile::modeRead| CFile::typeText ) ;

        if( !bFileOp ) return FALSE ;

        //On Saute les 3 premieres lignes
        for( int i = 0; bFileOp && i < 3; i++ )
        {
            bFileOp = PrefFile.ReadString( strValue ) ;
        }

        if( !bFileOp ) return FALSE ;

        for( ; PrefFile.ReadString( strValue );)
        {
            if( strValue.Right( 2 ) == _T(");") )
            {
                strValue = strLastValue + strValue ;
                strLastValue = "" ;
                ExtractInfo( &strValue ) ;
            }
            else
            {
                strLastValue = strValue ;
            }
        }

        PrefFile.Close() ;
    }
    catch( CFileException e )
    {
        return FALSE ;
    }

    return TRUE ;
}

//--------------------------------------------------------------------

void CNS47Information::ExtractInfo( CString * _strPrefLine )
{
    CString strLabel ;
    CString strValue ;
    int     nIndex ;
    int     nValueIndex ;
    long    lValue ;

    lValue          = _strPrefLine->GetLength() - 11 - 2 ;
    *_strPrefLine   = _strPrefLine->Mid( 11, lValue ) ;

    nIndex      = _strPrefLine->Find( '\"' ) ;
    strLabel    =  _strPrefLine->Left( nIndex ) ;

    if( !strcmp( strLabel, "browser.startup.page" ) )              nValueIndex = 1 ;
    else if( !strcmp( strLabel, "network.proxy.autoconfig_url" ) ) nValueIndex = 2 ;
    else if( !strcmp( strLabel, "network.proxy.http" ) )           nValueIndex = 3 ;
    else if( !strcmp( strLabel, "network.proxy.http_port" ) )      nValueIndex = 4 ;
    else if( !strcmp( strLabel, "network.proxy.no_proxies_on" ) )  nValueIndex = 5 ;
    else if( !strcmp( strLabel, "network.proxy.ssl" ) )            nValueIndex = 6 ;
    else if( !strcmp( strLabel, "network.proxy.ssl_port" ) )       nValueIndex = 7 ;
    else if( !strcmp( strLabel, "network.proxy.type" ) )           nValueIndex = 8 ;
    else if( !strcmp( strLabel, "network.proxy.ftp" ) )            nValueIndex = 9 ;
    else if( !strcmp( strLabel, "network.proxy.ftp_port" ) )       nValueIndex = 10 ;
    else if( !strcmp( strLabel, "li.enabled" ) )                   nValueIndex = 11 ;
    else nValueIndex = 0 ;

    if( nValueIndex > 0 )
    {
        nIndex          = _strPrefLine->Find( ',' ) + 1 ;
        *_strPrefLine   = _strPrefLine->Mid( nIndex ) ;

        nIndex      = _strPrefLine->Find( '\"' ) ;

        if( nIndex != -1 )
        {   // CHAR
            *_strPrefLine = _strPrefLine->Mid( nIndex + 1 ) ;
            strValue = _strPrefLine->Left( _strPrefLine->GetLength() - 1 ) ;
        }
        else
        {   // DIGIT
            lValue = atol( *_strPrefLine ) ;
        }

        switch( nValueIndex )
        {
        case 1 :
            m_lBrowserStartPageType = lValue ;
            break ;
        case 2 :
            m_strProxyAutoUrl       = strValue ;
            break ;
        case 3 :
            m_strProxyHTTPUrl       = strValue ;
            break ;
        case 4 :
            m_lProxyHTTPPort        = lValue ;
            break ;
        case 5 :
            strValue.Replace( ',', ';') ;
            m_strNoProxyUrl         = strValue ;
            break ;
        case 6 :
            m_strProxySSLUrl        = strValue ;
            break ;
        case 7 :
            m_lProxySSLPort         = lValue ;
            break ;
        case 8 :
            m_lProxyType            = lValue ;
            break ;
        case 9 :
            m_strProxyFTP           = strValue ;
            break ;
        case 10 :
            m_lProxyFTPPort         = lValue ;
            break ;
        case 11 :
            m_bIsRoamingAccessOn    = ( _strPrefLine->Find( "true" ) != -1 ) ;
            break ;
        default :
            ASSERT(FALSE) ;
        }
    }
}

//--------------------------------------------------------------------

BOOL CNS47Information::GetPrefFileFullPath( CString * _pstrPath )
{
    *_pstrPath = m_strPrefFileFullPath ;

    return ( !m_strPrefFileFullPath.IsEmpty() ) ;
}

//--------------------------------------------------------------------

BOOL CNS47Information::InitPrefFileFullPath()
{
    if( !IsNS47Version() ) return FALSE ;

	m_strPrefFileFullPath = GetExtendedProfileString( HKEY_CURRENT_USER , STR_MODELLER_KEY,
                          STR_MDL_NETSCAPE_ENTRY , _T("") ) ;
    
    return TRUE ;
}
