/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// HTTPGraphs.cpp: implementation of the CHTTPGraphs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTTPGraphs.h"
#include "HTTPStatsOcx.h"
#include "TimerList.h"
#include "GraphTypes.h"
#include "GraphWnd.h"
#include "SSDataDlg.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
extern HINSTANCE m_hInstance;

CHTTPGraphs::CHTTPGraphs():
	m_mapURLs(500),

	m_strDataList("HTTP Data List"),
	m_strBytesPerSecVTime("HTTP Monitored Bytes / Sec v Elapsed Time"),
	m_strRespTimeVNumResps("HTTP Response Time v Number Of Responses"),
	m_strErrorsVUsers("HTTP Errors v HTTP Requests"),
	m_strErrorsVTime("HTTP Errors v Elapsed Time"),
	m_strRespsVTime("HTTP Responses v Elapsed Time"),
	m_strRespTimeVTime("HTTP Response Time v Elapsed Time"),
	m_strSSDataTree("Single Step Results"),

	m_strUsersVTime("HTTP Active Users v Elapsed Time"),

	m_strTimerList("Timer List"),
	m_strTimersVUsers("Timer Values v Active Users"),
	m_strTimersVTime("Timer Values v Elapsed Time")
{
}

CHTTPGraphs::~CHTTPGraphs()
{
	for(int nNdx=0;nNdx<m_arHttp.GetSize();nNdx++)
		delete m_arHttp[nNdx];
	m_arHttp.RemoveAll();

	POSITION pos = m_listTimers.GetHeadPosition();
	while(pos)
		delete m_listTimers.GetNext(pos);
	m_listTimers.RemoveAll();

	pos = m_listVUs.GetHeadPosition();
	while(pos)
		delete m_listVUs.GetNext(pos);
	m_listVUs.RemoveAll();
}

void CHTTPGraphs::LoadRun(const CString& strRunFolder)
{
	CString strRunDir = strRunFolder;
	strRunDir.TrimRight("\\");
// removed because sometimes, the result tree was not refreshed
//	if(strRunDir == m_strLoadedRun)
//		return;

	m_strLoadedRun = strRunDir;
	LoadHttp();
	LoadTimers();
	LoadVUData();
	LoadSingleStepData();

	int nSlashPos = m_strLoadedRun.ReverseFind('\\');
	CString strRunName(m_strLoadedRun.Mid(nSlashPos+1));
	
	int nDay,nMonth,nYear,nHour,nMinute,nSecond;
	char *pszToken = strtok(strRunName.GetBuffer(0),"-");
	if(pszToken)
		nDay = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMonth = atoi(pszToken);

	if(pszToken = strtok(NULL," "))
		nYear = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nHour = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMinute = atoi(pszToken);

	if(pszToken = strtok(NULL,"."))
		nSecond = atoi(pszToken);

	CTime T(nYear,nMonth,nDay,nHour,nMinute,nSecond);
	m_dwTestStartTime = T.GetTime();
}

void CHTTPGraphs::GetAvailableGraphs(GraphInfoArray& arGraphs)
{
	arGraphs.RemoveAll();

	GraphInfo GI;
	GI.bFolder = false;
	if(m_arHttp.GetSize())
	{
		GI.strName = m_strDataList;
		arGraphs.Add(GI);
		GI.strName = m_strBytesPerSecVTime;
		arGraphs.Add(GI);
		GI.strName = m_strRespTimeVNumResps;
		arGraphs.Add(GI);
		GI.strName = m_strErrorsVUsers;
		arGraphs.Add(GI);
		GI.strName = m_strErrorsVTime;
		arGraphs.Add(GI);
		GI.strName = m_strRespsVTime;
		arGraphs.Add(GI);
		GI.strName = m_strRespTimeVTime;
		arGraphs.Add(GI);
	}

	if(m_SSFile.IsOpen())
	{
		GI.strName = m_strSSDataTree;
		arGraphs.Add(GI);
	}

	if(m_listVUs.GetCount())
	{
		BOOL bFound = false;
		POSITION pos = m_listVUs.GetHeadPosition();
		while(pos)
		{
			if(m_listVUs.GetNext(pos)->dwCount)
			{
				bFound = true;
				break;
			}
		}
		if(bFound)
		{
			GI.strName = m_strUsersVTime;
			arGraphs.Add(GI);
		}
	}

	if(m_listTimers.GetCount())
	{
		GI.strName = m_strTimerList;
		arGraphs.Add(GI);
		GI.strName = m_strTimersVUsers;
		arGraphs.Add(GI);
		GI.strName = m_strTimersVTime;
		arGraphs.Add(GI);
	}
}

CWnd* CHTTPGraphs::DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds)
{
	if(strName == m_strDataList)
	{
		return LoadHTTPDataList(pParent,rcBounds);
	}
	else
	if(strName == m_strTimerList)
	{
		return LoadTimerList(pParent,rcBounds);
	}
	else
	if(strName == m_strBytesPerSecVTime ||	//All http only graphs
		strName == m_strRespsVTime ||
		strName == m_strErrorsVUsers ||
		strName == m_strErrorsVTime ||
		strName == m_strRespTimeVTime ||
		strName == m_strRespTimeVNumResps)
	{
		return LoadHttpGraph(strName,rcBounds,pParent);
	}
	else				
	if(strName == m_strTimersVUsers ||		//all timer only graphs
		strName == m_strTimersVTime)
	{
		return LoadTimerGraph(strName,rcBounds,pParent);
	}
	else
	if(strName == m_strUsersVTime)
	{
		return LoadVUGraph(strName,rcBounds,pParent);
	}
	else
	if(strName == m_strSSDataTree)
	{
		return LoadSSDataTree(rcBounds,pParent);
	}
	
	return NULL;
}

CGraphWnd* CHTTPGraphs::LoadHttpGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	CHttpGraph *pGraph = NULL;
	if(strName == m_strBytesPerSecVTime)
		pGraph = new CThroughputGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strRespsVTime)
		pGraph = new CHitsPerSecondGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strErrorsVUsers)
		pGraph = new CErrorsPerUserGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strErrorsVTime)
		pGraph = new CErrorsTimeGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strRespTimeVTime)
		pGraph = new CPagesPerSecondGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strRespTimeVNumResps)
		pGraph = new CAvPageRTPerUserGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;
	if(!pGraph || !pGraph->AddTestRun(m_arHttp,&m_mapURLs,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();
	return pGraphWnd;
}

CGraphWnd* CHTTPGraphs::LoadTimerGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	CTimerGraph *pGraph = NULL;
	if(strName == m_strTimersVUsers)
		pGraph = new CRTATGraph(pGraphWnd->m_pChart);
	else
	if(strName == m_strTimersVTime)
		pGraph = new CRTTimeGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;
	if(!pGraph || !pGraph->AddTestRun(m_listTimers,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();

	return pGraphWnd;
}

CGraphWnd* CHTTPGraphs::LoadVUGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	CVUsersTimeGraph *pGraph = NULL;
	if(strName == m_strUsersVTime)
		pGraph = new CVUsersTimeGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;
	if(!pGraph || !pGraph->AddTestRun(m_listVUs,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();

	return pGraphWnd;
}

CTimerList* CHTTPGraphs::LoadTimerList(CWnd* pParent,const CRect& rcBounds)
{
	LoadTimers();

	CTimerList *pTimerList = new CTimerList(m_listTimers);
	pTimerList->Create(WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);

	return pTimerList;
}

CSSDataDlg* CHTTPGraphs::LoadSSDataTree(const CRect& rcBounds,CWnd* pParent)
{
	HINSTANCE hInst = AfxGetResourceHandle();
	AfxSetResourceHandle(m_hInstance);
	CSSDataDlg *pTree = new CSSDataDlg(&m_SSFile);
	pTree->Create(IDD_SS_DATA_DLG,pParent);
	pTree->MoveWindow(rcBounds);
	
	AfxSetResourceHandle(hInst);

	return pTree;
}

CHttpStatsOcx* CHTTPGraphs::LoadHTTPDataList(CWnd *pParent,const CRect& rcBounds)
{
	CHttpStatsOcx *pHttpStatsOcx = new CHttpStatsOcx;
	pHttpStatsOcx->Create("",WS_BORDER|WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);
	
	CString strStatTarget = m_strLoadedRun + "\\HTTP_ALL.stat";
	CString strURLTarget = m_strLoadedRun + "\\HTTP_ALL.urls";
	CFile fStatTarget(strStatTarget,CFile::modeCreate|CFile::modeWrite);
	CFile fUrlTarget(strURLTarget,CFile::modeCreate|CFile::modeWrite);

	//Concatenate all the http files in this test run directory
	DWORD dwRead=0;
	const DWORD dwBuffSize=4096;
	BYTE *pBuffer = new BYTE[dwBuffSize];

	CFileFind FF;
	if(!FF.FindFile(m_strLoadedRun + "\\TEW_*.stat"))
		return NULL;
	
	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		CString strStatFile = FF.GetFilePath();

		int nDotPos = strStatFile.ReverseFind('.');
		CString strUrlFile = strStatFile.Left(nDotPos) + ".urls";

		CFile fStats,fUrls;
			
		// Get the path for the url file and open it
		if(fUrls.Open(strUrlFile,CFile::modeRead|CFile::typeBinary) &&
			fStats.Open(strStatFile,CFile::modeRead|CFile::typeBinary))
		{
			DWORD dwLength = fUrls.GetLength();

			//copy urls file to concat file
			while(dwLength>0)
			{
				if(dwLength > dwBuffSize)
					dwRead = fUrls.Read(pBuffer,dwBuffSize);
				else
					dwRead = fUrls.Read(pBuffer,dwLength);

				fUrlTarget.Write(pBuffer,dwRead);
				dwLength -= dwRead;
			}
			fUrls.Close();

			//Copy stats to concat file
			dwLength = fStats.GetLength();

			while(dwLength>0)
			{
				if(dwLength > dwBuffSize)
					dwRead = fStats.Read(pBuffer,dwBuffSize);
				else
					dwRead = fStats.Read(pBuffer,dwLength);

				fStatTarget.Write(pBuffer,dwRead);
				dwLength -= dwRead;
			}
			fStats.Close();
		}
	}

	fStatTarget.Close();
	fUrlTarget.Close();
	delete []pBuffer;

	pHttpStatsOcx->LoadStatsFile(strStatTarget);

	DeleteFile(strStatTarget);
	DeleteFile(strURLTarget);

	return pHttpStatsOcx;
}

void CHTTPGraphs::LoadTimers()
{
	POSITION pos = m_listTimers.GetHeadPosition();
	while(pos)
		delete m_listTimers.GetNext(pos);
	m_listTimers.RemoveAll();

	CString strTimerFile(m_strLoadedRun + "\\Timer.txt");
    try
    {
        CStdioFile TimerFile(strTimerFile,CFile::modeRead);

		CArray <Timer *,Timer*> arTimers;
        CString strDescription;
        CString strTimer;
        while(TimerFile.ReadString(strTimer))
        {
			Timer *pTimer = new Timer;

			char *pszToken = strtok(strTimer.GetBuffer(0),",");
			if(pszToken)	//Timestamp
				pTimer->dwTimeStamp = atoi(pszToken);

			pszToken = strtok(NULL,",");
			if(pszToken)	//Value
				pTimer->dValue = ((double)atoi(pszToken))/100.0;	//Convert to Seconds

			pszToken = strtok(NULL,",");
			if(pszToken)	//Num Active Users
				pTimer->dwActiveUsers =	atoi(pszToken);

			pszToken = strtok(NULL,"|");
			if(pszToken)	//Test Name
				pTimer->strTestName = pszToken;

			pszToken = strtok(NULL,"|");
			if(pszToken)	//Thread Name
				pTimer->strThreadName = pszToken;

			pszToken = strtok(NULL,"|");
			if(pszToken)	//Script Name
				pTimer->strScriptName = pszToken;

			pszToken = strtok(NULL,",");
			if(pszToken)	//Timer name
				pTimer->strTimerName = pszToken;
            
			arTimers.Add(pTimer);
        }

		int nNumTimers = arTimers.GetSize();
		qsort(arTimers.GetData(),nNumTimers,sizeof(Timer*),&CompareTimerStart);

		for(int nNdx=0;nNdx<nNumTimers;nNdx++)
			m_listTimers.AddTail(arTimers[nNdx]);
    }
	catch(CFileException *e)
	{
		e->Delete();
	}
    catch (...)
    {
	}
}

int CHTTPGraphs::CompareTimerStart( const void *arg1, const void *arg2 )
{
	Timer *a = *(Timer **)arg1;
	Timer *b = *(Timer **)arg2;

	if(a->dwTimeStamp > b->dwTimeStamp)
		return 1;
	else
	if(a->dwTimeStamp < b->dwTimeStamp)
		return -1;
	
	return 0;
}

BOOL CHTTPGraphs::LoadCachedHttpData()
{
	CFile f;
	if(!f.Open(m_strLoadedRun+"\\HttpStats.cache",CFile::modeRead|CFile::shareDenyWrite))
		return false;

	DWORD dwNumStats=0;
	f.Read(&dwNumStats,sizeof(DWORD));

	if(!dwNumStats)
		return true;

	HttpData *pHttpData=NULL;
	m_arHttp.SetSize(dwNumStats);
	for(DWORD dwStat=0;dwStat<dwNumStats;dwStat++)
	{
		m_arHttp[dwStat] = pHttpData = new HttpData;
		f.Read(pHttpData,sizeof(HttpData));
	}

	DWORD dwBufferLen = f.GetLength()-f.GetPosition();
	if(!dwBufferLen)
		return true;

	LPBYTE pBuffer = new BYTE[dwBufferLen];
	f.Read(pBuffer,dwBufferLen);
	LPBYTE pBufferPos = pBuffer;

	DWORD dwNumUrls;
	memcpy(&dwNumUrls,pBufferPos,sizeof(DWORD));
	pBufferPos++;
	
	int nIndex;
	for(DWORD dwUrl=0;dwUrl<dwNumUrls;dwUrl++)
	{
		memcpy(&nIndex,pBufferPos,sizeof(int));
		pBufferPos+=sizeof(int);

		m_mapURLs.SetAt((char*)pBufferPos,nIndex);
		pBufferPos += strlen((char*)pBufferPos)+1;
	}
	
	delete []pBuffer;
	f.Close();
	return true;
}

void CHTTPGraphs::CacheHttpData()
{
	CFile f;
	if(!f.Open(m_strLoadedRun+"\\HttpStats.cache",CFile::modeCreate|CFile::modeWrite))
		return;

	DWORD dwNumStats=m_arHttp.GetSize();
	f.Write(&dwNumStats,sizeof(DWORD));

	HttpData *pHttpData=NULL;
	for(DWORD dwStat=0;dwStat<dwNumStats;dwStat++)
	{
		f.Write(m_arHttp[dwStat],sizeof(HttpData));
	}

	CString strUrl;
	int nIndex;
	POSITION pos = m_mapURLs.GetStartPosition();
	while(pos)
	{
		m_mapURLs.GetNextAssoc(pos,strUrl,nIndex);
	
		f.Write(&nIndex,sizeof(int));
		f.Write((LPCTSTR)strUrl,strUrl.GetLength()+1);
	}

	f.Close();
}

void CHTTPGraphs::LoadHttp()
{
	for(int nNdx=0;nNdx<m_arHttp.GetSize();nNdx++)
		delete m_arHttp[nNdx];
	m_arHttp.RemoveAll();

	if(LoadCachedHttpData())
		return;

	CFileFind FF;
	if(!FF.FindFile(m_strLoadedRun + "\\TEW_*.stat"))
		return;

	int nCount=0;
 	CFileStatus fsUrls,fsStat;
	BOOL bContinue = TRUE;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		CString strStatFile( FF.GetFilePath() );
		CString strUrlFile ( m_strLoadedRun + "\\" + FF.GetFileTitle() + ".urls");
		
		//Check url file exists
		if(!CFile::GetStatus(strUrlFile,fsUrls))
			continue;

		//Check stat file exists and get its size
		if(!CFile::GetStatus(strStatFile,fsStat) || !fsStat.m_size)
			continue;

		//Calculate number of stats in file
	    int nStatSize = sizeof(HttpData)-sizeof(int);
		int nNumStats = fsStat.m_size/nStatSize;

		//Resize data array to fit new stats
		m_arHttp.SetSize( m_arHttp.GetSize() + nNumStats);

		//Open the stats file
        HANDLE hStatFile = CreateFile(strStatFile,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(hStatFile == INVALID_HANDLE_VALUE)
            return;

        HANDLE hStatMap = CreateFileMapping(hStatFile,NULL,PAGE_READONLY,0,0,"StatsData");
        if(hStatMap == NULL)
            return;

        LPBYTE pStatBuffer = (LPBYTE)MapViewOfFile(hStatMap,FILE_MAP_READ,0,0,0);
        LPBYTE pStatBufPos = pStatBuffer;
		
		//Open the urls file
        HANDLE hUrlFile = CreateFile(strUrlFile,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(hUrlFile == INVALID_HANDLE_VALUE)
            return;

        HANDLE hUrlMap = CreateFileMapping(hUrlFile,NULL,PAGE_READONLY,0,0,"UrlData");
        if(hUrlMap == NULL)
            return;

        LPBYTE pUrlBuffer = (LPBYTE)MapViewOfFile(hUrlMap,FILE_MAP_READ,0,0,0);
        char *pUrlBufPos = (char*)pUrlBuffer;

		for(int nStat=0;nStat<nNumStats;nStat++)
        {
            int nIndex=-1;
            if(!m_mapURLs.Lookup(pUrlBufPos,nIndex))
            {
                nIndex = m_mapURLs.GetCount();
                m_mapURLs.SetAt(pUrlBufPos,nIndex);
            }
            pUrlBufPos += strlen(pUrlBufPos)+1;

            //Do the load
            HttpData *pData = m_arHttp[nStat+nCount] = new HttpData;
            memcpy(pData,&pStatBuffer[nStatSize*nStat],nStatSize);

            pData->nURLNdx = nIndex;
        }
        nCount += nNumStats;

        //Clean up
        UnmapViewOfFile(pStatBuffer);
        CloseHandle(hStatMap);
        CloseHandle(hStatFile);
        UnmapViewOfFile(pUrlBuffer);
        CloseHandle(hUrlMap);
        CloseHandle(hUrlFile);
    }
    
    //Sort data into time order
    qsort(m_arHttp.GetData(),(size_t)m_arHttp.GetSize(),sizeof(HttpData**),SortHttpData);

	CacheHttpData();
}

int CHTTPGraphs::SortHttpData( const void *arg1, const void *arg2 )
{
	HttpData *pStat1 = *(HttpData**)arg1;
	HttpData *pStat2 = *(HttpData**)arg2;
    DWORD dwTime1 = pStat1->dwTimeStamp;
    DWORD dwTime2 = pStat2->dwTimeStamp;

    if(dwTime1>dwTime2)
        return 1;

    return ((dwTime2>dwTime1) ? -1 : 0);
}

void CHTTPGraphs::LoadVUData()
{
	POSITION pos = m_listVUs.GetHeadPosition();
	while(pos)
		delete m_listVUs.GetNext(pos);
	m_listVUs.RemoveAll();

	CString strPath(m_strLoadedRun + "\\VUsersLog.txt");
    
    try
    {
        CStdioFile VUFile(strPath,CFile::modeRead);

		CString strTimeStamp;
		CString strCount;
        CString strVUData;
		char *pszToken = NULL;
        while(VUFile.ReadString(strVUData))
        {
			VUData *pVD = new VUData;
			try
			{
				if(!(pszToken = strtok(strVUData.GetBuffer(0),","))) throw -1;
				
				pVD->dwTimeStamp = atoi(pszToken);
				
				if(!(pszToken = strtok(NULL,","))) throw -1;

				pVD->dwCount = atoi(pszToken);
			}
			catch(int)
			{
				delete pVD;
				continue;
			}

			m_listVUs.AddTail(pVD);
		}

		VUFile.Close();
	}
	catch(CFileException *e)
	{
		e->Delete();
	}
	catch(...)
	{
	}
}

void CHTTPGraphs::LoadSingleStepData()
{
	m_SSFile.Open(m_strLoadedRun,true);
}