/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ModellerDoc.h : interface of the CModellerDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MODELLERDOC_H__9D2B8163_F0E1_11D3_AC83_00105A483E3C__INCLUDED_)
#define AFX_MODELLERDOC_H__9D2B8163_F0E1_11D3_AC83_00105A483E3C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BasedDocument.h"
#include "GatewayObject.h"
#include "SCLCaptureScriptView.h"
#include <VarStructures.h>

#include <SCLParser.h>
#include <set>
using std::set;

struct CTag;
class COutputFindView;

class CObjScript
{
public:
    CString m_strScript;
    BOOL    m_bModify;
} ;

enum enumNavigateMode
{
	ENM_UNKNOWN,
	ENM_GOTOURL,
	ENM_NAVIGATE
};

typedef set<CString> stringStlSet;

class CTLibrary;
class CReplayDlg;
class CModellerDoc : public CBasedDocument
{
protected: // create from serialization only
    CModellerDoc();
    DECLARE_DYNCREATE(CModellerDoc)

private:
	void GenerateVariable(CVarInfo &VarInfo,CString& strLine);
	static UINT WaitForBrowserThread(LPVOID pParam);
	void GetVarInfoFromVariable(CVariableInfo *pVariable,CVarInfo& VarInfo);
    void DeleteWorkingDirectory();
	
	BOOL CompileScript(const CString& strScript,BOOL bCompileDependencies=TRUE);
	BOOL Compile(BOOL bCompileDependencies);
    BOOL IsRecording()  ;
    void InitBrowserFromScript( CObjScript *pObjScript) ;
    void SetDefaultBrowser(CObjScript *pObjScript );

    //Save Files to Repository
    BOOL SaveToRepository() ;
    
	//Save files to Repository to do that we have to move the recent file to the Repository arborescence directory
    BOOL MoveFilesToRepositoryStorage() ;

	void ExportRequestTree();
private:
	CSclParser			m_sclParser;
    CGatewayObject      m_Gateway ;
    BOOL                m_bPause ;
	BOOL				m_bTimer ; 
    BOOL                m_bLocalRecord ;
    BOOL                m_bLocalReplay ;
	CString				m_strSCLCompilerPath;

    //Replay
    HWND                m_hOutputReplayWnd;
    HMODULE             m_hReplayLibrary;

	//DOM View
	CTag*				m_pHtmlRootTag;
	LPBYTE				m_pHtmlBody;
	DWORD				m_dwHtmlBodyLen;

	// Set for timers
	stringStlSet		m_timerSet ;

    // pointer on the views
    CSCLCaptureScriptView* m_pCaptureView;
	enumNavigateMode	m_eNavigateMode ;

public:
    void SetCaptureView(CSCLCaptureScriptView* _pCaptureView);
    BOOL Parse(BOOL _bForceParse = FALSE);
    BOOL RetrieveFromRepository() ;
	void SetOutputReplayWnd(HWND _hOutputReplayWnd);
    void StopReplay();
    void OnReplayStopped();

    //Error line when checking is made and double click used
    void SetErrorLine( int lRow ) ;

    //Insert Variable declaration in the script
    void Adressing( CVarResponse * _pVarResponse ) ;

    inline CTag* GetHtmlRootTag(){ return m_pHtmlRootTag; } 
	inline CSclParser *GetSCLParser(){	return &m_sclParser;}
	inline void GetHtmlBody(LPBYTE& pBody,LPDWORD dwLength)
	{
		pBody = m_pHtmlBody;
		*dwLength = m_dwHtmlBodyLen;
	}

    void OnUpdateGoToURL(long _lLine, CCmdUI* pCmdUI );
    void GoToURL(long _lLine);

    enumNavigateMode getNavigateMode() const {return m_eNavigateMode; }
    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CModellerDoc)
	public:
    virtual BOOL DoSave(LPCTSTR lpszPathName, BOOL bReplace = TRUE);
    virtual BOOL OnNewDocument();
    virtual void Serialize(CArchive& ar);
    virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
    virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
    virtual void OnCloseDocument();
	protected:
    virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

    // Implementation
public:
    virtual ~CModellerDoc();
#ifdef _DEBUG
    virtual void AssertValid() const;
    virtual void Dump(CDumpContext& dc) const;
#endif

    // Generated message map functions
public:
    //{{AFX_MSG(CModellerDoc)
    afx_msg void OnCaptureCheck();
    afx_msg void OnUpdateCaptureCheck(CCmdUI* pCmdUI);
    afx_msg void OnCapturePause();
    afx_msg void OnUpdateCapturePause(CCmdUI* pCmdUI);
    afx_msg void OnCaptureRecord();
    afx_msg void OnUpdateCaptureRecord(CCmdUI* pCmdUI);
    afx_msg void OnCaptureReplay();
    afx_msg void OnUpdateCaptureReplay(CCmdUI* pCmdUI);
    afx_msg void OnCaptureStop();
    afx_msg void OnUpdateCaptureStop(CCmdUI* pCmdUI);
    afx_msg void OnSclComment();
    afx_msg void OnUpdateSclComment(CCmdUI* pCmdUI);
    afx_msg void OnFileSave();
    afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
    afx_msg void OnUpdateVariableCreate(CCmdUI* pCmdUI);
    afx_msg void OnVariableCreate();
    afx_msg void OnUpdateVariableModify(CCmdUI* pCmdUI);
    afx_msg void OnVariableModify();
    afx_msg void OnUpdateViewRefresh(CCmdUI* pCmdUI);
    afx_msg void OnViewRefresh();
    afx_msg void OnFileSaveAs();
    afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
	afx_msg void OnCanonicalizeUrl();
	afx_msg void OnUpdateCanonicalizeUrl(CCmdUI* pCmdUI);
	afx_msg void OnToolsExporturltree();
	afx_msg void OnToolsNavigate();
	afx_msg void OnSclEndtimer();
	afx_msg void OnSclStarttimer();
	afx_msg void OnUpdateSclStarttimer(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSclEndtimer(CCmdUI* pCmdUI);
	//}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MODELLERDOC_H__9D2B8163_F0E1_11D3_AC83_00105A483E3C__INCLUDED_)
