/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarDBRowSelPPage.cpp : implementation file
//

#include "stdafx.h"
#include "VarDBRowSelPPage.h"
#include "ConstantesLib.h"
#include "htmlhelp.h"
#include "Help\VarDBRowSelPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVarDBRowSelPPage dialog
//---------------------------------------------------------------------------

CVarDBRowSelPPage::CVarDBRowSelPPage(CVarDBParam * _pVarDBParam)
        : CPropertyPage(CVarDBRowSelPPage::IDD)
{
    //{{AFX_DATA_INIT(CVarDBRowSelPPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pVarDBParam = _pVarDBParam ;
}

//---------------------------------------------------------------------------

CVarDBRowSelPPage::~CVarDBRowSelPPage()
{}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CVarDBRowSelPPage, CPropertyPage)
//{{AFX_MSG_MAP(CVarDBRowSelPPage)
ON_BN_CLICKED(IDC_ROWSEL_ALL, OnRowselAll)
ON_BN_CLICKED(IDC_ROWSEL_SOME, OnRowselSome)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
// CVarDBRowSelPPage message handlers
//---------------------------------------------------------------------------

BOOL CVarDBRowSelPPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    CSpinButtonCtrl *   ctlMaxRowSpin;
    //---
    ctlMaxRowSpin = static_cast<CSpinButtonCtrl *>( GetDlgItem( IDC_ROWMAX_SPIN ) ) ;
    ctlMaxRowSpin->SetRange32( 0, 0x7FFFFFFF );

    Values2Ctrl() ;

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarDBRowSelPPage::Ctrl2Values()
{
    CButton * pBut ;
    CWnd    * pEdit ;
    CString strValue ;

    pBut = static_cast<CButton*>(GetDlgItem(IDC_ROWSEL_ALL)) ;

    if (pBut->GetCheck())
    {
        m_pVarDBParam->m_lReqSelType = VAR_ROWSEL_ALL ;
        m_pVarDBParam->m_lMaxRowNumber = 0 ;
    }
    else
    {
        m_pVarDBParam-> m_lReqSelType = VAR_ROWSEL_SOME ;
        pEdit = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_EDIT)) ;

        pEdit->GetWindowText(strValue) ;
        m_pVarDBParam->m_lMaxRowNumber = atol( strValue ) ;
    }
}

//---------------------------------------------------------------------------

void CVarDBRowSelPPage::Values2Ctrl()
{
    CButton * pBut ;
    CWnd    * pEdit ;
    CString strValue ;

    if (m_pVarDBParam->m_lReqSelType == VAR_ROWSEL_SOME)
    {
        pBut = static_cast<CButton*>(GetDlgItem(IDC_ROWSEL_SOME)) ;
        pBut->SetCheck(1) ;

        pEdit = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_EDIT)) ;
        pEdit->EnableWindow(TRUE) ;

        strValue.Format("%ld", m_pVarDBParam->m_lReqSelType) ;
        pEdit->SetWindowText( strValue ) ;

        pEdit = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_SPIN)) ;
        pEdit->EnableWindow(TRUE) ;
    }
    else
    {
        pBut = static_cast<CButton*>(GetDlgItem(IDC_ROWSEL_ALL)) ;
        pBut->SetCheck(1) ;

        pEdit = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_EDIT)) ;
        pEdit->EnableWindow(FALSE) ;

        pEdit = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_SPIN)) ;
        pEdit->EnableWindow(FALSE) ;
    }
}

//---------------------------------------------------------------------------

LRESULT CVarDBRowSelPPage::OnWizardBack()
{
    switch( m_pVarDBParam->m_lReqType )
    {
    case VAR_DBSOURCE_TABLE :
        return IDD_VAR_DBCOLSEL_PPAGE ;
        break ;
    case VAR_DBSOURCE_REQUEST :
        return IDD_VAR_DBREQ_PPAGE ;
        break ;
    default :
        ASSERT(FALSE) ;
    }

    return -1 ;
}

//---------------------------------------------------------------------------

LRESULT CVarDBRowSelPPage::OnWizardNext()
{
    Ctrl2Values();

    return 0 ; //next page  in the array of VarPSheet
}

//---------------------------------------------------------------------------


void CVarDBRowSelPPage::OnRowselAll()
{
    CWnd    * pWnd ;

    pWnd = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_EDIT)) ;
    pWnd->EnableWindow(FALSE) ;

    pWnd = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_SPIN)) ;
    pWnd->EnableWindow(FALSE) ;
}

//---------------------------------------------------------------------------

void CVarDBRowSelPPage::OnRowselSome()
{
    CWnd    * pWnd ;

    pWnd = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_EDIT)) ;
    pWnd->EnableWindow(TRUE) ;

    pWnd = static_cast<CWnd*>(GetDlgItem(IDC_ROWMAX_SPIN)) ;
    pWnd->EnableWindow(TRUE) ;
}

//---------------------------------------------------------------------------

HBRUSH CVarDBRowSelPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

BOOL CVarDBRowSelPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp+="::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
