/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XMLDomTreeView.cpp : implementation of the CXMLDomTreeView class
//

#include "stdafx.h"
#include "Modeller.h"
#include "ModellerDoc.h"
#include "Constantes.h"
#include "VarAddrNameDlg.h"
#include "XMLDomTreeView.h"
#include "VariousObject.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




/////////////////////////////////////////////////////////////////////////////
// CXMLDomTreeView
//----------------------------------------------------------------------------

const UINT    WM_FINDREPLACE = RegisterWindowMessage( FINDMSGSTRING );

IMPLEMENT_DYNCREATE(CXMLDomTreeView, CTreeView)

//----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CXMLDomTreeView, CTreeView)
//{{AFX_MSG_MAP(CXMLDomTreeView)
ON_COMMAND(IDM_ADDRESS, OnAddressing)
ON_UPDATE_COMMAND_UI(IDM_SEARCH, OnUpdateSearch)
ON_COMMAND(IDM_SEARCH, OnInitSearch)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
ON_REGISTERED_MESSAGE( WM_FINDREPLACE, OnDoSearch )
	ON_COMMAND(IDM_CLIPBOARD_ADDRESS, OnClipboardAddress)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXMLDomTreeView construction/destruction
//----------------------------------------------------------------------------

CXMLDomTreeView::CXMLDomTreeView():
        m_nidTag(0),
        m_nidAttribute(0),
        m_nidText(0),
		m_nidNPI(0),
		m_nidComment(0),
        m_pFindDlg( NULL ),
		m_hMenuSelection(NULL),
		m_pSB_XMLTree(NULL),
		m_pMB_XMLTree(NULL)
{
    // TODO: add construction code here

}

//----------------------------------------------------------------------------


CXMLDomTreeView::~CXMLDomTreeView()
{
    if( m_pFindDlg != NULL )
    {
        delete m_pFindDlg ;
    }
}

//----------------------------------------------------------------------------

BOOL CXMLDomTreeView::PreCreateWindow(CREATESTRUCT& cs)
{
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs
	cs.style |= TVS_SHOWSELALWAYS;

    return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CXMLDomTreeView drawing
//----------------------------------------------------------------------------

void CXMLDomTreeView::OnDraw(CDC* pDC)
{
    // TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CXMLDomTreeView diagnostics
//----------------------------------------------------------------------------


#ifdef _DEBUG
void CXMLDomTreeView::AssertValid() const
{
    CTreeView::AssertValid();
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::Dump(CDumpContext& dc) const
{
    CTreeView::Dump(dc);
}

//----------------------------------------------------------------------------

#endif //_DEBUG


//----------------------------------------------------------------------------

void CXMLDomTreeView::InitTreeCtrlDisplay()
{
    CBitmap bmpTag;
    CBitmap bmpAttribute;
    CBitmap bmpText;
	CBitmap bmpNPI;
	CBitmap bmpComment;
    //----


    m_ImageList.Create( 16, 16, ILC_COLOR, 1, 1 );

    VERIFY(bmpTag.LoadBitmap(IDB_TAG));
    m_nidTag = m_ImageList.Add(&bmpTag, RGB(0,0,0));
    bmpTag.DeleteObject();


    VERIFY(bmpAttribute.LoadBitmap(IDB_ATTRIBUTE));
    m_nidAttribute = m_ImageList.Add(&bmpAttribute, RGB(0,0,0));
    bmpAttribute.DeleteObject();

    VERIFY(bmpText.LoadBitmap(IDB_TEXT));
    m_nidText = m_ImageList.Add(&bmpText, RGB(0,0,0));
    bmpText.DeleteObject();

    VERIFY(bmpNPI.LoadBitmap(IDB_NPI));
    m_nidNPI = m_ImageList.Add(&bmpNPI, RGB(0,0,0));
    bmpNPI.DeleteObject();

    VERIFY(bmpComment.LoadBitmap(IDB_COMMENT));
    m_nidComment = m_ImageList.Add(&bmpComment, RGB(0,0,0));
    bmpComment.DeleteObject();

    GetTreeCtrl().SetImageList(&m_ImageList , TVSIL_NORMAL);
	GetTreeCtrl().ModifyStyle(0,TVS_HASLINES|TVS_HASBUTTONS|TVS_LINESATROOT);
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_DISPLAY_DOM_URL )
    {
		CLongObject *pObj =  (CLongObject*)pHint;
        GetTreeCtrl().DeleteAllItems() ;
		if(pObj->m_lObject)
        {
			ShowDom();
			ExpandAllChildren(TVI_ROOT);
		}
	}
    else
        CTreeView::OnUpdate(pSender, lHint, pHint);
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::ShowDom()
{
	LPBYTE pBody;
	DWORD dwBodyLen;
	CModellerDoc* pDoc = static_cast<CModellerDoc*> (GetDocument() );
	if(!pDoc)
		return;

	pDoc->GetHtmlBody(pBody,&dwBodyLen);

	if(m_pMB_XMLTree)
	{
		delete m_pMB_XMLTree;
		m_pMB_XMLTree = NULL;
	}

	if(m_pSB_XMLTree)
	{
		delete m_pSB_XMLTree;
		m_pSB_XMLTree = NULL;
	}

	m_mapItemToType.RemoveAll();

	//Make a copy of buffer since normalizebuffer can modify it
	LPBYTE pBodyCopy = new BYTE[dwBodyLen+1];
	memcpy(pBodyCopy,pBody,dwBodyLen);
	pBodyCopy[dwBodyLen] = NULL;
	LPBYTE pBodyCopy2 = pBodyCopy;
			
	BOOL bNewBuffer=false;
	DWORD dwOffset=0;
	XmlParse<char>::XML_BUFFER_TYPE eBufferType;
	if(XmlParse<char>::NormalizeBuffer(pBodyCopy,NULL,eBufferType,bNewBuffer,dwOffset))
	{
		if(eBufferType == XmlParse<char>::OCTET_BUFFER)
		{
			XmlParse<char> parser((const char*)(pBodyCopy+dwOffset));
			m_pSB_XMLTree = new XmlParse<char>::XMLTree;
			try
			{
				parser.Parse(m_pSB_XMLTree);
			}
			catch(const XmlParse<char>::LexicalError& e)
			{
				CString str;
				str.Format("XML Lexical Error (S %d,%d)",e.m_dwLine,e.m_dwCol);
				AfxMessageBox(str);
			}
			catch(const XmlParse<char>::SyntaxError& e)
			{
				CString str;
				str.Format("XML Syntax Error (S %d,%d)",e.m_dwLine,e.m_dwCol);
				AfxMessageBox(str);
			}
		}
		else
		{
			XmlParse<wchar_t> parser((const wchar_t*)(pBodyCopy+dwOffset));
			m_pMB_XMLTree = new XmlParse<wchar_t>::XMLTree;
			try
			{
				parser.Parse(m_pMB_XMLTree);
			}
			catch(const XmlParse<wchar_t>::LexicalError& e)
			{
				CString str;
				str.Format("XML Lexical Error (W %d,%d)",e.m_dwLine,e.m_dwCol);
				AfxMessageBox(str);
			}
			catch(const XmlParse<wchar_t>::SyntaxError& e)
			{
				CString str;
				str.Format("XML Syntax Error (W %d,%d)",e.m_dwLine,e.m_dwCol);
				AfxMessageBox(str);
			}
		}
		if(bNewBuffer)
			XmlParse<char>::FreeBuf(pBodyCopy);
	}

	if(m_pSB_XMLTree && m_pSB_XMLTree->m_root.m_pRootElement)
	{
		InsertTag(m_pSB_XMLTree->m_root.m_pRootElement,TVI_ROOT);
	}
	else
	if(m_pMB_XMLTree && m_pMB_XMLTree->m_root.m_pRootElement)
	{
		InsertTag(m_pMB_XMLTree->m_root.m_pRootElement,TVI_ROOT);
	}

//	if(dwBodyLen)
//		delete []pBodyCopy2;
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::InsertTag(XmlParse<char>::XMLTree::NodeElement *pNode,HTREEITEM hParent)
{
	HTREEITEM hItem = InsertTagInTree( pNode,hParent);

	typedef XmlParse<char>::XMLTree::NodeElement::ChildElementIterator ChildElementIterator_;
	
	for(ChildElementIterator_ it = pNode->m_children.begin(); it!=pNode->m_children.end();it++)
	{
		if(dynamic_cast<XmlParse<char>::XMLTree::NodeComment*>(*it))
			InsertComment((XmlParse<char>::XMLTree::NodeComment*)(*it),hItem);

		if(dynamic_cast<XmlParse<char>::XMLTree::NodeText*>(*it))
			InsertText((XmlParse<char>::XMLTree::NodeText*)(*it),hItem);

		if(dynamic_cast<XmlParse<char>::XMLTree::NodeProcessingInstruction*>(*it))
			InsertNPI((XmlParse<char>::XMLTree::NodeProcessingInstruction*)(*it),hItem);

		if(dynamic_cast<XmlParse<char>::XMLTree::NodeElement*>(*it))
			InsertTag((XmlParse<char>::XMLTree::NodeElement*)(*it),hItem);
	}
}

void CXMLDomTreeView::InsertTag( XmlParse<wchar_t>::XMLTree::NodeElement* pNode ,HTREEITEM hParent)
{
	HTREEITEM hItem = InsertTagInTree( pNode,hParent);

	typedef XmlParse<wchar_t>::XMLTree::NodeElement::ChildElementIterator ChildElementIterator_;
	
	for(ChildElementIterator_ it = pNode->m_children.begin(); it!=pNode->m_children.end();it++)
	{
		if(dynamic_cast<XmlParse<wchar_t>::XMLTree::NodeComment*>(*it))
			InsertComment((XmlParse<wchar_t>::XMLTree::NodeComment*)(*it),hItem);

		if(dynamic_cast<XmlParse<wchar_t>::XMLTree::NodeText*>(*it))
			InsertText((XmlParse<wchar_t>::XMLTree::NodeText*)(*it),hItem);

		if(dynamic_cast<XmlParse<wchar_t>::XMLTree::NodeProcessingInstruction*>(*it))
			InsertNPI((XmlParse<wchar_t>::XMLTree::NodeProcessingInstruction*)(*it),hItem);

		if(dynamic_cast<XmlParse<wchar_t>::XMLTree::NodeElement*>(*it))
			InsertTag((XmlParse<wchar_t>::XMLTree::NodeElement*)(*it),hItem);
	}
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::InsertComment(XmlParse<char>::XMLTree::NodeComment* pNode,HTREEITEM hParent)
{
    HTREEITEM hItem = GetTreeCtrl().InsertItem(pNode->StringValue().c_str(),m_nidComment, m_nidComment,hParent);
	GetTreeCtrl().SetItemData(hItem,(DWORD)pNode);
	m_mapItemToType.SetAt(hItem,enComment);
}

void CXMLDomTreeView::InsertComment(XmlParse<wchar_t>::XMLTree::NodeComment* pNode,HTREEITEM hParent)
{
//    HTREEITEM hItem = GetTreeCtrl().InsertItem(pNode->StringValue().c_str(),m_nidComment, m_nidComment,hParent);
}

//----------------------------------------------------------------------------

HTREEITEM CXMLDomTreeView::InsertTagInTree(XmlParse<char>::XMLTree::NodeElement* pNode,HTREEITEM hParent)
{
    DWORD dwParent = 0L;

    HTREEITEM hItem = GetTreeCtrl().InsertItem(((XmlParse<char>::XmlString)pNode->m_strValue).c_str(),m_nidTag, m_nidTag,hParent);
	GetTreeCtrl().SetItemData(hItem,(DWORD)pNode);
	m_mapItemToType.SetAt(hItem,enTag);

	InsertAttribute( pNode ,hItem) ;
	return hItem;
}

HTREEITEM CXMLDomTreeView::InsertTagInTree(XmlParse<wchar_t>::XMLTree::NodeElement* pNode ,HTREEITEM hParent)
{
	HTREEITEM hItem = NULL;
/*	DWORD dwLen = ((XmlParse<wchar_t>::XmlString)pNode->m_strValue).size();
	CString str;

	WideCharToMultiByte(CP_ACP, 0,(LPCWSTR)((XmlParse<wchar_t>::XmlString)pNode->m_strValue).c_str(),dwLen,
                       str.GetBufferSetLength(dwLen/2),dwLen/2, NULL, NULL );
*/
	return hItem;
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::InsertAttribute(XmlParse<char>::XMLTree::NodeElement* pNode,HTREEITEM hParent)
{
	typedef XmlParse<char>::XMLTree::Attributes::const_iterator AttributeIterator;
	typedef XmlParse<char>::XMLTree::NodeAttribute* Attribute_;

	int n=0;
	CString strLabel;
	for(AttributeIterator it = pNode->m_attributes.begin(); it!=pNode->m_attributes.end();it++)
	{
		strLabel.Format("%s : %s",((XmlParse<char>::XmlString)((Attribute_)(*it))->m_strName).c_str(),
				((XmlParse<char>::XmlString)((Attribute_)(*it))->m_strValue).c_str());

		HTREEITEM hItem = GetTreeCtrl().InsertItem(strLabel,m_nidAttribute,m_nidAttribute,hParent);
		GetTreeCtrl().SetItemData(hItem,(DWORD)(*it));
		m_mapItemToType.SetAt(hItem,enAttribute);
	}
}

void CXMLDomTreeView::InsertAttribute(XmlParse<wchar_t>::XMLTree::NodeElement* pNode,HTREEITEM hParent) 
{
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::InsertText(XmlParse<char>::XMLTree::NodeText* pNode,HTREEITEM hParent)
{
    HTREEITEM hItem = GetTreeCtrl().InsertItem(pNode->StringValue().c_str(),m_nidText, m_nidText,hParent);
	GetTreeCtrl().SetItemData(hItem,(DWORD)pNode);
	m_mapItemToType.SetAt(hItem,enText);
}

void CXMLDomTreeView::InsertText(XmlParse<wchar_t>::XMLTree::NodeText* pNode,HTREEITEM hParent)
{

}

//----------------------------------------------------------------------------

void CXMLDomTreeView::InsertNPI( XmlParse<char>::XMLTree::NodeProcessingInstruction* pNode ,HTREEITEM hParent)
{
	CString strLabel;
	strLabel.Format("%s : %s",((XmlParse<char>::XmlString)pNode->m_strTarget).c_str(),pNode->StringValue().c_str());

    HTREEITEM hItem = GetTreeCtrl().InsertItem(strLabel,m_nidNPI, m_nidNPI,hParent);
	GetTreeCtrl().SetItemData(hItem,(DWORD)pNode);
	m_mapItemToType.SetAt(hItem,enNPI);
}
	
void CXMLDomTreeView::InsertNPI( XmlParse<wchar_t>::XMLTree::NodeProcessingInstruction* pNode ,HTREEITEM hParent)
{
}
//----------------------------------------------------------------------------

void CXMLDomTreeView::OnInitialUpdate()
{
    CTreeView::OnInitialUpdate();

    InitTreeCtrlDisplay() ;

	ShowDom();
	ExpandAllChildren(TVI_ROOT);
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::OnAddressing()
{
	if(!m_hMenuSelection)
		return;

	NodeType nType;
	m_mapItemToType.Lookup(m_hMenuSelection,nType);
	DWORD dwItemData = GetTreeCtrl().GetItemData(m_hMenuSelection);
	m_hMenuSelection=NULL;
	if(!dwItemData)
		return;

    CVarResponse    VarResponse ;
    CModellerDoc *  pDoc ;
    //---
    VarResponse.m_ResponseType = RT_BODY ;

    CVarAddrNameDlg Dlg( &VarResponse ) ;
    if( Dlg.DoModal() == IDCANCEL )
    {
		return;
	}

	VarResponse.m_strIdentifier = "XML:";
	if(m_pSB_XMLTree)
	{
		if(nType == enAttribute)
		{
			XmlParse<char>::XMLTree::NodeAttribute* pNode = (XmlParse<char>::XMLTree::NodeAttribute *)dwItemData;
			VarResponse.m_strIdentifier += pNode->GenerateAddress().c_str();
		}
		else
		{
			XmlParse<char>::XMLTree::NodeChild* pNode = (XmlParse<char>::XMLTree::NodeChild *)dwItemData;
			VarResponse.m_strIdentifier += pNode->GenerateAddress().c_str();
		}
	}
	else
	{
		if(nType == enAttribute)
		{
			XmlParse<wchar_t>::XMLTree::NodeAttribute* pNode = (XmlParse<wchar_t>::XMLTree::NodeAttribute *)dwItemData;
			VarResponse.m_strIdentifier += pNode->GenerateAddress().c_str();
		}
		else
		{
			XmlParse<wchar_t>::XMLTree::NodeChild* pNode = (XmlParse<wchar_t>::XMLTree::NodeChild *)dwItemData;
			VarResponse.m_strIdentifier += pNode->GenerateAddress().c_str();
		}
	}
	
    pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->Adressing( &VarResponse ) ;
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::OnUpdateSearch(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( TRUE ) ;
}

//----------------------------------------------------------------------------

void CXMLDomTreeView::OnInitSearch()
{
    m_pFindDlg = new CFindReplaceDialog();  // Must be created on the heap

    m_pFindDlg->m_fr.lStructSize    = sizeof(FINDREPLACE);
    m_pFindDlg->m_fr.hwndOwner      = this->m_hWnd;
    m_pFindDlg->Create( TRUE, "", "", FR_DOWN, this );
}

//----------------------------------------------------------------------------

LRESULT CXMLDomTreeView::OnDoSearch( WPARAM w , LPARAM pText )
{
    HTREEITEM       hFoundItem ;
    //---
    if( !m_pFindDlg->IsTerminating() )
    {
        hFoundItem      = FindItem( m_pFindDlg->GetFindString(),
                                    m_pFindDlg->MatchCase(),
                                    m_pFindDlg->SearchDown(),
                                    m_pFindDlg->MatchWholeWord() ) ;

        if( hFoundItem != NULL )
        {
            GetTreeCtrl().SelectItem(   hFoundItem ) ;
        }
    }
    else
    {
        m_pFindDlg = NULL ;
    }

    return 0 ;
}

//----------------------------------------------------------------------------
// FindItem         - Finds an item that contains the search string
// Returns          - Handle to the item or NULL
// str              - String to search for
// bCaseSensitive   - Should the search be case sensitive
// bDownDir         - Search direction - TRUE for down
// bWholeWord       - True if search should match whole words
// hItem            - Item to start searching from. NULL for
//                  - currently selected item
HTREEITEM CXMLDomTreeView::FindItem(   CString &str,
                                    BOOL bCaseSensitive /*= FALSE*/,
                                    BOOL bDownDir       /*= TRUE*/,
                                    BOOL bWholeWord     /*= FALSE*/,
                                    HTREEITEM hItem     /*= NULL*/)
{
    int lenSearchStr = str.GetLength();
    if( lenSearchStr == 0 ) return NULL;

    HTREEITEM htiSel = hItem ? hItem : GetTreeCtrl().GetSelectedItem();
    HTREEITEM htiCur = bDownDir ? GetNextItem( htiSel ) : GetPrevItem( htiSel );
    CString sSearch = str;

    if( htiCur == NULL && htiSel != NULL) // make sure it ends if we
        // started with no selection
    {
        if( bDownDir )
            htiCur = GetTreeCtrl().GetRootItem();
        else
            htiCur = GetLastItem( NULL );
    }

    if( !bCaseSensitive )
        sSearch.MakeLower();

    while( htiCur && htiCur != htiSel )
    {
        CString sItemText = GetTreeCtrl().GetItemText( htiCur );
        if( !bCaseSensitive )
            sItemText.MakeLower();

        int n;
        while( (n = sItemText.Find( sSearch )) != -1 )
        {
            // Search string found
            if( bWholeWord )
            {
                // Check preceding char
                if( n != 0 )
                {
                    if( isalpha(sItemText[n-1]) ||
                            sItemText[n-1] == '_' )
                    {
                        // Not whole word
                        sItemText = sItemText.Right(
                                        sItemText.GetLength() - n -
                                        lenSearchStr );
                        continue;
                    }
                }

                // Check succeeding char
                if( sItemText.GetLength() > n + lenSearchStr
                        && ( isalpha(sItemText[n+lenSearchStr]) ||
                             sItemText[n+lenSearchStr] == '_' ) )
                {
                    // Not whole word
                    sItemText = sItemText.Right( sItemText.GetLength()
                                                 - n - sSearch.GetLength() );
                    continue;
                }
            }

            if( IsFindValid( htiCur ) )
                return htiCur;
            else break;
        }


        htiCur = bDownDir ? GetNextItem( htiCur ) : GetPrevItem( htiCur );
        if( htiCur == NULL && htiSel != NULL)   // make sure it ends if we
            // started with no selection
        {
            if( bDownDir )
                htiCur = GetTreeCtrl().GetRootItem();
            else
                htiCur = GetLastItem( NULL );
        }
    }
    return NULL;
}

//----------------------------------------------------------------------------

// IsFindValid  - Virtual function used by FindItem to allow this
//        function to filter the result of FindItem
// Returns  - True if item matches the criteria
// Arg      - Handle of the item
BOOL CXMLDomTreeView::IsFindValid( HTREEITEM )
{
    return TRUE;
}

//----------------------------------------------------------------------------
// GetNextItem  - Get next item as if outline was completely expanded
// Returns      - The item immediately below the reference item
// hItem        - The reference item
//----------------------------------------------------------------------------
HTREEITEM CXMLDomTreeView::GetNextItem(HTREEITEM hItem)
{
    HTREEITEM hti = NULL;

    if ( GetTreeCtrl().ItemHasChildren(hItem))
        hti = GetTreeCtrl().GetChildItem(hItem);

    if (hti == NULL)
    {
        while ((hti = GetTreeCtrl().GetNextSiblingItem(hItem)) == NULL)
        {
            if ((hItem = GetTreeCtrl().GetParentItem(hItem)) == NULL)
                return NULL;
        }
    }
    return hti;
}

//----------------------------------------------------------------------------
// GetNextItem  - Get previous item as if outline was completely expanded
// Returns              - The item immediately above the reference item
// hItem                - The reference item
//----------------------------------------------------------------------------
HTREEITEM CXMLDomTreeView::GetPrevItem( HTREEITEM hItem )
{
    HTREEITEM       hti;

    hti = GetTreeCtrl().GetPrevSiblingItem(hItem);
    if( hti == NULL )
        hti = GetTreeCtrl().GetParentItem(hItem);
    else
        hti = GetLastItem(hti);
    return hti;
}

//----------------------------------------------------------------------------
// GetLastItem  - Gets last item in the branch
// Returns      - Last item
// hItem        - Node identifying the branch. NULL will
//                return the last item in outine
//----------------------------------------------------------------------------
HTREEITEM CXMLDomTreeView::GetLastItem( HTREEITEM hItem )
{
    // Last child of the last child of the last child ...
    HTREEITEM htiNext;

    if( hItem == NULL )
    {
        // Get the last item at the top level
        htiNext = GetTreeCtrl().GetRootItem();
        while( htiNext )
        {
            hItem = htiNext;
            htiNext = GetTreeCtrl().GetNextSiblingItem( htiNext );
        }
    }

    while( GetTreeCtrl().ItemHasChildren( hItem ) )
    {
        htiNext = GetTreeCtrl().GetChildItem( hItem );
        while( htiNext )
        {
            hItem = htiNext;
            htiNext = GetTreeCtrl().GetNextSiblingItem( htiNext );
        }
    }

    return hItem;
}

void CXMLDomTreeView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (point.x == -1 && point.y == -1)
		point = (CPoint) GetMessagePos();

	ScreenToClient(&point);

	CMenu menu;
	menu.LoadMenu(IDR_DOM_VIEW);
    CMenu* pPopup = menu.GetSubMenu(0);

	CModellerDoc *  pDoc = static_cast<CModellerDoc*>(GetDocument());
	if (pDoc)
	{
		if (pDoc->getNavigateMode() == ENM_NAVIGATE)
		{
			pPopup->RemoveMenu(IDM_ADDRESS,MF_BYCOMMAND);
		}
	}

	UINT uFlags;
	m_hMenuSelection = GetTreeCtrl().HitTest( point, &uFlags );
	if(m_hMenuSelection == NULL || GetTreeCtrl().GetItemData(m_hMenuSelection)==0)
	{
		pPopup->RemoveMenu(IDM_ADDRESS,MF_BYCOMMAND);
		pPopup->RemoveMenu(IDM_CLIPBOARD_ADDRESS,MF_BYCOMMAND);
		m_hMenuSelection = NULL;
	}
	
	ClientToScreen(&point);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y,this);
}

void CXMLDomTreeView::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Send WM_CONTEXTMENU to self
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());
	// Mark message as handled and suppress default handling
	*pResult = 1;
}

void CXMLDomTreeView::ExpandAllChildren(HTREEITEM hItem)
{
	HTREEITEM hChild = GetTreeCtrl().GetChildItem(hItem);
	while (hChild != NULL)
	{
		GetTreeCtrl().Expand(hChild,TVE_EXPAND);
		ExpandAllChildren(hChild);
		hChild = GetTreeCtrl().GetNextItem(hChild, TVGN_NEXT);
	}
}

void CXMLDomTreeView::OnClipboardAddress() 
{
	if(!m_hMenuSelection)
		return;

	NodeType nType;
	m_mapItemToType.Lookup(m_hMenuSelection,nType);
	DWORD dwItemData = GetTreeCtrl().GetItemData(m_hMenuSelection);
	m_hMenuSelection=NULL;
	if(!dwItemData)
		return;

	CString strIdentifier = "XML:";
	if(m_pSB_XMLTree)
	{
		if(nType == enAttribute)
		{
			XmlParse<char>::XMLTree::NodeAttribute* pNode = (XmlParse<char>::XMLTree::NodeAttribute *)dwItemData;
			strIdentifier += pNode->GenerateAddress().c_str();
		}
		else
		{
			XmlParse<char>::XMLTree::NodeChild* pNode = (XmlParse<char>::XMLTree::NodeChild *)dwItemData;
			strIdentifier += pNode->GenerateAddress().c_str();
		}
	}
	else
	{
		if(nType == enAttribute)
		{
			XmlParse<wchar_t>::XMLTree::NodeAttribute* pNode = (XmlParse<wchar_t>::XMLTree::NodeAttribute *)dwItemData;
			strIdentifier += pNode->GenerateAddress().c_str();
		}
		else
		{
			XmlParse<wchar_t>::XMLTree::NodeChild* pNode = (XmlParse<wchar_t>::XMLTree::NodeChild *)dwItemData;
			strIdentifier += pNode->GenerateAddress().c_str();
		}
	}
	if (OpenClipboard())
	{
		EmptyClipboard() ;
		HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE , strIdentifier.GetLength() + 1);
		char * pAddress = (char *)GlobalLock(hGlobal) ;
		lstrcpy(pAddress, LPCTSTR(strIdentifier));
		GlobalUnlock(hGlobal);
		SetClipboardData(CF_TEXT, hGlobal) ;
		CloseClipboard() ; 
	}

}
